@@ -1,14 +1,14 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2002
+// Copyright Bob Friesenhahn, 1999, 2002, 2003
 //
 // Simple C++ function wrappers for ImageMagick equivalents
 //
 
-#define MAGICK_IMPLEMENTATION
+#define MAGICKCORE_IMPLEMENTATION  1
 
+#include "Magick++/Include.h"
 #include <string>
-#include <iostream>
 
 using namespace std;
 
@@ -17,6 +17,5 @@ using namespace std;
 // Clone C++ string as allocated C string, de-allocating any existing string
 void Magick::CloneString( char **destination_, const std::string &source_ )
 {
-  CloneString( destination_, source_.c_str() );
+  MagickCore::CloneString( destination_, source_.c_str() );
 }
-
