@@ -96,8 +96,8 @@ namespace Magick
     Color           fillColor ( void ) const;
 
     // Fill pattern
-    void            fillPattern ( const MagickLib::Image *fillPattern_ );
-    const MagickLib::Image* fillPattern ( void  ) const;
+    void            fillPattern ( const MagickCore::Image *fillPattern_ );
+    const MagickCore::Image* fillPattern ( void  ) const;
 
     // Rule to use when filling drawn objects
     void            fillRule ( const FillRule &fillRule_ );
@@ -208,8 +208,8 @@ namespace Magick
     unsigned int    strokeMiterLimit ( void ) const;
 
     // Pattern image to use for stroked outlines
-    void            strokePattern ( const MagickLib::Image *strokePattern_ );
-    const MagickLib::Image* strokePattern ( void  ) const;
+    void            strokePattern ( const MagickCore::Image *strokePattern_ );
+    const MagickCore::Image* strokePattern ( void  ) const;
 
    // Stroke width for drawing vector objects (default one)
     void            strokeWidth ( double strokeWidth_ );
@@ -266,14 +266,14 @@ namespace Magick
     // Internal implementation methods.  Please do not use.
     //
 
-    MagickLib::DrawInfo*       drawInfo( void );
-    MagickLib::ImageInfo *     imageInfo( void );
-    MagickLib::QuantizeInfo *  quantizeInfo( void );
+    MagickCore::DrawInfo*       drawInfo( void );
+    MagickCore::ImageInfo *     imageInfo( void );
+    MagickCore::QuantizeInfo *  quantizeInfo( void );
 
     // Construct using raw structures
-    Options( const MagickLib::ImageInfo* imageInfo_,
-             const MagickLib::QuantizeInfo* quantizeInfo_,
-             const MagickLib::DrawInfo* drawInfo_ );
+    Options( const MagickCore::ImageInfo* imageInfo_,
+             const MagickCore::QuantizeInfo* quantizeInfo_,
+             const MagickCore::DrawInfo* drawInfo_ );
 
   protected:
 
@@ -282,9 +282,9 @@ namespace Magick
     // Assignment not supported
     Options& operator= ( const Options& );
 
-    MagickLib::ImageInfo*        _imageInfo;
-    MagickLib::QuantizeInfo*     _quantizeInfo;
-    MagickLib::DrawInfo*         _drawInfo;
+    MagickCore::ImageInfo*        _imageInfo;
+    MagickCore::QuantizeInfo*     _quantizeInfo;
+    MagickCore::DrawInfo*         _drawInfo;
   };
 } // namespace Magick
 
