@@ -10,7 +10,7 @@
 %                            A   A    V    SSSSS                              %
 %                                                                             %
 %                                                                             %
-%                        Read/Write AVS X Image Format.                       %
+%                        Read/Write AVS X Image Format                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -51,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -83,7 +84,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -131,7 +132,7 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -143,6 +144,8 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   width=ReadBlobMSBLong(image);
   height=ReadBlobMSBLong(image);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   if ((width == 0UL) || (height == 0UL))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   do
@@ -187,15 +190,9 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
       if (SyncImagePixels(image) == MagickFalse)
         break;
-      if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+      if ((image->previous == (Image *) NULL) &&
+          (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse))
+        break;
     }
     pixels=(unsigned char *) RelinquishMagickMemory(pixels);
     if (EOFBlob(image) != MagickFalse)
@@ -217,23 +214,20 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while ((width != 0UL) && (height != 0UL));
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -316,7 +310,7 @@ ModuleExport void UnregisterAVSImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -360,10 +354,10 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
     /*
       Write AVS header.
     */
-    if (image_info->colorspace == UndefinedColorspace)
+    if (image->colorspace != RGBColorspace)
       (void) SetImageColorspace(image,RGBColorspace);
-    (void) WriteBlobMSBLong(image,image->columns);
-    (void) WriteBlobMSBLong(image,image->rows);
+    (void) WriteBlobMSBLong(image,(unsigned int) image->columns);
+    (void) WriteBlobMSBLong(image,(unsigned int) image->rows);
     /*
       Allocate memory for pixels.
     */
@@ -390,29 +384,19 @@ static MagickBooleanType WriteAVSImage(const ImageInfo *image_info,Image *image)
         p++;
       }
       (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-      if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(SaveImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+      if ((image->previous == (Image *) NULL) &&
+          (SetImageProgress(image,SaveImageTag,y,image->rows) == MagickFalse))
+        break;
     }
     pixels=(unsigned char *) RelinquishMagickMemory(pixels);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
