@@ -10,7 +10,7 @@
 %                            DDDD   P      X   X                              %
 %                                                                             %
 %                                                                             %
-%                     Read/Write SMTPE DPX Image Format.                      %
+%                     Read/Write SMTPE DPX Image Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -52,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/profile.h"
 #include "magick/property.h"
@@ -363,7 +364,7 @@ static MagickBooleanType IsDPX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -557,7 +558,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     offset;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
 
   QuantumType
     quantum_type;
@@ -591,7 +592,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -1004,7 +1005,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   /*
@@ -1015,9 +1016,9 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
-  GetQuantumInfo(image_info,&quantum_info);
-  quantum_info.quantum=32;
-  quantum_info.pack=dpx.image.image_element[0].packing == 0 ? MagickTrue :
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  quantum_info->quantum=32;
+  quantum_info->pack=dpx.image.image_element[0].packing == 0 ? MagickTrue :
     MagickFalse;
   samples_per_pixel=1;
   quantum_type=GrayQuantum;
@@ -1054,6 +1055,11 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image->colorspace=Rec709YCbCrColorspace;
       break;
     }
+    case LumaComponentType:
+    {
+      image->colorspace=RGBColorspace;
+      break;
+    }
     default:
     {
       image->colorspace=RGBColorspace;
@@ -1062,14 +1068,17 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     }
   }
+  if (image->colorspace != UndefinedColorspace)
+    image->colorspace=image->colorspace;
   length=GetBytesPerRow(image->columns,samples_per_pixel,image->depth,
     dpx.image.image_element[0].packing == 0 ? MagickFalse : MagickTrue);
-  pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
-  if (pixels == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  if ((quantum_type == GrayQuantum) && (quantum_info.pack == MagickFalse) &&
+  pixels=GetQuantumPixels(quantum_info);
+  if ((quantum_type == GrayQuantum) && (quantum_info->pack == MagickFalse) &&
       (image->depth == 10) && (image->endian == MSBEndian))
     {
+      QuantumAny
+        scale;
+
       register long
         x;
 
@@ -1078,6 +1087,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       pixel=0UL;
       i=0;
+      scale=GetQuantumScale(image->depth);
       for (y=0; y < (long) image->rows; y++)
       {
         q=SetImagePixels(image,0,y,image->columns,1);
@@ -1090,17 +1100,19 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             case 0:
             {
               pixel=ReadBlobMSBLong(image);
-              q->red=ScaleAnyToQuantum((pixel >> 0) & 0x3ff,image->depth);
+              q->red=ScaleAnyToQuantum((pixel >> 0) & 0x3ff,image->depth,scale);
               break;
             }
             case 1:
             {
-              q->red=ScaleAnyToQuantum((pixel >> 10) & 0x3ff,image->depth);
+              q->red=ScaleAnyToQuantum((pixel >> 10) & 0x3ff,image->depth,
+                scale);
               break;
             }
             case 2:
             {
-              q->red=ScaleAnyToQuantum((pixel >> 20) & 0x3ff,image->depth);
+              q->red=ScaleAnyToQuantum((pixel >> 20) & 0x3ff,image->depth,
+                scale);
               break;
             }
           }
@@ -1110,15 +1122,9 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
     }
   else
@@ -1133,26 +1139,19 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       count=ReadBlob(image,length,pixels);
       if ((size_t) count != length)
         break;
-      status=ExportQuantumPixels(image,&quantum_info,quantum_type,pixels);
-      if (status == MagickFalse)
-        break;
+      (void) ImportQuantumPixels(image,quantum_info,quantum_type,pixels);
       if (SyncImagePixels(image) == MagickFalse)
         break;
-      if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+      status=SetImageProgress(image,LoadImageTag,y,image->rows);
+      if (status == MagickFalse)
+        break;
     }
-  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+  SetQuantumImageType(image,quantum_type);
+  quantum_info=DestroyQuantumInfo(quantum_info);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -1245,7 +1244,7 @@ ModuleExport void UnregisterDPXImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -1319,7 +1318,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     geometry_info;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
 
   QuantumType
     quantum_type;
@@ -1587,7 +1586,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   *dpx.film.prefix='\0';
   value=GetDPXProperty(image_info,image,"dpx:film.prefix");
   if (value != (const char *) NULL)
-    (void) CopyMagickString(dpx.film.offset,value,sizeof(dpx.film.prefix));
+    (void) CopyMagickString(dpx.film.prefix,value,sizeof(dpx.film.prefix));
   offset+=WriteBlob(image,sizeof(dpx.film.prefix),(unsigned char *)
     dpx.film.prefix);
   *dpx.film.count='\0';
@@ -1737,9 +1736,9 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   /*
     Convert pixel packets to DPX raster image.
   */
-  GetQuantumInfo(image_info,&quantum_info);
-  quantum_info.quantum=32;
-  quantum_info.pack=dpx.image.image_element[0].packing == 0 ? MagickTrue :
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  quantum_info->quantum=32;
+  quantum_info->pack=dpx.image.image_element[0].packing == 0 ? MagickTrue :
     MagickFalse;
   quantum_type=RGBQuantum;
   length=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
@@ -1748,22 +1747,21 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
       quantum_type=GrayQuantum;
       length=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
     }
-  pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
-  if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  pixels=GetQuantumPixels(quantum_info);
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    status=ImportQuantumPixels(image,&quantum_info,quantum_type,pixels);
-    if (status == MagickFalse)
-      break;
+    (void) ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
     count=WriteBlob(image,length,pixels);
     if (count != (ssize_t) length)
       break;
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  CloseBlob(image);
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  (void) CloseBlob(image);
   return(status);
 }
