@@ -10,7 +10,7 @@
 %                            F      P      X   X                              %
 %                                                                             %
 %                                                                             %
-%                     Read/Write FlashPIX Image Format.                       %
+%                     Read/Write FlashPIX Image Format                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -56,6 +56,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/pixel.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -134,7 +135,7 @@ static MagickBooleanType IsFPX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -194,7 +195,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     memory_limit;
 
   unsigned char
-    *scanline;
+    *pixels;
 
   unsigned int
     height,
@@ -215,14 +216,14 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
@@ -369,7 +370,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Create linear colormap.
       */
-      if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
+      if (AcquireImageColormap(image,MaxColormapSize) == MagickFalse)
         {
           FPX_ClearSystem();
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -391,9 +392,9 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate memory for the image and pixel buffer.
   */
-  scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
-    (tile_height+1UL)*colorspace.numberOfComponents*sizeof(*scanline));
-  if (scanline == (unsigned char *) NULL)
+  pixels=(unsigned char *) AcquireQuantumMemory(image->columns,(tile_height+
+    1UL)*colorspace.numberOfComponents*sizeof(*pixels));
+  if (pixels == (unsigned char *) NULL)
     {
       FPX_ClearSystem();
       (void) FPX_CloseImage(flashpix);
@@ -409,15 +410,15 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     fpx_info.components[i].horzSubSampFactor=1;
     fpx_info.components[i].vertSubSampFactor=1;
     fpx_info.components[i].columnStride=fpx_info.numberOfComponents;
-    fpx_info.components[i].lineStride=
-      image->columns*fpx_info.components[i].columnStride;
-    fpx_info.components[i].theData=scanline+i;
+    fpx_info.components[i].lineStride=image->columns*
+      fpx_info.components[i].columnStride;
+    fpx_info.components[i].theData=pixels+i;
   }
-  fpx_info.components[0].myColorType.myColor=
-    fpx_info.numberOfComponents > 2 ? NIFRGB_R : MONOCHROME;
+  fpx_info.components[0].myColorType.myColor=fpx_info.numberOfComponents > 2 ?
+    NIFRGB_R : MONOCHROME;
   red_component=(&fpx_info.components[0]);
-  fpx_info.components[1].myColorType.myColor=
-    fpx_info.numberOfComponents > 2 ? NIFRGB_G : ALPHA;
+  fpx_info.components[1].myColorType.myColor=fpx_info.numberOfComponents > 2 ?
+    NIFRGB_G : ALPHA;
   green_component=(&fpx_info.components[1]);
   fpx_info.components[2].myColorType.myColor=NIFRGB_B;
   blue_component=(&fpx_info.components[2]);
@@ -448,14 +449,14 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             (long) tile_height,&fpx_info);
         if (fpx_status == FPX_LOW_MEMORY_ERROR)
           {
-            scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
             (void) FPX_CloseImage(flashpix);
             FPX_ClearSystem();
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
       }
     /*
-      Transfer a FPX scanline.
+      Transfer a FPX pixels.
     */
     r=red_component->theData+(y % tile_height)*red_component->lineStride;
     g=green_component->theData+(y % tile_height)*green_component->lineStride;
@@ -487,16 +488,11 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,LoadImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
   return(GetFirstImageInList(image));
@@ -591,7 +587,7 @@ ModuleExport void UnregisterFPXImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -821,7 +817,10 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
     status;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
 
   register const PixelPacket
     *p;
@@ -830,6 +829,7 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
     i;
 
   size_t
+    length,
     memory_limit;
 
   unsigned char
@@ -851,12 +851,12 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
   image->depth=8;
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
@@ -878,7 +878,7 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   background_color.color3_value=0;
   background_color.color4_value=0;
   compression=NONE;
-  if (image->compression == JPEGCompression)
+  if (image_info->compression == JPEGCompression)
     compression=JPEG_UNSPECIFIED;
   {
 #if defined(macintosh)
@@ -959,20 +959,11 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   fpx_status=FPX_SetSummaryInformation(flashpix,&summary_info);
   if (fpx_status != FPX_OK)
     ThrowWriterException(DelegateError,"UnableToSetSummaryInfo");
-  /*
-    Allocate pixels.
-  */
-  pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
-    colorspace.numberOfComponents*sizeof(*pixels));
-  if (pixels == (unsigned char *) NULL)
-    {
-      (void) FPX_CloseImage(flashpix);
-      FPX_ClearSystem();
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    }
   /*
     Initialize FlashPix image description.
   */
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  pixels=GetQuantumPixels(quantum_info);
   fpx_info.numberOfComponents=colorspace.numberOfComponents;
   for (i=0; i < (long) fpx_info.numberOfComponents; i++)
   {
@@ -984,39 +975,33 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
       image->columns*fpx_info.components[i].columnStride;
     fpx_info.components[i].theData=pixels+i;
   }
-  fpx_info.components[0].myColorType.myColor=
-    fpx_info.numberOfComponents != 1 ? NIFRGB_R : MONOCHROME;
+  fpx_info.components[0].myColorType.myColor=fpx_info.numberOfComponents != 1
+    ? NIFRGB_R : MONOCHROME;
   fpx_info.components[1].myColorType.myColor=NIFRGB_G;
   fpx_info.components[2].myColorType.myColor=NIFRGB_B;
   fpx_info.components[3].myColorType.myColor=ALPHA;
   /*
-    Write image scanlines.
+    Write image pixelss.
   */
-  GetQuantumInfo(image_info,&quantum_info);
+  quantum_type=RGBQuantum;
+  if (image->matte != MagickFalse)
+    quantum_type=RGBAQuantum;
+  if (fpx_info.numberOfComponents == 1)
+    quantum_type=GrayQuantum;
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    if (fpx_info.numberOfComponents == 1)
-      (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,pixels);
-    else
-      if (image->matte == MagickFalse)
-        (void) ImportQuantumPixels(image,&quantum_info,RGBQuantum,pixels);
-      else
-        (void) ImportQuantumPixels(image,&quantum_info,RGBAQuantum,pixels);
+    length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
     fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
     if (fpx_status != FPX_OK)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SaveImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
+  quantum_info=DestroyQuantumInfo(quantum_info);
   if (image_info->view != (char *) NULL)
     {
       FPXAffineMatrix
@@ -1142,7 +1127,6 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
     }
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   return(MagickTrue);
 }
 #endif
