@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/resource_.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -97,6 +98,7 @@ ModuleExport unsigned long RegisterHISTOGRAMImage(void)
   entry=SetMagickInfo("HISTOGRAM");
   entry->encoder=(EncodeImageHandler *) WriteHISTOGRAMImage;
   entry->adjoin=MagickFalse;
+  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Histogram of the image");
   entry->module=ConstantString("HISTOGRAM");
   (void) RegisterMagickInfo(entry);
@@ -155,7 +157,7 @@ ModuleExport void UnregisterHISTOGRAMImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -230,7 +232,6 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  (void) SetImageDepth(image,image->depth);
   SetGeometry(image,&geometry);
   if (image_info->density == (char *) NULL)
     (void) ParseAbsoluteGeometry(HistogramDensity,&geometry);
@@ -240,7 +241,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     &image->exception);
   if (histogram_image == (Image *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  histogram_image->storage_class=DirectClass;
+  (void) SetImageStorageClass(histogram_image,DirectClass);
   /*
     Allocate histogram count arrays.
   */
@@ -277,15 +278,12 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   maximum=histogram[0].red;
   for (x=0; x < (long) histogram_image->columns; x++)
   {
-    if ((channel & RedChannel) != 0)
-      if (maximum < histogram[x].red)
-        maximum=histogram[x].red;
-    if ((channel & GreenChannel) != 0)
-      if (maximum < histogram[x].green)
-        maximum=histogram[x].green;
-    if ((channel & BlueChannel) != 0)
-      if (maximum < histogram[x].blue)
-        maximum=histogram[x].blue;
+    if (((channel & RedChannel) != 0) && (maximum < histogram[x].red))
+      maximum=histogram[x].red;
+    if (((channel & GreenChannel) != 0) && (maximum < histogram[x].green))
+      maximum=histogram[x].green;
+    if (((channel & BlueChannel) != 0) && (maximum < histogram[x].blue))
+      maximum=histogram[x].blue;
   }
   scale=(MagickRealType) histogram_image->rows/maximum;
   /*
@@ -331,25 +329,19 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
       }
     if (SyncImagePixels(histogram_image) == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,histogram_image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SaveImageTag,y,histogram_image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,SaveImageTag,y,histogram_image->rows);
+    if (status == MagickFalse)
+      break;
   }
   /*
-    Free resources.
+    Relinquish resources.
   */
   histogram=(MagickPixelPacket *) RelinquishMagickMemory(histogram);
   file=(FILE *) NULL;
   unique_file=AcquireUniqueFileResource(filename);
   if (unique_file != -1)
     file=fdopen(unique_file,"wb");
-  if (((unique_file != -1) && (file != (FILE *) NULL)) &&
-      (IsHistogramImage(image,&image->exception) != MagickFalse))
+  if ((unique_file != -1) && (file != (FILE *) NULL))
     {
       char
         *property;
