@@ -10,7 +10,7 @@
 %                        IIIII   CCCC   OOO   N   N                           %
 %                                                                             %
 %                                                                             %
-%                   Read Microsoft Windows Icon Format.                       %
+%                   Read Microsoft Windows Icon Format                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -52,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/nt-feature.h"
 #include "magick/quantize.h"
 #include "magick/quantum-private.h"
@@ -160,7 +161,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -216,7 +217,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -264,9 +265,13 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     icon_info.y_pixels=ReadBlobLSBLong(image);
     icon_info.number_colors=ReadBlobLSBLong(image);
     icon_info.colors_important=ReadBlobLSBLong(image);
-    image->matte=icon_info.bits_per_pixel == 32 ? MagickTrue : MagickFalse;
+    image->matte=MagickTrue;
     image->columns=(unsigned long) icon_file.directory[i].width;
+    if (image->columns == 0)
+      image->columns=256;
     image->rows=(unsigned long) icon_file.directory[i].height;
+    if (image->rows == 0)
+      image->rows=256;
     image->depth=8;
     if (image->debug != MagickFalse)
       {
@@ -306,7 +311,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
         Read Icon raster colormap.
       */
       number_colors=1UL << icon_info.bits_per_pixel;
-      if (AllocateImageColormap(image,number_colors) == MagickFalse)
+      if (AcquireImageColormap(image,number_colors) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       icon_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
         image->colors,4UL*sizeof(*icon_colormap));
@@ -371,14 +376,12 @@ static Image *ReadICONImage(const ImageInfo *image_info,
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,y,image->rows,
-                  image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+                image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -409,14 +412,12 @@ static Image *ReadICONImage(const ImageInfo *image_info,
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
-                  image->rows,image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+                image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -441,14 +442,12 @@ static Image *ReadICONImage(const ImageInfo *image_info,
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
-                  image->rows,image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+                image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -474,14 +473,12 @@ static Image *ReadICONImage(const ImageInfo *image_info,
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
-                  image->rows,image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+                image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -501,7 +498,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             q->blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             q->green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             q->red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            if (image->matte != MagickFalse)
+            if (icon_info.bits_per_pixel == 32)
               q->opacity=(Quantum) QuantumRange-ScaleCharToQuantum(
                 (unsigned char) ReadBlobByte(image));
             q++;
@@ -512,14 +509,12 @@ static Image *ReadICONImage(const ImageInfo *image_info,
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(-y-1L+image->rows,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,-y-1L+image->rows,
-                  image->rows,image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+                image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -527,7 +522,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
     (void) SyncImage(image);
-    if (image->matte != MagickTrue)
+    if (icon_info.bits_per_pixel != 32)
       {
         /*
           Read the ICON alpha mask.
@@ -576,23 +571,20 @@ static Image *ReadICONImage(const ImageInfo *image_info,
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -699,7 +691,7 @@ ModuleExport void UnregisterICONImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -790,15 +782,14 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     scene++;
     next=SyncNextImageInList(next);
   } while ((next != (Image *) NULL) && (image_info->adjoin != MagickFalse));
-
   scene=0;
-  next = image;
+  next=image;
   do
   {
     /*
       Initialize ICON raster file header.
     */
-    if (image_info->colorspace == UndefinedColorspace)
+    if (image->colorspace != RGBColorspace)
       (void) SetImageColorspace(next,RGBColorspace);
     icon_info.file_size=14+12+28;
     icon_info.offset_bits=icon_info.file_size;
@@ -811,7 +802,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
           Full color ICON raster.
         */
         icon_info.number_colors=0;
-        icon_info.bits_per_pixel=(unsigned short) 24;
+        icon_info.bits_per_pixel=32;
         icon_info.compression=(unsigned long) BI_RGB;
       }
     else
@@ -875,8 +866,8 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     /*
       Convert MIFF to ICON raster pixels.
     */
-    pixels=(unsigned char *) AcquireQuantumMemory((size_t)
-      icon_info.image_size,sizeof(*pixels));
+    pixels=(unsigned char *) AcquireQuantumMemory((size_t) icon_info.image_size,
+      sizeof(*pixels));
     if (pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     (void) ResetMagickMemory(pixels,0,(size_t) icon_info.image_size);
@@ -912,17 +903,14 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
                 byte=0;
               }
            }
-         if (bit != 0)
-           *q++=(unsigned char) (byte << (8-bit));
-         if (next->previous == (Image *) NULL)
-           if ((next->progress_monitor != (MagickProgressMonitor) NULL) &&
-               (QuantumTick(y,next->rows) != MagickFalse))
-             {
-               status=next->progress_monitor(SaveImageTag,y,next->rows,
-                 next->client_data);
-               if (status == MagickFalse)
-                 break;
-             }
+          if (bit != 0)
+            *q++=(unsigned char) (byte << (8-bit));
+          if (next->previous == (Image *) NULL)
+            {
+              status=SetImageProgress(next,SaveImageTag,y,next->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -956,17 +944,14 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
                 byte=0;
               }
            }
-         if (nibble != 0)
-           *q++=(unsigned char) (byte << 4);
-         if (next->previous == (Image *) NULL)
-           if ((next->progress_monitor != (MagickProgressMonitor) NULL) &&
-               (QuantumTick(y,next->rows) != MagickFalse))
-             {
-               status=next->progress_monitor(SaveImageTag,y,next->rows,
-                 next->client_data);
-               if (status == MagickFalse)
-                 break;
-             }
+          if (nibble != 0)
+            *q++=(unsigned char) (byte << 4);
+          if (next->previous == (Image *) NULL)
+            {
+              status=SetImageProgress(next,SaveImageTag,y,next->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -985,14 +970,11 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
           for (x=0; x < (long) next->columns; x++)
             *q++=(unsigned char) indexes[x];
           if (next->previous == (Image *) NULL)
-            if ((next->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,next->rows) != MagickFalse))
-              {
-                status=next->progress_monitor(SaveImageTag,y,next->rows,
-                  next->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(next,SaveImageTag,y,next->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -1013,7 +995,9 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
             *q++=ScaleQuantumToChar(p->blue);
             *q++=ScaleQuantumToChar(p->green);
             *q++=ScaleQuantumToChar(p->red);
-            if (icon_info.bits_per_pixel == 32)
+            if (next->matte == MagickFalse)
+              *q++=ScaleQuantumToChar(255);
+            else
               *q++=ScaleQuantumToChar(p->opacity);
             p++;
           }
@@ -1021,14 +1005,11 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
             for (x=3L*(long) next->columns; x < (long) bytes_per_line; x++)
               *q++=0x00;
           if (next->previous == (Image *) NULL)
-            if ((next->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,next->rows) != MagickFalse))
-              {
-                status=next->progress_monitor(SaveImageTag,y,next->rows,
-                  next->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(next,SaveImageTag,y,next->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         break;
       }
@@ -1046,7 +1027,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     icon_file.directory[scene].offset=(unsigned long) TellBlob(image);
     (void) WriteBlobLSBLong(image,(unsigned long) 40);
     (void) WriteBlobLSBLong(image,(unsigned long) icon_info.width);
-    (void) WriteBlobLSBLong(image,(unsigned long) icon_info.height * 2);
+    (void) WriteBlobLSBLong(image,(unsigned long) icon_info.height*2);
     (void) WriteBlobLSBShort(image,icon_info.planes);
     (void) WriteBlobLSBShort(image,icon_info.bits_per_pixel);
     (void) WriteBlobLSBLong(image,icon_info.compression);
@@ -1122,14 +1103,10 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     if (GetNextImageInList(next) == (Image *) NULL)
       break;
     next=SyncNextImageInList(next);
-    if (next->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=next->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(next),next->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(next,SaveImagesTag,scene++,
+      GetImageListLength(next));
+    if (status == MagickFalse)
+      break;
   } while ((next != (Image *) NULL) && (image_info->adjoin != MagickFalse));
   offset=SeekBlob(image,0,SEEK_SET);
   (void) WriteBlobLSBShort(image,0);
@@ -1149,7 +1126,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     (void) WriteBlobLSBLong(image,icon_file.directory[scene].offset);
     scene++;
     next=SyncNextImageInList(next);
-  } while (next && (image_info->adjoin != MagickFalse));
-  CloseBlob(image);
+  } while ((next != (Image *) NULL) && (image_info->adjoin != MagickFalse));
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
