@@ -80,11 +80,10 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadLABELImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
@@ -122,7 +121,7 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   (void) ResetImagePage(image,"0x0+0+0");
   property=InterpretImageProperties(image_info,image,image_info->filename);
   (void) SetImageProperty(image,"label",property);
@@ -168,10 +167,10 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       return((Image *) NULL);
     }
   if (image->columns == 0)
-    image->columns=(unsigned long) (metrics.width+draw_info->stroke_width+0.5);
+    image->columns=(unsigned long) (metrics.width+draw_info->stroke_width+1.5);
   if (image->columns == 0)
     image->columns=(unsigned long) (draw_info->pointsize+
-      draw_info->stroke_width+0.5);
+      draw_info->stroke_width+1.5);
   if (draw_info->gravity == UndefinedGravity)
     {
       (void) FormatMagickString(geometry,MaxTextExtent,"%+g%+g",
@@ -226,6 +225,7 @@ ModuleExport unsigned long RegisterLABELImage(void)
   entry=SetMagickInfo("LABEL");
   entry->decoder=(DecodeImageHandler *) ReadLABELImage;
   entry->adjoin=MagickFalse;
+  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Image label");
   entry->module=ConstantString("LABEL");
   (void) RegisterMagickInfo(entry);
