@@ -9,11 +9,11 @@
 %                  M   M  A   A    T    LLLLL  A   A  BBBB                    %
 %                                                                             %
 %                                                                             %
-%                        Read MATLAB Image Format.                            %
+%                        Read MATLAB Image Format                             %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
-%                                 June 2001                                   %
+%                                2001-2008                                    %
 %                                                                             %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -58,19 +58,27 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
-#include "magick/shear.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
+#include "magick/option.h"
+#include "magick/resource_.h"
+#include "magick/shear.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/module.h"
 #include "magick/transform.h"
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
+ #include "zlib.h"
+#endif
 
 /*
   Forward declaration.
 */
 static MagickBooleanType
   WriteMATImage(const ImageInfo *,Image *);
-
+
+
 /* Auto coloring method, sorry this creates some artefact inside data
 MinReal+j*MaxComplex = red  MaxReal+j*MaxComplex = black
 MinReal+j*0 = white          MaxReal+j*0 = black
@@ -160,7 +168,7 @@ typedef enum
 #define FLAG_GLOBAL  0x4
 #define FLAG_LOGICAL 0x2
 
-static const QuantumType z2qtype[4] = {GrayQuantum, BlueQuantum, GreenQuantum, RedQuantum};
+static const QuantumType z2qtype[4] = {IndexQuantum, BlueQuantum, GreenQuantum, RedQuantum};
 
 
 static void InsertComplexDoubleRow(double *p, int y, Image * image, double MinVal,
@@ -182,9 +190,9 @@ static void InsertComplexDoubleRow(double *p, int y, Image * image, double MinVa
   {
     if (*p > 0)
     {
-      f = (*p / MaxVal) * (MaxRGB - q->red);
-      if (f + q->red > MaxRGB)
-        q->red = MaxRGB;
+      f = (*p / MaxVal) * (QuantumRange - q->red);
+      if (f + q->red > QuantumRange)
+        q->red = QuantumRange;
       else
         q->red += (int) f;
       if ((int) f / 2.0 > q->green)
@@ -194,9 +202,9 @@ static void InsertComplexDoubleRow(double *p, int y, Image * image, double MinVa
     }
     if (*p < 0)
     {
-      f = (*p / MaxVal) * (MaxRGB - q->blue);
-      if (f + q->blue > MaxRGB)
-        q->blue = MaxRGB;
+      f = (*p / MaxVal) * (QuantumRange - q->blue);
+      if (f + q->blue > QuantumRange)
+        q->blue = QuantumRange;
       else
         q->blue += (int) f;
       if ((int) f / 2.0 > q->green)
@@ -235,9 +243,9 @@ static void InsertComplexFloatRow(float *p, int y, Image * image, double MinVal,
   {
     if (*p > 0)
     {
-      f = (*p / MaxVal) * (MaxRGB - q->red);
-      if (f + q->red > MaxRGB)
-        q->red = MaxRGB;
+      f = (*p / MaxVal) * (QuantumRange - q->red);
+      if (f + q->red > QuantumRange)
+        q->red = QuantumRange;
       else
         q->red += (int) f;
       if ((int) f / 2.0 > q->green)
@@ -247,9 +255,9 @@ static void InsertComplexFloatRow(float *p, int y, Image * image, double MinVal,
     }
     if (*p < 0)
     {
-      f = (*p / MaxVal) * (MaxRGB - q->blue);
-      if (f + q->blue > MaxRGB)
-        q->blue = MaxRGB;
+      f = (*p / MaxVal) * (QuantumRange - q->blue);
+      if (f + q->blue > QuantumRange)
+        q->blue = QuantumRange;
       else
         q->blue += (int) f;
       if ((int) f / 2.0 > q->green)
@@ -317,9 +325,8 @@ static void ReadBlobDoublesMSB(Image * image, size_t len, double *data)
     (void) SeekBlob(image, len, SEEK_CUR);
 }
 
-
 /* Calculate minimum and maximum from a given block of data */
-static void CalcMinMax(Image *image, MATHeader *MATLAB_HDR, unsigned long CellType, unsigned ldblk, void *BImgBuff, double *Min, double *Max)
+static void CalcMinMax(Image *image, int endian_indicator, int SizeX, int SizeY, unsigned long CellType, unsigned ldblk, void *BImgBuff, double *Min, double *Max)
 {
 MagickOffsetType filepos;
 int i, x;
@@ -328,7 +335,7 @@ void (*ReadBlobFloatsXXX)(Image * image, size_t len, float *data);
 double *dblrow;
 float *fltrow;
 
-  if (!strncmp(MATLAB_HDR->EndianIndicator, "IM", 2))
+  if (endian_indicator == LSBEndian)
   {    
     ReadBlobDoublesXXX = ReadBlobDoublesLSB;
     ReadBlobFloatsXXX = ReadBlobFloatsLSB;   
@@ -340,7 +347,7 @@ float *fltrow;
   }
 
   filepos = TellBlob(image);	   /* Please note that file seeking occurs only in the case of doubles */
-  for (i = 0; i < (long) MATLAB_HDR->SizeY; i++)
+  for (i = 0; i < SizeY; i++)
   {
     if (CellType==miDOUBLE)
     {
@@ -350,7 +357,7 @@ float *fltrow;
       {
         *Min = *Max = *dblrow;
       }
-      for (x = 0; x < (long) MATLAB_HDR->SizeX; x++)
+      for (x = 0; x < SizeX; x++)
       {
         if (*Min > *dblrow)
           *Min = *dblrow;
@@ -367,7 +374,7 @@ float *fltrow;
       {
         *Min = *Max = *fltrow;
       }
-    for (x = 0; x < (long) MATLAB_HDR->SizeX; x++)
+    for (x = 0; x < (long) SizeX; x++)
       {
         if (*Min > *fltrow)
           *Min = *fltrow;
@@ -386,15 +393,131 @@ static void FixSignedValues(PixelPacket *q, int y)
   while(y-->0)
   {
      /* Please note that negative values will overflow
-        Q=8; MaxRGB=255: <0;127> + 127+1 = <128; 255> 
+        Q=8; QuantumRange=255: <0;127> + 127+1 = <128; 255> 
 		       <-1;-128> + 127+1 = <0; 127> */
-    q->red += MaxRGB/2 + 1;
-    q->green += MaxRGB/ + 1;
-    q->blue += MaxRGB/ + 1;
+    q->red += QuantumRange/2 + 1;
+    q->green += QuantumRange/ + 1;
+    q->blue += QuantumRange/ + 1;
     q++;
   }
 }
 
+
+/** Fix whole row of logical/binary data. It means pack it. */
+static void FixLogical(unsigned char *Buff,int ldblk)
+{
+unsigned char mask=128;
+unsigned char *BuffL = Buff;
+unsigned char val = 0;
+
+  while(ldblk-->0)
+  {
+    if(*Buff++ != 0)
+      val |= mask;    
+
+    mask >>= 1;
+    if(mask==0)
+    {
+      *BuffL++ = val;
+      val = 0;
+      mask = 128;
+    }   
+      
+  }
+  *BuffL = val;
+}
+
+
+/** This procedure decompreses an image block for a new MATLAB format. */
+static Image *DecompressBlock(Image *orig, MagickOffsetType Size, ImageInfo *clone_info, ExceptionInfo *exception)
+{
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
+
+Image *image2;
+void *CacheBlock, *DecompressBlock;
+z_stream zip_info;
+FILE *mat_file;
+size_t magick_size;
+
+int status;
+
+  if(clone_info==NULL) return NULL;
+  if(clone_info->file)		/* Close file opened from previous transaction. */
+  {
+    fclose(clone_info->file);
+    clone_info->file = NULL;
+    (void) unlink(clone_info->filename);
+  }
+
+  CacheBlock = AcquireQuantumMemory((size_t)((Size<16384)?Size:16384),sizeof(unsigned char *));
+  if(CacheBlock==NULL) return NULL;
+  DecompressBlock = AcquireQuantumMemory((size_t)(4096),sizeof(unsigned char *));
+  if(DecompressBlock==NULL) 
+  {
+    RelinquishMagickMemory(CacheBlock);    
+    return NULL;
+  }
+
+  mat_file = fdopen(AcquireUniqueFileResource(clone_info->filename),"w");
+  if(!mat_file)
+  {
+    RelinquishMagickMemory(CacheBlock);
+    RelinquishMagickMemory(DecompressBlock);
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Gannot create file stream for PS image");
+    return NULL;
+  }
+
+  zip_info.zalloc = (alloc_func) NULL;
+  zip_info.zfree = (free_func) NULL;
+  zip_info.opaque = (voidpf) NULL;
+  inflateInit(&zip_info);
+  /* zip_info.next_out = 8*4;*/
+
+  zip_info.avail_in = 0;
+  zip_info.total_out = 0;
+  while(Size>0 && !EOFBlob(orig))
+  {    
+    magick_size = ReadBlob(orig, (Size<16384)?Size:16384, (unsigned char *) CacheBlock);
+    zip_info.next_in = (Bytef *) CacheBlock;
+    zip_info.avail_in = (uInt) magick_size;    
+
+    while(zip_info.avail_in>0)
+    {
+      zip_info.avail_out = 4096;    
+      zip_info.next_out = (Bytef *) DecompressBlock;
+      status = inflate(&zip_info,Z_NO_FLUSH);      
+      fwrite(DecompressBlock, 4096-zip_info.avail_out, 1, mat_file);
+
+      if(status == Z_STREAM_END) goto DblBreak;
+    }
+
+    Size -= magick_size;
+  }
+DblBreak:
+ 
+  (void)fclose(mat_file);
+  RelinquishMagickMemory(CacheBlock);
+  RelinquishMagickMemory(DecompressBlock);
+
+  if((clone_info->file=fopen(clone_info->filename,"rb"))==NULL) goto UnlinkFile;
+  if( (image2 = AcquireImage(clone_info))==NULL ) goto EraseFile;  
+  status = OpenBlob(clone_info,image2,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+  {
+    DeleteImageFromList(&image2);    
+EraseFile:
+    fclose(clone_info->file);
+    clone_info->file = NULL;
+UnlinkFile:
+    (void) unlink(clone_info->filename);
+    return NULL; 
+  }
+
+  return image2;
+#else
+  return NULL;
+#endif
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -407,9 +530,9 @@ static void FixSignedValues(PixelPacket *q, int y)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadMATImage() reads an MAT X image file and returns it.  It allocates
-%  the memory necessary for the new Image structure and returns a pointer to
-%  the new image.
+%  ReadMATImage() reads an MAT X image file and returns it.  It
+%  allocates the memory necessary for the new Image structure and returns a
+%  pointer to the new image.
 %
 %  The format of the ReadMATImage method is:
 %
@@ -417,6 +540,10 @@ static void FixSignedValues(PixelPacket *q, int y)
 %
 %  A description of each parameter follows:
 %
+%    o image:  Method ReadMATImage returns a pointer to the image after
+%      reading. A null image is returned if there is a memory shortage or if
+%      the image cannot be read.
+%
 %    o image_info: Specifies a pointer to a ImageInfo structure.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -432,14 +559,16 @@ static inline size_t MagickMin(const size_t x,const size_t y)
 
 static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  Image *image,
+  Image *image, *image2=NULL,
    *rotated_image;
   PixelPacket *q;
+
   unsigned int status;
   MATHeader MATLAB_HDR;
   unsigned long size;  
   unsigned long CellType;
-  QuantumInfo quantum_info;
+  QuantumInfo *quantum_info;
+  ImageInfo *clone_info;
   int i;
   long ldblk;
   unsigned char *BImgBuff = NULL;
@@ -447,9 +576,10 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned long Unknown6;
   unsigned z;
   int logging;
+  int sample_size;
   MagickOffsetType filepos=0x80;
   BlobInfo *blob;
-
+  
   unsigned long (*ReadBlobXXXLong)(Image *image);
   unsigned short (*ReadBlobXXXShort)(Image *image);
   void (*ReadBlobDoublesXXX)(Image * image, size_t len, double *data);
@@ -460,12 +590,13 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  logging=LogMagickEvent(CoderEvent,GetMagickModule(),"enter"); 
+  logging = LogMagickEvent(CoderEvent,GetMagickModule(),"enter"); 
 
   /*
      Open image file.
    */
-  image = AllocateImage(image_info);
+  image = AcquireImage(image_info);
+
   status = OpenBlob(image_info, image, ReadBinaryBlobMode, exception);
   if (status == MagickFalse)
     {
@@ -475,16 +606,13 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
      Read MATLAB image.
    */
-  status=ReadBlob(image, 124, (unsigned char *) &MATLAB_HDR.identific);
-  if (status == MagickFalse)
+  clone_info=CloneImageInfo(image_info);
+  if(ReadBlob(image,124,(unsigned char *) &MATLAB_HDR.identific) != 124)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   MATLAB_HDR.Version = ReadBlobLSBShort(image);
-  status=ReadBlob(image, 2, (unsigned char *) &MATLAB_HDR.EndianIndicator);
-  if (status == MagickFalse)
+  if(ReadBlob(image,2,(unsigned char *) &MATLAB_HDR.EndianIndicator) != 2)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
 
-  GetQuantumInfo(image_info,&quantum_info);
-
   if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),"  Endian %c%c",
         MATLAB_HDR.EndianIndicator[0],MATLAB_HDR.EndianIndicator[1]);
   if (!strncmp(MATLAB_HDR.EndianIndicator, "IM", 2))
@@ -512,8 +640,8 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   filepos = TellBlob(image);
   while(!EOFBlob(image)) /* object parser loop */
   {
-    SeekBlob(image,filepos,SEEK_SET);
-    //printf("pos=%X\n",TellBlob(image));
+    (void) SeekBlob(image,filepos,SEEK_SET);
+    /* printf("pos=%X\n",TellBlob(image)); */
 
     MATLAB_HDR.DataType = ReadBlobXXXLong(image);
     if(EOFBlob(image)) break;
@@ -521,106 +649,132 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     if(EOFBlob(image)) break;
     filepos += MATLAB_HDR.ObjectSize + 4 + 4;
 
-    if (MATLAB_HDR.DataType != miMATRIX) continue;  /* skip another objects. */    
- 
-    MATLAB_HDR.unknown1 = ReadBlobXXXLong(image);
-    MATLAB_HDR.unknown2 = ReadBlobXXXLong(image);  
+    image2 = image;
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
+    if(MATLAB_HDR.DataType == miCOMPRESSED)
+    {
+      image2 = DecompressBlock(image,MATLAB_HDR.ObjectSize,clone_info,exception);
+      if(image2==NULL) continue;
+      MATLAB_HDR.DataType = ReadBlobXXXLong(image2); /* replace compressed object type. */
+    }
+#endif    
 
-    MATLAB_HDR.unknown5 = ReadBlobXXXLong(image);
+    if(MATLAB_HDR.DataType!=miMATRIX) continue;  /* skip another objects. */
+ 
+    MATLAB_HDR.unknown1 = ReadBlobXXXLong(image2);
+    MATLAB_HDR.unknown2 = ReadBlobXXXLong(image2);  
+
+    MATLAB_HDR.unknown5 = ReadBlobXXXLong(image2);
     MATLAB_HDR.StructureClass = MATLAB_HDR.unknown5 & 0xFF;
     MATLAB_HDR.StructureFlag = (MATLAB_HDR.unknown5>>8) & 0xFF;  
 
-    MATLAB_HDR.unknown3 = ReadBlobXXXLong(image);
-    MATLAB_HDR.unknown4 = ReadBlobXXXLong(image);
-    MATLAB_HDR.DimFlag = ReadBlobXXXLong(image);
-    MATLAB_HDR.SizeX = ReadBlobXXXLong(image);
-    MATLAB_HDR.SizeY = ReadBlobXXXLong(image);  
+    MATLAB_HDR.unknown3 = ReadBlobXXXLong(image2);
+    if(image!=image2)
+      MATLAB_HDR.unknown4 = ReadBlobXXXLong(image2);	/* ??? don't understand why ?? */
+    MATLAB_HDR.unknown4 = ReadBlobXXXLong(image2);
+    MATLAB_HDR.DimFlag = ReadBlobXXXLong(image2);
+    MATLAB_HDR.SizeX = ReadBlobXXXLong(image2);
+    MATLAB_HDR.SizeY = ReadBlobXXXLong(image2);  
    
 
     switch(MATLAB_HDR.DimFlag)
-    {
-      case  8: z=1; break;	       /* 2D matrix*/
-      case 12: z=ReadBlobXXXLong(image); /* 3D matrix RGB*/
-  	       Unknown6 = ReadBlobXXXLong(image);
+    {     
+      case  8: z=1; break;			/* 2D matrix*/
+      case 12: z = ReadBlobXXXLong(image2);	/* 3D matrix RGB*/
+  	       Unknown6 = ReadBlobXXXLong(image2);
 	       if(z!=3) ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
 	       break;
       default: ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
     }  
 
-    MATLAB_HDR.Flag1 = ReadBlobXXXShort(image);
-    MATLAB_HDR.NameFlag = ReadBlobXXXShort(image);
+    MATLAB_HDR.Flag1 = ReadBlobXXXShort(image2);
+    MATLAB_HDR.NameFlag = ReadBlobXXXShort(image2);
 
     if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
           "MATLAB_HDR.StructureClass %d",MATLAB_HDR.StructureClass);
     if (MATLAB_HDR.StructureClass != mxCHAR_CLASS && 
-        MATLAB_HDR.StructureClass != mxSINGLE_CLASS &&	     /* float + complex float */
-        MATLAB_HDR.StructureClass != mxDOUBLE_CLASS &&	     /* double + complex double */
+        MATLAB_HDR.StructureClass != mxSINGLE_CLASS &&		/* float + complex float */
+        MATLAB_HDR.StructureClass != mxDOUBLE_CLASS &&		/* double + complex double */
         MATLAB_HDR.StructureClass != mxINT8_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT8_CLASS &&          /* uint8 + uint8 3D */
+        MATLAB_HDR.StructureClass != mxUINT8_CLASS &&		/* uint8 + uint8 3D */
         MATLAB_HDR.StructureClass != mxINT16_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT16_CLASS &&	     /* uint16 + uint16 3D */
+        MATLAB_HDR.StructureClass != mxUINT16_CLASS &&		/* uint16 + uint16 3D */
         MATLAB_HDR.StructureClass != mxINT32_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT32_CLASS &&	     /* uint32 + uint32 3D */
+        MATLAB_HDR.StructureClass != mxUINT32_CLASS &&		/* uint32 + uint32 3D */
         MATLAB_HDR.StructureClass != mxINT64_CLASS &&
-        MATLAB_HDR.StructureClass != mxUINT64_CLASS)	     /* uint64 + uint64 3D */
+        MATLAB_HDR.StructureClass != mxUINT64_CLASS)		/* uint64 + uint64 3D */
       ThrowReaderException(CoderError,"UnsupportedCellTypeInTheMatrix");
 
     switch (MATLAB_HDR.NameFlag)
     {
       case 0:
-        size = ReadBlobXXXLong(image);	/* Object name string size */
+        size = ReadBlobXXXLong(image2);	/* Object name string size */
         size = 4 * (long) ((size + 3 + 1) / 4);
-        (void) SeekBlob(image, size, SEEK_CUR);
+        (void) SeekBlob(image2, size, SEEK_CUR);
         break;
       case 1:
       case 2:
       case 3:
       case 4:
-        (void) ReadBlob(image, 4, (unsigned char *) &size); /* Object name string */
+        (void) ReadBlob(image2, 4, (unsigned char *) &size); /* Object name string */
         break;
       default:
         goto MATLAB_KO;
     }
 
-    CellType = ReadBlobXXXLong(image);    /* Additional object type */
+    CellType = ReadBlobXXXLong(image2);    /* Additional object type */
     if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
           "MATLAB_HDR.CellType: %ld",CellType);
   
-    (void) ReadBlob(image, 4, (unsigned char *) &size);     /* data size */
+    (void) ReadBlob(image2, 4, (unsigned char *) &size);     /* data size */
+
+      /* Image is gray when no complex flag is set and 2D Matrix */
+    if ((MATLAB_HDR.DimFlag == 8) &&
+        ((MATLAB_HDR.StructureFlag & FLAG_COMPLEX) == 0))
+      image->type=GrayscaleType;
 
     switch (CellType)
     {
       case miINT8:
       case miUINT8:
-        image->depth= 8;
+        sample_size = 8;
+        if(MATLAB_HDR.StructureFlag & FLAG_LOGICAL) 
+          image->depth = 1;
+        else
+          image->depth = 8;         /* Byte type cell */
         ldblk = (long) MATLAB_HDR.SizeX;      
         break;
       case miINT16:
       case miUINT16:
-        image->depth= 16;
+        sample_size = 16;
+        image->depth = 16;        /* Word type cell */
         ldblk = (long) (2 * MATLAB_HDR.SizeX);
         break;
       case miINT32:
       case miUINT32:
-        image->depth= 32;
+        sample_size = 32;
+        image->depth = 32;        /* Dword type cell */
         ldblk = (long) (4 * MATLAB_HDR.SizeX);      
         break;
       case miINT64:
       case miUINT64:
-        image->depth= 64;
+        sample_size = 64;
+        image->depth = 64;        /* Qword type cell */
         ldblk = (long) (8 * MATLAB_HDR.SizeX);      
         break;   
       case miSINGLE:
-        image->depth= 32;
-        quantum_info.format=FloatingPointQuantumFormat;
+        sample_size = 32;
+        image->depth = 32;        /* double type cell */
+        (void) SetImageOption(clone_info,"quantum:format","floating-point");
         if (MATLAB_HDR.StructureFlag & FLAG_COMPLEX)
-	{                         /* complex float type cell */
+	{					    /* complex float type cell */
 	}
         ldblk = (long) (4 * MATLAB_HDR.SizeX);
         break;
       case miDOUBLE:
-        image->depth= 64; 
-        quantum_info.format=FloatingPointQuantumFormat;
+        sample_size = 64; 
+        image->depth = 64;        /* double type cell */
+        (void) SetImageOption(clone_info,"quantum:format","floating-point");
         if (sizeof(double) != 8)
           ThrowReaderException(CoderError, "IncompatibleSizeOfDouble");
         if (MATLAB_HDR.StructureFlag & FLAG_COMPLEX)
@@ -631,13 +785,24 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       default:
         ThrowReaderException(CoderError, "UnsupportedCellTypeInTheMatrix");
     }
-
     image->columns = MATLAB_HDR.SizeX;
-    image->rows = MATLAB_HDR.SizeY;
-    image->colors = 1l >> 8;
+    image->rows = MATLAB_HDR.SizeY;    
+    quantum_info=AcquireQuantumInfo(clone_info,image);
+    image->colors = 1l << image->depth;
     if (image->columns == 0 || image->rows == 0)
       goto MATLAB_KO;
 
+    /* ----- Create gray palette ----- */
+
+    if (CellType==miUINT8 && z!=3)
+    {
+      if(image->colors>256) image->colors = 256;
+
+      if (!AcquireImageColormap(image, image->colors))
+      {
+ NoMemory:ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");}
+    }
+
     /*
       If ping is true, then only set image size and colors without
       reading any image data.
@@ -647,19 +812,19 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       unsigned long temp = image->columns;
       image->columns = image->rows;
       image->rows = temp;
-      goto done_reading; //!!!!!! BAD  !!!!
+      goto done_reading; /* !!!!!! BAD  !!!! */
     }  
 
   /* ----- Load raster data ----- */
-    BImgBuff = (unsigned char *) AcquireQuantumMemory(ldblk,sizeof(*BImgBuff));
+    BImgBuff = (unsigned char *) AcquireQuantumMemory((size_t) (ldblk),sizeof(unsigned char *));    /* Ldblk was set in the check phase */
     if (BImgBuff == NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      goto NoMemory;
 
     MinVal = 0;
     MaxVal = 0;
     if (CellType==miDOUBLE || CellType==miSINGLE)        /* Find Min and Max Values for floats */
     {
-      CalcMinMax(image, &MATLAB_HDR, CellType, ldblk, BImgBuff, &quantum_info.minimum, &quantum_info.maximum);
+      CalcMinMax(image2, image_info->endian,  MATLAB_HDR.SizeX, MATLAB_HDR.SizeY, CellType, ldblk, BImgBuff, &quantum_info->minimum, &quantum_info->maximum);
     }
 
     /* Main loop for reading all scanlines */
@@ -670,13 +835,46 @@ MATLAB_KO: ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
       {
         q=SetImagePixels(image,0,MATLAB_HDR.SizeY-i-1,image->columns,1);
-        if (q == (PixelPacket *)NULL) goto ExitLoop;
-        (void) ReadBlob(image, ldblk, (unsigned char *)BImgBuff);	  
-        (void) ExportQuantumPixels(image,&quantum_info,z2qtype[z],BImgBuff);
-        if (z<=1 &&			 // fix only during a last pass z==0 || z==1
+        if (q == (PixelPacket *)NULL)
+	{
+	  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+              "  MAT set image pixels returns unexpected NULL on a row %u.", (unsigned)(MATLAB_HDR.SizeY-i-1));
+	  goto done_reading;		/* Skip image rotation, when cannot set image pixels	  */
+	}
+        if(ReadBlob(image2,ldblk,(unsigned char *)BImgBuff) != (ssize_t) ldblk)
+	{
+	  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+             "  MAT cannot read scanrow %u from a file.", (unsigned)(MATLAB_HDR.SizeY-i-1));
+	  goto ExitLoop;
+	}
+        if((CellType==miINT8 || CellType==miUINT8) && (MATLAB_HDR.StructureFlag & FLAG_LOGICAL))
+        {
+          FixLogical((unsigned char *)BImgBuff,ldblk);
+          if(ImportQuantumPixels(image,quantum_info,z2qtype[z],BImgBuff) <= 0)
+	  {
+ImportQuantumPixelsFailed:
+	    if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+              "  MAT failed to ImportQuantumPixels for a row %u", (unsigned)(MATLAB_HDR.SizeY-i-1));
+	    break;
+	  }
+        }
+        else
+        {
+          if(ImportQuantumPixels(image,quantum_info,z2qtype[z],BImgBuff) <= 0)
+	    goto ImportQuantumPixelsFailed;
+
+
+          if (z<=1 &&			 /* fix only during a last pass z==0 || z==1 */
 	        (CellType==miINT8 || CellType==miINT16 || CellType==miINT32 || CellType==miINT64))
-	  FixSignedValues(q,MATLAB_HDR.SizeX);
-        if (!SyncImagePixels(image)) goto ExitLoop;
+	    FixSignedValues(q,MATLAB_HDR.SizeX);
+        }
+
+        if (!SyncImagePixels(image))
+	{
+	  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
+            "  MAT failed to sync image pixels for a row %u", (unsigned)(MATLAB_HDR.SizeY-i-1));
+	  goto ExitLoop;
+	}
       }
     } while(z-- >= 2);
 ExitLoop:
@@ -685,30 +883,39 @@ ExitLoop:
     /* Read complex part of numbers here */
     if (MATLAB_HDR.StructureFlag & FLAG_COMPLEX)
     {        /* Find Min and Max Values for complex parts of floats */
-      CellType = ReadBlobXXXLong(image);    /* Additional object type */
-      i = ReadBlobXXXLong(image);           /* size of a complex part - toss away*/
+      CellType = ReadBlobXXXLong(image2);    /* Additional object type */
+      i = ReadBlobXXXLong(image2);           /* size of a complex part - toss away*/
 
       if (CellType==miDOUBLE || CellType==miSINGLE)
       {
-        CalcMinMax(image, &MATLAB_HDR, CellType, ldblk, BImgBuff, &MinVal, &MaxVal);      
+        CalcMinMax(image2,  image_info->endian, MATLAB_HDR.SizeX, MATLAB_HDR.SizeY, CellType, ldblk, BImgBuff, &MinVal, &MaxVal);      
       }
 
       if (CellType==miDOUBLE)
         for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
 	{
-          ReadBlobDoublesXXX(image, ldblk, (double *)BImgBuff);
+          ReadBlobDoublesXXX(image2, ldblk, (double *)BImgBuff);
           InsertComplexDoubleRow((double *)BImgBuff, i, image, MinVal, MaxVal);
 	}
 
       if (CellType==miSINGLE)
         for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
 	{
-          ReadBlobFloatsXXX(image, ldblk, (float *)BImgBuff);
+          ReadBlobFloatsXXX(image2, ldblk, (float *)BImgBuff);
           InsertComplexFloatRow((float *)BImgBuff, i, image, MinVal, MaxVal);
 	}    
     }
 
       /* Image is gray when no complex flag is set and 2D Matrix AGAIN!!! */
+    if ((MATLAB_HDR.DimFlag == 8) &&
+        ((MATLAB_HDR.StructureFlag & FLAG_COMPLEX) == 0))
+      image->type=GrayscaleType;
+    if (image->depth == 1)
+      image->type=BilevelType;
+
+    if(image2==image)
+        image2 = NULL;    /* Remove shadow copy to an image before rotation. */
+
       /*  Rotate image. */
     rotated_image = RotateImage(image, 90.0, exception);
     if (rotated_image != (Image *) NULL)
@@ -719,6 +926,7 @@ ExitLoop:
 
       blob = rotated_image->blob;
       rotated_image->blob = image->blob;
+      rotated_image->colors = image->colors;
       image->blob = blob;
       AppendImageToList(&image,rotated_image);      
       DeleteImageFromList(&image);      
@@ -726,19 +934,35 @@ ExitLoop:
 
 done_reading:
 
+    if(image2!=NULL)
+      if(image2!=image)
+      {
+        DeleteImageFromList(&image2); 
+	if(clone_info)
+	{
+          if(clone_info->file)
+	  {
+            fclose(clone_info->file);
+            clone_info->file = NULL;
+            (void) unlink(clone_info->filename);
+	  }
+        }    
+      }
+
       /* Allocate next image structure. */    
-    AllocateNextImage(image_info,image);   
+    AcquireNextImage(image_info,image);
     if (image->next == (Image *) NULL) break;                
     image=SyncNextImageInList(image);
     image->columns=image->rows=0;
     image->colors=0;    
 
       /* row scan buffer is no longer needed */
-    BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+    RelinquishMagickMemory(BImgBuff);
+    BImgBuff = NULL;
   }
+    clone_info=DestroyImageInfo(clone_info);
 
-
-  BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+  RelinquishMagickMemory(BImgBuff);
   CloseBlob(image);
 
 
@@ -751,7 +975,7 @@ done_reading:
     */
     p=image;
     image=NULL;
-    while (p != (Image *) NULL)
+    while (p != (Image *)NULL)
       {
         Image *tmp=p;
         if ((p->rows == 0) || (p->columns == 0)) {
@@ -770,7 +994,20 @@ done_reading:
       p->scene=scene++;
   }
 
-  if (logging) (void) LogMagickEvent(CoderEvent,GetMagickModule(),"return");
+  if(clone_info != NULL)	/* cleanup garbage file from compression */
+  {
+    if(clone_info->file)
+    {
+      fclose(clone_info->file);
+      clone_info->file = NULL;
+      (void) unlink(clone_info->filename);
+    }
+    DestroyImageInfo(clone_info);
+    clone_info = NULL;
+  }
+  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),"return");
+  if(image==NULL)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   return (image);
 }
 
@@ -799,11 +1036,13 @@ done_reading:
 */
 ModuleExport unsigned long RegisterMATImage(void)
 {
-  MagickInfo * entry;
+  MagickInfo
+    *entry;
 
-  entry = SetMagickInfo("MAT");
+  entry=SetMagickInfo("MAT");
   entry->decoder=(DecodeImageHandler *) ReadMATImage;
   entry->encoder=(EncodeImageHandler *) WriteMATImage;
+  entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("MATLAB image format");
   entry->module=AcquireString("MAT");
@@ -868,6 +1107,7 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
   long y;
   unsigned z;
   const PixelPacket *p;
+
   unsigned int status;
   int logging;
   unsigned long DataSize;
@@ -882,7 +1122,7 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
     scene;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
 
   /*
     Open output image file.
@@ -915,11 +1155,8 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
-    if (image_info->colorspace == UndefinedColorspace)
+    if (image->colorspace != RGBColorspace)
       (void) SetImageColorspace(image,RGBColorspace);
-    pixels=(unsigned char *) AcquireQuantumMemory(image->rows,sizeof(*pixels));
-    if (pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
 
     is_gray = IsGrayImage(image,&image->exception);
     z = is_gray ? 0 : 3;
@@ -955,7 +1192,8 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
     /*
       Store image data.
     */
-    GetQuantumInfo(image_info,&quantum_info);
+    quantum_info=AcquireQuantumInfo(image_info,image);
+    pixels=GetQuantumPixels(quantum_info);
     do
     {
       for (y=0; y < (long)image->columns; y++)
@@ -963,26 +1201,22 @@ static MagickBooleanType WriteMATImage(const ImageInfo *image_info,Image *image)
         p=AcquireImagePixels(image,y,0,1,image->rows,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
-        (void) ImportQuantumPixels(image,&quantum_info,z2qtype[z],pixels);
+        (void) ExportQuantumPixels(image,quantum_info,z2qtype[z],pixels);
         (void) WriteBlob(image,image->rows,pixels);
       }    
       if (!SyncImagePixels(image))
         break;
     } while(z-- >= 2);
     while(padding-->0) (void) WriteBlobByte(image,0);
-    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+    quantum_info=DestroyQuantumInfo(quantum_info);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
