@@ -50,6 +50,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -91,6 +92,7 @@ ModuleExport unsigned long RegisterMATTEImage(void)
 
   entry=SetMagickInfo("MATTE");
   entry->encoder=(EncodeImageHandler *) WriteMATTEImage;
+  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("MATTE format");
   entry->module=ConstantString("MATTE");
   (void) RegisterMagickInfo(entry);
@@ -142,7 +144,7 @@ ModuleExport void UnregisterMATTEImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -196,15 +198,9 @@ static MagickBooleanType WriteMATTEImage(const ImageInfo *image_info,
     }
     if (SyncImagePixels(matte_image) == MagickFalse)
       break;
-    if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   (void) FormatMagickString(matte_image->filename,MaxTextExtent,
     "MIFF:%s",image->filename);
