@@ -10,7 +10,7 @@
 %                            M   M  P       CCCC                              %
 %                                                                             %
 %                                                                             %
-%              Read/Write Magick Persistant Cache Image Format.               %
+%              Read/Write Magick Persistant Cache Image Format                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -58,6 +58,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/profile.h"
 #include "magick/property.h"
@@ -132,7 +133,7 @@ static MagickBooleanType IsMPC(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -169,12 +170,6 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickStatusType
     flags;
 
-  QuantumInfo
-    quantum_info;
-
-  QuantumState
-    quantum_state;
-
   register long
     i;
 
@@ -201,7 +196,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -230,8 +225,6 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     quantum_depth=MAGICKCORE_QUANTUM_DEPTH;
     image->depth=8;
     image->compression=NoCompression;
-    GetQuantumInfo(image_info,&quantum_info);
-    InitializeQuantumState(&quantum_info,MSBEndian,&quantum_state);
     while ((isgraph(c) != MagickFalse) && (c != (int) ':'))
     {
       register char
@@ -622,6 +615,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     (void) ParseAbsoluteGeometry(geometry,&image->tile_offset);
                     geometry=DestroyString(geometry);
                   }
+                if (LocaleCompare(keyword,"type") == 0)
+                  {
+                    image->type=(ImageType) ParseMagickOption(MagickTypeOptions,                      MagickFalse,options);
+                    break;
+                  }
                 (void) SetImageProperty(image,keyword,options);
                 break;
               }
@@ -742,7 +740,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Create image colormap.
         */
-        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+        if (AcquireImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         if (image->colors != 0)
           {
@@ -777,11 +775,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
                 for (i=0; i < (long) image->colors; i++)
                 {
-                  pixel=PushCharPixel(&p);
+                  p=PushCharPixel(p,&pixel);
                   image->colormap[i].red=ScaleCharToQuantum(pixel);
-                  pixel=PushCharPixel(&p);
+                  p=PushCharPixel(p,&pixel);
                   image->colormap[i].green=ScaleCharToQuantum(pixel);
-                  pixel=PushCharPixel(&p);
+                  p=PushCharPixel(p,&pixel);
                   image->colormap[i].blue=ScaleCharToQuantum(pixel);
                 }
                 break;
@@ -793,11 +791,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
                 for (i=0; i < (long) image->colors; i++)
                 {
-                  pixel=PushShortPixel(&quantum_state,&p);
+                  p=PushShortPixel(MSBEndian,p,&pixel);
                   image->colormap[i].red=ScaleShortToQuantum(pixel);
-                  pixel=PushShortPixel(&quantum_state,&p);
+                  p=PushShortPixel(MSBEndian,p,&pixel);
                   image->colormap[i].green=ScaleShortToQuantum(pixel);
-                  pixel=PushShortPixel(&quantum_state,&p);
+                  p=PushShortPixel(MSBEndian,p,&pixel);
                   image->colormap[i].blue=ScaleShortToQuantum(pixel);
                 }
                 break;
@@ -809,11 +807,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
                 for (i=0; i < (long) image->colors; i++)
                 {
-                  pixel=PushLongPixel(&quantum_state,&p);
+                  p=PushLongPixel(MSBEndian,p,&pixel);
                   image->colormap[i].red=ScaleLongToQuantum(pixel);
-                  pixel=PushLongPixel(&quantum_state,&p);
+                  p=PushLongPixel(MSBEndian,p,&pixel);
                   image->colormap[i].green=ScaleLongToQuantum(pixel);
-                  pixel=PushLongPixel(&quantum_state,&p);
+                  p=PushLongPixel(MSBEndian,p,&pixel);
                   image->colormap[i].blue=ScaleLongToQuantum(pixel);
                 }
                 break;
@@ -831,11 +829,6 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
     /*
       Attach persistent pixel cache.
     */
@@ -854,23 +847,20 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (c != EOF);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -961,9 +951,9 @@ ModuleExport void UnregisterMPCImage(void)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
@@ -983,12 +973,6 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     offset,
     scene;
 
-  QuantumInfo
-    quantum_info;
-
-  QuantumState
-    quantum_state;
-
   register long
     i;
 
@@ -1033,6 +1017,12 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
       "columns=%lu  rows=%lu  depth=%lu\n",image->columns,image->rows,
       image->depth);
     (void) WriteBlobString(image,buffer);
+    if (image->type != UndefinedType)
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,"type=%s\n",
+          MagickOptionToMnemonic(MagickTypeOptions,image->type));
+        (void) WriteBlobString(image,buffer);
+      }
     if (image->colorspace != UndefinedColorspace)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"colorspace=%s\n",
@@ -1048,9 +1038,8 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
     if (image->compression != UndefinedCompression)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,
-          "compression=%s  quality=%lu\n",
-          MagickOptionToMnemonic(MagickCompressOptions,image->compression),
-          image->quality);
+          "compression=%s  quality=%lu\n",MagickOptionToMnemonic(
+          MagickCompressOptions,image->compression),image->quality);
         (void) WriteBlobString(image,buffer);
       }
     if (image->units != UndefinedResolution)
@@ -1199,13 +1188,6 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
           image->montage);
         (void) WriteBlobString(image,buffer);
       }
-    GetQuantumInfo(image_info,&quantum_info);
-    if (quantum_info.format == FloatingPointQuantumFormat)
-      {
-        (void) WriteBlobString(image,"quantum-format=floating-point\n");
-        quantum_info.scale=1.0/QuantumRange;
-      }
-    InitializeQuantumState(&quantum_info,MSBEndian,&quantum_state);
     ResetImagePropertyIterator(image);
     property=GetNextImageProperty(image);
     while (property != (const char *) NULL)
@@ -1292,11 +1274,11 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
                 pixel;
 
               pixel=ScaleQuantumToLong(image->colormap[i].red);
-              PopLongPixel(&quantum_state,pixel,&q);
+              q=PopLongPixel(MSBEndian,pixel,q);
               pixel=ScaleQuantumToLong(image->colormap[i].green);
-              PopLongPixel(&quantum_state,pixel,&q);
+              q=PopLongPixel(MSBEndian,pixel,q);
               pixel=ScaleQuantumToLong(image->colormap[i].blue);
-              PopLongPixel(&quantum_state,pixel,&q);
+              q=PopLongPixel(MSBEndian,pixel,q);
             }
             case 16:
             {
@@ -1304,11 +1286,11 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
                 pixel;
 
               pixel=ScaleQuantumToShort(image->colormap[i].red);
-              PopShortPixel(&quantum_state,pixel,&q);
+              q=PopShortPixel(MSBEndian,pixel,q);
               pixel=ScaleQuantumToShort(image->colormap[i].green);
-              PopShortPixel(&quantum_state,pixel,&q);
+              q=PopShortPixel(MSBEndian,pixel,q);
               pixel=ScaleQuantumToShort(image->colormap[i].blue);
-              PopShortPixel(&quantum_state,pixel,&q);
+              q=PopShortPixel(MSBEndian,pixel,q);
               break;
             }
             case 8:
@@ -1317,12 +1299,12 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
                 pixel;
 
               pixel=(unsigned char) ScaleQuantumToChar(image->colormap[i].red);
-              PopCharPixel(pixel,&q);
+              q=PopCharPixel(pixel,q);
               pixel=(unsigned char) ScaleQuantumToChar(
                 image->colormap[i].green);
-              PopCharPixel(pixel,&q);
+              q=PopCharPixel(pixel,q);
               pixel=(unsigned char) ScaleQuantumToChar(image->colormap[i].blue);
-              PopCharPixel(pixel,&q);
+              q=PopCharPixel(pixel,q);
               break;
             }
           }
@@ -1349,6 +1331,6 @@ static MagickBooleanType WriteMPCImage(const ImageInfo *image_info,Image *image)
       }
     scene++;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(status);
 }
