@@ -10,7 +10,7 @@
 %                        M   M  P      EEEEE   GGGG                           %
 %                                                                             %
 %                                                                             %
-%                       Read/Write MPEG Image Format.                         %
+%                       Read/Write MPEG Image Format                          %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -124,7 +124,7 @@ static MagickBooleanType IsMPEG(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -132,6 +132,8 @@ static MagickBooleanType IsMPEG(const unsigned char *magick,const size_t length)
 static Image *ReadMPEGImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+#define ReadMPEGIntermediateFormat "pam"
+
   Image
     *image,
     *images;
@@ -155,19 +157,19 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   (void) DestroyImageList(image);
   /*
     Convert MPEG to PPM with delegate.
   */
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   read_info=CloneImageInfo(image_info);
   (void) InvokeDelegate(read_info,image,"mpeg:decode",(char *) NULL,exception);
   image=DestroyImage(image);
@@ -177,9 +179,9 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
   images=NewImageList();
   for (i=(long) read_info->scene; ; i++)
   {
-    (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s%ld.ppm",
-      read_info->unique,i);
-    if (IsAccessible(read_info->filename) == MagickFalse)
+    (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s%ld.%s",
+      read_info->unique,i,ReadMPEGIntermediateFormat);
+    if (IsPathAccessible(read_info->filename) == MagickFalse)
       break;
     image=ReadImage(read_info,exception);
     if (image == (Image *) NULL)
@@ -192,13 +194,13 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
         break;
   }
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (i=0; ; i++)
   {
-    (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s%ld.ppm",
-      read_info->unique,i);
-    if (IsAccessible(read_info->filename) == MagickFalse)
+    (void) FormatMagickString(read_info->filename,MaxTextExtent,"%s%ld.%s",
+      read_info->unique,i,ReadMPEGIntermediateFormat);
+    if (IsPathAccessible(read_info->filename) == MagickFalse)
       break;
     (void) RelinquishUniqueFileResource(read_info->filename);
   }
@@ -234,6 +236,14 @@ ModuleExport unsigned long RegisterMPEGImage(void)
   MagickInfo
     *entry;
 
+  entry=SetMagickInfo("MOV");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
+  entry->magick=(IsImageFormatHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
+  entry->description=ConstantString("MPEG Video Stream");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPEG");
   entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
@@ -250,6 +260,14 @@ ModuleExport unsigned long RegisterMPEGImage(void)
   entry->description=ConstantString("MPEG Video Stream");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("MP4");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
+  entry->magick=(IsImageFormatHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
+  entry->description=ConstantString("MPEG-4 Video Stream");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M2V");
   entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
@@ -258,6 +276,22 @@ ModuleExport unsigned long RegisterMPEGImage(void)
   entry->description=ConstantString("MPEG Video Stream");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("M4V");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
+  entry->magick=(IsImageFormatHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
+  entry->description=ConstantString("Raw MPEG-4 Video");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("WMV");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
+  entry->magick=(IsImageFormatHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
+  entry->description=ConstantString("Windows Media Video");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
 
@@ -282,9 +316,13 @@ ModuleExport unsigned long RegisterMPEGImage(void)
 */
 ModuleExport void UnregisterMPEGImage(void)
 {
+  (void) UnregisterMagickInfo("WMV");
+  (void) UnregisterMagickInfo("M4V");
   (void) UnregisterMagickInfo("M2V");
-  (void) UnregisterMagickInfo("MPEG");
+  (void) UnregisterMagickInfo("MP4");
   (void) UnregisterMagickInfo("MPG");
+  (void) UnregisterMagickInfo("MPEG");
+  (void) UnregisterMagickInfo("MOV");
 }
 
 /*
@@ -304,11 +342,12 @@ ModuleExport void UnregisterMPEGImage(void)
 %
 %  The format of the WriteMPEGImage method is:
 %
-%      MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -328,222 +367,84 @@ static inline double MagickMin(const double x,const double y)
   return(y);
 }
 
-static MagickBooleanType WriteMPEGParameterFiles(const ImageInfo *image_info,
-  Image *image,const char *basename)
+static MagickBooleanType CopyDelegateFile(const char *source,
+  const char *destination)
 {
-  char
-    filename[MaxTextExtent];
-
-  double
-    delay,
-    q;
-
-  FILE
-    *file,
-    *parameter_file;
-
-  long
-    quant,
-    vertical_factor;
+  int
+    destination_file,
+    source_file;
 
   MagickBooleanType
-    mpeg;
+    status;
 
-  register Image
-    *p;
-
-  register long
+  register size_t
     i;
 
+  size_t
+    length,
+    quantum;
+
   ssize_t
     count;
 
-  static int
-    q_matrix[]=
-    {
-       8, 16, 19, 22, 26, 27, 29, 34,
-      16, 16, 22, 24, 27, 29, 34, 37,
-      19, 22, 26, 27, 29, 34, 34, 38,
-      22, 22, 26, 27, 29, 34, 37, 40,
-      22, 26, 27, 29, 32, 35, 40, 48,
-      26, 27, 29, 32, 35, 40, 48, 58,
-      26, 27, 29, 34, 38, 46, 56, 69,
-      27, 29, 35, 38, 46, 56, 69, 83
-    };
+  struct stat
+    attributes;
+
+  unsigned char
+    *buffer;
 
   /*
-    Write parameter file (see mpeg2encode documentation for details).
+    Return if destination file already exists and is not empty.
   */
-  file=MagickOpenStream(basename,"w");
-  if (file == (FILE *) NULL)
+  assert(source != (const char *) NULL);
+  assert(destination != (char *) NULL);
+  status=GetPathAttributes(destination,&attributes);
+  if ((status != MagickFalse) && (attributes.st_size != 0))
+    return(MagickTrue);
+  /*
+    Copy source file to destination.
+  */
+  destination_file=open(destination,O_WRONLY | O_BINARY | O_CREAT,S_MODE);
+  if (destination_file == -1)
     return(MagickFalse);
-  (void) fprintf(file,"MPEG\n");  /* comment */
-  (void) fprintf(file,"%s.%%d\n",image->filename); /* source frame file */
-  (void) fprintf(file,"-\n");  /* reconstructed frame file */
-  if (image->quality == UndefinedCompressionQuality)
-    (void) fprintf(file,"-\n");  /* default intra quant matrix */
-  else
+  source_file=open(source,O_RDONLY | O_BINARY);
+  if (source_file == -1)
     {
-      /*
-        Write intra quant matrix file.
-      */
-      (void) FormatMagickString(filename,MaxTextExtent,"%s.iqm",basename);
-      (void) fprintf(file,"%s\n",filename);
-      parameter_file=MagickOpenStream(filename,"w");
-      if (parameter_file == (FILE *) NULL)
-        return(MagickFalse);
-      if (image->quality >= 75)
-        {
-          q=MagickMax(2.0*(image->quality-75.0),1.0);
-          for (i=0; i < 64; i++)
-          {
-            quant=(long) MagickMin(MagickMax(q_matrix[i]/q,1.0),255.0);
-            (void) fprintf(parameter_file," %ld",quant);
-            if ((i % 8) == 7)
-              (void) fprintf(parameter_file,"\n");
-          }
-        }
-      else
-        {
-          q=MagickMax((75.0-image->quality)/8.0,1.0);
-          for (i=0; i < 64; i++)
-          {
-            quant=(long) MagickMin(MagickMax(q*q_matrix[i]+0.5,1.0),255.0);
-            (void) fprintf(parameter_file," %ld",quant);
-            if ((i % 8) == 7)
-              (void) fprintf(parameter_file,"\n");
-          }
-        }
-      (void) fclose(parameter_file);
+      (void) close(destination_file);
+      return(MagickFalse);
     }
-  if (image->quality == UndefinedCompressionQuality)
-    (void) fprintf(file,"-\n");  /* default non intra quant matrix */
-  else
+  quantum=(size_t) MagickMaxBufferExtent;
+  if ((fstat(source_file,&attributes) == 0) && (attributes.st_size != 0))
+    quantum=MagickMin((size_t) attributes.st_size,MagickMaxBufferExtent);
+  buffer=(unsigned char *) AcquireQuantumMemory(quantum,sizeof(*buffer));
+  if (buffer == (unsigned char *) NULL)
     {
-      /*
-        Write non intra quant matrix file.
-      */
-      (void) FormatMagickString(filename,MaxTextExtent,"%s.niq",basename);
-      (void) fprintf(file,"%s\n",filename);
-      parameter_file=MagickOpenStream(filename,"w");
-      if (parameter_file == (FILE *) NULL)
-        return(MagickFalse);
-      q=MagickMin(MagickMax(66.0-(2*image->quality)/3.0,1.0),255);
-      for (i=0; i < 64; i++)
-      {
-        (void) fprintf(parameter_file," %d",(int) q);
-        if ((i % 8) == 7)
-          (void) fprintf(parameter_file,"\n");
-      }
-      (void) fclose(parameter_file);
+      (void) close(source_file);
+      (void) close(destination_file);
+      return(MagickFalse);
     }
-  (void) fprintf(file,"%s.log\n",basename);  /* statistics log */
-  (void) fprintf(file,"1\n");  /* input picture file format */
-  count=0;
-  for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
+  length=0;
+  for (i=0; ; i+=count)
   {
-    delay=100.0*p->delay/MagickMax(1.0*p->ticks_per_second,1.0);
-    count+=(ssize_t) MagickMax((1.0*delay+1.0)/3.0,1.0);
+    count=(ssize_t) read(source_file,buffer,quantum);
+    if (count <= 0)
+      break;
+    length=(size_t) count;
+    count=(ssize_t) write(destination_file,buffer,length);
+    if ((size_t) count != length)
+      break;
   }
-  (void) fprintf(file,"%lu\n",(unsigned long) count); /* number of frames */
-  (void) fprintf(file,"0\n");  /* number of first frame */
-  (void) fprintf(file,"00:00:00:00\n");  /* timecode of first frame */
-  mpeg=LocaleCompare(image_info->magick,"M2V") != 0 ? MagickTrue : MagickFalse;
-  if (image->quality > 98)
-    (void) fprintf(file,"1\n");
-  else
-    (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 12 : 15);
-  if (image->quality > 98)
-    (void) fprintf(file,"1\n");
-  else
-    (void) fprintf(file,"3\n");
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* ISO/IEC 11172-2 stream */
-  (void) fprintf(file,"0\n");  /* select frame picture coding */
-  (void) fprintf(file,"%lu\n",image->columns+
-    ((image->columns & 0x01) != 0 ? 1 : 0));
-  (void) fprintf(file,"%lu\n",image->rows+((image->rows & 0x01) != 0 ? 1 : 0));
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 8 : 2);  /* aspect ratio */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 3 : 5);  /* frame rate code */
-  (void) fprintf(file,"%g\n",mpeg != MagickFalse ? 1152000.0 : 5000000.0);  /* bit rate */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 20 : 112);  /* vbv buffer size */
-  (void) fprintf(file,"0\n");  /* low delay */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* constrained parameter */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 4 : 1);  /* profile ID */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 8 : 4);  /* level ID */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* progressive sequence */
-  vertical_factor=2;
-  if (image_info->sampling_factor != (char *) NULL)
-    {
-      GeometryInfo
-        geometry_info;
-
-      long
-        horizontal_factor;
-
-      MagickStatusType
-        flags;
-
-      flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
-      horizontal_factor=(long) geometry_info.rho;
-      vertical_factor=(long) geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
-        vertical_factor=horizontal_factor;
-      if (mpeg != MagickFalse)
-        {
-          if ((horizontal_factor != 2) || (vertical_factor != 2))
-            {
-              (void) fclose(file);
-              return(MagickFalse);
-            }
-        }
-      else
-        if ((horizontal_factor != 2) ||
-            ((vertical_factor != 1) && (vertical_factor != 2)))
-          {
-            (void) fclose(file);
-            return(MagickFalse);
-          }
-    }
-  (void) fprintf(file,"%d\n",vertical_factor == 2 ? 1 : 2); /* chroma format */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 2);  /* video format */
-  (void) fprintf(file,"5\n");  /* color primaries */
-  (void) fprintf(file,"5\n");  /* transfer characteristics */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 5 : 4);  /* matrix coefficients */
-  (void) fprintf(file,"%lu\n",image->columns+
-    ((image->columns & 0x01) != 0 ? 1 : 0));
-  (void) fprintf(file,"%lu\n",image->rows+((image->rows & 0x01) != 0 ? 1 : 0));
-  (void) fprintf(file,"0\n");  /* intra dc precision */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 0 : 1);  /* top field */
-  (void) fprintf(file,"%d %d %d\n",mpeg != MagickFalse ? 1 : 0,
-    mpeg != MagickFalse ? 1 : 0, mpeg != MagickFalse ? 1 : 0);
-  (void) fprintf(file,"0 0 0\n");  /* concealment motion vector */
-  (void) fprintf(file,"%d %d %d\n",mpeg != MagickFalse ? 0 : 1,
-    mpeg != MagickFalse ? 0 : 1,mpeg != MagickFalse ? 0 : 1);
-  (void) fprintf(file,"%d 0 0\n",mpeg != MagickFalse ? 0 : 1);  /* intra vlc format */
-  (void) fprintf(file,"0 0 0\n");  /* alternate scan */
-  (void) fprintf(file,"0\n");  /* repeat first field */
-  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* progressive frame */
-  (void) fprintf(file,"0\n");  /* intra slice refresh period */
-  (void) fprintf(file,"0\n");  /* reaction parameter */
-  (void) fprintf(file,"0\n");  /* initial average activity */
-  (void) fprintf(file,"0\n");
-  (void) fprintf(file,"0\n");
-  (void) fprintf(file,"0\n");
-  (void) fprintf(file,"0\n");
-  (void) fprintf(file,"0\n");
-  (void) fprintf(file,"0\n");
-  (void) fprintf(file,"2 2 11 11\n");
-  (void) fprintf(file,"1 1 3 3\n");
-  (void) fprintf(file,"1 1 7 7\n");
-  (void) fprintf(file,"1 1 7 7\n");
-  (void) fprintf(file,"1 1 3 3\n");
-  (void) fclose(file);
-  return(MagickTrue);
+  (void) close(destination_file);
+  (void) close(source_file);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  return(i != 0 ? MagickTrue : MagickFalse);
 }
 
 static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
   Image *image)
 {
+#define WriteMPEGIntermediateFormat "jpg"
+
   char
     basename[MaxTextExtent],
     filename[MaxTextExtent];
@@ -552,8 +453,7 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
     delay;
 
   Image
-    *coalesce_image,
-    *next_image;
+    *coalesce_image;
 
   ImageInfo
     *write_info;
@@ -592,55 +492,20 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   /*
-    Determine if the sequence of images have identical page info.
-  */
-  coalesce_image=image;
-  for (next_image=image; next_image != (Image *) NULL; )
-  {
-    if ((image->columns != next_image->columns) ||
-        (image->rows != next_image->rows))
-      break;
-    if ((image->page.x != next_image->page.x) ||
-        (image->page.y != next_image->page.y))
-      break;
-    next_image=GetNextImageInList(next_image);
-  }
-  if (next_image != (Image *) NULL)
-    {
-      coalesce_image=CoalesceImages(image,&image->exception);
-      if (coalesce_image == (Image *) NULL)
-        return(MagickFalse);
-    }
-  /*
-    Write YUV files.
+    Write JPEG files.
   */
+  coalesce_image=CoalesceImages(image,&image->exception);
+  if (coalesce_image == (Image *) NULL)
+    return(MagickFalse);
   file=AcquireUniqueFileResource(basename);
   if (file != -1)
     file=close(file)-1;
   (void) FormatMagickString(coalesce_image->filename,MaxTextExtent,"%s",
     basename);
-  write_info=CloneImageInfo(image_info);
-  status=WriteMPEGParameterFiles(write_info,coalesce_image,basename);
-  if (status == MagickFalse)
-    {
-      if (coalesce_image != image)
-        coalesce_image=DestroyImage(coalesce_image);
-      (void) RelinquishUniqueFileResource(basename);
-      if (image->quality != UndefinedCompressionQuality)
-        {
-          (void) FormatMagickString(filename,MaxTextExtent,"%s.iqm",
-            basename);
-          (void) RelinquishUniqueFileResource(filename);
-          (void) FormatMagickString(filename,MaxTextExtent,"%s.niq",
-            basename);
-          (void) RelinquishUniqueFileResource(filename);
-        }
-      ThrowWriterException(CoderError,"UnableToWriteMPEGParameters");
-    }
   count=0;
-  write_info->interlace=PlaneInterlace;
+  write_info=CloneImageInfo(image_info);
   for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
   {
     char
@@ -662,12 +527,12 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
           Image
             *frame;
 
-          (void) FormatMagickString(p->filename,MaxTextExtent,"%s.%lu.yuv",
-            basename,p->scene);
-          (void) FormatMagickString(filename,MaxTextExtent,"%s.%lu.yuv",
-            basename,p->scene);
+          (void) FormatMagickString(p->filename,MaxTextExtent,"%s%lu.%s",
+            basename,p->scene,WriteMPEGIntermediateFormat);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s%lu.%s",
+            basename,p->scene,WriteMPEGIntermediateFormat);
           (void) FormatMagickString(previous_image,MaxTextExtent,
-            "%s.%lu.yuv",basename,p->scene);
+            "%s%lu.%s",basename,p->scene,WriteMPEGIntermediateFormat);
           frame=CloneImage(p,0,0,MagickTrue,&p->exception);
           if (frame == (Image *) NULL)
             break;
@@ -677,13 +542,13 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
         }
         case 1:
         {
-          blob=(unsigned char *)
-            FileToBlob(previous_image,~0UL,&length,&image->exception);
+          blob=(unsigned char *) FileToBlob(previous_image,~0UL,&length,
+            &image->exception);
         }
         default:
         {
-          (void) FormatMagickString(filename,MaxTextExtent,"%s.%lu.yuv",
-            basename,p->scene);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s%lu.%s",
+            basename,p->scene,WriteMPEGIntermediateFormat);
           if (length > 0)
             status=BlobToFile(filename,blob,length,&image->exception);
           break;
@@ -693,10 +558,10 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
         {
           if (status != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "%lu. Wrote YUV file for scene %lu:",i,p->scene);
+              "%lu. Wrote JPEG file for scene %lu:",i,p->scene);
           else
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "%lu. Failed to write YUV file for scene %lu:",i,p->scene);
+              "%lu. Failed to write JPEG file for scene %lu:",i,p->scene);
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",
             filename);
         }
@@ -708,14 +573,23 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
       break;
   }
   /*
-    Convert YUV to MPEG.
+    Convert JPEG to MPEG.
   */
+  (void) CopyMagickString(coalesce_image->magick_filename,basename,
+    MaxTextExtent);
   (void) CopyMagickString(coalesce_image->filename,basename,MaxTextExtent);
+  GetPathComponent(image_info->filename,ExtensionPath,coalesce_image->magick);
+  if (*coalesce_image->magick == '\0')
+    (void) CopyMagickString(coalesce_image->magick,image->magick,MaxTextExtent);
   status=InvokeDelegate(write_info,coalesce_image,(char *) NULL,"mpeg:encode",
     &image->exception);
+  (void) FormatMagickString(write_info->filename,MaxTextExtent,"%s.%s",
+    write_info->unique,coalesce_image->magick);
+  status=CopyDelegateFile(write_info->filename,image->filename);
+  (void) RelinquishUniqueFileResource(write_info->filename);
   write_info=DestroyImageInfo(write_info);
   /*
-    Free resources.
+    Relinquish resources.
   */
   count=0;
   for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
@@ -723,21 +597,14 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
     delay=100.0*p->delay/MagickMax(1.0*p->ticks_per_second,1.0);
     for (i=0; i < (long) MagickMax((1.0*delay+1.0)/3.0,1.0); i++)
     {
-      (void) FormatMagickString(p->filename,MaxTextExtent,"%s.%lu.yuv",
-        basename,count++);
+      (void) FormatMagickString(p->filename,MaxTextExtent,"%s%lu.%s",
+        basename,count++,WriteMPEGIntermediateFormat);
       (void) RelinquishUniqueFileResource(p->filename);
     }
     (void) CopyMagickString(p->filename,image_info->filename,MaxTextExtent);
   }
   (void) RelinquishUniqueFileResource(basename);
-  (void) FormatMagickString(filename,MaxTextExtent,"%s.iqm",basename);
-  (void) RelinquishUniqueFileResource(filename);
-  (void) FormatMagickString(filename,MaxTextExtent,"%s.niq",basename);
-  (void) RelinquishUniqueFileResource(filename);
-  (void) FormatMagickString(filename,MaxTextExtent,"%s.log",basename);
-  (void) RelinquishUniqueFileResource(filename);
-  if (coalesce_image != image)
-    coalesce_image=DestroyImage(coalesce_image);
+  coalesce_image=DestroyImageList(coalesce_image);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),"exit");
   return(status);
