@@ -53,6 +53,8 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
 #include "magick/property.h"
@@ -68,11 +70,11 @@
 */
 #define PALM_IS_COMPRESSED_FLAG  0x8000
 #define PALM_HAS_COLORMAP_FLAG  0x4000
+#define PALM_HAS_FOUR_BYTE_FIELD  0x0200
 #define PALM_HAS_TRANSPARENCY_FLAG  0x2000
 #define PALM_IS_INDIRECT  0x1000
 #define PALM_IS_FOR_SCREEN  0x0800
 #define PALM_IS_DIRECT_COLOR  0x0400
-
 #define PALM_COMPRESSION_SCANLINE  0x00
 #define PALM_COMPRESSION_RLE  0x01
 #define PALM_COMPRESSION_NONE  0xFF
@@ -179,7 +181,6 @@ static MagickBooleanType
 %
 %    o pixel: a pointer to the PixelPacket to be matched.
 %
-%
 */
 static int FindColor(PixelPacket *pixel)
 {
@@ -306,7 +307,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -318,6 +319,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   {
     image->columns=ReadBlobMSBShort(image);
     image->rows=ReadBlobMSBShort(image);
+    if (EOFBlob(image) != MagickFalse)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     /*
       Copied from coders/pnm.c. TODO other checks ...
     */
@@ -337,7 +340,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       Initialize image colormap.
     */
     if ((bits_per_pixel < 16) &&
-        !AllocateImageColormap(image,1L << bits_per_pixel))
+        (AcquireImageColormap(image,1L << bits_per_pixel) == MagickFalse))
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     GetMagickPixelPacket(image,&transpix);
     if (bits_per_pixel == 16)  /* Direct Color */
@@ -353,17 +356,21 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       }
     if (bits_per_pixel == 8)
       {
+        IndexPacket
+          index;
+
         if (flags & PALM_HAS_COLORMAP_FLAG)
           {
             count=(ssize_t) ReadBlobMSBShort(image);
             for (i=0; i < (long) count; i++)
             {
               ReadBlobByte(image);
-              image->colormap[255-i].red=
+              index=ConstrainColormapIndex(image,255-i);
+              image->colormap[(int) index].red=
                 ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-              image->colormap[255-i].green=
+              image->colormap[(int) index].green=
                 ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-              image->colormap[255-i].blue=
+              image->colormap[(int) index].blue=
                 ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
           }
         }
@@ -371,9 +378,13 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         {
           for (i=0; i < (long) (1L << bits_per_pixel); i++)
           {
-            image->colormap[255-i].red=ScaleCharToQuantum(PalmPalette[i][0]);
-            image->colormap[255-i].green=ScaleCharToQuantum(PalmPalette[i][1]);
-            image->colormap[255-i].blue=ScaleCharToQuantum(PalmPalette[i][2]);
+            index=ConstrainColormapIndex(image,255-i);
+            image->colormap[(int) index].red=
+              ScaleCharToQuantum(PalmPalette[i][0]);
+            image->colormap[(int) index].green=
+              ScaleCharToQuantum(PalmPalette[i][1]);
+            image->colormap[(int) index].blue=
+              ScaleCharToQuantum(PalmPalette[i][2]);
           }
         }
       }
@@ -486,14 +497,20 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
           if (SyncImagePixels(image) == MagickFalse)
             break;
         }
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
     if (flags & PALM_HAS_TRANSPARENCY_FLAG)
       {
         if (bits_per_pixel != 16)
           SetMagickPixelPacket(image,image->colormap+(mask-transparentIndex),
             (const IndexPacket *) NULL,&transpix);
-        (void) PaintTransparentImage(image,&transpix,(Quantum)
-          TransparentOpacity);
+        (void) TransparentPaintImage(image,&transpix,(Quantum)
+          TransparentOpacity,MagickFalse);
       }
     one_row=(unsigned char *) RelinquishMagickMemory(one_row);
     if (compressionType == PALM_COMPRESSION_SCANLINE)
@@ -523,24 +540,20 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         /*
           Allocate next image structure. Copied from coders/pnm.c
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             (void) DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        /* TODO Analyze this ...
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }*/
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (nextDepthOffset != 0);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -620,7 +633,8 @@ ModuleExport void UnregisterPALMImage(void)
 %
 %  The format of the WritePALMImage method is:
 %
-%      MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePALMImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -635,12 +649,6 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
   int
     y;
 
-  const char
-    *value;
-
-  Image
-    *map;
-
   ExceptionInfo
     exception;
 
@@ -649,7 +657,8 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
 
   MagickOffsetType
     currentOffset,
-    offset;
+    offset,
+    scene;
 
   MagickSizeType
     cc;
@@ -658,7 +667,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
     transpix;
 
   QuantizeInfo
-    quantize_info;
+    *quantize_info;
 
   register IndexPacket
     *indexes;
@@ -676,10 +685,10 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
     *lastrow,
     *one_row,
     *ptr,
-    version = 0;
+    version;
 
   unsigned int
-    transparentIndex = 0;
+    transparentIndex;
 
   unsigned long
     count,
@@ -689,7 +698,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
 
   unsigned short
     color16,
-    flags = 0;
+    flags;
 
 
   /*
@@ -705,19 +714,22 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
   if (status == MagickFalse)
     return(status);
   GetExceptionInfo(&exception);
+  quantize_info=AcquireQuantizeInfo(image_info);
+  flags=0;
   currentOffset=0;
+  transparentIndex=0;
   transpix.red=0;
   transpix.green=0;
   transpix.blue=0;
   transpix.opacity=0;
+  version=0;
+  scene=0;
   do
   {
-    value=GetImageProperty(image, "comment");
-    if (value != (const char *) NULL)
-      if (LocaleCompare("COLORMAP",value) == 0)
-        flags|=PALM_HAS_COLORMAP_FLAG;
-    count = GetNumberColors(image, NULL, &exception);
-    for (bits_per_pixel=1;  (1UL << bits_per_pixel) < count;  bits_per_pixel*=2);
+    if (image->colorspace != RGBColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
+    count=GetNumberColors(image,NULL,&exception);
+    for (bits_per_pixel=1;  (1UL << bits_per_pixel) < count;  bits_per_pixel*=2) ;
     if (image_info->depth > 100)
       bits_per_pixel=image_info->depth-100;
     if (bits_per_pixel < 16)
@@ -728,30 +740,34 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
         (void) SetImageType(image,PaletteType);
         (void) SortColormapByIntensity(image);
       }
-    if (bits_per_pixel > 8)
+    if ((image->storage_class == PseudoClass) && (image->colors > 256))
+      (void) SetImageStorageClass(image,DirectClass);
+    if (image->storage_class == PseudoClass)
+      flags|=PALM_HAS_COLORMAP_FLAG;
+    else
       flags|=PALM_IS_DIRECT_COLOR;
     (void) WriteBlobMSBShort(image,(unsigned short) image->columns); /* width */
-    (void) WriteBlobMSBShort(image,(unsigned short) image->rows );  /* height */
+    (void) WriteBlobMSBShort(image,(unsigned short) image->rows);  /* height */
     bytes_per_row=((image->columns+(16/bits_per_pixel-1))/(16/
       bits_per_pixel))*2;
     (void) WriteBlobMSBShort(image,(unsigned short) bytes_per_row);
-    if ((image->compression == RLECompression) ||
-        (image->compression == FaxCompression))
+    if ((image_info->compression == RLECompression) ||
+        (image_info->compression == FaxCompression))
       flags|=PALM_IS_COMPRESSED_FLAG;
     (void) WriteBlobMSBShort(image, flags);
     (void) WriteBlobByte(image,(unsigned char) bits_per_pixel);
     if (bits_per_pixel > 1)
       version=1;
-    if ((image->compression == RLECompression) ||
-        (image->compression == FaxCompression))
+    if ((image_info->compression == RLECompression) ||
+        (image_info->compression == FaxCompression))
       version=2;
     (void) WriteBlobByte(image,version);
     (void) WriteBlobMSBShort(image,0);  /* nextDepthOffset */
     (void) WriteBlobByte(image,(unsigned char) transparentIndex);
-    if (image->compression == RLECompression)
+    if (image_info->compression == RLECompression)
       (void) WriteBlobByte(image,PALM_COMPRESSION_RLE);
     else
-      if (image->compression == FaxCompression)
+      if (image_info->compression == FaxCompression)
         (void) WriteBlobByte(image,PALM_COMPRESSION_SCANLINE);
       else
         (void) WriteBlobByte(image,PALM_COMPRESSION_NONE);
@@ -770,10 +786,9 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
       {
         if (flags & PALM_HAS_COLORMAP_FLAG)  /* Write out colormap */
           {
-            GetQuantizeInfo(&quantize_info);
-            quantize_info.dither=IsPaletteImage(image,&image->exception);
-            quantize_info.number_colors=image->colors;
-            (void) QuantizeImage(&quantize_info,image);
+            quantize_info->dither=IsPaletteImage(image,&image->exception);
+            quantize_info->number_colors=image->colors;
+            (void) QuantizeImage(quantize_info,image);
             (void) WriteBlobMSBShort(image,(unsigned short) image->colors);
             for (count = 0; count < image->colors; count++)
             {
@@ -789,9 +804,13 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
           }
       else  /* Map colors to Palm standard colormap */
         {
-          map=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,&exception);
-          (void) SetImageColorspace(map,map->colorspace);
-          (void) MapImage(image,map,image_info->dither);
+          Image
+            *affinity_image;
+
+          affinity_image=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,
+            &exception);
+          (void) SetImageColorspace(affinity_image,affinity_image->colorspace);
+          (void) RemapImage(quantize_info,image,affinity_image);
           for (y=0; y < (long) image->rows; y++)
           {
             p=GetImagePixels(image,0,y,image->columns,1);
@@ -800,13 +819,13 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
               indexes[x]=(IndexPacket) FindColor(&image->colormap[(long)
                 indexes[x]]);
           }
-          (void) DestroyImage(map);
+          affinity_image=DestroyImage(affinity_image);
         }
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)
       (void) WriteBlobMSBShort(image,0);  /* fill in size later */
     lastrow=(unsigned char *) NULL;
-    if (image->compression == FaxCompression)
+    if (image_info->compression == FaxCompression)
       lastrow=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
         sizeof(*lastrow));
       /* TODO check whether memory really was acquired? */
@@ -867,7 +886,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
           if ((image->columns % (8/bits_per_pixel)) != 0)
             *ptr++=byte;
         }
-      if (image->compression == RLECompression)
+      if (image_info->compression == RLECompression)
         {
           x=0;
           while (x < (long) bytes_per_row)
@@ -882,7 +901,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
           }
         }
       else
-        if (image->compression == FaxCompression)
+        if (image_info->compression == FaxCompression)
           {
             char
               tmpbuf[8],
@@ -925,7 +944,8 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
     if (flags & PALM_IS_COMPRESSED_FLAG)  /* fill in size now */
       {
         offset=SeekBlob(image,currentOffset+offset,SEEK_SET);
-        (void) WriteBlobMSBShort(image,(unsigned short) (GetBlobSize(image)-currentOffset-offset));
+        (void) WriteBlobMSBShort(image,(unsigned short) (GetBlobSize(image)-
+          currentOffset-offset));
       }
     if (one_row != (unsigned char *) NULL) 
       one_row=(unsigned char *) RelinquishMagickMemory(one_row);
@@ -945,8 +965,13 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
     currentOffset=(MagickOffsetType) GetBlobSize(image);
     offset=SeekBlob(image,currentOffset,SEEK_SET);
     image=SyncNextImageInList(image);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  quantize_info=DestroyQuantizeInfo(quantize_info);
+  (void) CloseBlob(image);
   (void) DestroyExceptionInfo(&exception);
   return(MagickTrue);
 }
