@@ -10,7 +10,7 @@
 %                            P       CCCC  X   X                              %
 %                                                                             %
 %                                                                             %
-%                Read/Write ZSoft IBM PC Paintbrush Image Format.             %
+%                Read/Write ZSoft IBM PC Paintbrush Image Format              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -190,7 +191,7 @@ static MagickBooleanType IsPCX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -278,7 +279,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -286,7 +287,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   /*
-    Determine if this is a PCX file.
+    Determine if this a PCX file.
   */
   page_table=(MagickOffsetType *) NULL;
   if (LocaleCompare(image_info->magick,"DCX") == 0)
@@ -362,7 +363,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           ((pcx_info.bits_per_pixel*pcx_info.planes) == 1))
         image->colors=(unsigned long) MagickMin(1UL << (1UL*
           pcx_info.bits_per_pixel*pcx_info.planes),256UL);
-    if (AllocateImageColormap(image,image->colors) == MagickFalse)
+    if (AcquireImageColormap(image,image->colors) == MagickFalse)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((pcx_info.bits_per_pixel >= 8) && (pcx_info.planes != 1))
       image->storage_class=DirectClass;
@@ -620,14 +621,11 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        {
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
     if (image->storage_class == PseudoClass)
       (void) SyncImage(image);
@@ -660,25 +658,22 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   }
   if (page_table != (MagickOffsetType *) NULL)
     page_table=(MagickOffsetType *) RelinquishMagickMemory(page_table);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -775,7 +770,7 @@ ModuleExport void UnregisterPCXImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -878,7 +873,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   page_table=(MagickOffsetType *) NULL;
   if ((LocaleCompare(image_info->magick,"DCX") == 0) ||
@@ -942,8 +937,8 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
         if (image->matte != MagickFalse)
           pcx_info.planes++;
       }
-    pcx_info.bytes_per_line=(unsigned short)
-      (((unsigned long) image->columns*pcx_info.bits_per_pixel+7)/8);
+    pcx_info.bytes_per_line=(unsigned short) (((unsigned long) image->columns*
+      pcx_info.bits_per_pixel+7)/8);
     pcx_info.palette_info=1;
     pcx_info.colormap_signature=0x0c;
     /*
@@ -1049,110 +1044,101 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
           }
           if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
             break;
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
+          if (image->previous == (Image *) NULL)
             {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
+              status=SetImageProgress(image,SaveImageTag,y,image->rows);
               if (status == MagickFalse)
                 break;
             }
         }
       }
     else
-      if (pcx_info.bits_per_pixel > 1)
-        for (y=0; y < (long) image->rows; y++)
-        {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-          if (p == (const PixelPacket *) NULL)
-            break;
-          indexes=GetIndexes(image);
-          q=pcx_pixels;
-          for (x=0; x < (long) image->columns; x++)
-            *q++=(unsigned char) indexes[x];
-          if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
-            break;
-          if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(SaveImageTag,y,image->rows,
-                  image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
-        }
-      else
-        {
-          IndexPacket
-            polarity;
-
-          register unsigned char
-            bit,
-            byte;
-
-          /*
-            Convert PseudoClass image to a PCX monochrome image.
-          */
-          polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
-            ((Quantum) QuantumRange/2) ? 1 : 0);
-          if (image->colors == 2)
-            polarity=(IndexPacket) (
-              PixelIntensityToQuantum(&image->colormap[0]) <
-              PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
+      {
+        if (pcx_info.bits_per_pixel > 1)
           for (y=0; y < (long) image->rows; y++)
           {
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
             indexes=GetIndexes(image);
-            bit=0;
-            byte=0;
             q=pcx_pixels;
             for (x=0; x < (long) image->columns; x++)
-            {
-              byte<<=1;
-              if (indexes[x] == polarity)
-                byte|=0x01;
-              bit++;
-              if (bit == 8)
-                {
-                  *q++=byte;
-                  bit=0;
-                  byte=0;
-                }
-              p++;
-            }
-            if (bit != 0)
-              *q++=byte << (8-bit);
+              *q++=(unsigned char) indexes[x];
             if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                  (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (status == MagickFalse)
+                  break;
+              }
+          }
+        else
+          {
+            IndexPacket
+              polarity;
+
+            register unsigned char
+              bit,
+              byte;
+
+            /*
+              Convert PseudoClass image to a PCX monochrome image.
+            */
+            polarity=(IndexPacket) (PixelIntensityToQuantum(
+              &image->colormap[0]) < ((Quantum) QuantumRange/2) ? 1 : 0);
+            if (image->colors == 2)
+              polarity=(IndexPacket) (
+                PixelIntensityToQuantum(&image->colormap[0]) <
+                PixelIntensityToQuantum(&image->colormap[1]) ? 1 : 0);
+            for (y=0; y < (long) image->rows; y++)
+            {
+              p=AcquireImagePixels(image,0,y,image->columns,1,
+                &image->exception);
+              if (p == (const PixelPacket *) NULL)
+                break;
+              indexes=GetIndexes(image);
+              bit=0;
+              byte=0;
+              q=pcx_pixels;
+              for (x=0; x < (long) image->columns; x++)
+              {
+                byte<<=1;
+                if (indexes[x] == polarity)
+                  byte|=0x01;
+                bit++;
+                if (bit == 8)
+                  {
+                    *q++=byte;
+                    bit=0;
+                    byte=0;
+                  }
+                p++;
+              }
+              if (bit != 0)
+                *q++=byte << (8-bit);
+              if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
+                break;
+              if (image->previous == (Image *) NULL)
                 {
-                  status=image->progress_monitor(SaveImageTag,y,image->rows,
-                    image->client_data);
+                  status=SetImageProgress(image,SaveImageTag,y,image->rows);
                   if (status == MagickFalse)
                     break;
                 }
+            }
           }
-        }
-    (void) WriteBlobByte(image,pcx_info.colormap_signature);
-    (void) WriteBlob(image,3*256,pcx_colormap);
+        (void) WriteBlobByte(image,pcx_info.colormap_signature);
+        (void) WriteBlob(image,3*256,pcx_colormap);
+      }
     pcx_pixels=(unsigned char *) RelinquishMagickMemory(pcx_pixels);
     pcx_colormap=(unsigned char *) RelinquishMagickMemory(pcx_colormap);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
     if (scene >= 1023)
       break;
   } while (image_info->adjoin != MagickFalse);
@@ -1168,8 +1154,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
       for (i=0; i <= (long) scene; i++)
         (void) WriteBlobLSBLong(image,(unsigned long) page_table[i]);
-      page_table=(MagickOffsetType *)
-        RelinquishMagickMemory(page_table);
+      page_table=(MagickOffsetType *) RelinquishMagickMemory(page_table);
     }
   if (status == MagickFalse)
     {
@@ -1181,6 +1166,6 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
         FileOpenError,"UnableToWriteFile","`%s': %s",image->filename,message);
       message=DestroyString(message);
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
