@@ -10,7 +10,7 @@
 %                            P      DDDD   BBBB                               %
 %                                                                             %
 %                                                                             %
-%               Read/Write Palm Database ImageViewer Image Format.            %
+%               Read/Write Palm Database ImageViewer Image Format             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -60,6 +60,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/property.h"
 #include "magick/quantum-private.h"
 #include "magick/quantum-private.h"
@@ -150,7 +151,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image: The address of a structure of type Image.
+%    o image: the address of a structure of type Image.
 %
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
 %      the decoding process.
@@ -250,7 +251,7 @@ static MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -314,7 +315,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -322,7 +323,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   /*
-    Determine if this is a PDB image file.
+    Determine if this a PDB image file.
   */
   count=ReadBlob(image,32,(unsigned char *) pdb_info.name);
   pdb_info.attributes=(short) ReadBlobMSBShort(image);
@@ -347,7 +348,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Read record header.
   */
   img_offset=(long) ReadBlobMSBLong(image); /* TS */
-  attributes=ReadBlobByte(image);
+  attributes=(unsigned char) ReadBlobByte(image);
   count=ReadBlob(image,3,(unsigned char *) tag);
   if (count != 3  ||  memcmp(tag,"\x6f\x80\x00",3) != 0)
     ThrowReaderException(CorruptImageError,"CorruptImage");
@@ -355,13 +356,13 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (pdb_info.number_records > 1)
     {
       comment_offset=(long) ReadBlobMSBLong(image);
-      attributes=ReadBlobByte(image);
+      attributes=(unsigned char) ReadBlobByte(image);
       count=ReadBlob(image,3,(unsigned char *) tag);
       if (count != 3  ||  memcmp(tag,"\x6f\x80\x01",3) != 0)
         ThrowReaderException(CorruptImageError,"CorruptImage");
     }
 
-  num_pad_bytes = img_offset - TellBlob( image );
+  num_pad_bytes = (unsigned long) (img_offset - TellBlob( image ));
   while (num_pad_bytes--) ReadBlobByte( image );
   /*
     Read image header.
@@ -386,11 +387,11 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->depth=8;
   image->storage_class=PseudoClass;
   bits_per_pixel=pdb_image.type == 0 ? 2UL : pdb_image.type == 2 ? 4UL : 1UL;
-  if (AllocateImageColormap(image,1 << bits_per_pixel) == MagickFalse)
+  if (AcquireImageColormap(image,1 << bits_per_pixel) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   if (SetImageExtent(image,0,0) == MagickFalse)
@@ -452,14 +453,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -492,14 +488,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -526,14 +517,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -559,7 +545,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       unsigned long
         length;
 
-      num_pad_bytes = comment_offset - TellBlob( image );
+      num_pad_bytes = (unsigned long) (comment_offset - TellBlob( image ));
       while (num_pad_bytes--) ReadBlobByte( image );
 
       /*
@@ -590,7 +576,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) SetImageProperty(image,"comment",comment);
       comment=DestroyString(comment);
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -675,7 +661,7 @@ ModuleExport void UnregisterPDBImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -755,7 +741,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
 
   if (image -> colors <= 2  ||  GetImageType( image, &image -> exception ) == BilevelType) { /* TS */
@@ -830,7 +816,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    if (image_info->colorspace == UndefinedColorspace)
+    if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   /*
     Convert to GRAY raster scanline.
@@ -846,7 +832,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
+    (void) ExportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
     for (x=0; x < pdb_image.width; x++)
     {
       if (x < (long) image->columns)
@@ -894,14 +880,9 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
         buffer[literal+repeat]=0x00;
       }
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SaveImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   q=EncodeRLE(q,buffer,literal,repeat);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -946,6 +927,6 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   runlength=(unsigned char *) RelinquishMagickMemory(runlength);
   if (pdb_info.number_records > 1)
     (void) WriteBlobString(image,comment);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
