@@ -10,7 +10,7 @@
 %                        P      IIIII   CCCC    T                             %
 %                                                                             %
 %                                                                             %
-%               Read/Write Apple Macintosh QuickDraw/PICT Format.             %
+%               Read/Write Apple Macintosh QuickDraw/PICT Format              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -55,6 +55,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/profile.h"
 #include "magick/resource_.h"
 #include "magick/quantum-private.h"
@@ -335,20 +336,20 @@ static MagickBooleanType
 %
 %      unsigned char *DecodeImage(Image *blob,Image *image,
 %        unsigned long bytes_per_line,const int bits_per_pixel,
-%        unsigned long *extent)
+%        unsigned size_t extent)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o blob,image: The address of a structure of type Image.
+%    o blob,image: the address of a structure of type Image.
 %
 %    o bytes_per_line: This integer identifies the number of bytes in a
 %      scanline.
 %
-%    o bits_per_pixel: The number of bits in a pixel.
+%    o bits_per_pixel: the number of bits in a pixel.
 %
-%    o extent: The number of pixels allocated.
+%    o extent: the number of pixels allocated.
 %
 */
 
@@ -421,8 +422,7 @@ static unsigned char *ExpandBuffer(unsigned char *pixels,
 }
 
 static unsigned char *DecodeImage(Image *blob,Image *image,
-  unsigned long bytes_per_line,const unsigned int bits_per_pixel,
-  unsigned long *extent)
+  unsigned long bytes_per_line,const unsigned int bits_per_pixel,size_t *extent)
 {
   long
     j,
@@ -439,7 +439,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
     *q;
 
   size_t
-    length;
+    length,
+    row_bytes;
 
   ssize_t
     count;
@@ -448,9 +449,6 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
     *pixels,
     *scanline;
 
-  unsigned short
-    row_bytes;
-
   unsigned long
     bytes_per_pixel,
     scanline_length,
@@ -473,9 +471,9 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
       width*=image->matte ? 4 : 3;
   if (bytes_per_line == 0)
     bytes_per_line=width;
-  row_bytes=(unsigned short) (image->columns | 0x8000);
+  row_bytes=(size_t) (image->columns | 0x8000);
   if (image->storage_class == DirectClass)
-    row_bytes=(unsigned short) ((4*image->columns) | 0x8000);
+    row_bytes=(size_t) ((4*image->columns) | 0x8000);
   /*
     Allocate pixel and scanline buffer.
   */
@@ -499,6 +497,13 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
         number_pixels=bytes_per_line;
         count=ReadBlob(blob,(size_t) number_pixels,scanline);
         p=ExpandBuffer(scanline,&number_pixels,bits_per_pixel);
+        if ((q+number_pixels) > (pixels+(*extent)))
+          {
+            (void) ThrowMagickException(&image->exception,GetMagickModule(),
+              CorruptImageError,"UnableToUncompressImage","`%s'",
+              image->filename);
+            break;
+          }
         (void) CopyMagickMemory(q,p,(size_t) number_pixels);
       }
       scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -517,7 +522,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
     if (scanline_length >= row_bytes)
       {
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
-           CorruptImageError,"UnableToUncompressImage","`%s'",image->filename);
+          CorruptImageError,"UnableToUncompressImage","`%s'",image->filename);
         break;
       }
     count=ReadBlob(blob,scanline_length,scanline);
@@ -527,7 +532,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           length=(size_t) ((scanline[j] & 0xff)+1);
           number_pixels=length*bytes_per_pixel;
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
-          (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+          if ((q-pixels+number_pixels) <= *extent)
+            (void) CopyMagickMemory(q,p,(size_t) number_pixels);
           q+=number_pixels;
           j+=(long) (length*bytes_per_pixel+1);
         }
@@ -538,7 +544,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
           for (i=0; i < (long) length; i++)
           {
-            (void) CopyMagickMemory(q,p,(size_t) number_pixels);
+            if ((q-pixels+number_pixels) <= *extent)
+              (void) CopyMagickMemory(q,p,(size_t) number_pixels);
             q+=number_pixels;
           }
           j+=bytes_per_pixel+1;
@@ -569,11 +576,11 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The address of a structure of type Image.
+%    o image: the address of a structure of type Image.
 %
 %    o scanline: A pointer to an array of characters to pack.
 %
-%    o bytes_per_line: The number of bytes in a scanline.
+%    o bytes_per_line: the number of bytes in a scanline.
 %
 %    o pixels: A pointer to an array of characters where the packed
 %      characters are stored.
@@ -768,7 +775,7 @@ static MagickBooleanType IsPICT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -827,6 +834,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     i;
 
   size_t
+    extent,
     length;
 
   ssize_t
@@ -835,9 +843,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   StringInfo
     *profile;
 
-  unsigned long
-    extent;
-
   /*
     Open image file.
   */
@@ -848,7 +853,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   image->depth=8;
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -865,7 +870,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     (void) ReadBlobByte(image);  /* skip header */
   (void) ReadBlobMSBShort(image);  /* skip picture size */
   ReadRectangle(image,frame);
-  while ((c=ReadBlobByte(image)) == 0);
+  while ((c=ReadBlobByte(image)) == 0) ;
   if (c != 0x11)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   version=ReadBlobByte(image);
@@ -878,6 +883,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   else
     if (version != 1)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if ((frame.left < 0) || (frame.right < 0) || (frame.top < 0) ||
+      (frame.bottom < 0) || (frame.left >= frame.right) ||
+      (frame.top >= frame.bottom))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   /*
     Create black canvas.
   */
@@ -1100,7 +1109,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                     flags=1L*ReadBlobMSBShort(image);
                     tile_image->colors=1UL*ReadBlobMSBShort(image)+1;
                   }
-                status=AllocateImageColormap(tile_image,tile_image->colors);
+                status=AcquireImageColormap(tile_image,tile_image->colors);
                 if (status == MagickFalse)
                   {
                     tile_image=DestroyImage(tile_image);
@@ -1232,15 +1241,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   if (p < pixels)
                     break;
                 }
-              if (destination.bottom == (long) image->rows)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,tile_image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(LoadImageTag,y,
-                      tile_image->rows,image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+              status=SetImageProgress(image,LoadImageTag,y,tile_image->rows);
+              if (status == MagickFalse)
+                break;
             }
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
             if (jpeg == MagickFalse)
@@ -1249,13 +1252,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 (void) CompositeImage(image,CopyCompositeOp,tile_image,
                   destination.left,destination.top);
             tile_image=DestroyImage(tile_image);
-            if (image->progress_monitor != (MagickProgressMonitor) NULL)
-              {
-                status=image->progress_monitor(LoadImageTag,destination.bottom,
-                  image->rows,image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
             break;
           }
           case 0xa1:
@@ -1363,7 +1359,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         file=(FILE *) NULL;
         unique_file=AcquireUniqueFileResource(read_info->filename);
         if (unique_file != -1)
-          file=MagickOpenStream(read_info->filename,"wb");
+          file=OpenMagickStream(read_info->filename,"wb");
         if ((unique_file == -1) || (file == (FILE *) NULL))
           {
             (void) CopyMagickString(image->filename,read_info->filename,
@@ -1394,7 +1390,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           MagickMax(image->columns,tile_image->columns),
           MagickMax(image->rows,tile_image->rows));
         (void) TransformImage(&image,(char *) NULL,geometry);
-        if (image_info->colorspace == UndefinedColorspace)
+        if (image->colorspace != RGBColorspace)
           (void) SetImageColorspace(image,tile_image->colorspace);
         (void) CompositeImage(image,CopyCompositeOp,tile_image,frame.left,
           frame.right);
@@ -1426,7 +1422,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         continue;
       }
   }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 #endif
@@ -1523,7 +1519,7 @@ ModuleExport void UnregisterPICTImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -1610,7 +1606,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize image info.
@@ -1645,7 +1641,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   y_resolution=image->y_resolution != 0.0 ? image->y_resolution :
     DefaultResolution;
   storage_class=image->storage_class;
-  if (image->compression == JPEGCompression)
+  if (image_info->compression == JPEGCompression)
     storage_class=DirectClass;
   if ((storage_class == DirectClass) || (image->matte != MagickFalse))
     {
@@ -1729,7 +1725,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   (void) WriteBlobMSBShort(image,(unsigned short) crop_rectangle.left);
   (void) WriteBlobMSBShort(image,(unsigned short) crop_rectangle.bottom);
   (void) WriteBlobMSBShort(image,(unsigned short) crop_rectangle.right);
-  if (image->compression == JPEGCompression)
+  if (image_info->compression == JPEGCompression)
     {
       Image
         *jpeg_image;
@@ -1746,7 +1742,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       jpeg_image=CloneImage(image,0,0,MagickTrue,&image->exception);
       if (jpeg_image == (Image *) NULL)
         {
-          CloseBlob(image);
+          (void) CloseBlob(image);
           return(MagickFalse);
         }
       DestroyBlob(jpeg_image);
@@ -1903,7 +1899,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         }
     }
   else
-    if (image->compression == JPEGCompression)
+    if (image_info->compression == JPEGCompression)
       {
         (void) ResetMagickMemory(scanline,0,row_bytes);
         for (y=0; y < (long) image->rows; y++)
@@ -1968,6 +1964,6 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   packed_scanline=(unsigned char *) RelinquishMagickMemory(packed_scanline);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
