@@ -10,7 +10,7 @@
 %                            P      SSSSS  33333                              %
 %                                                                             %
 %                                                                             %
-%                     Write Postscript Level III Format.                      %
+%                     Write Postscript Level III Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -57,6 +57,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/property.h"
 #include "magick/quantum-private.h"
@@ -184,7 +185,7 @@ ModuleExport void UnregisterPS3Image(void)
 %
 %    o image_info: Specifies a pointer to a ImageInfo structure.
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 
@@ -244,14 +245,11 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
         p++;
       }
     if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+      {
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
@@ -289,8 +287,7 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
-  pack=(unsigned long) (IsMonochromeImage(image,&image->exception) !=
-    MagickFalse ? 1 : 8);
+  pack=IsMonochromeImage(image,&image->exception) == MagickFalse ? 1UL : 8UL;
   padded_columns=((image->columns+pack-1)/pack)*pack;
   *length=(size_t) padded_columns*image->rows/pack;
   *pixels=(unsigned char *) AcquireQuantumMemory(*length,sizeof(**pixels));
@@ -326,15 +323,9 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
           p++;
         }
       }
-    if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
@@ -381,14 +372,11 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
     for (x=0; x < (long) image->columns; x++)
       *q++=(unsigned char) indexes[x];
     if (image->previous == (Image *) NULL)
-     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-         (QuantumTick(y,image->rows) != MagickFalse))
-       {
-         status=image->progress_monitor(SaveImageTag,y,image->rows,
-           image->client_data);
-         if (status == MagickFalse)
-           break;
-       }
+      {
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
@@ -851,7 +839,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(MagickFalse);
-  compression=image->compression;
+  compression=image_info->compression;
   switch (compression)
   {
     case FaxCompression:
@@ -867,7 +855,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JPEGLibraryIsNotAvailable","`%s'",
+        MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (JPEG)",
         image->filename);
       break;
     }
@@ -877,7 +865,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"ZipLibraryIsNotAvailable","`%s'",image->filename);
+        MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (ZLIB)",
+        image->filename);
       break;
     }
 #endif
@@ -1117,7 +1106,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobByte(image,'\n');
       }
     /*
-      Push a dictionary for our own def's if this is an EPS.
+      Push a dictionary for our own def's if this an EPS.
     */
     if (LocaleCompare(image_info->magick,"PS3") != 0)
       (void) WriteBlobString(image,"userdict begin\n");
@@ -1127,7 +1116,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if ((image->matte != MagickFalse) &&
         (WritePS3MaskImage(image_info,image,compression) == MagickFalse))
       {
-        CloseBlob(image);
+        (void) CloseBlob(image);
         return(MagickFalse);
       }
     /*
@@ -1288,7 +1277,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               status=SerializeImageChannel(image_info,image,&pixels,&length);
               if (status == MagickFalse)
                 {
-                  CloseBlob(image);
+                  (void) CloseBlob(image);
                   return(MagickFalse);
                 }
               switch (compression)
@@ -1372,7 +1361,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               status=SerializeImage(image_info,image,&pixels,&length);
               if (status == MagickFalse)
                 {
-                  CloseBlob(image);
+                  (void) CloseBlob(image);
                   return(MagickFalse);
                 }
               switch (compression)
@@ -1450,7 +1439,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             /*
               Color map - uncompressed.
             */
-            if (compression != NoCompression)
+            if ((compression != NoCompression) &&
+                (compression != UndefinedCompression))
               {
                 for (i=0; i < (long) image->colors; i++)
                 {
@@ -1479,7 +1469,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             status=SerializeImageIndexes(image_info,image,&pixels,&length);
             if (status == MagickFalse)
               {
-                CloseBlob(image);
+                (void) CloseBlob(image);
                 return(MagickFalse);
               }
             switch (compression)
@@ -1520,7 +1510,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,'\n');
     if (status == MagickFalse)
       {
-        CloseBlob(image);
+        (void) CloseBlob(image);
         return(MagickFalse);
       }
     /*
@@ -1538,7 +1528,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     offset=SeekBlob(image,stop,SEEK_SET);
     (void) WriteBlobString(image,"%%EndData\n");
     /*
-      End private dictionary if this is an EPS.
+      End private dictionary if this an EPS.
     */
     if (LocaleCompare(image_info->magick,"PS3") != 0)
       (void) WriteBlobString(image,"end\n");
@@ -1546,14 +1536,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 1)
@@ -1568,6 +1554,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
