@@ -10,7 +10,7 @@
 %                            P       W W   P                                  %
 %                                                                             %
 %                                                                             %
-%                    Read Seattle Film Works Image Format.                    %
+%                    Read Seattle Film Works Image Format                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -51,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/resource_.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -114,7 +115,7 @@ static MagickBooleanType IsPWP(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -164,7 +165,7 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  pwp_image=AllocateImage(image_info);
+  pwp_image=AcquireImage(image_info);
   image=pwp_image;
   status=OpenBlob(image_info,pwp_image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -224,7 +225,7 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Link image into image list.
         */
-        for (p=image; p->next != (Image *) NULL; p=GetNextImageInList(p));
+        for (p=image; p->next != (Image *) NULL; p=GetNextImageInList(p)) ;
         next_image->previous=p;
         next_image->scene=p->scene+1;
         p->next=next_image;
@@ -232,17 +233,14 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->number_scenes != 0)
       if (next_image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(LoadImagesTag,TellBlob(pwp_image),
-          GetBlobSize(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,LoadImagesTag,TellBlob(pwp_image),
+      GetBlobSize(pwp_image));
+    if (status == MagickFalse)
+      break;
   }
   (void) RelinquishUniqueFileResource(read_info->filename);
   read_info=DestroyImageInfo(read_info);
-  CloseBlob(pwp_image);
+  (void) CloseBlob(pwp_image);
   pwp_image=DestroyImage(pwp_image);
   if (EOFBlob(image) != MagickFalse)
     {
@@ -254,7 +252,7 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         "UnexpectedEndOfFile","`%s': %s",image->filename,message);
       message=DestroyString(message);
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
