@@ -10,7 +10,7 @@
 %                            R  R   LLLLL  EEEEE                              %
 %                                                                             %
 %                                                                             %
-%                          Read URT RLE Image Format.                         %
+%                          Read URT RLE Image Format                          %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -51,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -113,7 +114,7 @@ static MagickBooleanType IsRLE(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -189,7 +190,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -197,7 +198,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   /*
-    Determine if this is a RLE file.
+    Determine if this a RLE file.
   */
   count=ReadBlob(image,2,(unsigned char *) magick);
   if ((count == 0) || (memcmp(magick,"\122\314",2) != 0))
@@ -454,14 +455,11 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,y,image->rows,
-                  image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,LoadImageTag,y,image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
       }
     else
@@ -471,7 +469,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (number_colormaps == 0)
           map_length=256;
-        if (AllocateImageColormap(image,map_length) == MagickFalse)
+        if (AcquireImageColormap(image,map_length) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         p=colormap;
         if (number_colormaps == 1)
@@ -510,14 +508,11 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(LoadImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                {
+                  status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                  if (status == MagickFalse)
+                    break;
+                }
             }
             (void) SyncImage(image);
           }
@@ -542,14 +537,11 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(LoadImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                {
+                  status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                  if (status == MagickFalse)
+                    break;
+                }
             }
             image->colormap=(PixelPacket *)
               RelinquishMagickMemory(image->colormap);
@@ -579,23 +571,20 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while ((count != 0) && (memcmp(magick,"\122\314",2) == 0));
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
