@@ -10,7 +10,7 @@
 %                            SSSSS   GGG   IIIII                              %
 %                                                                             %
 %                                                                             %
-%                      Read/Write Irix RGB Image Format.                      %
+%                      Read/Write Irix RGB Image Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/property.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -154,7 +155,7 @@ static MagickBooleanType IsSGI(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -309,7 +310,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -574,14 +575,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(LoadImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                {
+                  status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                  if (status == MagickFalse)
+                    break;
+                }
             }
           }
         else
@@ -605,14 +603,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                  (QuantumTick(y,image->rows) != MagickFalse))
-                {
-                  status=image->progress_monitor(LoadImageTag,y,image->rows,
-                    image->client_data);
-                  if (status == MagickFalse)
-                    break;
-                }
+              {
+                status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                if (status == MagickFalse)
+                  break;
+              }
           }
       }
     else
@@ -620,7 +615,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Create grayscale map.
         */
-        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+        if (AcquireImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         /*
           Convert SGI image to PseudoClass pixel packets.
@@ -645,14 +640,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(y,image->rows) != MagickFalse))
-                  {
-                    status=image->progress_monitor(LoadImageTag,y,image->rows,
-                      image->client_data);
-                    if (status == MagickFalse)
-                      break;
-                  }
+                {
+                  status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                  if (status == MagickFalse)
+                    break;
+                }
             }
           }
         else
@@ -672,14 +664,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                  (QuantumTick(y,image->rows) != MagickFalse))
-                {
-                  status=image->progress_monitor(LoadImageTag,y,image->rows,
-                    image->client_data);
-                  if (status == MagickFalse)
-                    break;
-                }
+              {
+                status=SetImageProgress(image,LoadImageTag,y,image->rows);
+                if (status == MagickFalse)
+                  break;
+              }
           }
         (void) SyncImage(image);
       }
@@ -702,23 +691,20 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (iris_info.magic == 0x01DA);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -804,7 +790,7 @@ ModuleExport void UnregisterSGIImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -865,6 +851,9 @@ static size_t SGIEncode(unsigned char *pixels,size_t length,
 
 static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
 {
+  CompressionType
+    compression;
+
   const char
     *value;
 
@@ -918,13 +907,14 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize SGI raster file header.
     */
-    if (image_info->colorspace == UndefinedColorspace)
+    if (image->colorspace != RGBColorspace)
       (void) SetImageColorspace(image,RGBColorspace);
     (void) ResetMagickMemory(&iris_info,0,sizeof(iris_info));
     iris_info.magic=0x01DA;
+    compression=image_info->compression;
     if (image->depth > 8)
-      image->compression=NoCompression;
-    if (image->compression == NoCompression)
+      compression=NoCompression;
+    if (compression == NoCompression)
       iris_info.storage=(unsigned char) 0x00;
     else
       iris_info.storage=(unsigned char) 0x01;
@@ -1017,16 +1007,13 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           p++;
         }
       if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(SaveImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        {
+          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
-    switch (image->compression)
+    switch (compression)
     {
       case NoCompression:
       {
@@ -1112,7 +1099,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobMSBLong(image,runlength[i]);
         (void) WriteBlob(image,number_packets,packets);
         /*
-          Free resources.
+          Relinquish resources.
         */
         runlength=(unsigned long *) RelinquishMagickMemory(runlength);
         packets=(unsigned char *) RelinquishMagickMemory(packets);
@@ -1124,15 +1111,11 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
