@@ -10,7 +10,7 @@
 %                       V    IIIII   CCCC  A   A  R  R                        %
 %                                                                             %
 %                                                                             %
-%                    Read/Write VICAR Rasterfile Format.                      %
+%                    Read/Write VICAR Rasterfile Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -52,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -90,7 +91,6 @@ static MagickBooleanType
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsVICAR(const unsigned char *magick,
   const size_t length)
@@ -132,7 +132,7 @@ static MagickBooleanType IsVICAR(const unsigned char *magick,
 %      reading.  A null image is returned if there is a memory shortage or if
 %      the image cannot be read.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -159,7 +159,10 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     value_expected;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
 
   register PixelPacket
     *q;
@@ -167,11 +170,11 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   ssize_t
     count;
 
-  ssize_t
+  size_t
     length;
 
   unsigned char
-    *scanline;
+    *pixels;
 
   /*
     Open image file.
@@ -183,7 +186,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -276,11 +279,11 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
   image->depth=8;
-  if (AllocateImageColormap(image,256) == MagickFalse)
+  if (AcquireImageColormap(image,256) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   /*
@@ -291,34 +294,29 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
       InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
-  GetQuantumInfo(image_info,&quantum_info);
-  scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
-    sizeof(*scanline));
-  if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(CorruptImageError,"UnableToReadImageData");
+  quantum_type=IndexQuantum;
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  pixels=GetQuantumPixels(quantum_info);
+  length=GetQuantumExtent(image,quantum_info,IndexQuantum);
   for (y=0; y < (long) image->rows; y++)
   {
     q=SetImagePixels(image,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
       break;
-    count=ReadBlob(image,image->columns,scanline);
-    (void) ExportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
+    count=ReadBlob(image,length,pixels);
+    (void) ImportQuantumPixels(image,quantum_info,quantum_type,pixels);
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(image,LoadImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  SetQuantumImageType(image,quantum_type);
+  quantum_info=DestroyQuantumInfo(quantum_info);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -412,11 +410,10 @@ ModuleExport void UnregisterVICARImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   Image *image)
@@ -431,13 +428,19 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
     status;
 
   QuantumInfo
-    quantum_info;
+    *quantum_info;
 
   register const PixelPacket
     *p;
 
+  ssize_t
+    count;
+
+  size_t
+    length;
+
   unsigned char
-    *scanline;
+    *pixels;
 
   /*
     Open output image file.
@@ -451,7 +454,7 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   /*
     Write header.
@@ -464,35 +467,25 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
     image->rows,image->columns);
   (void) WriteBlob(image,MaxTextExtent,(unsigned char *) header);
   /*
-    Allocate memory for scanline.
+    Write VICAR pixels.
   */
-  scanline=(unsigned char *) AcquireQuantumMemory(image->columns,
-    sizeof(*scanline));
-  if (scanline == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  /*
-    Write VICAR scanline.
-  */
-  GetQuantumInfo(image_info,&quantum_info);
   image->depth=8;
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  pixels=GetQuantumPixels(quantum_info);
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
-    (void) WriteBlob(image,image->columns,scanline);
-    if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    length=ExportQuantumPixels(image,quantum_info,GrayQuantum,pixels);
+    count=WriteBlob(image,length,pixels);
+    if (count != (ssize_t) length)
+      break;
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-  CloseBlob(image);
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
