@@ -54,6 +54,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/montage.h"
 #include "magick/quantum-private.h"
 #include "magick/resize.h"
@@ -89,7 +90,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -137,7 +138,7 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   filelist=(char **) AcquireMagickMemory(sizeof(*filelist));
   if (filelist == (char **) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -189,13 +190,9 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "thumbnail geometry: %ldx%ld",next_image->columns,next_image->rows);
     AppendImageToList(&images,next_image);
-    if (images->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=images->progress_monitor(LoadImagesTag,i,(MagickSizeType)
-          number_files,images->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    status=SetImageProgress(images,LoadImagesTag,i,number_files);
+    if (status == MagickFalse)
+      break;
   }
   read_info=DestroyImageInfo(read_info);
   filelist=(char **) RelinquishMagickMemory(filelist);
@@ -246,6 +243,7 @@ ModuleExport unsigned long RegisterVIDImage(void)
   entry=SetMagickInfo("VID");
   entry->decoder=(DecodeImageHandler *) ReadVIDImage;
   entry->encoder=(EncodeImageHandler *) WriteVIDImage;
+  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Visual Image Directory");
   entry->module=ConstantString("VID");
   (void) RegisterMagickInfo(entry);
@@ -295,7 +293,7 @@ ModuleExport void UnregisterVIDImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
