@@ -10,7 +10,7 @@
 %                             W   W   M   M  F                                %
 %                                                                             %
 %                                                                             %
-%                        Read Windows Metafile Format.                        %
+%                        Read Windows Metafile Format                         %
 %                                                                             %
 %                              Software Design                                %
 %                              Bob Friesenhahn                                %
@@ -58,6 +58,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/paint.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -67,7 +68,7 @@
 #include "magick/module.h"
 #include "wand/MagickWand.h"
 
-#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMFLITE_DELEGATE)
 
 #define ERR(API)  ((API)->err != wmf_E_None)
 #define XC(x) ((double) x)
@@ -132,7 +133,7 @@
 #define CENTIMETERS_PER_INCH  2.54
 #define POINTS_PER_INCH       72
 
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
 # define wmf_api_create(api,flags,options) wmf_lite_create(api,flags,options)
 # define wmf_api_destroy(api) wmf_lite_destroy(api)
 # undef WMF_FONT_PSNAME
@@ -217,7 +218,7 @@ typedef enum
 }
 magick_arc_t;
 
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
 static void  lite_font_init (wmfAPI* API, wmfAPI_Options* options);
 static void  lite_font_map(wmfAPI* API,wmfFont* font);
 static float lite_font_stringwidth(wmfAPI* API, wmfFont* font, char* str);
@@ -243,7 +244,7 @@ static void         ipa_draw_line(wmfAPI * API, wmfDrawLine_t * draw_line);
 static void         ipa_draw_pie(wmfAPI * API, wmfDrawArc_t * draw_arc);
 static void         ipa_draw_pixel(wmfAPI * API, wmfDrawPixel_t * draw_pixel);
 static void         ipa_draw_polygon(wmfAPI * API, wmfPolyLine_t * poly_line);
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
 static void         ipa_draw_polypolygon(wmfAPI * API, wmfPolyPoly_t* polypolygon);
 #endif
 static void         ipa_draw_rectangle(wmfAPI * API, wmfDrawRectangle_t * draw_rect);
@@ -262,7 +263,7 @@ static void         ipa_udata_init(wmfAPI * API, wmfUserData_t * userdata);
 static void         ipa_udata_set(wmfAPI * API, wmfUserData_t * userdata);
 static int          magick_progress_callback(void* wand,float quantum);
 static void         util_draw_arc(wmfAPI * API, wmfDrawArc_t * draw_arc,magick_arc_t finish);
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
 /*static int          util_font_weight( const char* font );*/
 #endif
 static double       util_pointsize( wmfAPI* API, wmfFont* font, char* str, double font_height);
@@ -276,7 +277,7 @@ int magick_progress_callback(void *context,float quantum)
     *image;
 
   MagickBooleanType
-    status = MagickTrue;
+    status;
 
   MagickOffsetType
     offset;
@@ -288,10 +289,8 @@ int magick_progress_callback(void *context,float quantum)
   assert(image->signature == MagickSignature);
   offset=(MagickOffsetType) floor((double) (100.0*quantum));
   span=100;
-  if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-      (QuantumTick(offset,span) != MagickFalse))
-    status=image->progress_monitor(LoadImagesTag,offset,span,
-      image->client_data);
+  status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+    GetBlobSize(image));
   return(status == MagickTrue ? 0 : 1);
 }
 
@@ -544,7 +543,7 @@ static void ipa_bmp_draw(wmfAPI *API, wmfBMP_Draw_t *bmp_draw)
       overlaid over non-white backgrounds.
     */
     QueryMagickColor( "white", &white, &exception );
-    PaintTransparentImage( image, &white, QuantumRange );
+    TransparentPaintImage( image, &white, QuantumRange, MagickFalse );
   }
 
   width = fabs(bmp_draw->pixel_width * (double) bmp_draw->crop.w);
@@ -634,7 +633,7 @@ static void ipa_bmp_free(wmfAPI * API, wmfBMP * bmp)
 }
 
 /*
-  This is called by wmf_play() the *first* time the meta file is played
+  This called by wmf_play() the *first* time the meta file is played
  */
 static void ipa_device_open(wmfAPI * API)
 {
@@ -657,7 +656,7 @@ static void ipa_device_open(wmfAPI * API)
 }
 
 /*
-  This is called by wmf_api_destroy()
+  This called by wmf_api_destroy()
  */
 static void ipa_device_close(wmfAPI * API)
 {
@@ -667,7 +666,7 @@ static void ipa_device_close(wmfAPI * API)
 }
 
 /*
-  This is called from the beginning of each play for initial page setup
+  This called from the beginning of each play for initial page setup
  */
 static void ipa_device_begin(wmfAPI * API)
 {
@@ -775,7 +774,7 @@ static void ipa_device_begin(wmfAPI * API)
 }
 
 /*
-  This is called from the end of each play for page termination
+  This called from the end of each play for page termination
  */
 static void ipa_device_end(wmfAPI * API)
 {
@@ -942,8 +941,8 @@ static void util_draw_arc(wmfAPI * API,
           DrawPathStart(WmfDrawingWand);
           DrawPathMoveToAbsolute(WmfDrawingWand, XC(O.x+start.x), 
             YC(O.y+start.y));
-          DrawPathEllipticArcAbsolute(WmfDrawingWand, Rx, Ry, 0, 0, 1, 
-            XC(O.x+end.x), YC(O.y+end.y));
+          DrawPathEllipticArcAbsolute(WmfDrawingWand, Rx, Ry, 0, MagickFalse,
+            MagickTrue, XC(O.x+end.x), YC(O.y+end.y));
           DrawPathLineToAbsolute(WmfDrawingWand, XC(O.x), YC(O.y));
           DrawPathClose(WmfDrawingWand);
           DrawPathFinish(WmfDrawingWand);
@@ -1049,7 +1048,7 @@ static void ipa_draw_polygon(wmfAPI * API, wmfPolyLine_t * polyline)
 }
 
 /* Draw a polypolygon.  A polypolygon is a list of polygons */
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
 static void ipa_draw_polypolygon(wmfAPI * API, wmfPolyPoly_t* polypolygon)
 {
   if (TO_FILL(polypolygon) || TO_DRAW(polypolygon))
@@ -1242,7 +1241,7 @@ static void ipa_functions(wmfAPI *API)
   FR->draw_line = ipa_draw_line;
   FR->poly_line = ipa_poly_line;
   FR->draw_polygon = ipa_draw_polygon;
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
   FR->draw_polypolygon = ipa_draw_polypolygon;
 #endif
   FR->draw_rectangle = ipa_draw_rectangle;
@@ -1383,7 +1382,7 @@ static void ipa_draw_text(wmfAPI * API, wmfDrawText_t * draw_text)
               double
                 text_width = metrics.width * (ddata->scale_y / ddata->scale_x);
 
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
               point.x -= text_width / 2;
 #else
               point.x += bbox_width / 2 - text_width / 2;
@@ -1873,7 +1872,7 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
       return;
     }
 
-  DrawSetStrokeAntialias(WmfDrawingWand, 1 );
+  DrawSetStrokeAntialias(WmfDrawingWand, MagickTrue );
   DrawSetStrokeWidth(WmfDrawingWand, (unsigned long) MagickMax(0.0, pen_width));
 
   {
@@ -2056,7 +2055,7 @@ static double util_pointsize( wmfAPI* API, wmfFont* font, char* str, double font
   return floor(pointsize);
 }
 
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
 /* Estimate weight based on font name */
 /*
 static int util_font_weight( const char* font )
@@ -2093,7 +2092,7 @@ static int util_font_weight( const char* font )
  * Returns width of string in points, assuming (unstretched) font size of 1pt
  * (similar to wmf_ipa_font_stringwidth)
  *
- * This is extremely odd at best, particularly since player/meta.h has access
+ * This extremely odd at best, particularly since player/meta.h has access
  * to the corrected font_height (as drawtext.font_height) when it invokes the
  * stringwidth callback.  It should be possible to compute the real stringwidth!
  */
@@ -2342,7 +2341,7 @@ static void lite_font_init( wmfAPI* API, wmfAPI_Options* options)
   ((wmf_magick_font_t*)font_data->user_data)->pointsize = 0;
 }
 
-#endif /* MAGICKCORE_WMF_DELEGATElite */
+#endif /* MAGICKCORE_WMFLITE_DELEGATE */
 
 /* BLOB read byte */
 static int ipa_blob_read(void* wand)
@@ -2400,7 +2399,7 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
   wmfD_Rect
     bbox;
 
-  image = AllocateImage(image_info);
+  image = AcquireImage(image_info);
   if (OpenBlob(image_info,image,ReadBinaryBlobMode,exception) == MagickFalse)
     {
       if (image->debug != MagickFalse)
@@ -2449,7 +2448,7 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
   ddata->image = image;
   ddata->image_info = image_info;
 
-#if defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)
   /* Must initialize font subystem for WMFlite interface */
   lite_font_init (API,&wmf_api_options); /* similar to wmf_ipa_font_init in src/font.c */
   /* wmf_arg_fontdirs (API,options); */ /* similar to wmf_arg_fontdirs in src/wmf.c */
@@ -2555,7 +2554,7 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
 
   /* Compute bounding box scale factors and origin translations
    *
-   * This is all just a hack since libwmf does not currently seem to
+   * This all just a hack since libwmf does not currently seem to
    * provide the mapping between LOGICAL coordinates and DEVICE
    * coordinates. This mapping is necessary in order to know
    * where to place the logical bounding box within the image.
@@ -2639,7 +2638,7 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
     printf("dc->Window.height = %d\n", dc->Window.height);
     printf("dc->pixel_width   = %g\n", dc->pixel_width);
     printf("dc->pixel_height  = %g\n", dc->pixel_height);
-#if defined(MAGICKCORE_WMF_DELEGATElite)  /* Only in libwmf 0.3 */
+#if defined(MAGICKCORE_WMFLITE_DELEGATE)  /* Only in libwmf 0.3 */
     printf("dc->Ox            = %.d\n", dc->Ox);
     printf("dc->Oy            = %.d\n", dc->Oy);
     printf("dc->width         = %.d\n", dc->width);
@@ -2661,7 +2660,7 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
   if (image_info->ping != MagickFalse)
     {
       wmf_api_destroy(API);
-      CloseBlob(image);
+      (void) CloseBlob(image);
       if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
           "leave ReadWMFImage()");
@@ -2739,10 +2738,6 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
       "  Rendering WMF vectors");
   DrawRender(ddata->draw_wand);
 
-  /* Cleanup allocated data */
-  wmf_api_destroy(API);
-  CloseBlob(image);
-
   /* Check for and report any rendering error */
   if (image->exception.severity != UndefinedException)
     (void) ThrowMagickException(exception,GetMagickModule(),CoderWarning,
@@ -2751,11 +2746,15 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
 
+  /* Cleanup allocated data */
+  wmf_api_destroy(API);
+  (void) CloseBlob(image);
+
   /* Return image */
   return image;
 }
 /* #endif */
-#endif /* MAGICKCORE_WMF_DELEGATE || MAGICKCORE_WMF_DELEGATElite */
+#endif /* MAGICKCORE_WMF_DELEGATE || MAGICKCORE_WMFLITE_DELEGATE */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -2786,7 +2785,7 @@ ModuleExport unsigned long RegisterWMFImage(void)
     *entry;
 
   entry = SetMagickInfo("WMZ");
-#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMFLITE_DELEGATE)
   entry->decoder=ReadWMFImage;
 #endif
   entry->description=ConstantString("Compressed Windows Meta File");
@@ -2795,7 +2794,7 @@ ModuleExport unsigned long RegisterWMFImage(void)
   entry->module=ConstantString("WMZ");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("WMF");
-#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMF_DELEGATElite)
+#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMFLITE_DELEGATE)
   entry->decoder=ReadWMFImage;
 #endif
   entry->description=ConstantString("Windows Meta File");
