@@ -9,7 +9,7 @@
 %                            W   W  P       GGG                               %
 %                                                                             %
 %                                                                             %
-%                       Read WordPerfect Image Format.                        %
+%                       Read WordPerfect Image Format                         %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
@@ -731,7 +731,7 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
 
   /* Obtain temporary file */
   AcquireUniqueFilename(postscript_file);
-  ps_file=MagickOpenStream(postscript_file,"wb");
+  ps_file=OpenMagickStream(postscript_file,"wb");
   if (ps_file == (FILE *) NULL)
     goto FINISH;
 
@@ -945,7 +945,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   image->depth=8;
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -970,6 +970,8 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   if (Header.EncryptKey!=0)
     ThrowReaderException(CoderError,"EncryptedWPGImageFileNotSupported");
 
+  image->columns = 1;
+  image->rows = 1;
   image->colors = 0;
   bpp=0;
   BitmapHeader2.RotAngle=0;
@@ -1018,17 +1020,17 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               WPG_Palette.NumOfEntries=ReadBlobLSBShort(image);
 
               image->colors=WPG_Palette.NumOfEntries;
-              if (!AllocateImageColormap(image,image->colors))
+              if (!AcquireImageColormap(image,image->colors))
                 goto NoMemory;
               for (i=WPG_Palette.StartIndex;
                    i < (int)WPG_Palette.NumOfEntries; i++)
                 {
-                  image->colormap[i].red=
-                    ScaleCharToQuantum(ReadBlobByte(image));
-                  image->colormap[i].green=
-                    ScaleCharToQuantum(ReadBlobByte(image));
-                  image->colormap[i].blue=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                  image->colormap[i].red=ScaleCharToQuantum((unsigned char)
+                    ReadBlobByte(image));
+                  image->colormap[i].green=ScaleCharToQuantum((unsigned char)
+                    ReadBlobByte(image));
+                  image->colormap[i].blue=ScaleCharToQuantum((unsigned char)
+                    ReadBlobByte(image));
                 }
               break;
      
@@ -1071,7 +1073,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if ((image->colors == 0) && (bpp != 24))
                 {
                   image->colors=1 << bpp;
-                  if (!AllocateImageColormap(image,image->colors))
+                  if (!AcquireImageColormap(image,image->colors))
                     {
                     NoMemory:
                       ThrowReaderException(ResourceLimitError,
@@ -1149,7 +1151,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                 }
 
               /* Allocate next image structure. */
-              AllocateNextImage(image_info,image);
+              AcquireNextImage(image_info,image);
               image->depth=8;
               if (image->next == (Image *) NULL)
                 goto Finish;
@@ -1202,18 +1204,18 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               WPG_Palette.NumOfEntries=ReadBlobLSBShort(image);
 
               image->colors=WPG_Palette.NumOfEntries;
-              if (AllocateImageColormap(image,image->colors) == MagickFalse)
+              if (AcquireImageColormap(image,image->colors) == MagickFalse)
                 ThrowReaderException(ResourceLimitError,
                   "MemoryAllocationFailed");
               for (i=WPG_Palette.StartIndex;
                    i < (int)WPG_Palette.NumOfEntries; i++)
                 {
-                  image->colormap[i].red=
-                    ScaleCharToQuantum(ReadBlobByte(image));
-                  image->colormap[i].green=
-                    ScaleCharToQuantum(ReadBlobByte(image));
-                  image->colormap[i].blue=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                  image->colormap[i].red=ScaleCharToQuantum((char)
+                    ReadBlobByte(image));
+                  image->colormap[i].green=ScaleCharToQuantum((char)
+                    ReadBlobByte(image));
+                  image->colormap[i].blue=ScaleCharToQuantum((char)
+                    ReadBlobByte(image));
                   (void) ReadBlobByte(image);   /*Opacity??*/
                 }
               break;
@@ -1251,7 +1253,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if ((image->colors == 0) && (bpp != 24))
                 {
                   image->colors=1 << bpp;
-                  if (!AllocateImageColormap(image,image->colors))
+                  if (!AcquireImageColormap(image,image->colors))
                     goto NoMemory;
                 }
               else
@@ -1322,12 +1324,12 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     
 
               /* Allocate next image structure. */
-              AllocateNextImage(image_info,image);
+              AcquireNextImage(image_info,image);
               image->depth=8;
               if (image->next == (Image *) NULL)
                 goto Finish;
               image=SyncNextImageInList(image);
-              image->columns=image->rows=0;
+              image->columns=image->rows=1;
               image->colors=0;
               break;
 
@@ -1354,7 +1356,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
    }
 
  Finish:
-  CloseBlob(image);
+  (void) CloseBlob(image);
 
   {
     Image
@@ -1385,6 +1387,9 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     for (p=image; p != (Image *) NULL; p=p->next)
       p->scene=(unsigned long) scene++;
   }
+  if (image == (Image *) NULL)
+    ThrowReaderException(CorruptImageError,
+      "ImageFileDoesNotContainAnyImageData");
   return(image);
 }
 
