@@ -10,7 +10,7 @@
 %                            X   X  BBBB   M   M                              %
 %                                                                             %
 %                                                                             %
-%                  Read/Write X Windows System Bitmap Format.                 %
+%                  Read/Write X Windows System Bitmap Format                  %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -52,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -120,7 +121,7 @@ static MagickBooleanType IsXBM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -211,7 +212,7 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -264,7 +265,7 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image structure.
   */
-  if (AllocateImageColormap(image,image->colors) == MagickFalse)
+  if (AcquireImageColormap(image,image->colors) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Initialize colormap.
@@ -277,7 +278,7 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->colormap[1].blue=(Quantum) 0;
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   if (SetImageExtent(image,0,0) == MagickFalse)
@@ -368,19 +369,13 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(LoadImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,LoadImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   data=(unsigned char *) RelinquishMagickMemory(data);
   (void) SyncImage(image);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -467,7 +462,7 @@ ModuleExport void UnregisterXBMImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -491,9 +486,6 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
   register long
     x;
 
-  register IndexPacket
-    *indexes;
-
   ssize_t
     count;
 
@@ -513,7 +505,7 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   /*
     Write X bitmap header.
@@ -546,7 +538,6 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; x++)
     {
       byte>>=1;
@@ -592,18 +583,12 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
         bit=0;
         byte=0;
       };
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   (void) CopyMagickString(buffer,"};\n",MaxTextExtent);
   (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
