@@ -51,6 +51,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/pixel-private.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -80,7 +81,7 @@
 %
 %    o image:  The image.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -91,6 +92,7 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *image;
 
   IndexPacket
+    index,
     *indexes;
 
   MagickBooleanType
@@ -102,6 +104,9 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  PixelPacket
+    pixel;
+
   register long
     x;
 
@@ -118,11 +123,16 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   if (image->columns == 0)
     image->columns=1;
   if (image->rows == 0)
     image->rows=1;
+  if (SetImageExtent(image,0,0) == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
   (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
   status=QueryMagickColor((char *) image_info->filename,&color,exception);
   if (status == MagickFalse)
@@ -132,38 +142,21 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   image->colorspace=color.colorspace;
   image->matte=color.matte;
-  if ((image->colorspace == RGBColorspace) && (image->matte == MagickFalse))
-    {
-      if (AllocateImageColormap(image,1) == MagickFalse)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      (void) QueryColorDatabase((char *) image_info->filename,
-        &image->background_color,exception);
-      image->colormap[0]=image->background_color;
-      color.index=0.0;
-    }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
+  index=0;
+  SetPixelPacket(image,&color,&pixel,&index);
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    q=SetImagePixels(image,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; x++)
-    {
-      q->red=RoundToQuantum(color.red);
-      q->green=RoundToQuantum(color.green);
-      q->blue=RoundToQuantum(color.blue);
-      if (image->matte)
-        q->opacity=RoundToQuantum(color.opacity);
-      if ((image->storage_class == PseudoClass) ||
-          (image->colorspace == CMYKColorspace))
-        indexes[x]=(IndexPacket) RoundToQuantum(color.index);
-      q++;
-    }
+      *q++=pixel;
+    if (image->colorspace == CMYKColorspace)
+      {
+        indexes=GetIndexes(image);
+        for (x=0; x < (long) image->columns; x++)
+          indexes[x]=index;
+      }
     if (SyncImagePixels(image) == MagickFalse)
       break;
   }
@@ -201,6 +194,7 @@ ModuleExport unsigned long RegisterXCImage(void)
   entry=SetMagickInfo("XC");
   entry->decoder=(DecodeImageHandler *) ReadXCImage;
   entry->adjoin=MagickFalse;
+  entry->format_type=ExplicitFormatType;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
   entry->description=ConstantString("Constant image uniform color");
