@@ -10,7 +10,7 @@
 %                            X   X  P      M   M                              %
 %                                                                             %
 %                                                                             %
-%                  Read/Write X Windows system Pixmap Format.                 %
+%                  Read/Write X Windows system Pixmap Format                  %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -54,6 +54,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/pixel-private.h"
 #include "magick/quantize.h"
 #include "magick/quantum-private.h"
@@ -129,7 +130,7 @@ static MagickBooleanType IsXPM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -267,7 +268,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -339,7 +340,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   xpm_colors=NewSplayTree(CompareXPMColor,RelinquishMagickMemory,
     (void *(*)(void *)) NULL);
-  if (AllocateImageColormap(image,image->colors) == MagickFalse)
+  if (AcquireImageColormap(image,image->colors) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Read image colormap.
@@ -418,10 +419,10 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         ThrowReaderException(CorruptImageError,"NotEnoughPixelData");
     }
   /*
-    Free resources.
+    Relinquish resources.
   */
   xpm_colors=DestroySplayTree(xpm_colors);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -526,7 +527,7 @@ ModuleExport void UnregisterXPMImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -580,8 +581,8 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
     symbol[MaxTextExtent];
 
   Image
-    *picon,
-    *map;
+    *affinity_image,
+    *picon;
 
   ImageInfo
     *blob_info;
@@ -598,6 +599,9 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   MagickPixelPacket
     pixel;
 
+  QuantizeInfo
+    *quantize_info;
+
   RectangleInfo
     geometry;
 
@@ -630,7 +634,7 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   SetGeometry(image,&geometry);
   (void) ParseMetaGeometry(PiconGeometry,&geometry.x,&geometry.y,
@@ -641,15 +645,19 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   (void) AcquireUniqueFilename(blob_info->filename);
   if ((image_info->type != TrueColorType) &&
       (IsGrayImage(image,&image->exception) != MagickFalse))
-    map=BlobToImage(blob_info,Graymap,GraymapExtent,&image->exception);
+    affinity_image=BlobToImage(blob_info,Graymap,GraymapExtent,
+      &image->exception);
   else
-    map=BlobToImage(blob_info,Colormap,ColormapExtent,&image->exception);
+    affinity_image=BlobToImage(blob_info,Colormap,ColormapExtent,
+      &image->exception);
   (void) RelinquishUniqueFileResource(blob_info->filename);
   blob_info=DestroyImageInfo(blob_info);
-  if ((picon == (Image *) NULL) || (map == (Image *) NULL))
+  if ((picon == (Image *) NULL) || (affinity_image == (Image *) NULL))
     return(MagickFalse);
-  status=MapImage(picon,map,image_info->dither);
-  map=DestroyImage(map);
+  quantize_info=AcquireQuantizeInfo(image_info);
+  status=RemapImage(quantize_info,picon,affinity_image);
+  quantize_info=DestroyQuantizeInfo(quantize_info);
+  affinity_image=DestroyImage(affinity_image);
   transparent=MagickFalse;
   if (picon->storage_class == PseudoClass)
     {
@@ -738,11 +746,8 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
     pixel.colorspace=RGBColorspace;
     pixel.depth=8;
     pixel.opacity=(MagickRealType) OpaqueOpacity;
-    (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickFalse,name,
+    (void) QueryMagickColorname(image,&pixel,XPMCompliance,name,
       &image->exception);
-    if (LocaleNCompare(name,"rgb",3) == 0)
-      (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickTrue,name,
-        &image->exception);
     if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
@@ -790,19 +795,13 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
     (void) FormatMagickString(buffer,MaxTextExtent,"\"%s\n",
       y == (long) (picon->rows-1) ? "" : ",");
     (void) WriteBlobString(image,buffer);
-    if (QuantumTick(y,picon->rows) != MagickFalse)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,picon->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,picon->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,y,picon->rows);
+    if (status == MagickFalse)
+      break;
   }
   picon=DestroyImage(picon);
   (void) WriteBlobString(image,"};\n");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
 
@@ -825,7 +824,7 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -882,7 +881,7 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image_info->colorspace == UndefinedColorspace)
+  if (image->colorspace != RGBColorspace)
     (void) SetImageColorspace(image,RGBColorspace);
   opacity=(-1);
   if (image->matte == MagickFalse)
@@ -978,11 +977,8 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
     pixel.colorspace=RGBColorspace;
     pixel.depth=8;
     pixel.opacity=(MagickRealType) OpaqueOpacity;
-    (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickFalse,name,
+    (void) QueryMagickColorname(image,&pixel,XPMCompliance,name,
       &image->exception);
-    if (LocaleNCompare(name,"rgb",3) == 0)
-      (void) QueryMagickColorname(image,&pixel,XPMCompliance,MagickTrue,name,
-        &image->exception);
     if (i == opacity)
       (void) CopyMagickString(name,"None",MaxTextExtent);
     /*
@@ -1038,6 +1034,6 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
         }
   }
   (void) WriteBlobString(image,"};\n");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
