@@ -10,39 +10,26 @@
 %                            X   X   W W   DDDD                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                Read/Write X Windows System Window Dump Format               %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,14 +39,41 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "define.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/color-private.h"
+#include "magick/colorspace.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
+#include "magick/image.h"
+#include "magick/image-private.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/module.h"
+#if defined(MAGICKCORE_X11_DELEGATE)
+#include "magick/xwindow-private.h"
+#if !defined(vms)
+#include <X11/XWDFile.h>
+#else
+#include "XWDFile.h"
+#endif
+#endif
 
 /*
   Forward declarations.
 */
-static unsigned int
+#if defined(MAGICKCORE_X11_DELEGATE)
+static MagickBooleanType
   WriteXWDImage(const ImageInfo *,Image *);
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,17 +86,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsXWD returns True if the image format type, identified by the
+%  IsXWD() returns MagickTrue if the image format type, identified by the
 %  magick string, is XWD.
 %
 %  The format of the IsXWD method is:
 %
-%      unsigned int IsXWD(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsXWD returns True if the image format type is XWD.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -90,22 +102,21 @@ static unsigned int
 %
 %
 */
-static unsigned int IsXWD(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 {
   if (length < 8)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick+1,"\000\000",2) == 0)
     {
       if (memcmp(magick+4,"\007\000\000",3) == 0)
-        return(True);
+        return(MagickTrue);
       if (memcmp(magick+5,"\000\000\007",3) == 0)
-        return(True);
+        return(MagickTrue);
     }
-  return(False);
+  return(MagickFalse);
 }
 
-#if defined(HasX11)
-#include "xwindow.h"
+#if defined(MAGICKCORE_X11_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -117,7 +128,7 @@ static unsigned int IsXWD(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadXWDImage reads an X Window System window dump image file and
+%  ReadXWDImage() reads an X Window System window dump image file and
 %  returns it.  It allocates the memory necessary for the new Image structure
 %  and returns a pointer to the new image.
 %
@@ -127,18 +138,17 @@ static unsigned int IsXWD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadXWDImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
 static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+#define CheckOverflowException(length,width,height) \
+   (((height) != 0) && ((length)/((size_t) height) != ((size_t) width)))
+
   char
     *comment;
 
@@ -149,11 +159,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     index;
 
   int
-    status;
+    x_status;
 
   long
     y;
 
+  MagickStatusType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -170,9 +183,11 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     pixel;
 
   size_t
-    count,
     length;
 
+  ssize_t
+    count;
+
   unsigned long
     lsb_first;
 
@@ -190,19 +205,24 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+  image=AcquireImage(image_info);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
      Read in header information.
   */
-  count=ReadBlob(image,sz_XWDheader,(char *) &header);
+  count=ReadBlob(image,sz_XWDheader,(unsigned char *) &header);
   if (count == 0)
-    ThrowReaderException(CorruptImageWarning,"Unable to read dump file header",
-      image);
+    ThrowReaderException(CorruptImageError,"UnableToReadImageHeader");
   image->columns=header.pixmap_width;
   image->rows=header.pixmap_height;
   image->depth=8;
@@ -210,34 +230,31 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Ensure the header byte-order is most-significant byte first.
   */
   lsb_first=1;
-  if (*(char *) &lsb_first)
+  if ((int) (*(char *) &lsb_first) != 0)
     MSBOrderLong((unsigned char *) &header,sz_XWDheader);
   /*
     Check to see if the dump file is in the proper format.
   */
   if (header.file_version != XWD_FILE_VERSION)
-    ThrowReaderException(CorruptImageWarning,"XWD file format version mismatch",
-      image);
+    ThrowReaderException(CorruptImageError,"FileFormatVersionMismatch");
   if (header.header_size < sz_XWDheader)
-    ThrowReaderException(CorruptImageWarning,"XWD header size is too small",
-      image);
-  length=header.header_size-sz_XWDheader;
-  comment=(char *) AcquireMemory(length+1);
+    ThrowReaderException(CorruptImageError,"CorruptImage");
+  length=(size_t) header.header_size-sz_XWDheader;
+  comment=(char *) AcquireQuantumMemory(length+1,sizeof(*comment));
   if (comment == (char *) NULL)
-    ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",image);
-  count=ReadBlob(image,length,comment);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  count=ReadBlob(image,length,(unsigned char *) comment);
   comment[length]='\0';
-  (void) SetImageAttribute(image,"comment",comment);
-  LiberateMemory((void **) &comment);
+  (void) SetImageProperty(image,"comment",comment);
+  comment=DestroyString(comment);
   if (count == 0)
-    ThrowReaderException(CorruptImageWarning,
-      "Unable to read window name from dump file",image);
+    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
   /*
     Initialize the X image.
   */
-  ximage=(XImage *) AcquireMemory(sizeof(XImage));
+  ximage=(XImage *) AcquireMagickMemory(sizeof(*ximage));
   if (ximage == (XImage *) NULL)
-    ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   ximage->depth=(int) header.pixmap_depth;
   ximage->format=(int) header.pixmap_format;
   ximage->xoffset=(int) header.xoffset;
@@ -253,9 +270,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ximage->red_mask=header.red_mask;
   ximage->green_mask=header.green_mask;
   ximage->blue_mask=header.blue_mask;
-  status=XInitImage(ximage);
-  if (status == False)
-    ThrowReaderException(CorruptImageWarning,"Invalid XWD header",image);
+  if ((ximage->depth < 0) || (ximage->width < 0) || (ximage->height < 0) ||
+      (ximage->bitmap_pad < 0) || (ximage->bytes_per_line < 0))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if ((ximage->bits_per_pixel > 32) || (ximage->bitmap_unit > 32))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  x_status=XInitImage(ximage);
+  if (x_status == 0)
+    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
   /*
     Read colormap.
   */
@@ -265,62 +287,73 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       XWDColor
         color;
 
-      colors=(XColor *)
-        AcquireMemory((unsigned int) header.ncolors*sizeof(XColor));
+      length=(size_t) header.ncolors;
+      colors=(XColor *) AcquireQuantumMemory(length,sizeof(*colors));
       if (colors == (XColor *) NULL)
-        ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (long) header.ncolors; i++)
       {
-        count=ReadBlob(image,sz_XWDColor,(char *) &color);
+        count=ReadBlob(image,sz_XWDColor,(unsigned char *) &color);
         if (count == 0)
-          ThrowReaderException(CorruptImageWarning,
-            "Unable to read color map from dump file",image);
+          ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
         colors[i].pixel=color.pixel;
         colors[i].red=color.red;
         colors[i].green=color.green;
         colors[i].blue=color.blue;
-        colors[i].flags=color.flags;
+        colors[i].flags=(char) color.flags;
       }
       /*
         Ensure the header byte-order is most-significant byte first.
       */
       lsb_first=1;
-      if (*(char *) &lsb_first)
+      if ((int) (*(char *) &lsb_first) != 0)
         for (i=0; i < (long) header.ncolors; i++)
         {
           MSBOrderLong((unsigned char *) &colors[i].pixel,
-            sizeof(unsigned long));
-          MSBOrderShort((unsigned char *) &colors[i].red,
-            3*sizeof(unsigned short));
+            sizeof(colors[i].pixel));
+          MSBOrderShort((unsigned char *) &colors[i].red,3*
+            sizeof(colors[i].red));
         }
     }
   /*
     Allocate the pixel buffer.
   */
-  if (ximage->format == ZPixmap)
-    length=ximage->bytes_per_line*ximage->height;
-  else
-    length=ximage->bytes_per_line*ximage->height*ximage->depth;
-  ximage->data=(char *) AcquireMemory(length);
+  length=(size_t) ximage->bytes_per_line*ximage->height;
+  if (CheckOverflowException(length,ximage->bytes_per_line,ximage->height))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if (ximage->format != ZPixmap)
+    {
+      size_t
+        extent;
+
+      extent=length;
+      length*=ximage->depth;
+      if (CheckOverflowException(length,extent,ximage->depth))
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
+  ximage->data=(char *) AcquireQuantumMemory(length,sizeof(*ximage->data));
   if (ximage->data == (char *) NULL)
-    ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",image);
-  count=ReadBlob(image,length,ximage->data);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  count=ReadBlob(image,length,(unsigned char *) ximage->data);
   if (count == 0)
-    ThrowReaderException(CorruptImageWarning,"Unable to read dump pixmap",
-      image);
+    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
   /*
     Convert image to MIFF format.
   */
-  image->columns=ximage->width;
-  image->rows=ximage->height;
+  image->columns=(unsigned long) ximage->width;
+  image->rows=(unsigned long) ximage->height;
   if ((colors == (XColor *) NULL) || (ximage->red_mask != 0) ||
       (ximage->green_mask != 0) || (ximage->blue_mask != 0))
     image->storage_class=DirectClass;
   else
     image->storage_class=PseudoClass;
+  if (SetImageExtent(image,0,0) == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
   image->colors=header.ncolors;
-  if (!image_info->ping)
+  if (image_info->ping == MagickFalse)
     switch (image->storage_class)
     {
       case DirectClass:
@@ -342,21 +375,21 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         red_mask=ximage->red_mask;
         red_shift=0;
-        while ((red_mask & 0x01) == 0)
+        while ((red_mask != 0) && ((red_mask & 0x01) == 0))
         {
           red_mask>>=1;
           red_shift++;
         }
         green_mask=ximage->green_mask;
         green_shift=0;
-        while ((green_mask & 0x01) == 0)
+        while ((green_mask != 0) && ((green_mask & 0x01) == 0))
         {
           green_mask>>=1;
           green_shift++;
         }
         blue_mask=ximage->blue_mask;
         blue_shift=0;
-        while ((blue_mask & 0x01) == 0)
+        while ((blue_mask != 0) && ((blue_mask & 0x01) == 0))
         {
           blue_mask>>=1;
           blue_shift++;
@@ -373,18 +406,19 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (x=0; x < (long) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
-              index=(unsigned short) ((pixel >> red_shift) & red_mask);
-              q->red=XDownscale(colors[index].red);
-              index=(unsigned short) ((pixel >> green_shift) & green_mask);
-              q->green=XDownscale(colors[index].green);
-              index=(unsigned short) ((pixel >> blue_shift) & blue_mask);
-              q->blue=XDownscale(colors[index].blue);
+              index=(IndexPacket) ((pixel >> red_shift) & red_mask);
+              q->red=ScaleShortToQuantum(colors[(long) index].red);
+              index=(IndexPacket) ((pixel >> green_shift) & green_mask);
+              q->green=ScaleShortToQuantum(colors[(long) index].green);
+              index=(IndexPacket) ((pixel >> blue_shift) & blue_mask);
+              q->blue=ScaleShortToQuantum(colors[(long) index].blue);
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
+              break;
+            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            if (status == MagickFalse)
               break;
-            if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
           }
         else
           for (y=0; y < (long) image->rows; y++)
@@ -396,17 +430,21 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
               color=(pixel >> red_shift) & red_mask;
-              q->red=XDownscale((color*65535L)/red_mask);
+              color=(color*65535UL)/red_mask;
+              q->red=ScaleShortToQuantum((unsigned short) color);
               color=(pixel >> green_shift) & green_mask;
-              q->green=XDownscale((color*65535L)/green_mask);
+              color=(color*65535UL)/green_mask;
+              q->green=ScaleShortToQuantum((unsigned short) color);
               color=(pixel >> blue_shift) & blue_mask;
-              q->blue=XDownscale((color*65535L)/blue_mask);
+              color=(color*65535UL)/blue_mask;
+              q->blue=ScaleShortToQuantum((unsigned short) color);
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
+              break;
+            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            if (status == MagickFalse)
               break;
-            if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
           }
         break;
       }
@@ -415,14 +453,13 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Convert X image to PseudoClass packets.
         */
-        if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
+        if (AcquireImageColormap(image,image->colors) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i < (long) image->colors; i++)
         {
-          image->colormap[i].red=XDownscale(colors[i].red);
-          image->colormap[i].green=XDownscale(colors[i].green);
-          image->colormap[i].blue=XDownscale(colors[i].blue);
+          image->colormap[i].red=ScaleShortToQuantum(colors[i].red);
+          image->colormap[i].green=ScaleShortToQuantum(colors[i].green);
+          image->colormap[i].blue=ScaleShortToQuantum(colors[i].blue);
         }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -432,15 +469,16 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           indexes=GetIndexes(image);
           for (x=0; x < (long) image->columns; x++)
           {
-            index=
-              ConstrainColormapIndex(image,XGetPixel(ximage,(int) x,(int) y));
+            index=ConstrainColormapIndex(image,XGetPixel(ximage,(int) x,
+              (int) y));
             indexes[x]=index;
-            *q++=image->colormap[index];
+            *q++=image->colormap[(long) index];
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
+            break;
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          if (status == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,y,image->rows);
         }
         break;
       }
@@ -449,20 +487,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Free image and colormap.
   */
   if (header.ncolors != 0)
-    LiberateMemory((void **) &colors);
-  LiberateMemory((void **) &ximage->data);
-  LiberateMemory((void **) &ximage);
-  if (EOFBlob(image))
-    ThrowReaderException(CorruptImageWarning,"Unexpected end-of-file",image);
-  CloseBlob(image);
-  return(image);
-}
-#else
-static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  ThrowException(exception,MissingDelegateWarning,
-    "X11 library is not available",image_info->filename);
-  return((Image *) NULL);
+    colors=(XColor *) RelinquishMagickMemory(colors);
+  ximage->data=DestroyString(ximage->data);
+  ximage=(XImage *) RelinquishMagickMemory(ximage);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
+  (void) CloseBlob(image);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -477,8 +509,8 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterXWDImage adds attributes for the XWD image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterXWDImage() adds properties for the XWD image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -486,22 +518,25 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterXWDImage method is:
 %
-%      RegisterXWDImage(void)
+%      unsigned long RegisterXWDImage(void)
 %
 */
-ModuleExport void RegisterXWDImage(void)
+ModuleExport unsigned long RegisterXWDImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("XWD");
-  entry->decoder=ReadXWDImage;
-  entry->encoder=WriteXWDImage;
-  entry->magick=IsXWD;
-  entry->adjoin=False;
-  entry->description=AcquireString("X Windows system window dump (color)");
-  entry->module=AcquireString("XWD");
+#if defined(MAGICKCORE_X11_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadXWDImage;
+  entry->encoder=(EncodeImageHandler *) WriteXWDImage;
+#endif
+  entry->magick=(IsImageFormatHandler *) IsXWD;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString("X Windows system window dump (color)");
+  entry->module=ConstantString("XWD");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -515,7 +550,7 @@ ModuleExport void RegisterXWDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterXWDImage removes format registrations made by the
+%  UnregisterXWDImage() removes format registrations made by the
 %  XWD module from the list of supported formats.
 %
 %  The format of the UnregisterXWDImage method is:
@@ -525,12 +560,10 @@ ModuleExport void RegisterXWDImage(void)
 */
 ModuleExport void UnregisterXWDImage(void)
 {
-#if defined(HasX11)
   (void) UnregisterMagickInfo("XWD");
-#endif
 }
 
-#if defined(HasX11)
+#if defined(MAGICKCORE_X11_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -542,37 +575,38 @@ ModuleExport void UnregisterXWDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteXWDImage writes an image to a file in X window dump
+%  WriteXWDImage() writes an image to a file in X window dump
 %  rasterfile format.
 %
 %  The format of the WriteXWDImage method is:
 %
-%      unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteXWDImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
-%
-%    o image:  A pointer to a Image structure.
+%    o image_info: the image info.
 %
+%    o image:  The image.
 %
 */
-static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
 {
-  int
+  const char
+    *value;
+
+  long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
   register IndexPacket
     *indexes;
 
-  register int
+  register long
     x;
 
   register long
@@ -581,12 +615,12 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   register unsigned char
     *q;
 
+  size_t
+    length;
+
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     bits_per_pixel,
     bytes_per_line,
@@ -603,14 +637,21 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize XWD file header.
   */
-  xwd_info.header_size=(CARD32) (sz_XWDheader+strlen(image->filename)+1);
+  xwd_info.header_size=(CARD32) sz_XWDheader;
+  value=GetImageProperty(image,"comment");
+  if (value != (const char *) NULL)
+    xwd_info.header_size+=(CARD32) strlen(value);
+  xwd_info.header_size++;
   xwd_info.file_version=(CARD32) XWD_FILE_VERSION;
   xwd_info.pixmap_format=(CARD32) ZPixmap;
   xwd_info.pixmap_depth=(CARD32) (image->storage_class == DirectClass ? 24 : 8);
@@ -621,7 +662,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   xwd_info.bitmap_unit=(CARD32) (image->storage_class == DirectClass ? 32 : 8);
   xwd_info.bitmap_bit_order=(CARD32) MSBFirst;
   xwd_info.bitmap_pad=(CARD32) (image->storage_class == DirectClass ? 32 : 8);
-  bits_per_pixel=(image->storage_class == DirectClass ? 24 : 8);
+  bits_per_pixel=(unsigned long) (image->storage_class == DirectClass ? 24 : 8);
   xwd_info.bits_per_pixel=(CARD32) bits_per_pixel;
   bytes_per_line=(CARD32) ((((xwd_info.bits_per_pixel*
     xwd_info.pixmap_width)+((xwd_info.bitmap_pad)-1))/
@@ -648,10 +689,12 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
     Write XWD header.
   */
   lsb_first=1;
-  if (*(char *) &lsb_first)
+  if ((int) (*(char *) &lsb_first) != 0)
     MSBOrderLong((unsigned char *) &xwd_info,sizeof(xwd_info));
-  (void) WriteBlob(image,sz_XWDheader,(char *) &xwd_info);
-  (void) WriteBlob(image,strlen(image->filename)+1,(char *) image->filename);
+  (void) WriteBlob(image,sz_XWDheader,(unsigned char *) &xwd_info);
+  if (value != (const char *) NULL)
+    (void) WriteBlob(image,strlen(value),(unsigned char *) value);
+  (void) WriteBlob(image,1,(const unsigned char *) "\0");
   if (image->storage_class == PseudoClass)
     {
       XColor
@@ -663,22 +706,24 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
       /*
         Dump colormap to file.
       */
-      colors=(XColor *) AcquireMemory(image->colors*sizeof(XColor));
+      colors=(XColor *) AcquireQuantumMemory((size_t) image->colors,
+        sizeof(*colors));
       if (colors == (XColor *) NULL)
-        ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
-          image);
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (long) image->colors; i++)
       {
-        colors[i].pixel=i;
-        colors[i].red=XUpscale(image->colormap[i].red);
-        colors[i].green=XUpscale(image->colormap[i].green);
-        colors[i].blue=XUpscale(image->colormap[i].blue);
-        colors[i].flags=DoRed | DoGreen | DoBlue;
-        colors[i].pad=0;
-        if (*(char *) &lsb_first)
+        colors[i].pixel=(unsigned long) i;
+        colors[i].red=ScaleQuantumToShort(image->colormap[i].red);
+        colors[i].green=ScaleQuantumToShort(image->colormap[i].green);
+        colors[i].blue=ScaleQuantumToShort(image->colormap[i].blue);
+        colors[i].flags=(char) (DoRed | DoGreen | DoBlue);
+        colors[i].pad='\0';
+        if ((int) (*(char *) &lsb_first) != 0)
           {
-            MSBOrderLong((unsigned char *) &colors[i].pixel,sizeof(long));
-            MSBOrderShort((unsigned char *) &colors[i].red,3*sizeof(short));
+            MSBOrderLong((unsigned char *) &colors[i].pixel,
+              sizeof(colors[i].pixel));
+            MSBOrderShort((unsigned char *) &colors[i].red,
+              3*sizeof(colors[i].red));
           }
       }
       for (i=0; i < (long) image->colors; i++)
@@ -687,18 +732,21 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
         color.red=colors[i].red;
         color.green=colors[i].green;
         color.blue=colors[i].blue;
-        color.flags=colors[i].flags;
-        (void) WriteBlob(image,sz_XWDColor,(char *) &color);
+        color.flags=(CARD8) colors[i].flags;
+        (void) WriteBlob(image,sz_XWDColor,(unsigned char *) &color);
       }
-      LiberateMemory((void **) &colors);
+      colors=(XColor *) RelinquishMagickMemory(colors);
     }
   /*
     Allocate memory for pixels.
   */
-  pixels=(unsigned char *)
-    AcquireMemory(image->columns*sizeof(PixelPacket));
+  length=3*bytes_per_line;
+  if (image->storage_class == PseudoClass)
+    length=bytes_per_line;
+  pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) ResetMagickMemory(pixels,0,length);
   /*
     Convert MIFF to XWD raster pixels.
   */
@@ -708,35 +756,30 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
     q=pixels;
-    for (x=0; x < (long) image->columns; x++)
-    {
-      if (image->storage_class == PseudoClass)
-        *q++=indexes[x];
-      else
-        {
-          *q++=Downscale(p->red);
-          *q++=Downscale(p->green);
-          *q++=Downscale(p->blue);
-        }
-      p++;
-    }
+    if (image->storage_class == PseudoClass)
+      {
+        indexes=GetIndexes(image);
+        for (x=0; x < (long) image->columns; x++)
+          *q++=(unsigned char) indexes[x];
+      }
+    else
+      for (x=0; x < (long) image->columns; x++)
+      {
+        *q++=ScaleQuantumToChar(p->red);
+        *q++=ScaleQuantumToChar(p->green);
+        *q++=ScaleQuantumToChar(p->blue);
+        p++;
+      }
     for (x=0; x < (long) scanline_pad; x++)
-      *q++=0;
-    (void) WriteBlob(image,q-pixels,(char *) pixels);
-    if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+      *q++='\0';
+    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  LiberateMemory((void **) &pixels);
-  CloseBlob(image);
-  return(True);
-}
-#else
-static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateWarning,"X11 library is not available",
-    image->filename);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+  (void) CloseBlob(image);
+  return(MagickTrue);
 }
 #endif
