@@ -10,7 +10,7 @@
 %                              Y     UUU     V                                %
 %                                                                             %
 %                                                                             %
-%            Read/Write Raw CCIR 601 4:1:1 or 4:2:2 Image Format.             %
+%            Read/Write Raw CCIR 601 4:1:1 or 4:2:2 Image Format              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/resize.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
@@ -88,7 +89,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -142,7 +143,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   image->depth=8;
@@ -284,18 +285,16 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (interlace == NoInterlace)
         if (SyncImagePixels(chroma_image) == MagickFalse)
           break;
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
+      if (image->previous == (Image *) NULL)
         {
-          status=image->progress_monitor(LoadImageTag,y,image->rows,
-            image->client_data);
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
           if (status == MagickFalse)
             break;
         }
     }
     if (interlace == PartitionInterlace)
       {
-        CloseBlob(image);
+        (void) CloseBlob(image);
         AppendImageFormat("U",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
@@ -325,7 +324,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
       if (interlace == PartitionInterlace)
         {
-          CloseBlob(image);
+          (void) CloseBlob(image);
           AppendImageFormat("V",image->filename);
           status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
           if (status == MagickFalse)
@@ -378,8 +377,6 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     resize_image=DestroyImage(resize_image);
     image->colorspace=YCbCrColorspace;
-    if (image_info->colorspace == UndefinedColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
         MaxTextExtent);
@@ -404,24 +401,21 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -509,7 +503,7 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -602,16 +596,13 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       Sample image to an even width and height, if necessary.
     */
     image->depth=8;
-    if (image_info->colorspace == UndefinedColorspace)
-      (void) SetImageColorspace(image,RGBColorspace);
     width=image->columns+(image->columns & (horizontal_factor-1));
     height=image->rows+(image->rows & (vertical_factor-1));
     yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
       &image->exception);
     if (yuv_image == (Image *) NULL)
       ThrowWriterException(ResourceLimitError,image->exception.reason);
-    if (image_info->colorspace == UndefinedColorspace)
-      (void) SetImageColorspace(yuv_image,YCbCrColorspace);
+    (void) SetImageColorspace(yuv_image,YCbCrColorspace);
     /*
       Downsample image.
     */
@@ -619,8 +610,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       height/vertical_factor,TriangleFilter,1.0,&image->exception);
     if (chroma_image == (Image *) NULL)
       ThrowWriterException(ResourceLimitError,image->exception.reason);
-    if (image_info->colorspace == UndefinedColorspace)
-      (void) SetImageColorspace(chroma_image,YCbCrColorspace);
+    (void) SetImageColorspace(chroma_image,YCbCrColorspace);
     if (interlace == NoInterlace)
       {
         /*
@@ -648,14 +638,11 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             x++;
           }
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(SaveImageTag,y,image->rows,
-                  image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         yuv_image=DestroyImage(yuv_image);
       }
@@ -676,22 +663,25 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(SaveImageTag,y,image->rows,
-                  image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         yuv_image=DestroyImage(yuv_image);
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,SaveImageTag,1,3);
+            if (status == MagickFalse)
+              break;
+          }
         /*
           Initialize U channel.
         */
         if (interlace == PartitionInterlace)
           {
-            CloseBlob(image);
+            (void) CloseBlob(image);
             AppendImageFormat("U",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
@@ -710,12 +700,18 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
         }
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,SaveImageTag,2,3);
+            if (status == MagickFalse)
+              break;
+          }
         /*
           Initialize V channel.
         */
         if (interlace == PartitionInterlace)
           {
-            CloseBlob(image);
+            (void) CloseBlob(image);
             AppendImageFormat("V",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
@@ -734,6 +730,12 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
         }
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,SaveImageTag,2,3);
+            if (status == MagickFalse)
+              break;
+          }
       }
     chroma_image=DestroyImage(chroma_image);
     if (interlace == PartitionInterlace)
@@ -742,15 +744,11 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
