@@ -10,49 +10,35 @@
 %                        I    C      C      C      M   M                      %
 %                      IIIII   CCCC   CCCC   CCCC  M   M                      %
 %                                                                             %
-%                          X11 Compatibility Methods                          %
-%                                                                             %
+%                     MagickCore X11 Compatibility Methods                    %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                December 1994                                %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %
 */
 
-#include "magick.h"
-#include "define.h"
-#if defined(HasX11)
-#include "xwindow.h"
+#include "magick/studio.h"
+#if defined(MAGICKCORE_X11_DELEGATE)
+#include "magick/xwindow-private.h"
 
 #if defined(PRE_R6_ICCCM)
 /*
@@ -114,7 +100,7 @@ Status XrmCombineFileDatabase(const char *filename,XrmDatabase *target,
     source;
 
   source=XrmGetFileDatabase(filename);
-  if (!override)
+  if (override == MagickFalse)
     XrmMergeDatabases(source,target);
   return(1);
 }
@@ -141,17 +127,17 @@ Bool XSupportsLocale()
 */
 XClassHint *XAllocClassHint)
 {
-  return((XClassHint *) AcquireMemory(sizeof(XClassHint)));
+  return((XClassHint *) AcquireMagickMemory(sizeof(XClassHint)));
 }
 
 XIconSize *XAllocIconSize)
 {
-  return((XIconSize *) AcquireMemory(sizeof(XIconSize)));
+  return((XIconSize *) AcquireMagickMemory(sizeof(XIconSize)));
 }
 
 XSizeHints *XAllocSizeHints)
 {
-  return((XSizeHints *) AcquireMemory(sizeof(XSizeHints)));
+  return((XSizeHints *) AcquireMagickMemory(sizeof(XSizeHints)));
 }
 
 Status XReconfigureWMWindow(Display *display,Window window,int screen_number,
@@ -162,18 +148,18 @@ Status XReconfigureWMWindow(Display *display,Window window,int screen_number,
 
 XStandardColormap *XAllocStandardColormap)
 {
-  return((XStandardColormap *) AcquireMemory(sizeof(XStandardColormap)));
+  return((XStandardColormap *) AcquireMagickMemory(sizeof(XStandardColormap)));
 }
 
 XWMHints *XAllocWMHints)
 {
-  return((XWMHints *) AcquireMemory(sizeof(XWMHints)));
+  return((XWMHints *) AcquireMagickMemory(sizeof(XWMHints)));
 }
 
 Status XGetGCValues(Display *display,GC gc,unsigned long mask,
   XGCValues *values)
 {
-  return(True);
+  return(MagickTrue);
 }
 
 Status XGetRGBColormaps(Display *display,Window window,
@@ -199,27 +185,27 @@ Status XGetWMColormapWindows(Display *display,Window window,
     leftover,
     number_items;
 
-  property=XInternAtom(display,"WM_COLORMAP_WINDOWS",False);
+  property=XInternAtom(display,"WM_COLORMAP_WINDOWS",MagickFalse);
   if (property == None)
-    return(False);
+    return(MagickFalse);
   /*
     Get the window property.
   */
   *data=(Atom) NULL;
-  status=XGetWindowProperty(display,window,property,0L,1000000L,False,
+  status=XGetWindowProperty(display,window,property,0L,1000000L,MagickFalse,
     XA_WINDOW,&actual_type,&actual_format,&number_items,&leftover,
     (unsigned char **) &data);
   if (status != Success)
-    return(False);
+    return(MagickFalse);
   if ((actual_type != XA_WINDOW) || (actual_format != 32))
     {
       if (data != (Atom *) NULL)
         XFree((char *) data);
-      return(False);
+      return(MagickFalse);
     }
   *colormap_windows=(Window *) data;
   *number_windows=(int) number_items;
-  return(True);
+  return(MagickTrue);
 }
 
 Status XGetWMName(Display *display,Window window,XTextProperty *text_property)
@@ -228,12 +214,12 @@ Status XGetWMName(Display *display,Window window,XTextProperty *text_property)
     *window_name;
 
   if (XFetchName(display,window,&window_name) == 0)
-    return(False);
+    return(MagickFalse);
   text_property->value=(unsigned char *) window_name;
   text_property->encoding=XA_STRING;
   text_property->format=8;
   text_property->nitems=strlen(window_name);
-  return(True);
+  return(MagickTrue);
 }
 
 char *XResourceManagerString(Display *display)
@@ -272,10 +258,10 @@ Status XSetWMProtocols(Display *display,Window window,Atom *protocols,
   Atom
     wm_protocols;
 
-  wm_protocols=XInternAtom(display,"WM_PROTOCOLS",False);
+  wm_protocols=XInternAtom(display,"WM_PROTOCOLS",MagickFalse);
   XChangeProperty(display,window,wm_protocols,XA_ATOM,32,PropModeReplace,
-    (unsigned char *) protocols, count);
-  return(True);
+    (unsigned char *) protocols,count);
+  return(MagickTrue);
 }
 
 int XStringListToTextProperty(char **argv,int argc,XTextProperty *property)
@@ -290,7 +276,7 @@ int XStringListToTextProperty(char **argv,int argc,XTextProperty *property)
      protocol;
 
   number_bytes=0;
-  for (i=0; i < argc; i++)
+  for (i=0; i < (long) argc; i++)
     number_bytes+=(unsigned int) ((argv[i] ? strlen(argv[i]) : 0)+1);
   protocol.encoding=XA_STRING;
   protocol.format=8;
@@ -300,9 +286,10 @@ int XStringListToTextProperty(char **argv,int argc,XTextProperty *property)
   protocol.value=NULL;
   if (number_bytes <= 0)
     {
-      protocol.value=(unsigned char *) AcquireMemory(1);
-      if (!protocol.value)
-        return(False);
+      protocol.value=(unsigned char *) AcquireQuantumMemory(1UL,
+        sizeof(*protocol.value));
+      if (protocol.value == MagickFalse)
+        return(MagickFalse);
       *protocol.value='\0';
     }
   else
@@ -310,27 +297,27 @@ int XStringListToTextProperty(char **argv,int argc,XTextProperty *property)
       register char
         *buffer;
 
-      buffer=(char *) AcquireMemory(number_bytes);
+      buffer=(char *) AcquireQuantumMemory(number_bytes,sizeof(*buffer));
       if (buffer == (char *) NULL)
-        return(False);
+        return(MagickFalse);
       protocol.value=(unsigned char *) buffer;
-      for (i=0; i < argc; i++)
+      for (i=0; i < (long) argc; i++)
       {
         char
           *argument;
 
         argument=argv[i];
-        if (!argument)
+        if (argument == MagickFalse)
           *buffer++='\0';
         else
           {
-            (void) strncpy(buffer,argument,MaxTextExtent-1);
+            (void) CopyMagickString(buffer,argument,MaxTextExtent);
             buffer+=(strlen(argument)+1);
           }
       }
     }
   *property=protocol;
-  return(True);
+  return(MagickTrue);
 }
 
 VisualID XVisualIDFromVisual(Visual *visual)
