@@ -10,7 +10,7 @@
 %          A   A  N   N  N   N   OOO     T    A   A    T    EEEEE             %
 %                                                                             %
 %                                                                             %
-%                  ImageMagick Image Annotation Methods                       %
+%                   MagickCore Image Annotation Methods                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -137,7 +137,6 @@ static MagickBooleanType
 %    %o   output image filename.
 %    %p   page number of the image.
 %    %q   image depth (8 or 16).
-%    %p   page number of the image.
 %    %q   image depth (8 or 16).
 %    %s   image scene number.
 %    %t   image filename without any extension.
@@ -152,9 +151,9 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
 */
 MagickExport MagickBooleanType AnnotateImage(Image *image,
@@ -404,7 +403,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     if (annotate->decorate == OverlineDecoration)
       {
         annotate_info->affine.ty-=(draw_info->affine.sy*(metrics.ascent+
-          metrics.descent-metrics.underline_position+2));
+          metrics.descent-metrics.underline_position));
         (void) CloneString(&annotate_info->primitive,primitive);
         (void) DrawImage(image,annotate_info);
       }
@@ -412,7 +411,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
       if (annotate->decorate == UnderlineDecoration)
         {
           annotate_info->affine.ty-=(draw_info->affine.sy*
-            metrics.underline_position+2);
+            metrics.underline_position);
           (void) CloneString(&annotate_info->primitive,primitive);
           (void) DrawImage(image,annotate_info);
         }
@@ -431,7 +430,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
       }
   }
   /*
-    Free resources.
+    Relinquish resources.
   */
   annotate_info=DestroyDrawInfo(annotate_info);
   annotate=DestroyDrawInfo(annotate);
@@ -466,9 +465,9 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o text: The text.
+%    o text: the text.
 %
 %    o count: return the number of characters generated by the encoding.
 %
@@ -566,7 +565,7 @@ static unsigned long *EncodeSJIS(const char *text,size_t *count)
 %
 %  A description of each parameter follows:
 %
-%    o text: The text.
+%    o text: the text.
 %
 %    o count: return the number of characters generated by the encoding.
 %
@@ -628,7 +627,7 @@ static unsigned long *EncodeText(const char *text,size_t *count)
 %    o unicode:  EncodeUnicode() returns a pointer to an unsigned short array
 %      representing the encoded version of the ASCII string.
 %
-%    o text: The text.
+%    o text: the text.
 %
 %    o count: return the number of characters generated by the encoding.
 %
@@ -755,7 +754,7 @@ static unsigned long *EncodeUnicode(const char *text,size_t *count)
 %
 %    o image:  The image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
 %    o metrics: Return the font metrics in this structure.
 %
@@ -829,11 +828,17 @@ MagickExport long FormatMagickCaption(Image *image,DrawInfo *draw_info,
 %
 %    character width
 %    character height
-%    ascent
-%    descent
+%    ascender
+%    descender
 %    text width
 %    text height
 %    maximum horizontal advance
+%    bounds: x1
+%    bounds: y1
+%    bounds: x2
+%    bounds: y2
+%    origin: x
+%    origin: y
 %    underline position
 %    underline thickness
 %
@@ -847,9 +852,9 @@ MagickExport long FormatMagickCaption(Image *image,DrawInfo *draw_info,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
 %    o metrics: Return the font metrics in this structure.
 %
@@ -912,7 +917,7 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   metrics->height=(double) number_lines*(long) (metrics->ascent-
     metrics->descent+0.5);
   /*
-    Free resources.
+    Relinquish resources.
   */
   annotate_info->text=(char *) NULL;
   annotate_info=DestroyDrawInfo(annotate_info);
@@ -938,11 +943,17 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
 %
 %    character width
 %    character height
-%    ascent
-%    descent
+%    ascender
+%    descender
 %    text width
 %    text height
 %    maximum horizontal advance
+%    bounds: x1
+%    bounds: y1
+%    bounds: x2
+%    bounds: y2
+%    origin: x
+%    origin: y
 %    underline position
 %    underline thickness
 %
@@ -953,9 +964,9 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
 %    o metrics: Return the font metrics in this structure.
 %
@@ -985,6 +996,16 @@ MagickExport MagickBooleanType GetTypeMetrics(Image *image,
   offset.x=0.0;
   offset.y=0.0;
   status=RenderType(image,annotate_info,&offset,metrics);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),"Metrics: text: %s; "
+      "width: %g; height: %g; ascent: %g; descent: %g; max advance: %g; "
+      "bounds: %g,%g  %g,%g; origin: %g,%g; pixels per em: %g,%g; "
+      "underline position: %g; underline thickness: %g",annotate_info->text,
+      metrics->width,metrics->height,metrics->ascent,metrics->descent,
+      metrics->max_advance,metrics->bounds.x1,metrics->bounds.y1,
+      metrics->bounds.x2,metrics->bounds.y2,metrics->origin.x,metrics->origin.y,
+      metrics->pixels_per_em.x,metrics->pixels_per_em.y,
+      metrics->underline_position,metrics->underline_thickness);
   annotate_info=DestroyDrawInfo(annotate_info);
   return(status);
 }
@@ -1010,9 +1031,9 @@ MagickExport MagickBooleanType GetTypeMetrics(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
 %    o offset: (x,y) location of text relative to image.
 %
@@ -1035,11 +1056,11 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
   if (draw_info->font != (char *) NULL)
     {
       if (*draw_info->font == '@')
-        return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
+        return(RenderFreetype(image,draw_info,draw_info->encoding,offset,metrics));
       if (*draw_info->font == '-')
         return(RenderX11(image,draw_info,offset,metrics));
-      if (IsAccessible(draw_info->font) != MagickFalse)
-        return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
+      if (IsPathAccessible(draw_info->font) != MagickFalse)
+        return(RenderFreetype(image,draw_info,draw_info->encoding,offset,metrics));
       type_info=GetTypeInfo(draw_info->font,&image->exception);
       if (type_info == (const TypeInfo *) NULL)
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
@@ -1065,7 +1086,7 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
       draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
     {
-      status=RenderFreetype(image,draw_info,(const char *) NULL,offset,
+      status=RenderFreetype(image,draw_info,draw_info->encoding,offset,
         metrics);
       return(status);
     }
@@ -1101,11 +1122,11 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
-%    o encoding: The font encoding.
+%    o encoding: the font encoding.
 %
 %    o offset: (x,y) location of text relative to image.
 %
@@ -1240,25 +1261,12 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   long
     y;
 
-  MagickBooleanType
-    active;
-
-  MagickRealType
-    fill_opacity;
-
-  PixelPacket
-    fill_color;
-
   PointInfo
     point,
     resolution;
 
   register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
+    i;
 
   size_t
     length;
@@ -1301,7 +1309,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       return(RenderPostscript(image,draw_info,offset,metrics));
     }
   if ((draw_info->metrics != (char *) NULL) &&
-      (IsAccessible(draw_info->metrics) != MagickFalse))
+      (IsPathAccessible(draw_info->metrics) != MagickFalse))
     (void) FT_Attach_File(face,draw_info->metrics);
   encoding_type=ft_encoding_unicode;
   status=FT_Select_Charmap(face,encoding_type);
@@ -1362,7 +1370,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       if ((flags & SigmaValue) == 0)
         resolution.y=resolution.x;
     }
-  (void) FT_Set_Char_Size(face,(FT_F26Dot6) (64.0*draw_info->pointsize),
+  status=FT_Set_Char_Size(face,(FT_F26Dot6) (64.0*draw_info->pointsize),
     (FT_F26Dot6) (64.0*draw_info->pointsize),(FT_UInt) resolution.x,
     (FT_UInt) resolution.y);
   metrics->pixels_per_em.x=face->size->metrics.x_ppem;
@@ -1473,7 +1481,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       if (image->storage_class != DirectClass)
         (void) SetImageStorageClass(image,DirectClass);
       if (image->matte == MagickFalse)
-        (void) SetImageOpacity(image,OpaqueOpacity);
+        (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
     }
   point.x=0.0;
   point.y=0.0;
@@ -1534,36 +1542,59 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     point.y=offset->y-bitmap->top;
     if (draw_info->render != MagickFalse)
       {
-        long
-          x_offset,
-          y_offset;
-
-        register unsigned char
-          *p;
+        MagickBooleanType
+          status;
 
         ViewInfo
-          *image_view;
+          **image_view;
 
         /*
           Rasterize the glyph.
         */
-        p=bitmap->bitmap.buffer;
-        image_view=OpenCacheView(image);
+        status=MagickTrue;
+        image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp parallel for schedule(static,64) shared(status)
+#endif
         for (y=0; y < (long) bitmap->bitmap.rows; y++)
         {
+          long
+            x_offset,
+            y_offset;
+
+          MagickBooleanType
+            active;
+
+          MagickRealType
+            fill_opacity;
+
+          PixelPacket
+            fill_color;
+
+          register long
+            id,
+            x;
+
+          register PixelPacket
+            *q;
+
+          register unsigned char
+            *p;
+
+          if (status == MagickFalse)
+            continue;
           x_offset=(long) (point.x+0.5);
           y_offset=(long) (point.y+y+0.5);
           if ((y_offset < 0) || (y_offset >= (long) image->rows))
-            {
-              p+=bitmap->bitmap.width;
-              continue;
-            }
-          q=GetCacheViewPixels(image_view,x_offset,y_offset,
+            continue;
+          id=GetCacheViewThreadId();
+          q=GetCacheViewPixels(image_view[id],x_offset,y_offset,
             bitmap->bitmap.width,1);
           active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
+          p=bitmap->bitmap.buffer+y*bitmap->bitmap.width;
           for (x=0; x < (long) bitmap->bitmap.width; x++)
           {
-            x_offset=(long) (point.x+x+0.5);
+            x_offset++;
             if ((*p == 0) || (x_offset < 0) ||
                 (x_offset >= (long) image->columns))
               {
@@ -1575,26 +1606,27 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             if (draw_info->text_antialias == MagickFalse)
               fill_opacity=fill_opacity > 0.5 ? 1.0 : 0.0;
             if (active == MagickFalse)
-              q=GetCacheViewPixels(image_view,x_offset,y_offset,1,1);
+              q=GetCacheViewPixels(image_view[id],x_offset,y_offset,1,1);
             if (q == (PixelPacket *) NULL)
               {
                 p++;
                 q++;
                 continue;
               }
-            fill_color=GetFillColor(draw_info,x_offset,y_offset);
+            (void) GetFillColor(draw_info,x_offset,y_offset,&fill_color);
             fill_opacity=QuantumRange-fill_opacity*(QuantumRange-
               fill_color.opacity);
             MagickCompositeOver(&fill_color,fill_opacity,q,q->opacity,q);
-            if (active == MagickFalse)
-              (void) SyncCacheView(image_view);
+            if ((active == MagickFalse) &&
+                (SyncCacheViewPixels(image_view[id]) == MagickFalse))
+              status=MagickFalse;
             p++;
             q++;
           }
-          if (SyncCacheView(image_view) == MagickFalse)
-            break;
+          if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+            status=MagickFalse;
         }
-        image_view=CloseCacheView(image_view);
+        image_view=DestroyCacheViewThreadSet(image_view);
       }
     if ((bitmap->left+bitmap->bitmap.width) > metrics->width)
       metrics->width=bitmap->left+bitmap->bitmap.width;
@@ -1656,7 +1688,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->origin.x/=64.0;
   metrics->origin.y/=64.0;
   /*
-    Free resources.
+    Relinquish resources.
   */
   text=(unsigned long *) RelinquishMagickMemory(text);
   annotate_info=DestroyDrawInfo(annotate_info);
@@ -1670,7 +1702,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   TypeMetric *metrics)
 {
   (void) ThrowMagickException(&image->exception,GetMagickModule(),
-    MissingDelegateWarning,"FreeTypeLibraryIsNotAvailable","`%s'",
+    MissingDelegateWarning,"DelegateLibrarySupportNotBuiltIn","`%s' (Freetype)",
     draw_info->font);
   return(RenderPostscript(image,draw_info,offset,metrics));
 }
@@ -1697,9 +1729,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
 %    o offset: (x,y) location of text relative to image.
 %
@@ -1925,11 +1957,11 @@ static MagickBooleanType RenderPostscript(Image *image,
         Render fill color.
       */
       if (image->matte == MagickFalse)
-        (void) SetImageOpacity(image,OpaqueOpacity);
+        (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       if (annotate_image->matte == MagickFalse)
-        (void) SetImageOpacity(annotate_image,OpaqueOpacity);
+        (void) SetImageAlphaChannel(annotate_image,OpaqueAlphaChannel);
       fill_color=draw_info->fill;
-      annotate_view=OpenCacheView(annotate_image);
+      annotate_view=AcquireCacheView(annotate_image);
       for (y=0; y < (long) annotate_image->rows; y++)
       {
         q=GetCacheViewPixels(annotate_view,0,y,annotate_image->columns,1);
@@ -1937,7 +1969,7 @@ static MagickBooleanType RenderPostscript(Image *image,
           break;
         for (x=0; x < (long) annotate_image->columns; x++)
         {
-          fill_color=GetFillColor(draw_info,x,y);
+          (void) GetFillColor(draw_info,x,y,&fill_color);
           q->opacity=RoundToQuantum(QuantumRange-(((QuantumRange-
             (MagickRealType) PixelIntensityToQuantum(q))*(QuantumRange-
             fill_color.opacity))/QuantumRange));
@@ -1946,10 +1978,10 @@ static MagickBooleanType RenderPostscript(Image *image,
           q->blue=fill_color.blue;
           q++;
         }
-        if (SyncCacheView(annotate_view) == MagickFalse)
+        if (SyncCacheViewPixels(annotate_view) == MagickFalse)
           break;
       }
-      annotate_view=CloseCacheView(annotate_view);
+      annotate_view=DestroyCacheView(annotate_view);
       (void) CompositeImage(image,OverCompositeOp,annotate_image,
         (long) (offset->x+0.5),(long) (offset->y-(metrics->ascent+
         metrics->descent)+0.5));
@@ -1979,9 +2011,9 @@ static MagickBooleanType RenderPostscript(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o draw_info: The draw info.
+%    o draw_info: the draw info.
 %
 %    o offset: (x,y) location of text relative to image.
 %
@@ -2028,6 +2060,9 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
 
   if (display == (Display *) NULL)
     {
+      ImageInfo
+        *image_info;
+
       /*
         Open X server connection.
       */
@@ -2042,8 +2077,10 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
         Get user defaults from X resource database.
       */
       (void) XSetErrorHandler(XError);
+      image_info=AcquireImageInfo();
       resource_database=XGetResourceDatabase(display,GetClientName());
-      XGetResourceInfo(resource_database,GetClientName(),&resource_info);
+      XGetResourceInfo(image_info,resource_database,GetClientName(),
+        &resource_info);
       resource_info.close_server=MagickFalse;
       resource_info.colormap=PrivateColormap;
       resource_info.font=AcquireString(draw_info->font);
@@ -2176,7 +2213,9 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
 static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
-  ThrowBinaryException(MissingDelegateError,"XWindowLibraryIsNotAvailable",
-    draw_info->font);
+  (void) ThrowMagickException(&image->exception,GetMagickModule(),
+    MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (X11)",
+    image->filename);
+  return(MagickFalse);
 }
 #endif
