@@ -10,7 +10,7 @@
 %           A   A  R  R     T    IIIII  F      A   A  CCCCC    T              %
 %                                                                             %
 %                                                                             %
-%                        ImageMagick Artifact Methods                         %
+%                         MagickCore Artifact Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -65,7 +65,7 @@
 #include "magick/quantum.h"
 #include "magick/resource_.h"
 #include "magick/splay-tree.h"
-#include "magick/signature.h"
+#include "magick/signature-private.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -92,9 +92,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o clone_image: The clone image.
+%    o clone_image: the clone image.
 %
 */
 MagickExport MagickBooleanType CloneImageArtifacts(Image *image,
@@ -135,9 +135,9 @@ MagickExport MagickBooleanType CloneImageArtifacts(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o artifact: The image artifact.
+%    o artifact: the image artifact.
 %
 */
 MagickExport MagickBooleanType DefineImageArtifact(Image *image,
@@ -182,9 +182,9 @@ MagickExport MagickBooleanType DefineImageArtifact(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o artifact: The image artifact.
+%    o artifact: the image artifact.
 %
 */
 MagickExport MagickBooleanType DeleteImageArtifact(Image *image,
@@ -220,7 +220,7 @@ MagickExport MagickBooleanType DeleteImageArtifact(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport void DestroyImageArtifacts(Image *image)
@@ -254,9 +254,9 @@ MagickExport void DestroyImageArtifacts(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o key: The key.
+%    o key: the key.
 %
 */
 MagickExport const char *GetImageArtifact(const Image *image,
@@ -306,7 +306,7 @@ MagickExport const char *GetImageArtifact(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport char *GetNextImageArtifact(const Image *image)
@@ -341,9 +341,9 @@ MagickExport char *GetNextImageArtifact(const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o artifact: The image artifact.
+%    o artifact: the image artifact.
 %
 */
 MagickExport char *RemoveImageArtifact(Image *image,const char *artifact)
@@ -384,7 +384,7 @@ MagickExport char *RemoveImageArtifact(Image *image,const char *artifact)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport void ResetImageArtifactIterator(const Image *image)
@@ -419,11 +419,11 @@ MagickExport void ResetImageArtifactIterator(const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o artifact: The image artifact.
+%    o artifact: the image artifact.
 %
-%    o values: The image artifact values.
+%    o values: the image artifact values.
 %
 */
 MagickExport MagickBooleanType SetImageArtifact(Image *image,
