@@ -43,19 +43,15 @@ typedef IndexPacket
   *(*GetIndexesFromHandler)(const Image *);
 
 typedef MagickBooleanType
+  (*AcquireOnePixelFromHandler)(const Image *,const VirtualPixelMethod,
+    const long,const long,PixelPacket *,ExceptionInfo *),
+  (*GetOnePixelFromHandler)(Image *,const long,const long,PixelPacket *),
   (*SyncPixelHandler)(Image *);
 
 typedef const PixelPacket
   *(*AcquirePixelHandler)(const Image *,const VirtualPixelMethod,const long,
     const long,const unsigned long,const unsigned long,ExceptionInfo *);
 
-typedef PixelPacket
-  (*AcquireOnePixelFromHandler)(const Image *,const VirtualPixelMethod,
-    const long,const long,ExceptionInfo *);
-
-typedef PixelPacket
-  (*GetOnePixelFromHandler)(Image *,const long,const long);
-
 typedef PixelPacket
   *(*GetPixelHandler)(Image *,const long,const long,const unsigned long,
     const unsigned long);
@@ -103,13 +99,11 @@ typedef struct _CacheMethods
     sync_pixel_handler;
 } CacheMethods;
 
-typedef struct _NexusInfo NexusInfo;
+typedef struct _NexusInfo
+   NexusInfo;
 
 typedef struct _CacheInfo
 {
-  unsigned long
-    id;
-
   NexusInfo
     *nexus_info;
 
@@ -122,7 +116,7 @@ typedef struct _CacheInfo
   ColorspaceType
     colorspace;
 
-  CacheType
+  volatile CacheType
     type;
 
   MagickBooleanType
@@ -166,14 +160,22 @@ typedef struct _CacheInfo
   MagickBooleanType
     debug;
 
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  pthread_t
+#elif defined(__WINDOWS__)
+  DWORD
+#else
   unsigned long
-    thread;
+#endif
+    id;
 
-  long
+  volatile long
     reference_count;
 
   SemaphoreInfo
-    *semaphore;
+    *semaphore,
+    *nexus_semaphore,
+    *disk_semaphore;
 
   unsigned long
     signature;
