@@ -30,7 +30,7 @@ extern MagickExport const PixelPacket
     ExceptionInfo *);
 
 extern MagickExport MagickSizeType
-  GetPixelCacheArea(const Image *);
+  GetPixelCacheExtent(const Image *);
 
 extern MagickExport MagickBooleanType
   PersistCache(Image *,const char *,const MagickBooleanType,MagickOffsetType *,
