@@ -10,7 +10,7 @@
 %                   CCCC   OOO   DDDD    EEEEE  R  R                          %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Coder Methods                         %
+%                     MagickCore Image Coder Methods                          %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -82,6 +82,7 @@ static const char
     "  <coder magick=\"BMP2\" name=\"BMP\" />"
     "  <coder magick=\"BMP3\" name=\"BMP\" />"
     "  <coder magick=\"B\" name=\"GRAY\" />"
+    "  <coder magick=\"BRF\" name=\"BRAILLE\" />"
     "  <coder magick=\"CMYKA\" name=\"CMYK\" />"
     "  <coder magick=\"C\" name=\"GRAY\" />"
     "  <coder magick=\"CR2\" name=\"DNG\" />"
@@ -120,6 +121,7 @@ static const char
     "  <coder magick=\"IPTC\" name=\"META\" />"
     "  <coder magick=\"IPTCTEXT\" name=\"META\" />"
     "  <coder magick=\"IPTCWTEXT\" name=\"META\" />"
+    "  <coder magick=\"ISOBRL\" name=\"BRAILLE\" />"
     "  <coder magick=\"JBG\" name=\"JBIG\" />"
     "  <coder magick=\"JNG\" name=\"PNG\" />"
     "  <coder magick=\"JPC\" name=\"JP2\" />"
@@ -128,9 +130,12 @@ static const char
     "  <coder magick=\"K\" name=\"GRAY\" />"
     "  <coder magick=\"LOGO\" name=\"MAGICK\" />"
     "  <coder magick=\"M2V\" name=\"MPEG\" />"
+    "  <coder magick=\"M4V\" name=\"MPEG\" />"
     "  <coder magick=\"M\" name=\"GRAY\" />"
     "  <coder magick=\"MNG\" name=\"PNG\" />"
+    "  <coder magick=\"MOV\" name=\"MPEG\" />"
     "  <coder magick=\"MPG\" name=\"MPEG\" />"
+    "  <coder magick=\"MP4\" name=\"MPEG\" />"
     "  <coder magick=\"MPRI\" name=\"MPR\" />"
     "  <coder magick=\"MRW\" name=\"DNG\" />"
     "  <coder magick=\"MSVG\" name=\"SVG\" />"
@@ -145,6 +150,7 @@ static const char
     "  <coder magick=\"PBM\" name=\"PNM\" />"
     "  <coder magick=\"PCDS\" name=\"PCD\" />"
     "  <coder magick=\"PCT\" name=\"PICT\" />"
+    "  <coder magick=\"PDFA\" name=\"PDF\" />"
     "  <coder magick=\"PEF\" name=\"DNG\" />"
     "  <coder magick=\"PFA\" name=\"TTF\" />"
     "  <coder magick=\"PFB\" name=\"TTF\" />"
@@ -159,6 +165,7 @@ static const char
     "  <coder magick=\"PNG8\" name=\"PNG\" />"
     "  <coder magick=\"PPM\" name=\"PNM\" />"
     "  <coder magick=\"PTIF\" name=\"TIFF\" />"
+    "  <coder magick=\"RADIAL-GRADIENT\" name=\"GRADIENT\" />"
     "  <coder magick=\"RAF\" name=\"DNG\" />"
     "  <coder magick=\"RAS\" name=\"SUN\" />"
     "  <coder magick=\"RGBA\" name=\"RGB\" />"
@@ -171,9 +178,11 @@ static const char
     "  <coder magick=\"TIFF64\" name=\"TIFF\" />"
     "  <coder magick=\"TIF\" name=\"TIFF\" />"
     "  <coder magick=\"TTC\" name=\"TTF\" />"
+    "  <coder magick=\"UBRL\" name=\"BRAILLE\" />"
     "  <coder magick=\"VDA\" name=\"TGA\" />"
     "  <coder magick=\"VST\" name=\"TGA\" />"
     "  <coder magick=\"WMFWIN32\" name=\"EMF\" />"
+    "  <coder magick=\"WMV\" name=\"MPEG\" />"
     "  <coder magick=\"X3F\" name=\"DNG\" />"
     "  <coder magick=\"XTRNARRAY\" name=\"XTRN\" />"
     "  <coder magick=\"XTRNBLOB\" name=\"XTRN\" />"
@@ -230,7 +239,7 @@ MagickExport void DestroyCoderList(void)
     coder_list=DestroySplayTree(coder_list);
   instantiate_coder=MagickFalse;
   RelinquishSemaphoreInfo(coder_semaphore);
-  coder_semaphore=DestroySemaphoreInfo(coder_semaphore);
+  DestroySemaphoreInfo(&coder_semaphore);
 }
 
 /*
@@ -253,7 +262,7 @@ MagickExport void DestroyCoderList(void)
 %
 %  A description of each parameter follows:
 %
-%    o name: The coder name.
+%    o name: the coder name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -610,103 +619,173 @@ static void *DestroyCoderNode(void *coder_info)
 static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
-  CoderInfo
-    *coder_info = (CoderInfo *) NULL;
+  char
+    keyword[MaxTextExtent],
+    *token;
 
   const char
-    *attribute;
+    *q;
+
+  CoderInfo
+    *coder_info;
 
   MagickBooleanType
     status;
 
-  XMLTreeInfo
-    *coder,
-    *coder_map,
-    *include;
-
   /*
     Load the coder map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading coder map \"%s\" ...",filename);
+    "Loading coder configuration file \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (coder_list == (SplayTreeInfo *) NULL)
     {
-      coder_list=NewSplayTree(CompareSplayTreeString,(void *(*)(void *)) NULL,
+      coder_list=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
         DestroyCoderNode);
       if (coder_list == (SplayTreeInfo *) NULL)
         {
-          (void) ThrowMagickException(exception,GetMagickModule(),
-             ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
+          ThrowFileException(exception,ResourceLimitError,
+            "MemoryAllocationFailed",filename);
           return(MagickFalse);
         }
     }
-  coder_map=NewXMLTree(xml,exception);
-  if (coder_map == (XMLTreeInfo *) NULL)
-    return(MagickFalse);
   status=MagickTrue;
-  include=GetXMLTreeChild(coder_map,"include");
-  while (include != (XMLTreeInfo *) NULL)
+  coder_info=(CoderInfo *) NULL;
+  token=AcquireString(xml);
+  for (q=(char *) xml; *q != '\0'; )
   {
     /*
-      Process include element.
+      Interpret XML.
     */
-    attribute=GetXMLTreeAttribute(include,"file");
-    if (attribute != (const char *) NULL)
+    GetMagickToken(q,&q,token);
+    if (*token == '\0')
+      break;
+    (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
       {
-        if (depth > 200)
-          (void) ThrowMagickException(exception,GetMagickModule(),
-            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",attribute);
-        else
-          {
-            char
-              path[MaxTextExtent],
-              *xml;
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
+      {
+        /*
+          Comment element.
+        */
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
+    if (LocaleCompare(keyword,"<include") == 0)
+      {
+        /*
+          Include element.
+        */
+        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
+        {
+          (void) CopyMagickString(keyword,token,MaxTextExtent);
+          GetMagickToken(q,&q,token);
+          if (*token != '=')
+            continue;
+          GetMagickToken(q,&q,token);
+          if (LocaleCompare(keyword,"file") == 0)
+            {
+              if (depth > 200)
+                (void) ThrowMagickException(exception,GetMagickModule(),
+                  ConfigureError,"IncludeNodeNestedTooDeeply","`%s'",token);
+              else
+                {
+                  char
+                    path[MaxTextExtent],
+                    *xml;
 
-            GetPathComponent(filename,HeadPath,path);
-            if (*path != '\0')
-              (void) ConcatenateMagickString(path,DirectorySeparator,
-                MaxTextExtent);
-            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
-            xml=FileToString(path,~0,exception);
-            if (xml != (char *) NULL)
-              {
-                status=LoadCoderList(xml,path,depth+1,exception);
-                xml=DestroyString(xml);
-              }
-          }
+                  GetPathComponent(filename,HeadPath,path);
+                  if (*path != '\0')
+                    (void) ConcatenateMagickString(path,DirectorySeparator,
+                      MaxTextExtent);
+                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  xml=FileToString(path,~0,exception);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadCoderList(xml,path,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
+                }
+            }
+        }
+        continue;
+      }
+    if (LocaleCompare(keyword,"<coder") == 0)
+      {
+        /*
+          Coder element.
+        */
+        coder_info=(CoderInfo *) AcquireMagickMemory(sizeof(*coder_info));
+        if (coder_info == (CoderInfo *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+        (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
+        coder_info->path=ConstantString(filename);
+        coder_info->signature=MagickSignature;
+        continue;
       }
-    include=GetNextXMLTreeTag(include);
-  }
-  coder=GetXMLTreeChild(coder_map,"coder");
-  while (coder != (XMLTreeInfo *) NULL)
-  {
-    /*
-      Process coder element.
-    */
-    coder_info=(CoderInfo *) AcquireMagickMemory(sizeof(*coder_info));
     if (coder_info == (CoderInfo *) NULL)
-      ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-    (void) ResetMagickMemory(coder_info,0,sizeof(*coder_info));
-    coder_info->path=ConstantString(filename);
-    coder_info->signature=MagickSignature;
-    attribute=GetXMLTreeAttribute(coder,"magick");
-    if (attribute != (const char *) NULL)
-      coder_info->magick=ConstantString(attribute);
-    attribute=GetXMLTreeAttribute(coder,"name");
-    if (attribute != (const char *) NULL)
-      coder_info->name=ConstantString(attribute);
-    attribute=GetXMLTreeAttribute(coder,"stealth");
-    if (attribute != (const char *) NULL)
-      coder_info->stealth=IsMagickTrue(attribute);
-    status=AddValueToSplayTree(coder_list,coder_info->magick,coder_info);
-    if (status == MagickFalse)
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
-    coder=GetNextXMLTreeTag(coder);
+      continue;
+    if (LocaleCompare(keyword,"/>") == 0)
+      {
+        status=AddValueToSplayTree(coder_list,ConstantString(
+          coder_info->magick),coder_info);
+        if (status == MagickFalse)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            coder_info->magick);
+        coder_info=(CoderInfo *) NULL;
+      }
+    GetMagickToken(q,(const char **) NULL,token);
+    if (*token != '=')
+      continue;
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    switch (*keyword)
+    {
+      case 'M':
+      case 'm':
+      {
+        if (LocaleCompare((char *) keyword,"magick") == 0)
+          {
+            coder_info->magick=ConstantString(token);
+            break;
+          }
+        break;
+      }
+      case 'N':
+      case 'n':
+      {
+        if (LocaleCompare((char *) keyword,"name") == 0)
+          {
+            coder_info->name=ConstantString(token);
+            break;
+          }
+        break;
+      }
+      case 'S':
+      case 's':
+      {
+        if (LocaleCompare((char *) keyword,"stealth") == 0)
+          {
+            coder_info->stealth=IsMagickTrue(token);
+            break;
+          }
+        break;
+      }
+      default:
+        break;
+    }
   }
-  coder_map=DestroyXMLTree(coder_map);
+  token=(char *) RelinquishMagickMemory(token);
   return(status);
 }
 
@@ -731,7 +810,7 @@ static MagickBooleanType LoadCoderList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: the font file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
