@@ -10,7 +10,7 @@
 %          CCCC   OOO   N   N  F      IIIII   GGG    UUU   R  R   EEEEE       %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Configure Methods                     %
+%                      MagickCore Image Configure Methods                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -128,7 +128,7 @@ MagickExport void DestroyConfigureList(void)
   configure_list=(LinkedListInfo *) NULL;
   instantiate_configure=MagickFalse;
   RelinquishSemaphoreInfo(configure_semaphore);
-  configure_semaphore=DestroySemaphoreInfo(configure_semaphore);
+  DestroySemaphoreInfo(&configure_semaphore);
 }
 
 /*
@@ -151,7 +151,7 @@ MagickExport void DestroyConfigureList(void)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 
@@ -191,7 +191,7 @@ MagickExport LinkedListInfo *DestroyConfigureOptions(LinkedListInfo *options)
 %    o configure_info: GetConfigureInfo() searches the configure list for the
 %      specified name and if found returns attributes for that element.
 %
-%    o name: The configure name.
+%    o name: the configure name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -213,7 +213,7 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
     return((const ConfigureInfo *) GetValueFromLinkedList(configure_list,0));
   /*
-    Search for named configure.
+    Search for configure tag.
   */
   AcquireSemaphoreInfo(&configure_semaphore);
   ResetLinkedListIterator(configure_list);
@@ -224,9 +224,9 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
       break;
     p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   }
-  if (p == (ConfigureInfo *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
-      "NoSuchElement","`%s'",name);
+  if (p != (ConfigureInfo *) NULL)
+    (void) InsertValueInLinkedList(configure_list,0,
+      RemoveElementByValueFromLinkedList(configure_list,p));
   RelinquishSemaphoreInfo(configure_semaphore);
   return(p);
 }
@@ -426,6 +426,52 @@ MagickExport char **GetConfigureList(const char *pattern,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t C o n f i g u r e O p t i o n                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetConfigureOption() returns the value associated with the configure option.
+%
+%  The format of the GetConfigureOption method is:
+%
+%      char *GetConfigureOption(const char *option)
+%
+%  A description of each parameter follows:
+%
+%    o configure_info:  The configure info.
+%
+*/
+MagickExport char *GetConfigureOption(const char *option)
+{
+  const char
+    *value;
+
+  const ConfigureInfo
+    *configure_info;
+
+  ExceptionInfo
+    *exception;
+
+  assert(option != (const char *) NULL);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",option);
+  exception=AcquireExceptionInfo();
+  configure_info=GetConfigureInfo(option,exception);
+  exception=DestroyExceptionInfo(exception);
+  if (configure_info == (ConfigureInfo *) NULL)
+    return((char *) NULL);
+  value=GetConfigureValue(configure_info);
+  if ((value == (const char *) NULL) || (*value == '\0'))
+    return((char *) NULL);
+  return(ConstantString(value));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  G e t C o n f i g u r e O p t i o n s                                      %
 %                                                                             %
 %                                                                             %
@@ -442,7 +488,7 @@ MagickExport char **GetConfigureList(const char *pattern,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The configure file name.
+%    o filename: the configure file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -531,7 +577,7 @@ MagickExport LinkedListInfo *GetConfigureOptions(const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The configure file name.
+%    o filename: the configure file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -581,14 +627,17 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
       }
   }
 #if defined(MAGICKCORE_INSTALLED_SUPPORT)
-#if defined(MAGICKCORE_CONFIGURE_PATH)
-  (void) AppendValueToLinkedList(paths,ConstantString(MAGICKCORE_CONFIGURE_PATH));
-#endif
 #if defined(MAGICKCORE_SHARE_CONFIGURE_PATH)
-  (void) AppendValueToLinkedList(paths,ConstantString(MAGICKCORE_SHARE_CONFIGURE_PATH));
+  (void) AppendValueToLinkedList(paths,ConstantString(
+    MAGICKCORE_SHARE_CONFIGURE_PATH));
+#endif
+#if defined(MAGICKCORE_CONFIGURE_PATH)
+  (void) AppendValueToLinkedList(paths,ConstantString(
+    MAGICKCORE_CONFIGURE_PATH));
 #endif
 #if defined(MAGICKCORE_DOCUMENTATION_PATH)
-  (void) AppendValueToLinkedList(paths,ConstantString(MAGICKCORE_DOCUMENTATION_PATH));
+  (void) AppendValueToLinkedList(paths,ConstantString(
+    MAGICKCORE_DOCUMENTATION_PATH));
 #endif
 #if defined(MAGICKCORE_SHARE_PATH)
   (void) AppendValueToLinkedList(paths,ConstantString(MAGICKCORE_SHARE_PATH));
@@ -596,7 +645,9 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
 #if defined(__WINDOWS__) && !(defined(MAGICKCORE_CONFIGURE_PATH) || defined(MAGICKCORE_SHARE_CONFIGURE_PATH))
   {
     char
-      *registry_key,
+      *registry_key;
+
+    unsigned char
       *key_value;
 
     /*
@@ -604,11 +655,15 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
     */
     registry_key="ConfigurePath";
     key_value=NTRegistryKeyLookup(registry_key);
-    if (key_value != (char *) NULL)
+    if (key_value == (unsigned char *) NULL)
+      ThrowMagickException(exception,GetMagickModule(),ConfigureError,
+        "RegistryKeyLookupFailed","`%s'",registry_key);
+    else
       {
-        (void) FormatMagickString(path,MaxTextExtent,"%s%s",key_value,
+        (void) FormatMagickString(path,MaxTextExtent,"%s%s",(char *) key_value,
           DirectorySeparator);
         (void) AppendValueToLinkedList(paths,ConstantString(path));
+        key_value=(unsigned char *) RelinquishMagickMemory(key_value);
       }
   }
 #endif
@@ -653,12 +708,12 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
       */
       (void) CopyMagickString(prefix,GetClientPath(),MaxTextExtent);
       ChopPathComponents(prefix,1);
-      (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/",prefix,
-        MAGICKCORE_CONFIGURE_RELATIVE_PATH);
-      (void) AppendValueToLinkedList(paths,ConstantString(path));
       (void) FormatMagickString(path,MaxTextExtent,"%s/share/%s/",prefix,
         MAGICKCORE_SHARE_CONFIGURE_RELATIVE_PATH);
       (void) AppendValueToLinkedList(paths,ConstantString(path));
+      (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/",prefix,
+        MAGICKCORE_CONFIGURE_RELATIVE_PATH);
+      (void) AppendValueToLinkedList(paths,ConstantString(path));
 #endif
     }
 #endif
@@ -674,8 +729,8 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
         /*
           Search $HOME/.magick.
         */
-        (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",home,
-          *home == '/' ? "/.magick" : "",DirectorySeparator);
+        (void) FormatMagickString(path,MaxTextExtent,"%s%s.magick%s",home,
+          DirectorySeparator,DirectorySeparator);
         (void) AppendValueToLinkedList(paths,ConstantString(path));
         home=DestroyString(home);
       }
@@ -906,27 +961,24 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
 static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
-  const char
-    *attribute;
+  char
+    keyword[MaxTextExtent],
+    *token;
 
   ConfigureInfo
-    *configure_info = (ConfigureInfo *) NULL;
+    *configure_info;
+
+  const char
+    *q;
 
   MagickBooleanType
     status;
 
-  XMLTreeInfo
-    *configure,
-    *configure_map,
-    *include;
-
   /*
     Load the configure map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading configure map \"%s\" ...",filename);
-  if (xml == (const char *) NULL)
-    return(MagickFalse);
+    "Loading configure file \"%s\" ...",filename);
   if (configure_list == (LinkedListInfo *) NULL)
     {
       configure_list=NewLinkedList(0);
@@ -937,75 +989,148 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
-  configure_map=NewXMLTree(xml,exception);
-  if (configure_map == (XMLTreeInfo *) NULL)
-    return(MagickFalse);
   status=MagickTrue;
-  include=GetXMLTreeChild(configure_map,"include");
-  while (include != (XMLTreeInfo *) NULL)
+  configure_info=(ConfigureInfo *) NULL;
+  if (xml == (char *) NULL)
+    token=AcquireString(ConfigureMap);
+  else
+    token=AcquireString((char *) xml);
+  for (q=(char *) xml; *q != '\0'; )
   {
     /*
-      Process include element.
+      Interpret XML.
     */
-    attribute=GetXMLTreeAttribute(include,"file");
-    if (attribute != (const char *) NULL)
+    GetMagickToken(q,&q,token);
+    if (*token == '\0')
+      break;
+    (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
       {
-        if (depth > 200)
-          (void) ThrowMagickException(exception,GetMagickModule(),
-            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",filename);
-        else
-          {
-            char
-              path[MaxTextExtent],
-              *xml;
-
-            GetPathComponent(filename,HeadPath,path);
-            if (*path != '\0')
-              (void) ConcatenateMagickString(path,DirectorySeparator,
-                MaxTextExtent);
-            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
-            xml=FileToString(path,~0,exception);
-            if (xml != (char *) NULL)
-              {
-                status=LoadConfigureList(xml,path,depth+1,exception);
-                xml=DestroyString(xml);
-              }
-          }
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
       }
-    include=GetNextXMLTreeTag(include);
-  }
-  configure=GetXMLTreeChild(configure_map,"configure");
-  while (configure != (XMLTreeInfo *) NULL)
-  {
-    const char
-      *attribute;
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
+      {
+        /*
+          Comment element.
+        */
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
+    if (LocaleCompare(keyword,"<include") == 0)
+      {
+        /*
+          Include element.
+        */
+        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
+        {
+          (void) CopyMagickString(keyword,token,MaxTextExtent);
+          GetMagickToken(q,&q,token);
+          if (*token != '=')
+            continue;
+          GetMagickToken(q,&q,token);
+          if (LocaleCompare(keyword,"file") == 0)
+            {
+              if (depth > 200)
+                (void) ThrowMagickException(exception,GetMagickModule(),
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
+              else
+                {
+                  char
+                    path[MaxTextExtent],
+                    *xml;
 
-    /*
-      Process configure element.
-    */
-    configure_info=(ConfigureInfo *) AcquireMagickMemory(
-      sizeof(*configure_info));
+                  GetPathComponent(filename,HeadPath,path);
+                  if (*path != '\0')
+                    (void) ConcatenateMagickString(path,DirectorySeparator,
+                      MaxTextExtent);
+                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  xml=FileToString(path,~0,exception);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadConfigureList(xml,path,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
+                }
+            }
+        }
+        continue;
+      }
+    if (LocaleCompare(keyword,"<configure") == 0)
+      {
+        /*
+          Configure element.
+        */
+        configure_info=(ConfigureInfo *) AcquireMagickMemory(
+          sizeof(*configure_info));
+        if (configure_info == (ConfigureInfo *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+        (void) ResetMagickMemory(configure_info,0,sizeof(*configure_info));
+        configure_info->path=ConstantString(filename);
+        configure_info->signature=MagickSignature;
+        continue;
+      }
     if (configure_info == (ConfigureInfo *) NULL)
-      ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-    (void) ResetMagickMemory(configure_info,0,sizeof(*configure_info));
-    configure_info->path=ConstantString(filename);
-    configure_info->signature=MagickSignature;
-    attribute=GetXMLTreeAttribute(configure,"name");
-    if (attribute != (const char *) NULL)
-      configure_info->name=ConstantString(attribute);
-    attribute=GetXMLTreeAttribute(configure,"stealth");
-    if (attribute != (const char *) NULL)
-      configure_info->stealth=IsMagickTrue(attribute);
-    attribute=GetXMLTreeAttribute(configure,"value");
-    if (attribute != (const char *) NULL)
-      configure_info->value=ConstantString(attribute);
-    status=AppendValueToLinkedList(configure_list,configure_info);
-    if (status == MagickFalse)
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
-    configure=GetNextXMLTreeTag(configure);
+      continue;
+    if (LocaleCompare(keyword,"/>") == 0)
+      {
+        status=AppendValueToLinkedList(configure_list,configure_info);
+        if (status == MagickFalse)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            configure_info->name);
+        configure_info=(ConfigureInfo *) NULL;
+      }
+    /*
+      Parse configure element.
+    */
+    GetMagickToken(q,(const char **) NULL,token);
+    if (*token != '=')
+      continue;
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    switch (*keyword)
+    {
+      case 'N':
+      case 'n':
+      {
+        if (LocaleCompare((char *) keyword,"name") == 0)
+          {
+            configure_info->name=ConstantString(token);
+            break;
+          }
+        break;
+      }
+      case 'S':
+      case 's':
+      {
+        if (LocaleCompare((char *) keyword,"stealth") == 0)
+          {
+            configure_info->stealth=IsMagickTrue(token);
+            break;
+          }
+        break;
+      }
+      case 'V':
+      case 'v':
+      {
+        if (LocaleCompare((char *) keyword,"value") == 0)
+          {
+            configure_info->value=ConstantString(token);
+            break;
+          }
+        break;
+      }
+      default:
+        break;
+    }
   }
-  configure_map=DestroyXMLTree(configure_map);
+  token=(char *) RelinquishMagickMemory(token);
   return(status);
 }
 
@@ -1030,7 +1155,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: the font file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
