@@ -10,7 +10,7 @@
 %     CCCC   OOO   N   N  SSSSS    T    IIIII    T     UUU     T    EEEEE     %
 %                                                                             %
 %                                                                             %
-%                       Methods to Consitute an Image                         %
+%                  MagickCore Methods to Consitute an Image                   %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
@@ -73,12 +73,6 @@
 static SemaphoreInfo
   *constitute_semaphore = (SemaphoreInfo *) NULL;
 
-/*
-  Forward declarations.
-*/
-static Image
-  *ReadImages(const ImageInfo *,ExceptionInfo *);
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -90,7 +84,7 @@ static Image
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ConstituteImage() returns an image from the the pixel data you supply.
+%  ConstituteImage() returns an image from the pixel data you supply.
 %  The pixel data must be in scanline order top-to-bottom.  The data can be
 %  char, short int, int, float, or double.  Float and double require the
 %  pixels to be normalized [0..1], otherwise [0..QuantumRange].  For example, to
@@ -146,7 +140,7 @@ MagickExport Image *ConstituteImage(const unsigned long columns,
   assert(pixels != (void *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage((ImageInfo *) NULL);
+  image=AcquireImage((ImageInfo *) NULL);
   if (image == (Image *) NULL)
     return((Image *) NULL);
   if ((columns == 0) || (rows == 0))
@@ -184,7 +178,7 @@ MagickExport Image *ConstituteImage(const unsigned long columns,
 MagickExport void DestroyConstitute(void)
 {
   if (constitute_semaphore != (SemaphoreInfo *) NULL)
-    constitute_semaphore=DestroySemaphoreInfo(constitute_semaphore);
+    DestroySemaphoreInfo(&constitute_semaphore);
 }
 
 /*
@@ -263,6 +257,126 @@ MagickExport Image *PingImage(const ImageInfo *image_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P i n g I m a g e s                                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PingImages() pings one or more images and returns them as an image list.
+%
+%  The format of the PingImage method is:
+%
+%      Image *PingImages(const ImageInfo *image_info,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image_info: the image info.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+*/
+MagickExport Image *PingImages(const ImageInfo *image_info,
+  ExceptionInfo *exception)
+{
+  char
+    filename[MaxTextExtent];
+
+  Image
+    *image,
+    *images;
+
+  ImageInfo
+    *read_info;
+
+  /*
+    Ping image list from a file.
+  */
+  assert(image_info != (ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  if (*image_info->filename == '@')
+    {
+      char
+        *command,
+        **image_list;
+
+      int
+        number_image_list;
+
+      register long
+        i;
+
+      /*
+        A file with a list of image filenames to read.
+      */
+      command=FileToString(image_info->filename+1,~0,exception);
+      if (command == (char *) NULL)
+        return((Image *) NULL);
+      StripString(command);
+      image_list=StringToArgv(command,&number_image_list);
+      command=DestroyString(command);
+      read_info=CloneImageInfo(image_info);
+      images=NewImageList();
+      for (i=1; i < number_image_list; i++)
+      {
+        (void) CopyMagickString(read_info->filename,image_list[i],
+          MaxTextExtent);
+        image=PingImage(read_info,exception);
+        if (image == (Image *) NULL)
+          continue;
+        AppendImageToList(&images,image);
+      }
+      read_info=DestroyImageInfo(read_info);
+      for (i=1; i < number_image_list; i++)
+        image_list[i]=DestroyString(image_list[i]);
+      image_list=(char **) RelinquishMagickMemory(image_list);
+      return(images);
+    }
+  (void) InterpretImageFilename(filename,MaxTextExtent,image_info->filename,
+    (int) image_info->scene);
+  if (LocaleCompare(filename,image_info->filename) != 0)
+    {
+      ExceptionInfo
+        *sans;
+
+      long
+        extent,
+        scene;
+
+      /*
+        Images of the form image-%d.png[1-5].
+      */
+      read_info=CloneImageInfo(image_info);
+      sans=AcquireExceptionInfo();
+      (void) SetImageInfo(read_info,MagickFalse,sans);
+      sans=DestroyExceptionInfo(sans);
+      (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
+      images=NewImageList();
+      extent=(long) (read_info->scene+read_info->number_scenes);
+      for (scene=read_info->scene; scene < (long) extent; scene++)
+      {
+        (void) InterpretImageFilename(read_info->filename,MaxTextExtent,
+          filename,(int) scene);
+        image=PingImage(read_info,exception);
+        if (image == (Image *) NULL)
+          continue;
+        AppendImageToList(&images,image);
+      }
+      read_info=DestroyImageInfo(read_info);
+      return(images);
+    }
+  return(PingImage(image_info,exception));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e a d I m a g e                                                         %
 %                                                                             %
 %                                                                             %
@@ -317,7 +431,8 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     *read_info;
 
   MagickStatusType
-    flags;
+    flags,
+    thread_support;
 
   /*
     Determine image type from filename prefix or suffix (e.g. image.jpg).
@@ -329,8 +444,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  if (*image_info->filename == '@')
-    return(ReadImages(image_info,exception));
   read_info=CloneImageInfo(image_info);
   (void) CopyMagickString(magick_filename,read_info->filename,MaxTextExtent);
   (void) SetImageInfo(read_info,MagickFalse,exception);
@@ -341,6 +454,22 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   */
   GetExceptionInfo(&sans_exception);
   magick_info=GetMagickInfo(read_info->magick,&sans_exception);
+  if (magick_info != (const MagickInfo *) NULL)
+    {
+      if (GetMagickEndianSupport(magick_info) == MagickFalse)
+        read_info->endian=UndefinedEndian;
+      else
+        if ((image_info->endian == UndefinedEndian) &&
+            (GetMagickRawSupport(magick_info) != MagickFalse))
+          {
+            unsigned long
+              lsb_first;
+  
+            lsb_first=1;
+            read_info->endian=(*(char *) &lsb_first) == 1 ? LSBEndian :
+              MSBEndian;
+         }
+    }
   (void) DestroyExceptionInfo(&sans_exception);
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetMagickSeekableStream(magick_info) != MagickFalse))
@@ -348,7 +477,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       MagickBooleanType
         status;
 
-      image=AllocateImage(read_info);
+      image=AcquireImage(read_info);
       (void) CopyMagickString(image->filename,read_info->filename,
         MaxTextExtent);
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
@@ -381,10 +510,11 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetImageDecoder(magick_info) != (DecodeImageHandler *) NULL))
     {
-      if ((GetMagickThreadSupport(magick_info) & DecoderThreadSupport) == 0)
+      thread_support=GetMagickThreadSupport(magick_info);      
+      if ((thread_support & DecoderThreadSupport) == 0)
         AcquireSemaphoreInfo(&constitute_semaphore);
       image=GetImageDecoder(magick_info)(read_info,exception);
-      if ((GetMagickThreadSupport(magick_info) & DecoderThreadSupport) == 0)
+      if ((thread_support & DecoderThreadSupport) == 0)
         RelinquishSemaphoreInfo(constitute_semaphore);
     }
   else
@@ -392,7 +522,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       delegate_info=GetDelegateInfo(read_info->magick,(char *) NULL,exception);
       if (delegate_info == (const DelegateInfo *) NULL)
         {
-          if (IsAccessible(read_info->filename) != MagickFalse)
+          if (IsPathAccessible(read_info->filename) != MagickFalse)
             (void) ThrowMagickException(exception,GetMagickModule(),
               MissingDelegateError,"NoDecodeDelegateForThisImageFormat","`%s'",
               read_info->filename);
@@ -404,7 +534,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       /*
         Let our decoding delegate process the image.
       */
-      image=AllocateImage(read_info);
+      image=AcquireImage(read_info);
       if (image == (Image *) NULL)
         {
           read_info=DestroyImageInfo(read_info);
@@ -426,20 +556,21 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       if ((magick_info == (const MagickInfo *) NULL) ||
           (GetImageDecoder(magick_info) == (DecodeImageHandler *) NULL))
         {
-          if (IsAccessible(read_info->filename) != MagickFalse)
+          if (IsPathAccessible(read_info->filename) != MagickFalse)
             (void) ThrowMagickException(exception,GetMagickModule(),
               MissingDelegateError,"NoDecodeDelegateForThisImageFormat","`%s'",
               read_info->filename);
           else
-            ThrowFileException(exception,FileOpenError,"UnableToOpenBlob",
+            ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
               read_info->filename);
           read_info=DestroyImageInfo(read_info);
           return((Image *) NULL);
         }
-      if ((GetMagickThreadSupport(magick_info) & DecoderThreadSupport) == 0)
+      thread_support=GetMagickThreadSupport(magick_info);      
+      if ((thread_support & DecoderThreadSupport) == 0)
         AcquireSemaphoreInfo(&constitute_semaphore);
       image=(Image *) (GetImageDecoder(magick_info))(read_info,exception);
-      if ((GetMagickThreadSupport(magick_info) & DecoderThreadSupport) == 0)
+      if ((thread_support & DecoderThreadSupport) == 0)
         RelinquishSemaphoreInfo(constitute_semaphore);
     }
   if (read_info->temporary != MagickFalse)
@@ -486,6 +617,9 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     }
   for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
   {
+    char
+      timestamp[MaxTextExtent];
+
     const StringInfo
       *profile;
 
@@ -494,7 +628,9 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       next->magick_columns=next->columns;
     if (next->magick_rows == 0)
       next->magick_rows=next->rows;
-    (void) CopyMagickString(next->magick,magick,MaxTextExtent);
+    if ((LocaleCompare(magick,"HTTP") != 0) &&
+        (LocaleCompare(magick,"FTP") != 0))
+      (void) CopyMagickString(next->magick,magick,MaxTextExtent);
     (void) CopyMagickString(next->magick_filename,magick_filename,
       MaxTextExtent);
     if (IsBlobTemporary(image) != MagickFalse)
@@ -544,7 +680,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       next->page.width=next->columns;
     if (next->page.height == 0)
       next->page.height=next->rows;
-    (void) SyncImageOptions(read_info,next);
     if (*read_info->filename != '\0')
       {
         const char
@@ -603,17 +738,23 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       profile=GetImageProfile(next,"icm");
     if (profile != (const StringInfo *) NULL)
       {
-        image->color_profile.length=GetStringInfoLength(profile);
-        image->color_profile.info=GetStringInfoDatum(profile);
+        next->color_profile.length=GetStringInfoLength(profile);
+        next->color_profile.info=GetStringInfoDatum(profile);
       }
     profile=GetImageProfile(next,"iptc");
     if (profile == (const StringInfo *) NULL)
       profile=GetImageProfile(next,"8bim");
     if (profile != (const StringInfo *) NULL)
       {
-        image->iptc_profile.length=GetStringInfoLength(profile);
-        image->iptc_profile.info=GetStringInfoDatum(profile);
+        next->iptc_profile.length=GetStringInfoLength(profile);
+        next->iptc_profile.info=GetStringInfoDatum(profile);
       }
+    (void) FormatMagickTime(GetBlobProperties(next)->st_mtime,MaxTextExtent,
+      timestamp);
+    (void) SetImageProperty(next,"modify-date",timestamp);
+    (void) FormatMagickTime(GetBlobProperties(next)->st_ctime,MaxTextExtent,
+      timestamp);
+    (void) SetImageProperty(next,"create-date",timestamp);
     if (read_info->verbose != MagickFalse)
       (void) IdentifyImage(next,stdout,MagickFalse);
     image=next;
@@ -627,14 +768,13 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   R e a d I m a g e s                                                       %
+%   R e a d I m a g e s                                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadImages() reads a list of image names from a file and then returns the
-%  images as a linked list.
+%  ReadImages() reads one or more images and returns them as an image list.
 %
 %  The format of the ReadImage method is:
 %
@@ -642,38 +782,24 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image: Method ReadImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: The list of filenames are defined in the filename member of
-%      this structure.
+%    o image_info: the image info.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
-static Image *ReadImages(const ImageInfo *image_info,ExceptionInfo *exception)
+MagickExport Image *ReadImages(const ImageInfo *image_info,
+  ExceptionInfo *exception)
 {
   char
-    *command,
-    **images;
+    filename[MaxTextExtent];
 
   Image
-    *image;
+    *image,
+    *images;
 
   ImageInfo
     *read_info;
 
-  int
-    number_images;
-
-  register Image
-    *next;
-
-  register long
-    i;
-
   /*
     Read image list from a file.
   */
@@ -683,30 +809,78 @@ static Image *ReadImages(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  command=FileToString(image_info->filename+1,~0,exception);
-  if (command == (char *) NULL)
-    return((Image *) NULL);
-  StripString(command);
-  images=StringToArgv(command,&number_images);
-  command=DestroyString(command);
-  /*
-    Read the images into a linked list.
-  */
-  read_info=CloneImageInfo(image_info);
-  image=NewImageList();
-  for (i=1; i < number_images; i++)
-  {
-    (void) CopyMagickString(read_info->filename,images[i],MaxTextExtent);
-    next=ReadImage(read_info,exception);
-    if (next == (Image *) NULL)
-      continue;
-    AppendImageToList(&image,next);
-  }
-  read_info=DestroyImageInfo(read_info);
-  for (i=1; i < number_images; i++)
-    images[i]=DestroyString(images[i]);
-  images=(char **) RelinquishMagickMemory(images);
-  return(image);
+  if (*image_info->filename == '@')
+    {
+      char
+        *command,
+        **image_list;
+
+      int
+        number_image_list;
+
+      register long
+        i;
+
+      /*
+        A file with a list of image filenames to read.
+      */
+      command=FileToString(image_info->filename+1,~0,exception);
+      if (command == (char *) NULL)
+        return((Image *) NULL);
+      StripString(command);
+      image_list=StringToArgv(command,&number_image_list);
+      command=DestroyString(command);
+      read_info=CloneImageInfo(image_info);
+      images=NewImageList();
+      for (i=1; i < number_image_list; i++)
+      {
+        (void) CopyMagickString(read_info->filename,image_list[i],
+          MaxTextExtent);
+        image=ReadImage(read_info,exception);
+        if (image == (Image *) NULL)
+          continue;
+        AppendImageToList(&images,image);
+      }
+      read_info=DestroyImageInfo(read_info);
+      for (i=1; i < number_image_list; i++)
+        image_list[i]=DestroyString(image_list[i]);
+      image_list=(char **) RelinquishMagickMemory(image_list);
+      return(images);
+    }
+  (void) InterpretImageFilename(filename,MaxTextExtent,image_info->filename,
+    (int) image_info->scene);
+  if (LocaleCompare(filename,image_info->filename) != 0)
+    {
+      ExceptionInfo
+        *sans;
+
+      long
+        extent,
+        scene;
+
+      /*
+        Images of the form image-%d.png[1-5].
+      */
+      read_info=CloneImageInfo(image_info);
+      sans=AcquireExceptionInfo();
+      (void) SetImageInfo(read_info,MagickFalse,sans);
+      sans=DestroyExceptionInfo(sans);
+      (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
+      images=NewImageList();
+      extent=(long) (read_info->scene+read_info->number_scenes);
+      for (scene=read_info->scene; scene < (long) extent; scene++)
+      {
+        (void) InterpretImageFilename(read_info->filename,MaxTextExtent,
+          filename,(int) scene);
+        image=ReadImage(read_info,exception);
+        if (image == (Image *) NULL)
+          continue;
+        AppendImageToList(&images,image);
+      }
+      read_info=DestroyImageInfo(read_info);
+      return(images);
+    }
+  return(ReadImage(image_info,exception));
 }
 
 /*
@@ -732,9 +906,9 @@ static Image *ReadImages(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o content: The image encoded in Base64.
+%    o content: the image encoded in Base64.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -758,14 +932,14 @@ MagickExport Image *ReadInlineImage(const ImageInfo *image_info,
     *p;
 
   image=NewImageList();
-  for (p=content; (*p != ',') && (*p != '\0'); p++);
+  for (p=content; (*p != ',') && (*p != '\0'); p++) ;
   if (*p == '\0')
-    ThrowReaderException(CorruptImageWarning,"CorruptImage");
+    ThrowReaderException(CorruptImageError,"CorruptImage");
   p++;
   length=0;
   blob=Base64Decode(p,&length);
   if (length == 0)
-    ThrowReaderException(CorruptImageWarning,"CorruptImage");
+    ThrowReaderException(CorruptImageError,"CorruptImage");
   read_info=CloneImageInfo(image_info);
   (void) SetImageInfoProgressMonitor(read_info,(MagickProgressMonitor) NULL,
     (void *) NULL);
@@ -798,49 +972,11 @@ MagickExport Image *ReadInlineImage(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
-
-static void SyncImageInfo(ImageInfo *image_info,Image *image,
-  ExceptionInfo *exception)
-{
-  const MagickInfo
-    *magick_info;
-
-  QuantumInfo
-    quantum_info;
-
-  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
-  if (image_info->extract != (char *) NULL)
-    (void) ParseAbsoluteGeometry(image_info->extract,&image->extract_info);
-  if (image_info->depth != 0)
-    (void) SetImageDepth(image,image_info->depth);
-  else
-    if (image->taint != MagickFalse)
-      image->depth=MAGICKCORE_QUANTUM_DEPTH;
-  if (image_info->compression != UndefinedCompression)
-    image->compression=image_info->compression;
-  if (image_info->quality != UndefinedCompressionQuality)
-    image->quality=image_info->quality;
-  if (image_info->interlace != UndefinedInterlace)
-    image->interlace=image_info->interlace;
-  if (image_info->orientation != UndefinedOrientation)
-    image->orientation=image_info->orientation;
-  if (image_info->endian != UndefinedEndian)
-    image->endian=image_info->endian;
-  magick_info=GetMagickInfo(image_info->magick,exception);
-  if ((magick_info == (const MagickInfo *) NULL) ||
-      (GetMagickEndianSupport(magick_info) == MagickFalse))
-    image->endian=UndefinedEndian;
-  GetQuantumInfo(image_info,&quantum_info);
-  if ((quantum_info.format == UndefinedQuantumFormat) &&
-      (IsHighDynamicRangeImage(image,exception) != MagickFalse))
-    (void) SetImageOption(image_info,"quantum:format","floating-point");
-}
-
 MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
   Image *image)
 {
@@ -863,6 +999,9 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
     status,
     temporary;
 
+  MagickStatusType
+    thread_support;
+
   /*
     Determine image type from filename prefix or suffix (e.g. image.jpg).
   */
@@ -891,18 +1030,34 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       (void) SetImageInfo(write_info,MagickTrue,sans_exception);
     }
   (void) CopyMagickString(filename,image->filename,MaxTextExtent);
-  SyncImageInfo(write_info,image,sans_exception);
+  (void) CopyMagickString(image->filename,write_info->filename,MaxTextExtent);
+  magick_info=GetMagickInfo(write_info->magick,sans_exception);
+  if (magick_info != (const MagickInfo *) NULL)
+    {
+      if (GetMagickEndianSupport(magick_info) == MagickFalse)
+        image->endian=UndefinedEndian;
+      else
+        if ((image_info->endian == UndefinedEndian) &&
+            (GetMagickRawSupport(magick_info) != MagickFalse))
+          {
+            unsigned long
+              lsb_first;
+  
+            lsb_first=1;
+            image->endian=(*(char *) &lsb_first) == 1 ? LSBEndian : MSBEndian;
+         }
+    }
   (void) SyncImageProfiles(image);
-  if ((GetPreviousImageInList(image) == (Image *) NULL) &&
+  if ((write_info->page == (char *) NULL) &&
+      (GetPreviousImageInList(image) == (Image *) NULL) &&
       (GetNextImageInList(image) == (Image *) NULL) &&
-      (write_info->page == (char *) NULL) &&
       (IsTaintImage(image) == MagickFalse))
     {
       delegate_info=GetDelegateInfo(image->magick,write_info->magick,
         &image->exception);
       if ((delegate_info != (const DelegateInfo *) NULL) &&
           (GetDelegateMode(delegate_info) == 0) &&
-          (IsAccessible(image->magick_filename) != MagickFalse))
+          (IsPathAccessible(image->magick_filename) != MagickFalse))
         {
           /*
             Process image with bi-modal delegate.
@@ -919,7 +1074,6 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
     }
   status=MagickFalse;
   temporary=MagickFalse;
-  magick_info=GetMagickInfo(write_info->magick,sans_exception);
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetMagickSeekableStream(magick_info) != MagickFalse))
     {
@@ -950,10 +1104,11 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       /*
         Call appropriate image writer based on image type.
       */
-      if ((GetMagickThreadSupport(magick_info) & EncoderThreadSupport) == 0)
+      thread_support=GetMagickThreadSupport(magick_info);      
+      if ((thread_support & EncoderThreadSupport) == 0)
         AcquireSemaphoreInfo(&constitute_semaphore);
       status=GetImageEncoder(magick_info)(write_info,image);
-      if ((GetMagickThreadSupport(magick_info) & EncoderThreadSupport) == 0)
+      if ((thread_support & EncoderThreadSupport) == 0)
         RelinquishSemaphoreInfo(constitute_semaphore);
     }
   else
@@ -994,10 +1149,11 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
               /*
                 Call appropriate image writer based on image type.
               */
-              if ((GetMagickThreadSupport(magick_info) & EncoderThreadSupport) == 0)
+              thread_support=GetMagickThreadSupport(magick_info);      
+              if ((thread_support & EncoderThreadSupport) == 0)
                 AcquireSemaphoreInfo(&constitute_semaphore);
               status=GetImageEncoder(magick_info)(write_info,image);
-              if ((GetMagickThreadSupport(magick_info) & EncoderThreadSupport) == 0)
+              if ((thread_support & EncoderThreadSupport) == 0)
                 RelinquishSemaphoreInfo(constitute_semaphore);
             }
         }
@@ -1046,11 +1202,11 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o images: The image list.
+%    o images: the image list.
 %
-%    o filename: The image filename.
+%    o filename: the image filename.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -1058,6 +1214,9 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
 MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   Image *images,const char *filename,ExceptionInfo *exception)
 {
+  BlobInfo
+    *blob;
+
   ExceptionInfo
     *sans_exception;
 
@@ -1070,9 +1229,6 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   register Image
     *p;
 
-  /*
-    Write converted imagess.
-  */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(images != (Image *) NULL);
@@ -1082,6 +1238,9 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   write_info=CloneImageInfo(image_info);
   images=GetFirstImageInList(images);
+  blob=CloneBlobInfo(images->blob);  /* thread specific I/O handler */
+  DestroyBlob(images);
+  images->blob=blob;
   if (filename != (const char *) NULL)
     for (p=images; p != (Image *) NULL; p=GetNextImageInList(p))
       (void) CopyMagickString(p->filename,filename,MaxTextExtent);
@@ -1101,11 +1260,14 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
         /*
           Generate consistent scene numbers.
         */
-        i=0;
+        i=(long) images->scene;
         for (p=images; p != (Image *) NULL; p=GetNextImageInList(p))
           p->scene=(unsigned long) i++;
         break;
       }
+  /*
+    Write images.
+  */
   status=MagickTrue;
   for (p=images; p != (Image *) NULL; p=GetNextImageInList(p))
   {
