@@ -10,7 +10,7 @@
 %            DDDD   EEEEE   CCCC   OOO   R  R   A   A    T    EEEEE           %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Decoration Methods                    %
+%                     MagickCore Image Decoration Methods                     %
 %                                                                             %
 %                                Software Design                              %
 %                                  John Cristy                                %
@@ -41,6 +41,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/cache-view.h"
 #include "magick/color-private.h"
 #include "magick/colorspace-private.h"
 #include "magick/composite.h"
@@ -50,6 +51,7 @@
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/pixel-private.h"
 #include "magick/quantum.h"
 
@@ -85,7 +87,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o border_info:  Define the width and height of the border.
 %
@@ -148,7 +150,7 @@ MagickExport Image *BorderImage(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o frame_info: Define the width and height of the frame and its bevels.
 %
@@ -164,6 +166,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     *frame_image;
 
   long
+    progress,
     y;
 
   MagickBooleanType
@@ -192,6 +195,9 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     height,
     width;
 
+  ViewInfo
+    **frame_view;
+
   /*
     Check frame geometry.
   */
@@ -276,10 +282,13 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
       ConvertRGBToCMYK(&shadow);
       ConvertRGBToCMYK(&trough);
     }
+  status=MagickTrue;
+  progress=0;
+  frame_view=AcquireCacheViewThreadSet(frame_image);
   height=(unsigned long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
     frame_info->inner_bevel);
-  q=SetImagePixels(frame_image,0,0,frame_image->columns,height);
-  frame_indexes=GetIndexes(frame_image);
+  q=SetCacheViewPixels(frame_view[0],0,0,frame_image->columns,height);
+  frame_indexes=GetCacheViewIndexes(frame_view[0]);
   if (q != (PixelPacket *) NULL)
     {
       /*
@@ -369,20 +378,40 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
           frame_indexes++;
         }
       }
-      (void) SyncImagePixels(frame_image);
+      (void) SyncCacheViewPixels(frame_view[0]);
     }
   /*
     Draw sides of ornamental border.
   */
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *frame_indexes;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
     /*
       Initialize scanline with matte color.
     */
-    q=SetImagePixels(frame_image,0,frame_info->y+y,frame_image->columns,1);
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=SetCacheViewPixels(frame_view[id],0,frame_info->y+y,frame_image->columns,
+      1);
     if (q == (PixelPacket *) NULL)
-      break;
-    frame_indexes=GetIndexes(frame_image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    frame_indexes=GetCacheViewIndexes(frame_view[id]);
     for (x=0; x < (long) frame_info->outer_bevel; x++)
     {
       SetPixelPacket(frame_image,&highlight,q,frame_indexes);
@@ -429,20 +458,24 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
       q++;
       frame_indexes++;
     }
-    if (SyncImagePixels(frame_image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(frame_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(FrameImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,FrameImageTag,progress++,image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
   height=(unsigned long) (frame_info->inner_bevel+frame_info->height-
     frame_info->y-image->rows-bevel_width+frame_info->outer_bevel);
-  q=SetImagePixels(frame_image,0,(long) (frame_image->rows-height),
+  q=SetCacheViewPixels(frame_view[0],0,(long) (frame_image->rows-height),
     frame_image->columns,height);
   if (q != (PixelPacket *) NULL)
     {
@@ -534,8 +567,9 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
           frame_indexes++;
         }
       }
-      (void) SyncImagePixels(frame_image);
+      (void) SyncCacheViewPixels(frame_view[0]);
     }
+  frame_view=DestroyCacheViewThreadSet(frame_view);
   x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
     frame_info->inner_bevel);
   y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
@@ -567,7 +601,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o raise_info: Define the width and height of the raise area.
 %
@@ -584,6 +618,10 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
 #define RaiseImageTag  "Raise/Image"
 #define TroughFactor  ScaleCharToQuantum(135)
 
+  long
+    progress,
+    y;
+
   MagickBooleanType
     status;
 
@@ -591,14 +629,8 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     foreground,
     background;
 
-  long
-    y;
-
-  register long
-    x;
-
-  register PixelPacket
-    *q;
+  ViewInfo
+    **image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -618,11 +650,33 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     }
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
+  /*
+    Raise image.
+  */
+  status=MagickTrue;
+  progress=0;
+  image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) raise_info->height; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFalse;
+        continue;
+      }
     for (x=0; x < y; x++)
     {
       q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*
@@ -659,22 +713,39 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(RaiseImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+        proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  for ( ; y < (long) (image->rows-raise_info->height); y++)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
+  for (y=(long) raise_info->height; y < (long) (image->rows-raise_info->height); y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFalse;
+        continue;
+      }
     for (x=0; x < (long) raise_info->width; x++)
     {
       q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*
@@ -700,22 +771,39 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(RaiseImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+        proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  for ( ; y < (long) image->rows; y++)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
+  for (y=(long) (image->rows-raise_info->height); y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFalse;
+        continue;
+      }
     for (x=0; x < (long) (image->rows-y); x++)
     {
       q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*
@@ -749,16 +837,21 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(RaiseImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  return(MagickTrue);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  return(status);
 }
