@@ -44,6 +44,7 @@
 */
 #include "magick/studio.h"
 #include "magick/annotate.h"
+#include "magick/artifact.h"
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/coder.h"
@@ -78,6 +79,7 @@
 #include "magick/quantize.h"
 #include "magick/quantum.h"
 #include "magick/random_.h"
+#include "magick/registry.h"
 #include "magick/resize.h"
 #include "magick/resource_.h"
 #include "magick/signature.h"
@@ -115,9 +117,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o file: The file, typically stdout.
+%    o file: the file, typically stdout.
 %
 %    o verbose: A value other than zero prints more detailed information
 %      about the image.
@@ -126,9 +128,9 @@
 MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   const MagickBooleanType verbose)
 {
-#define IdentifyFormat "    %s:\n      Min: " QuantumFormat  \
-  " (%g)\n      Max: " QuantumFormat " (%g)\n"  \
-  "      Mean: %g (%g)\n      Standard deviation: %g (%g)\n"
+#define IdentifyFormat "    %s:\n      min: " QuantumFormat  \
+  " (%g)\n      max: " QuantumFormat " (%g)\n"  \
+  "      mean: %g (%g)\n      standard deviation: %g (%g)\n"
 
   char
     color[MaxTextExtent],
@@ -139,7 +141,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
     colorspace;
 
   const char
+    *artifact,
+    *name,
     *property,
+    *registry,
     *value;
 
   const MagickInfo
@@ -155,9 +160,6 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   ExceptionInfo
     *exception;
 
-  Image
-    *p;
-
   ImageType
     type;
 
@@ -208,6 +210,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           (image->page.x != 0) || (image->page.y != 0))
         (void) fprintf(file,"%lux%lu%+ld%+ld ",image->page.width,
           image->page.height,image->page.x,image->page.y);
+      (void) fprintf(file,"%lu-bit ",image->depth);
+      if (image->type != UndefinedType)
+        (void) fprintf(file,"%s ",MagickOptionToMnemonic(MagickTypeOptions,
+          (long) image->type));
       if (image->storage_class == DirectClass)
         {
           (void) fprintf(file,"DirectClass ");
@@ -223,10 +229,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         else
           (void) fprintf(file,"PseudoClass %lu=>%luc ",image->total_colors,
             image->colors);
-      (void) fprintf(file,"%lu-bit ",image->depth);
       if (image->error.mean_error_per_pixel != 0.0)
-        (void) fprintf(file,"%ld/%f/%fdb ",
-          (long) (image->error.mean_error_per_pixel+0.5),
+        (void) fprintf(file,"%ld/%f/%fdb ",(long)
+          (image->error.mean_error_per_pixel+0.5),
           image->error.normalized_mean_error,
           image->error.normalized_maximum_error);
       if (GetBlobSize(image) != 0)
@@ -235,8 +240,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           (void) fprintf(file,"%s ",format);
         }
       if (elapsed_time > 0.06)
-        (void) fprintf(file,"%0.3fu %ld:%02ld",user_time,
-          (long) (elapsed_time/60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
+        (void) fprintf(file,"%0.3fu %ld:%02ld",user_time,(long)
+          (elapsed_time/60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
       (void) fprintf(file,"\n");
       (void) fflush(file);
       return(ferror(file) != 0 ? MagickFalse : MagickTrue);
@@ -267,8 +272,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   else
     (void) fprintf(file,"  Format: %s (%s)\n",image->magick,
       GetMagickDescription(magick_info));
-  (void) fprintf(file,"  Class: %s\n",
-    MagickOptionToMnemonic(MagickClassOptions,(long) image->storage_class));
+  (void) fprintf(file,"  Class: %s\n",MagickOptionToMnemonic(MagickClassOptions,
+    (long) image->storage_class));
   (void) fprintf(file,"  Geometry: %lux%lu%+ld%+ld\n",image->columns,
     image->rows,image->tile_offset.x,image->tile_offset.y);
   if ((image->magick_columns != 0) || (image->magick_rows != 0))
@@ -276,77 +281,97 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         (image->magick_rows != image->rows))
       (void) fprintf(file,"  Base geometry: %lux%lu\n",image->magick_columns,
         image->magick_rows);
+  if ((image->x_resolution != 0.0) && (image->y_resolution != 0.0))
+    {
+      (void) fprintf(file,"  Resolution: %gx%g\n",image->x_resolution,
+        image->y_resolution);
+      (void) fprintf(file,"  Print size: %gx%g\n",(double) image->columns/
+        image->x_resolution,(double) image->rows/image->y_resolution);
+    }
+  (void) fprintf(file,"  Units: %s\n",MagickOptionToMnemonic(
+    MagickResolutionOptions,(long) image->units));
   (void) fprintf(file,"  Type: %s\n",MagickOptionToMnemonic(MagickTypeOptions,
     (long) type));
+  if (image->type != UndefinedType)
+    (void) fprintf(file,"  Base type: %s\n",MagickOptionToMnemonic(
+      MagickTypeOptions,(long) image->type));
   (void) fprintf(file,"  Endianess: %s\n",MagickOptionToMnemonic(
     MagickEndianOptions,(long) image->endian));
   /*
     Detail channel depth and extrema.
   */
-  colorspace=image->colorspace;
-  if (IsGrayImage(image,&image->exception) != MagickFalse)
-    colorspace=GRAYColorspace;
-  (void) fprintf(file,"  Colorspace: %s\n",
-    MagickOptionToMnemonic(MagickColorspaceOptions,(long) colorspace));
+  (void) fprintf(file,"  Colorspace: %s\n",MagickOptionToMnemonic(
+    MagickColorspaceOptions,(long) image->colorspace));
   if (ping == MagickFalse)
     {
       ChannelStatistics
         *channel_statistics;
 
-      (void) fprintf(file,"  Depth: %lu-bit\n",GetImageDepth(image,
-        &image->exception));
+      unsigned long
+        depth;
+
+      depth=GetImageDepth(image,&image->exception);
+      if (image->depth == depth)
+        (void) fprintf(file,"  Depth: %lu-bit\n",image->depth);
+      else
+        (void) fprintf(file,"  Depth: %lu/%lu-bit\n",image->depth,depth);
       channel_statistics=GetImageChannelStatistics(image,&image->exception);
       (void) fprintf(file,"  Channel depth:\n");
+      colorspace=image->colorspace;
+      if (IsGrayImage(image,&image->exception) != MagickFalse)
+        colorspace=GRAYColorspace;
       switch (colorspace)
       {
         case RGBColorspace:
         default:
         {
-          (void) fprintf(file,"    Red: %lu-bit\n",
+          (void) fprintf(file,"    red: %lu-bit\n",
             channel_statistics[RedChannel].depth);
-          (void) fprintf(file,"    Green: %lu-bit\n",
+          (void) fprintf(file,"    green: %lu-bit\n",
             channel_statistics[GreenChannel].depth);
-          (void) fprintf(file,"    Blue: %lu-bit\n",
+          (void) fprintf(file,"    blue: %lu-bit\n",
             channel_statistics[BlueChannel].depth);
           if (image->matte != MagickFalse)
-            (void) fprintf(file,"    Alpha: %lu-bit\n",
+            (void) fprintf(file,"    alpha: %lu-bit\n",
               channel_statistics[OpacityChannel].depth);
           break;
         }
         case CMYKColorspace:
         {
-          (void) fprintf(file,"    Cyan: %lu-bit\n",
+          (void) fprintf(file,"    cyan: %lu-bit\n",
             channel_statistics[CyanChannel].depth);
-          (void) fprintf(file,"    Magenta: %lu-bit\n",
+          (void) fprintf(file,"    magenta: %lu-bit\n",
             channel_statistics[MagentaChannel].depth);
-          (void) fprintf(file,"    Yellow: %lu-bit\n",
+          (void) fprintf(file,"    yellow: %lu-bit\n",
             channel_statistics[YellowChannel].depth);
-          (void) fprintf(file,"    Black: %lu-bit\n",
+          (void) fprintf(file,"    black: %lu-bit\n",
             channel_statistics[BlackChannel].depth);
           if (image->matte != MagickFalse)
-            (void) fprintf(file,"    Alpha: %lu-bit\n",
+            (void) fprintf(file,"    alpha: %lu-bit\n",
               channel_statistics[OpacityChannel].depth);
           break;
         }
         case GRAYColorspace:
         {
-          (void) fprintf(file,"    Gray: %lu-bit\n",
+          (void) fprintf(file,"    gray: %lu-bit\n",
             channel_statistics[GrayChannel].depth);
           if (image->matte != MagickFalse)
-            (void) fprintf(file,"    Alpha: %lu-bit\n",
+            (void) fprintf(file,"    alpha: %lu-bit\n",
               channel_statistics[OpacityChannel].depth);
           break;
         }
       }
-      scale=QuantumRange/((unsigned long) QuantumRange >> ((unsigned long)
-        MAGICKCORE_QUANTUM_DEPTH-channel_statistics[AllChannels].depth));
+      scale=1.0;
+      if (image->depth <= MAGICKCORE_QUANTUM_DEPTH)
+        scale=QuantumRange/((unsigned long) QuantumRange >> ((unsigned long)
+          MAGICKCORE_QUANTUM_DEPTH-image->depth));
       (void) fprintf(file,"  Channel statistics:\n");
       switch (colorspace)
       {
         case RGBColorspace:
         default:
         {
-          (void) fprintf(file,IdentifyFormat,"Red",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"red",(Quantum)
             (channel_statistics[RedChannel].minima/scale),(double)
             channel_statistics[RedChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[RedChannel].maxima/scale),(double)
@@ -356,7 +381,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[RedChannel].standard_deviation/(double) scale,
             channel_statistics[RedChannel].standard_deviation/(double)
             QuantumRange);
-          (void) fprintf(file,IdentifyFormat,"Green",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"green",(Quantum)
             (channel_statistics[GreenChannel].minima/scale),(double)
             channel_statistics[GreenChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[GreenChannel].maxima/scale),(double)
@@ -366,7 +391,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[GreenChannel].standard_deviation/(double) scale,
             channel_statistics[GreenChannel].standard_deviation/(double)
             QuantumRange);
-          (void) fprintf(file,IdentifyFormat,"Blue",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"blue",(Quantum)
             (channel_statistics[BlueChannel].minima/scale),(double)
             channel_statistics[BlueChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[BlueChannel].maxima/scale),(double)
@@ -380,7 +405,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         }
         case CMYKColorspace:
         {
-          (void) fprintf(file,IdentifyFormat,"Cyan",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"cyan",(Quantum)
             (channel_statistics[CyanChannel].minima/scale),(double)
             channel_statistics[CyanChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[CyanChannel].maxima/scale),(double)
@@ -390,7 +415,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[CyanChannel].standard_deviation/(double) scale,
             channel_statistics[CyanChannel].standard_deviation/(double)
             QuantumRange);
-          (void) fprintf(file,IdentifyFormat,"Magenta",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"magenta",(Quantum)
             (channel_statistics[MagentaChannel].minima/scale),(double)
             channel_statistics[MagentaChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[MagentaChannel].maxima/scale),(double)
@@ -400,7 +425,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[MagentaChannel].standard_deviation/(double)
             scale,channel_statistics[MagentaChannel].standard_deviation/(double)
             QuantumRange);
-          (void) fprintf(file,IdentifyFormat,"Yellow",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"yellow",(Quantum)
             (channel_statistics[YellowChannel].minima/scale),(double)
             channel_statistics[YellowChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[YellowChannel].maxima/scale),(double)
@@ -410,7 +435,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             channel_statistics[YellowChannel].standard_deviation/(double) scale,
             channel_statistics[YellowChannel].standard_deviation/(double)
             QuantumRange);
-          (void) fprintf(file,IdentifyFormat,"Black",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"black",(Quantum)
             (channel_statistics[BlackChannel].minima/scale),(double)
             channel_statistics[BlackChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[BlackChannel].maxima/scale),(double)
@@ -424,7 +449,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         }
         case GRAYColorspace:
         {
-          (void) fprintf(file,IdentifyFormat,"Gray",(Quantum)
+          (void) fprintf(file,IdentifyFormat,"gray",(Quantum)
             (channel_statistics[GrayChannel].minima/scale),(double)
             channel_statistics[GrayChannel].minima/(double) QuantumRange,
             (Quantum) (channel_statistics[GrayChannel].maxima/scale),(double)
@@ -438,23 +463,24 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         }
       }
       if (image->matte != MagickFalse)
-        (void) fprintf(file,IdentifyFormat,"Opacity",(Quantum)
-          (channel_statistics[OpacityChannel].minima/scale),(double)
-          channel_statistics[OpacityChannel].minima/(double) QuantumRange,
-          (Quantum) (channel_statistics[OpacityChannel].maxima/scale),(double)
-          channel_statistics[OpacityChannel].maxima/(double) QuantumRange,
-          channel_statistics[OpacityChannel].mean/(double) scale,
-          channel_statistics[OpacityChannel].mean/(double) QuantumRange,
-          channel_statistics[OpacityChannel].standard_deviation/(double) scale,
-          channel_statistics[OpacityChannel].standard_deviation/(double)
+        (void) fprintf(file,IdentifyFormat,"alpha",(Quantum)
+          ((QuantumRange-channel_statistics[AlphaChannel].maxima)/scale),
+          (double) (QuantumRange-channel_statistics[AlphaChannel].maxima)/
+          (double) QuantumRange, (Quantum) ((QuantumRange-
+          channel_statistics[AlphaChannel].minima)/scale),(double)
+          (QuantumRange-channel_statistics[AlphaChannel].minima)/(double)
+          QuantumRange,(QuantumRange-channel_statistics[AlphaChannel].mean)/
+          (double) scale,(QuantumRange-channel_statistics[AlphaChannel].mean)/
+          (double) QuantumRange,
+          channel_statistics[AlphaChannel].standard_deviation/(double) scale,
+          channel_statistics[AlphaChannel].standard_deviation/(double)
           QuantumRange);
       channel_statistics=(ChannelStatistics *) RelinquishMagickMemory(
         channel_statistics);
-      if (colorspace == CMYKColorspace)
+      if (image->colorspace == CMYKColorspace)
         (void) fprintf(file,"  Total ink density: %.0f%%\n",100.0*
           GetImageTotalInkDensity(image)/(double) QuantumRange);
       x=0;
-      p=NewImageList();
       if (image->matte != MagickFalse)
         {
           register const IndexPacket
@@ -490,19 +516,25 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
 
               GetMagickPixelPacket(image,&pixel);
               SetMagickPixelPacket(image,p,indexes+x,&pixel);
-              (void) QueryMagickColorname(image,&pixel,SVGCompliance,
-                MagickFalse,tuple,&image->exception);
-              (void) fprintf(file,"  Alpha: %s  ",tuple);
-              (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickTrue,
-                tuple,&image->exception);
+              (void) QueryMagickColorname(image,&pixel,SVGCompliance,tuple,
+                &image->exception);
+              (void) fprintf(file,"  Alpha: %s ",tuple);
+              GetColorTuple(&pixel,MagickTrue,tuple);
               (void) fprintf(file,"  %s\n",tuple);
             }
         }
-      if ((ping == MagickFalse) &&
-          (IsHistogramImage(image,&image->exception) != MagickFalse))
+      if (ping == MagickFalse)
         {
-          (void) fprintf(file,"  Histogram:\n");
-          (void) GetNumberColors(image,file,&image->exception);
+          artifact=GetImageArtifact(image,"identify:unique");
+          if ((artifact != (const char *) NULL) &&
+              (IsMagickTrue(artifact) != MagickFalse))
+            (void) fprintf(file,"  Colors: %lu\n",GetNumberColors(image,
+              (FILE *) NULL,&image->exception));
+          if (IsHistogramImage(image,&image->exception) != MagickFalse)
+            {
+              (void) fprintf(file,"  Histogram:\n");
+              (void) GetNumberColors(image,file,&image->exception);
+            }
         }
     }
   if (image->storage_class == PseudoClass)
@@ -545,12 +577,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
                   tuple);
               }
             (void) ConcatenateMagickString(tuple,")",MaxTextExtent);
-            (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickFalse,
-              color,&image->exception);
-            (void) QueryMagickColorname(image,&pixel,SVGCompliance,MagickTrue,
-              hex,&image->exception);
-            (void) fprintf(file,"  %8ld: %s %s %s\n",i,tuple,hex,
-              color);
+            (void) QueryMagickColorname(image,&pixel,SVGCompliance,color,
+              &image->exception);
+            GetColorTuple(&pixel,MagickTrue,hex);
+            (void) fprintf(file,"  %8ld: %s %s %s\n",i,tuple,hex,color);
             p++;
           }
         }
@@ -562,7 +592,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
     (void) fprintf(file,"  Normalized mean error: %g\n",
       image->error.normalized_mean_error);
   if (image->error.normalized_maximum_error != 0.0)
-    (void) fprintf(file,"  Normalized maximum error: %gn",
+    (void) fprintf(file,"  Normalized maximum error: %g\n",
       image->error.normalized_maximum_error);
   (void) fprintf(file,"  Rendering intent: %s\n",MagickOptionToMnemonic(
     MagickIntentOptions,(long) image->rendering_intent));
@@ -591,13 +621,6 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
     (void) fprintf(file,"  Tile geometry: %lux%lu%+ld%+ld\n",
       image->extract_info.width,image->extract_info.height,
       image->extract_info.x,image->extract_info.y);
-  if ((image->x_resolution != 0.0) && (image->y_resolution != 0.0))
-    (void) fprintf(file,"  Resolution: %gx%g\n",image->x_resolution,
-      image->y_resolution);
-  (void) fprintf(file,"  Units: %s\n",
-    MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-  (void) FormatMagickSize(GetBlobSize(image),format);
-  (void) fprintf(file,"  Filesize: %s\n",format);
   (void) fprintf(file,"  Interlace: %s\n",MagickOptionToMnemonic(
     MagickInterlaceOptions,(long) image->interlace));
   (void) QueryColorname(image,&image->background_color,SVGCompliance,color,
@@ -626,22 +649,18 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
       image->ticks_per_second);
   if (image->iterations != 1)
     (void) fprintf(file,"  Iterations: %lu\n",image->iterations);
-  p=image;
-  while (p->previous != (Image *) NULL)
-    p=p->previous;
-  for (i=1; GetNextImageInList(p) != (Image *) NULL; i++)
-    p=GetNextImageInList(p);
-  if (i > 1)
-    (void) fprintf(file,"  Scene: %lu of %ld\n",image->scene,i);
+  if ((image->next != (Image *) NULL) || (image->previous != (Image *) NULL))
+    (void) fprintf(file,"  Scene: %lu of %lu\n",image->scene,
+      GetImageListLength(image));
   else
     if (image->scene != 0)
       (void) fprintf(file,"  Scene: %lu\n",image->scene);
-  (void) fprintf(file,"  Compression: %s\n",
-    MagickOptionToMnemonic(MagickCompressOptions,(long) image->compression));
+  (void) fprintf(file,"  Compression: %s\n",MagickOptionToMnemonic(
+    MagickCompressOptions,(long) image->compression));
   if (image->quality != UndefinedCompressionQuality)
     (void) fprintf(file,"  Quality: %lu\n",image->quality);
-  (void) fprintf(file,"  Orientation: %s\n",
-    MagickOptionToMnemonic(MagickOrientationOptions,(long) image->orientation));
+  (void) fprintf(file,"  Orientation: %s\n",MagickOptionToMnemonic(
+    MagickOrientationOptions,(long) image->orientation));
   if (image->montage != (char *) NULL)
     (void) fprintf(file,"  Montage: %s\n",image->montage);
   if (image->directory != (char *) NULL)
@@ -698,51 +717,57 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
       }
       image_info=DestroyImageInfo(image_info);
     }
-  /*
-    Display image properties.
-  */
   (void) GetImageProperty(image,"exif:*");
   ResetImagePropertyIterator(image);
   property=GetNextImageProperty(image);
-  while (property != (const char *) NULL)
-  {
-    (void) fprintf(file,"  %c",toupper(*property));
-    if (strlen(property) > 1)
-      (void) fprintf(file,"%s: ",property+1);
-    if (strlen(property) > 80)
-      (void) fputc('\n',file);
-    value=GetImageProperty(image,property);
-    if (value != (const char *) NULL)
-      (void) fprintf(file,"%s\n",value);
-    property=GetNextImageProperty(image);
-  }
+  if (property != (const char *) NULL)
+    {
+      /*
+        Display image properties.
+      */
+      (void) fprintf(file,"  Properties:\n");
+      while (property != (const char *) NULL)
+      {
+        (void) fprintf(file,"    %c",*property);
+        if (strlen(property) > 1)
+          (void) fprintf(file,"%s: ",property+1);
+        if (strlen(property) > 80)
+          (void) fputc('\n',file);
+        value=GetImageProperty(image,property);
+        if (value != (const char *) NULL)
+          (void) fprintf(file,"%s\n",value);
+        property=GetNextImageProperty(image);
+      }
+    }
   (void) FormatMagickString(key,MaxTextExtent,"8BIM:1999,2998:#1");
   value=GetImageProperty(image,key);
   if (value != (const char *) NULL)
     {
+      /*
+        Display clipping path.
+      */
       (void) fprintf(file,"  Clipping path: ");
       if (strlen(value) > 80)
         (void) fputc('\n',file);
       (void) fprintf(file,"%s\n",value);
     }
-  if (image->profiles != (void *) NULL)
+  ResetImageProfileIterator(image);
+  name=GetNextImageProfile(image);
+  if (name != (char *) NULL)
     {
-      const char
-        *name;
-
       const StringInfo
         *profile;
 
       /*
         Identify image profiles.
       */
-      ResetImageProfileIterator(image);
-      for (name=GetNextImageProfile(image); name != (char *) NULL; )
+      (void) fprintf(file,"  Profiles:\n");
+      while (name != (char *) NULL)
       {
         profile=GetImageProfile(image,name);
         if (profile == (StringInfo *) NULL)
           continue;
-        (void) fprintf(file,"  Profile-%s: %lu bytes\n",name,(unsigned long)
+        (void) fprintf(file,"    Profile-%s: %lu bytes\n",name,(unsigned long)
           GetStringInfoLength(profile));
 #if defined(MAGICKCORE_LCMS_DELEGATE)
         if ((LocaleCompare(name,"icc") == 0) ||
@@ -760,12 +785,12 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
 
                 name=cmsTakeProductName(icc_profile);
                 if (name != (const char *) NULL)
-                  (void) fprintf(file,"    %s\n",name);
+                  (void) fprintf(file,"      %s\n",name);
                 (void) cmsCloseProfile(icc_profile);
               }
           }
 #endif
-        if (LocaleCompare(name,"iptc") == 0)
+        if (LocaleCompare(name,"8bim") == 0)
           {
             char
               *attribute,
@@ -783,12 +808,14 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               j;
 
             size_t
-              length;
+              length,
+              profile_length;
 
             /*
               Identify IPTC data.
             */
-            for (i=0; i < (long) GetStringInfoLength(profile); i+=(long) length)
+            profile_length=GetStringInfoLength(profile);
+            for (i=0; i < (long) profile_length; i+=(long) length)
             {
               length=1;
               sentinel=GetStringInfoDatum(profile)[i++];
@@ -870,8 +897,8 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
                       {
                         (void) fputs(attribute_list[j],file);
                         (void) fputs("\n",file);
-                        attribute_list[j]=(char *)
-                          RelinquishMagickMemory(attribute_list[j]);
+                        attribute_list[j]=(char *) RelinquishMagickMemory(
+                          attribute_list[j]);
                       }
                       attribute_list=(char **) RelinquishMagickMemory(
                         attribute_list);
@@ -885,21 +912,66 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         name=GetNextImageProfile(image);
       }
     }
-  if (image->taint != MagickFalse)
-    (void) fprintf(file,"  Tainted: True\n");
-  else
-    (void) fprintf(file,"  Tainted: False\n");
+  ResetImageArtifactIterator(image);
+  artifact=GetNextImageArtifact(image);
+  if (artifact != (const char *) NULL)
+    {
+      /*
+        Display image artifacts.
+      */
+      (void) fprintf(file,"  Artifacts:\n");
+      while (artifact != (const char *) NULL)
+      {
+        (void) fprintf(file,"    %c",*artifact);
+        if (strlen(artifact) > 1)
+          (void) fprintf(file,"%s: ",artifact+1);
+        if (strlen(artifact) > 80)
+          (void) fputc('\n',file);
+        value=GetImageArtifact(image,artifact);
+        if (value != (const char *) NULL)
+          (void) fprintf(file,"%s\n",value);
+        artifact=GetNextImageArtifact(image);
+      }
+    }
+  ResetImageRegistryIterator();
+  registry=GetNextImageRegistry();
+  if (registry != (const char *) NULL)
+    {
+      /*
+        Display image registry.
+      */
+      (void) fprintf(file,"  Registry:\n");
+      while (registry != (const char *) NULL)
+      {
+        (void) fprintf(file,"    %c",*registry);
+        if (strlen(registry) > 1)
+          (void) fprintf(file,"%s: ",registry+1);
+        if (strlen(registry) > 80)
+          (void) fputc('\n',file);
+        value=(const char *) GetImageRegistry(StringRegistryType,registry,
+          &image->exception);
+        if (value != (const char *) NULL)
+          (void) fprintf(file,"%s\n",value);
+        registry=GetNextImageRegistry();
+      }
+    }
+  (void) fprintf(file,"  Tainted: %s\n",MagickOptionToMnemonic(
+    MagickBooleanOptions,(long) image->taint));
+  (void) FormatMagickSize(GetBlobSize(image),format);
+  (void) fprintf(file,"  Filesize: %s\n",format);
+  (void) FormatMagickSize((MagickSizeType) image->columns*image->rows,format);
+  (void) fprintf(file,"  Number pixels: %s\n",format);
   if (elapsed_time > 0.06)
     {
-      (void) fprintf(file,"  User time: %0.3fu\n",user_time);
-      (void) fprintf(file,"  Elapsed time: %ld:%02ld\n",
-      (long) (elapsed_time/60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
       (void) FormatMagickSize((MagickSizeType) ((double) image->columns*
-        image->rows/elapsed_time),format);
+        image->rows/elapsed_time+0.5),format);
       (void) fprintf(file,"  Pixels per second: %s\n",format);
+      (void) fprintf(file,"  User time: %0.3fu\n",user_time);
+      (void) fprintf(file,"  Elapsed time: %ld:%02ld\n",(long) (elapsed_time/
+        60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
     }
-  (void) fprintf(file,"  Version: %s\n",
-    GetMagickVersion((unsigned long *) NULL));
+  (void) fprintf(file,"  Version: %s\n",GetMagickVersion(
+    (unsigned long *) NULL));
   (void) fflush(file);
   return(ferror(file) != 0 ? MagickFalse : MagickTrue);
 }
