@@ -10,14 +10,14 @@
 %                  LLLLL   OOO    CCCC  A   A  LLLLL  EEEEE                   %
 %                                                                             %
 %                                                                             %
-%                      ImageMagick Image Locale Methods                       %
+%                      MagickCore Image Locale Methods                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,17 +49,18 @@
 #include "magick/locale_.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt-feature.h"
 #include "magick/semaphore.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
 
 /*
   Define declarations.
 */
 #define LocaleFilename  "locale.xml"
+#define MaxRecursionDepth  200
 
 /*
   Static declarations.
@@ -109,7 +110,6 @@ static MagickBooleanType
 %
 %      DestroyLocaleList(void)
 %
-%
 */
 MagickExport void DestroyLocaleList(void)
 {
@@ -118,7 +118,7 @@ MagickExport void DestroyLocaleList(void)
     locale_list=DestroySplayTree(locale_list);
   instantiate_locale=MagickFalse;
   RelinquishSemaphoreInfo(locale_semaphore);
-  locale_semaphore=DestroySemaphoreInfo(locale_semaphore);
+  DestroySemaphoreInfo(&locale_semaphore);
 }
 
 /*
@@ -141,8 +141,7 @@ MagickExport void DestroyLocaleList(void)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
-%
+%    o image: the image.
 %
 */
 
@@ -179,18 +178,14 @@ MagickExport LinkedListInfo *DestroyLocaleOptions(LinkedListInfo *messages)
 %
 %  A description of each parameter follows:
 %
-%    o tag: The locale tag.
+%    o tag: the locale tag.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
   ExceptionInfo *exception)
 {
-  register const LocaleInfo
-    *p;
-
   assert(exception != (ExceptionInfo *) NULL);
   if ((locale_list == (SplayTreeInfo *) NULL) ||
       (instantiate_locale == MagickFalse))
@@ -204,20 +199,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
       ResetSplayTreeIterator(locale_list);
       return((const LocaleInfo *) GetNextValueInSplayTree(locale_list));
     }
-  /*
-    Search for named tag.
-  */
-  AcquireSemaphoreInfo(&locale_semaphore);
-  ResetSplayTreeIterator(locale_list);
-  p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
-  while (p != (const LocaleInfo *) NULL)
-  {
-    if (LocaleCompare(tag,p->tag) == 0)
-      break;
-    p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
-  }
-  RelinquishSemaphoreInfo(locale_semaphore);
-  return(p);
+  return((const LocaleInfo *) GetValueFromSplayTree(locale_list,tag));
 }
 
 /*
@@ -293,8 +275,8 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
   p=GetLocaleInfo_("*",exception);
   if (p == (const LocaleInfo *) NULL)
     return((const LocaleInfo **) NULL);
-  messages=(const LocaleInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfNodesInSplayTree(locale_list)+1)*sizeof(*messages));
+  messages=(const LocaleInfo **) AcquireQuantumMemory((size_t)
+    GetNumberOfNodesInSplayTree(locale_list)+1UL,sizeof(*messages));
   if (messages == (const LocaleInfo **) NULL)
     return((const LocaleInfo **) NULL);
   /*
@@ -306,7 +288,7 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
   for (i=0; p != (const LocaleInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->tag,pattern) != MagickFalse))
+        (GlobExpression(p->tag,pattern,MagickTrue) != MagickFalse))
       messages[i++]=p;
     p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   }
@@ -390,8 +372,8 @@ MagickExport char **GetLocaleList(const char *pattern,
     return((char **) NULL);
   AcquireSemaphoreInfo(&locale_semaphore);
   RelinquishSemaphoreInfo(locale_semaphore);
-  messages=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfNodesInSplayTree(locale_list)+1)*sizeof(*messages));
+  messages=(char **) AcquireQuantumMemory((size_t)
+    GetNumberOfNodesInSplayTree(locale_list)+1UL,sizeof(*messages));
   if (messages == (char **) NULL)
     return((char **) NULL);
   AcquireSemaphoreInfo(&locale_semaphore);
@@ -399,8 +381,8 @@ MagickExport char **GetLocaleList(const char *pattern,
   for (i=0; p != (const LocaleInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->tag,pattern) != MagickFalse))
-      messages[i++]=ConstantString(AcquireString(p->tag));
+        (GlobExpression(p->tag,pattern,MagickTrue) != MagickFalse))
+      messages[i++]=ConstantString(p->tag);
     p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   }
   RelinquishSemaphoreInfo(locale_semaphore);
@@ -432,7 +414,6 @@ MagickExport char **GetLocaleList(const char *pattern,
 %
 %    o tag: Return a message that matches this tag in the current locale.
 %
-%
 */
 MagickExport const char *GetLocaleMessage(const char *tag)
 {
@@ -443,14 +424,14 @@ MagickExport const char *GetLocaleMessage(const char *tag)
     *locale_info;
 
   ExceptionInfo
-    exception;
+    *exception;
 
   if ((tag == (const char *) NULL) || (*tag == '\0'))
     return(tag);
-  GetExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
   (void) FormatMagickString(name,MaxTextExtent,"%s/",tag);
-  locale_info=GetLocaleInfo_(name,&exception);
-  DestroyExceptionInfo(&exception);
+  locale_info=GetLocaleInfo_(name,exception);
+  exception=DestroyExceptionInfo(exception);
   if (locale_info != (const LocaleInfo *) NULL)
     return(locale_info->message);
   return(tag);
@@ -477,11 +458,10 @@ MagickExport const char *GetLocaleMessage(const char *tag)
 %
 %  A description of each parameter follows:
 %
-%    o filename: The locale file tag.
+%    o filename: the locale file tag.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
   ExceptionInfo *exception)
@@ -517,7 +497,7 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",element,filename);
         (void) LogMagickEvent(LocaleEvent,GetMagickModule(),
           "Searching for locale file: \"%s\"",path);
-        xml=FileToStringInfo(path);
+        xml=ConfigureFileToStringInfo(path);
         if (xml != (StringInfo *) NULL)
           (void) AppendValueToLinkedList(messages,xml);
         element=(const char *) GetNextValueInLinkedList(paths);
@@ -534,7 +514,7 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
       {
         xml=StringToStringInfo(blob);
         (void) AppendValueToLinkedList(messages,xml);
-        blob=(char *) RelinquishMagickMemory(blob);
+        blob=DestroyString(blob);
       }
   }
 #endif
@@ -606,23 +586,30 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
           char
             *locale;
 
-          locale=setlocale(LC_CTYPE,0);
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LC_ALL");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LC_MESSAGES");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LC_CTYPE");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LANG");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale="C";
+          register const char
+            *p;
+
+          locale=(char *) NULL;
+          p=setlocale(LC_CTYPE,(const char *) NULL);
+          if (p != (const char *) NULL)
+            locale=ConstantString(p);
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LC_ALL");
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LC_MESSAGES");
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LC_CTYPE");
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LANG");
+          if (locale == (char *) NULL)
+            locale=ConstantString("C");
           (void) LoadLocaleLists(LocaleFilename,locale,exception);
+          locale=DestroyString(locale);
           instantiate_locale=MagickTrue;
         }
       RelinquishSemaphoreInfo(locale_semaphore);
     }
-  return((MagickBooleanType) (locale_list != (SplayTreeInfo *) NULL));
+  return(locale_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -648,7 +635,6 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
   ExceptionInfo *exception)
@@ -726,7 +712,6 @@ MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 static void ChopLocaleComponents(char *path,const unsigned long components)
@@ -759,11 +744,11 @@ static void *DestroyLocaleNode(void *locale_info)
 
   p=(LocaleInfo *) locale_info;
   if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
+    p->path=DestroyString(p->path);
   if (p->tag != (char *) NULL)
-    p->tag=(char *) RelinquishMagickMemory(p->tag);
+    p->tag=DestroyString(p->tag);
   if (p->message != (char *) NULL)
-    p->message=(char *) RelinquishMagickMemory(p->message);
+    p->message=DestroyString(p->message);
   return(RelinquishMagickMemory(p));
 }
 
@@ -773,14 +758,16 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
   char
     keyword[MaxTextExtent],
     message[MaxTextExtent],
-    *q,
     tag[MaxTextExtent],
     *token;
 
-  LocaleInfo
-    *locale_info = (LocaleInfo *) NULL;
+  const char
+    *q;
 
-  MagickStatusType
+  LocaleInfo
+    *locale_info;
+
+  MagickBooleanType
     status;
 
   register char
@@ -789,11 +776,13 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
   /*
     Read the locale configure file.
   */
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+    "Loading locale configure file \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (locale_list == (SplayTreeInfo *) NULL)
     {
-      locale_list=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
+      locale_list=NewSplayTree(CompareSplayTreeString,(void *(*)(void *)) NULL,
         DestroyLocaleNode);
       if (locale_list == (SplayTreeInfo *) NULL)
         {
@@ -803,6 +792,7 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
+  locale_info=(LocaleInfo *) NULL;
   *tag='\0';
   *message='\0';
   *keyword='\0';
@@ -816,13 +806,30 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+        {
+          GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+        {
           GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -860,9 +867,12 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
-                  status|=LoadLocaleList(xml,path,locale,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
+                  xml=FileToString(path,~0,exception);
+                  if (xml == (char *) NULL)
+                    {
+                      status=LoadLocaleList(xml,path,locale,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
                 }
             }
         }
@@ -911,7 +921,7 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
               (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
             }
         }
-        for (p=q; (*q != '<') && (*q != '\0'); q++);
+        for (p=(char *) q; (*q != '<') && (*q != '\0'); q++);
         while (isspace((int) ((unsigned char) *p)) != 0)
           p++;
         q--;
@@ -920,15 +930,13 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         (void) CopyMagickString(message,p,(size_t) (q-p+2));
         locale_info=(LocaleInfo *) AcquireMagickMemory(sizeof(*locale_info));
         if (locale_info == (LocaleInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) ResetMagickMemory(locale_info,0,sizeof(*locale_info));
-        locale_info->path=ConstantString(AcquireString(filename));
-        locale_info->tag=ConstantString(AcquireString(tag));
-        locale_info->message=ConstantString(AcquireString(message));
+        locale_info->path=ConstantString(filename);
+        locale_info->tag=ConstantString(tag);
+        locale_info->message=ConstantString(message);
         locale_info->signature=MagickSignature;
-        status=AddValueToSplayTree(locale_list,
-          ConstantString(AcquireString(locale_info->tag)),locale_info);
+        status=AddValueToSplayTree(locale_list,locale_info->tag,locale_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",
@@ -964,14 +972,12 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
         (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
         continue;
       }
-    GetMagickToken(q,(char **) NULL,token);
+    GetMagickToken(q,(const char **) NULL,token);
     if (*token != '=')
       continue;
   }
   token=(char *) RelinquishMagickMemory(token);
-  if (locale_list == (SplayTreeInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  return(status);
 }
 
 /*
@@ -995,18 +1001,17 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file tag.
+%    o filename: the font file tag.
 %
-%    o locale: The actual locale.
+%    o locale: the actual locale.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadLocaleLists(const char *filename,
   const char *locale,ExceptionInfo *exception)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   return(LoadLocaleList(LocaleMap,"built-in",locale,0,exception));
 #else
   const StringInfo
@@ -1023,8 +1028,8 @@ static MagickBooleanType LoadLocaleLists(const char *filename,
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    status|=LoadLocaleList((const char *) option->datum,option->path,locale,0,
-      exception);
+    status|=LoadLocaleList((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),locale,0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyLocaleOptions(options);
@@ -1035,8 +1040,8 @@ static MagickBooleanType LoadLocaleLists(const char *filename,
       option=(const StringInfo *) GetNextValueInLinkedList(options);
       while (option != (const StringInfo *) NULL)
       {
-        status|=LoadLocaleList((const char *) option->datum,option->path,locale,
-          0,exception);
+        status|=LoadLocaleList((const char *) GetStringInfoDatum(option),
+          GetStringInfoPath(option),locale,0,exception);
         option=(const StringInfo *) GetNextValueInLinkedList(options);
       }
       options=DestroyLocaleOptions(options);
