@@ -10,7 +10,7 @@
 %                            M   M  A   A   CCCC                              %
 %                                                                             %
 %                                                                             %
-%                   Macintosh Utility Methods for ImageMagick                 %
+%                    Macintosh Utility Methods for MagickCore                 %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
@@ -71,6 +71,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -134,11 +135,11 @@ static MACErrorHookPtr
 %
 %    o picture: Specifies a pointer to a PicHandle structure.
 %
-%    o codec: The code type is returned in this CodecType pointer structure.
+%    o codec: the code type is returned in this CodecType pointer structure.
 %
-%    o depth: The image depth is returned as an integer pointer.
+%    o depth: the image depth is returned as an integer pointer.
 %
-%    o colormap_id: The colormap ID is returned in this short pointer.
+%    o colormap_id: the colormap ID is returned in this short pointer.
 %
 %
 */
@@ -1169,10 +1170,10 @@ MagickExport struct dirent *readdir(DIR *entry)
 %  A description of each parameter follows:
 %
 %    o image:  Method ReadPICTImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a a memory shortage or
+%      reading.  A null image is returned if there is a memory shortage or
 %      if the image cannot be read.
 %
-%    o image_info: The image info..
+%    o image_info: the image info..
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -1211,6 +1212,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
     y;
 
   MagickBooleanType
+    proceed,
     status;
 
   PicHandle
@@ -1243,7 +1245,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   /*
     Open image file.
   */
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(NULL);
@@ -1322,7 +1324,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
         Colormapped PICT image.
       */
       number_colors=(*(picture_info.theColorTable))->ctSize;
-      if (!AllocateImageColormap(image,number_colors))
+      if (!AcquireImageColormap(image,number_colors))
         {
           if (picture_info.theColorTable != nil)
             DisposeHandle((Handle) picture_info.theColorTable);
@@ -1373,14 +1375,9 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,LoadImageTag,y,image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   UnlockPixels(GetGWorldPixMap(graphic_world));
   SetGWorld(port,device);
