@@ -9,7 +9,7 @@
 %                    M   M  A   A   GGGG  IIIII   CCCC                        %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Magic Methods                         %
+%                      MagickCore Image Magic Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                              Bob Friesenhahn                                %
@@ -79,6 +79,7 @@ static const char
     "  <magic name=\"CGM\" offset=\"0\" target=\"BEGMF\" />"
     "  <magic name=\"DCM\" offset=\"128\" target=\"DICM\" />"
     "  <magic name=\"DCX\" offset=\"0\" target=\"\\261\\150\\336\\72\" />"
+    "  <magic name=\"DDS\" offset=\"0\" target=\"DDS \" />"
     "  <magic name=\"DIB\" offset=\"0\" target=\"\\050\\000\" />"
     "  <magic name=\"DJVU\" offset=\"0\" target=\"AT&TFORM\" />"
     "  <magic name=\"DOT\" offset=\"0\" target=\"digraph\" />"
@@ -223,7 +224,7 @@ MagickExport void DestroyMagicList(void)
     magic_list=DestroyLinkedList(magic_list,DestroyMagicElement);
   instantiate_magic=MagickFalse;
   RelinquishSemaphoreInfo(magic_semaphore);
-  magic_semaphore=DestroySemaphoreInfo(magic_semaphore);
+  DestroySemaphoreInfo(&magic_semaphore);
 }
 
 /*
@@ -250,7 +251,7 @@ MagickExport void DestroyMagicList(void)
 %    o magic: A binary string generally representing the first few characters
 %      of the image file or blob.
 %
-%    o length: The length of the binary signature.
+%    o length: the length of the binary signature.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -274,7 +275,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   if (length == 0)
     return((const MagicInfo *) NULL);
   /*
-    Search for requested magic.
+    Search for magic tag.
   */
   AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
@@ -287,8 +288,11 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
       break;
     p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
+  if (p != (const MagicInfo *) NULL)
+    (void) InsertValueInLinkedList(magic_list,0,
+      RemoveElementByValueFromLinkedList(magic_list,p));
   RelinquishSemaphoreInfo(magic_semaphore);
-  return((const MagicInfo *) p);
+  return(p);
 }
 
 /*
@@ -658,153 +662,217 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
 static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
-  const char
-    *attribute;
+  char
+    keyword[MaxTextExtent],
+    *token;
 
-  MagicInfo
-    *magic_info = (MagicInfo *) NULL;
+  const char
+    *q;
 
   MagickBooleanType
     status;
 
-  XMLTreeInfo
-    *magic,
-    *magic_map,
-    *include;
+  MagicInfo
+    *magic_info;
 
   /*
     Load the magic map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading magic map \"%s\" ...",filename);
-  if (xml == (const char *) NULL)
+    "Loading magic configure file \"%s\" ...",filename);
+  if (xml == (char *) NULL)
     return(MagickFalse);
   if (magic_list == (LinkedListInfo *) NULL)
-    {
-      magic_list=NewLinkedList(0);
-      if (magic_list == (LinkedListInfo *) NULL)
-        {
-          ThrowFileException(exception,ResourceLimitError,
-            "MemoryAllocationFailed",filename);
-          return(MagickFalse);
-        }
-    }
-  magic_map=NewXMLTree(xml,exception);
-  if (magic_map == (XMLTreeInfo *) NULL)
-    return(MagickFalse);
+    magic_list=NewLinkedList(0);
   status=MagickTrue;
-  include=GetXMLTreeChild(magic_map,"include");
-  while (include != (XMLTreeInfo *) NULL)
+  magic_info=(MagicInfo *) NULL;
+  token=AcquireString(xml);
+  for (q=(char *) xml; *q != '\0'; )
   {
     /*
-      Process include element.
+      Interpret XML.
     */
-    attribute=GetXMLTreeAttribute(include,"file");
-    if (attribute != (const char *) NULL)
+    GetMagickToken(q,&q,token);
+    if (*token == '\0')
+      break;
+    (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
       {
-        if (depth > 200)
-          (void) ThrowMagickException(exception,GetMagickModule(),
-            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",filename);
-        else
-          {
-            char
-              path[MaxTextExtent],
-              *xml;
-
-            GetPathComponent(filename,HeadPath,path);
-            if (*path != '\0')
-              (void) ConcatenateMagickString(path,DirectorySeparator,
-                MaxTextExtent);
-            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
-            xml=FileToString(path,~0,exception);
-            if (xml != (char *) NULL)
-              {
-                status=LoadMagicList(xml,path,depth+1,exception);
-                xml=DestroyString(xml);
-              }
-          }
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
       }
-    include=GetNextXMLTreeTag(include);
-  }
-  magic=GetXMLTreeChild(magic_map,"magic");
-  while (magic != (XMLTreeInfo *) NULL)
-  {
-    const char
-      *attribute;
-
-    /*
-      Process magic element.
-    */
-    magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(*magic_info));
-    if (magic_info == (MagicInfo *) NULL)
-      ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-    (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
-    magic_info->path=ConstantString(filename);
-    magic_info->signature=MagickSignature;
-    attribute=GetXMLTreeAttribute(magic,"name");
-    if (attribute != (const char *) NULL)
-      magic_info->name=ConstantString(attribute);
-    attribute=GetXMLTreeAttribute(magic,"offset");
-    if (attribute != (const char *) NULL)
-      magic_info->offset=(MagickOffsetType) atol(attribute);
-    attribute=GetXMLTreeAttribute(magic,"stealth");
-    if (attribute != (const char *) NULL)
-      magic_info->stealth=IsMagickTrue(attribute);
-    attribute=GetXMLTreeAttribute(magic,"target");
-    if (attribute != (const char *) NULL)
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
-        const char
-          *p;
-
-        register unsigned char
-          *q;
-
-        magic_info->target=ConstantString(attribute);
-        magic_info->magic=(unsigned char *) ConstantString(attribute);
-        q=magic_info->magic;
-        for (p=magic_info->target; *p != '\0'; )
+        /*
+          Comment element.
+        */
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
+    if (LocaleCompare(keyword,"<include") == 0)
+      {
+        /*
+          Include element.
+        */
+        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
-          if (*p == '\\')
+          (void) CopyMagickString(keyword,token,MaxTextExtent);
+          GetMagickToken(q,&q,token);
+          if (*token != '=')
+            continue;
+          GetMagickToken(q,&q,token);
+          if (LocaleCompare(keyword,"file") == 0)
             {
-              p++;
-              if (isdigit((int) ((unsigned char) *p)) != 0)
+              if (depth > 200)
+                (void) ThrowMagickException(exception,GetMagickModule(),
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
+              else
                 {
                   char
-                    *end;
+                    path[MaxTextExtent],
+                    *xml;
 
-                  *q++=(unsigned char) strtol(p,&end,8);
-                  p+=(end-p);
+                  GetPathComponent(filename,HeadPath,path);
+                  if (*path != '\0')
+                    (void) ConcatenateMagickString(path,DirectorySeparator,
+                      MaxTextExtent);
+                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  xml=FileToString(path,~0,exception);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadMagicList(xml,path,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
+                }
+            }
+        }
+        continue;
+      }
+    if (LocaleCompare(keyword,"<magic") == 0)
+      {
+        /*
+          Magic element.
+        */
+        magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(*magic_info));
+        if (magic_info == (MagicInfo *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+        (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
+        magic_info->path=ConstantString(filename);
+        magic_info->signature=MagickSignature;
+        continue;
+      }
+    if (magic_info == (MagicInfo *) NULL)
+      continue;
+    if (LocaleCompare(keyword,"/>") == 0)
+      {
+        status=AppendValueToLinkedList(magic_list,magic_info);
+        if (status == MagickFalse)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            magic_info->name);
+        magic_info=(MagicInfo *) NULL;
+      }
+    GetMagickToken(q,(const char **) NULL,token);
+    if (*token != '=')
+      continue;
+    GetMagickToken(q,&q,token);
+    GetMagickToken(q,&q,token);
+    switch (*keyword)
+    {
+      case 'N':
+      case 'n':
+      {
+        if (LocaleCompare((char *) keyword,"name") == 0)
+          {
+            magic_info->name=ConstantString(token);
+            break;
+          }
+        break;
+      }
+      case 'O':
+      case 'o':
+      {
+        if (LocaleCompare((char *) keyword,"offset") == 0)
+          {
+            magic_info->offset=(MagickOffsetType) atol(token);
+            break;
+          }
+        break;
+      }
+      case 'S':
+      case 's':
+      {
+        if (LocaleCompare((char *) keyword,"stealth") == 0)
+          {
+            magic_info->stealth=IsMagickTrue(token);
+            break;
+          }
+        break;
+      }
+      case 'T':
+      case 't':
+      {
+        if (LocaleCompare((char *) keyword,"target") == 0)
+          {
+            const char
+              *p;
+
+            register unsigned char
+              *q;
+
+            magic_info->target=ConstantString(token);
+            magic_info->magic=(unsigned char *) ConstantString(token);
+            q=magic_info->magic;
+            for (p=magic_info->target; *p != '\0'; )
+            {
+              if (*p == '\\')
+                {
+                  p++;
+                  if (isdigit((int) ((unsigned char) *p)) != 0)
+                    {
+                      char
+                        *end;
+
+                      *q++=(unsigned char) strtol(p,&end,8);
+                      p+=(end-p);
+                      magic_info->length++;
+                      continue;
+                    }
+                  switch (*p)
+                  {
+                    case 'b': *q='\b'; break;
+                    case 'f': *q='\f'; break;
+                    case 'n': *q='\n'; break;
+                    case 'r': *q='\r'; break;
+                    case 't': *q='\t'; break;
+                    case 'v': *q='\v'; break;
+                    case 'a': *q='a'; break;
+                    case '?': *q='\?'; break;
+                    default: *q=(unsigned char) (*p); break;
+                  }
+                  p++;
+                  q++;
                   magic_info->length++;
                   continue;
                 }
-              switch (*p)
-              {
-                case 'b': *q='\b'; break;
-                case 'f': *q='\f'; break;
-                case 'n': *q='\n'; break;
-                case 'r': *q='\r'; break;
-                case 't': *q='\t'; break;
-                case 'v': *q='\v'; break;
-                case 'a': *q='a'; break;
-                case '?': *q='\?'; break;
-                default: *q=(unsigned char) (*p); break;
-              }
-              p++;
-              q++;
+              *q++=(unsigned char) (*p++);
               magic_info->length++;
-              continue;
             }
-          *q++=(unsigned char) (*p++);
-          magic_info->length++;
-        }
+            SubstituteString((char **) &magic_info->target,"&amp;","&");
+            break;
+          }
+        break;
       }
-    status=AppendValueToLinkedList(magic_list,magic_info);
-    if (status == MagickFalse)
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
-    magic=GetNextXMLTreeTag(magic);
+      default:
+        break;
+    }
   }
-  magic_map=DestroyXMLTree(magic_map);
+  token=(char *) RelinquishMagickMemory(token);
   return(status);
 }
 
@@ -829,7 +897,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: the font file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
