@@ -10,7 +10,7 @@
 %                    M   M  EEEEE  M   M   OOO   R  R     Y                   %
 %                                                                             %
 %                                                                             %
-%                    ImageMagick Memory Allocation Methods                    %
+%                     MagickCore Memory Allocation Methods                    %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -44,10 +44,11 @@
 %  written by Yoo C. Chung.
 %
 %  By default, ANSI memory methods are called (e.g. malloc).  Use the
-%  custom memory allocator by defining MAGICKCORE_EMBEDDABLE_SUPPORT.  The custom memory
-%  allocator has the advantage of using memory-mapped heap allocation which
-%  permits the virtual memory to be returned to the system in all cases, unlike
-%  the ANSI memory methods that typically only memory-map large allocations.
+%  custom memory allocator by defining MAGICKCORE_EMBEDDABLE_SUPPORT.  The
+%  custom memory allocator has the advantage of using memory-mapped heap
+%  allocation which permits the virtual memory to be returned to the system in
+%  all cases, unlike the ANSI memory methods that typically only memory-map
+%  large allocations.
 %
 */
 
@@ -119,10 +120,25 @@ typedef struct _MemoryInfo
     *segments[MaxSegments],
     segment_pool[MaxSegments];
 } MemoryInfo;
+
+typedef struct _MagickMemoryMethods
+{
+  AcquireMemoryHandler
+    acquire_memory_handler;
+
+  ResizeMemoryHandler
+    resize_memory_handler;
+
+  DestroyMemoryHandler
+    destroy_memory_handler;
+} MagickMemoryMethods;
 
 /*
   Global declarations.
 */
+static MagickMemoryMethods
+  memory_methods = { malloc, realloc, free };
+
 #if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 static MemoryInfo
   memory_info;
@@ -161,7 +177,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o size: The size of the memory in bytes to allocate.
+%    o size: the size of the memory in bytes to allocate.
 %
 */
 
@@ -309,7 +325,7 @@ static void *AcquireBlock(size_t size)
 %
 %  A description of each parameter follows:
 %
-%    o size: The size of the memory in bytes to allocate.
+%    o size: the size of the memory in bytes to allocate.
 %
 */
 MagickExport void *AcquireMagickMemory(const size_t size)
@@ -318,7 +334,7 @@ MagickExport void *AcquireMagickMemory(const size_t size)
     *memory;
 
 #if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
-  memory=malloc(size == 0 ? 1UL : size);
+  memory=memory_methods.acquire_memory_handler(size == 0 ? 1UL : size);
 #else
   if (free_segments == (DataSegmentInfo *) NULL)
     {
@@ -376,9 +392,9 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 %
 %  A description of each parameter follows:
 %
-%    o count: The number of quantum elements to allocate.
+%    o count: the number of quantum elements to allocate.
 %
-%    o quantum: The number of bytes in each quantum.
+%    o quantum: the number of bytes in each quantum.
 %
 */
 MagickExport void *AcquireQuantumMemory(const size_t count,const size_t quantum)
@@ -417,11 +433,11 @@ MagickExport void *AcquireQuantumMemory(const size_t count,const size_t quantum)
 %
 %  A description of each parameter follows:
 %
-%    o destination: The destination.
+%    o destination: the destination.
 %
-%    o source: The source.
+%    o source: the source.
 %
-%    o size: The size of the memory in bytes to allocate.
+%    o size: the size of the memory in bytes to allocate.
 %
 */
 MagickExport void *CopyMagickMemory(void *destination,const void *source,
@@ -435,11 +451,9 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
 
   assert(destination != (void *) NULL);
   assert(source != (const void *) NULL);
-  if ((size == 0) || (source == destination))
-    return(destination);
   p=(const unsigned char *) source;
   q=(unsigned char *) destination;
-  if ((q >= (p+size)) || (p >= (q+size)))
+  if (((q+size) < p) || (q > (p+size)))
     return(memcpy(destination,source,size));
   return(memmove(destination,source,size));
 }
@@ -472,13 +486,13 @@ MagickExport void DestroyMagickMemory(void)
   RelinquishSemaphoreInfo(memory_semaphore);
   for (i=0; i < (long) memory_info.number_segments; i++)
     if (memory_info.segments[i]->mapped == MagickFalse)
-      free(memory_info.segments[i]->allocation);
+      memory_methods.destroy_memory_handler(memory_info.segments[i]->allocation);
     else
       (void) UnmapBlob(memory_info.segments[i]->allocation,
         memory_info.segments[i]->length);
   free_segments=(DataSegmentInfo *) NULL;
   (void) ResetMagickMemory(&memory_info,0,sizeof(memory_info));
-  memory_semaphore=DestroySemaphoreInfo(memory_semaphore);
+  DestroySemaphoreInfo(&memory_semaphore);
 #endif
 }
 
@@ -503,7 +517,7 @@ MagickExport void DestroyMagickMemory(void)
 %
 %  A description of each parameter follows:
 %
-%    o size: The size of the memory in bytes we require.
+%    o size: the size of the memory in bytes we require.
 %
 */
 static MagickBooleanType ExpandHeap(size_t size)
@@ -531,7 +545,7 @@ static MagickBooleanType ExpandHeap(size_t size)
   segment=MapBlob(-1,IOMode,0,blocksize);
   mapped=segment != (void *) NULL ? MagickTrue : MagickFalse;
   if (segment == (void *) NULL)
-    segment=(void *) malloc(blocksize);
+    segment=(void *) memory_methods.acquire_memory_handler(blocksize);
   if (segment == (void *) NULL)
     return(MagickFalse);
   segment_info=(DataSegmentInfo *) free_segments;
@@ -563,6 +577,48 @@ static MagickBooleanType ExpandHeap(size_t size)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M a g i c k M e m o r y M e t h o d s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickMemoryMethods() gets the methods to acquire, resize, and destroy
+%  memory.
+%
+%  The format of the GetMagickMemoryMethods() method is:
+%
+%      void GetMagickMemoryMethods(AcquireMemoryHandler *acquire_memory_handler,
+%        ResizeMemoryHandler *resize_memory_handler,
+%        DestroyMemoryHandler *destroy_memory_handler)
+%
+%  A description of each parameter follows:
+%
+%    o acquire_memory_handler: method to acquire memory (e.g. malloc).
+%
+%    o resize_memory_handler: method to resize memory (e.g. realloc).
+%
+%    o destroy_memory_handler: method to destroy memory (e.g. free).
+%
+*/
+MagickExport void GetMagickMemoryMethods(
+  AcquireMemoryHandler *acquire_memory_handler,
+  ResizeMemoryHandler *resize_memory_handler,
+  DestroyMemoryHandler *destroy_memory_handler)
+{
+  assert(acquire_memory_handler != (AcquireMemoryHandler *) NULL);
+  assert(resize_memory_handler != (ResizeMemoryHandler *) NULL);
+  assert(destroy_memory_handler != (DestroyMemoryHandler *) NULL);
+  *acquire_memory_handler=memory_methods.acquire_memory_handler;
+  *resize_memory_handler=memory_methods.resize_memory_handler;
+  *destroy_memory_handler=memory_methods.destroy_memory_handler;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e l i n q u i s h M a g i c k M e m o r y                               %
 %                                                                             %
 %                                                                             %
@@ -586,7 +642,7 @@ MagickExport void *RelinquishMagickMemory(void *memory)
   if (memory == (void *) NULL)
     return((void *) NULL);
 #if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
-  free(memory);
+  memory_methods.destroy_memory_handler(memory);
 #else
   assert((SizeOfBlock(memory) % (4*sizeof(size_t))) == 0);
   assert((*BlockHeader(NextBlock(memory)) & PreviousBlockBit) != 0);
@@ -682,7 +738,7 @@ MagickExport void *ResetMagickMemory(void *memory,int byte,const size_t size)
 %
 %    o memory: A pointer to a memory allocation.
 %
-%    o size: The new size of the allocated memory.
+%    o size: the new size of the allocated memory.
 %
 */
 
@@ -714,7 +770,7 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
   if (memory == (void *) NULL)
     return(AcquireMagickMemory(size));
 #if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
-  block=realloc(memory,size == 0 ? 1UL : size);
+  block=memory_methods.resize_memory_handler(memory,size == 0 ? 1UL : size);
   if (block == (void *) NULL)
     memory=RelinquishMagickMemory(memory);
 #else
@@ -761,9 +817,9 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
 %
 %    o memory: A pointer to a memory allocation.
 %
-%    o count: The number of quantum elements to allocate.
+%    o count: the number of quantum elements to allocate.
 %
-%    o quantum: The number of bytes in each quantum.
+%    o quantum: the number of bytes in each quantum.
 %
 */
 MagickExport void *ResizeQuantumMemory(void *memory,const size_t count,
@@ -775,8 +831,54 @@ MagickExport void *ResizeQuantumMemory(void *memory,const size_t count,
   size=count*quantum;
   if ((count == 0) || (quantum != (size/count)))
     {
+      memory=RelinquishMagickMemory(memory);
       errno=ENOMEM;
       return((void *) NULL);
     }
   return(ResizeMagickMemory(memory,size));
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t M a g i c k M e m o r y M e t h o d s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetMagickMemoryMethods() sets the methods to acquire, resize, and destroy
+%  memory.
+%
+%  The format of the SetMagickMemoryMethods() method is:
+%
+%      SetMagickMemoryMethods(AcquireMemoryHandler acquire_memory_handler,
+%        ResizeMemoryHandler resize_memory_handler,
+%        DestroyMemoryHandler destroy_memory_handler)
+%
+%  A description of each parameter follows:
+%
+%    o acquire_memory_handler: method to acquire memory (e.g. malloc).
+%
+%    o resize_memory_handler: method to resize memory (e.g. realloc).
+%
+%    o destroy_memory_handler: method to destroy memory (e.g. free).
+%
+*/
+MagickExport void SetMagickMemoryMethods(
+  AcquireMemoryHandler acquire_memory_handler,
+  ResizeMemoryHandler resize_memory_handler,
+  DestroyMemoryHandler destroy_memory_handler)
+{
+  /*
+    Set memory methods.
+  */
+  if (acquire_memory_handler != (AcquireMemoryHandler) NULL)
+    memory_methods.acquire_memory_handler=acquire_memory_handler;
+  if (resize_memory_handler != (ResizeMemoryHandler) NULL)
+    memory_methods.resize_memory_handler=resize_memory_handler;
+  if (destroy_memory_handler != (DestroyMemoryHandler) NULL)
+    memory_methods.destroy_memory_handler=destroy_memory_handler;
+}
