@@ -9,7 +9,7 @@
 %                        M   M  IIIII  M   M  EEEEE                           %
 %                                                                             %
 %                                                                             %
-%                         ImageMagick Mime Methods                            %
+%                          MagickCore Mime Methods                            %
 %                                                                             %
 %                              Software Design                                %
 %                                 July 2000                                   %
@@ -172,7 +172,7 @@ MagickExport void DestroyMimeList(void)
     mime_list=DestroyLinkedList(mime_list,DestroyMimeElement);
   instantiate_mime=MagickFalse;
   RelinquishSemaphoreInfo(mime_semaphore);
-  mime_semaphore=DestroySemaphoreInfo(mime_semaphore);
+  DestroySemaphoreInfo(&mime_semaphore);
 }
 
 /*
@@ -203,7 +203,7 @@ MagickExport void DestroyMimeList(void)
 %    o magic: A binary string generally representing the first few characters
 %      of the image file or blob.
 %
-%    o length: The length of the binary signature.
+%    o length: the length of the binary signature.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -245,7 +245,7 @@ MagickExport const MimeInfo *GetMimeInfo(const char *filename,
   if (length == 0)
     return((const MimeInfo *) NULL);
   /*
-    Search for requested mime type.
+    Search for mime tag.
   */
   mime_info=(const MimeInfo *) NULL;
   lsb_first=1;
@@ -370,6 +370,9 @@ MagickExport const MimeInfo *GetMimeInfo(const char *filename,
     }
     p=(const MimeInfo *) GetNextValueInLinkedList(mime_list);
   }
+  if (p != (const MimeInfo *) NULL)
+    (void) InsertValueInLinkedList(mime_list,0,
+      RemoveElementByValueFromLinkedList(mime_list,p));
   RelinquishSemaphoreInfo(mime_semaphore);
   return(mime_info);
 }
@@ -985,7 +988,7 @@ static MagickBooleanType LoadMimeList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: the font file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
