@@ -10,7 +10,7 @@
 %               M   M   OOO   N   N  IIIII    T     OOO   R  R                %
 %                                                                             %
 %                                                                             %
-%                    ImageMagick Progress Monitor Methods                     %
+%                     MagickCore Progress Monitor Methods                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -74,7 +74,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o progress_monitor: Specifies a pointer to a method to monitor progress of
 %      an image operation.
@@ -123,7 +123,7 @@ MagickExport MagickProgressMonitor SetImageProgressMonitor(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o progress_monitor: Specifies a pointer to a method to monitor progress of
 %      an image operation.
