@@ -10,7 +10,7 @@
 %               M   M   OOO   N   N    T    A   A   GGG   EEEEE               %
 %                                                                             %
 %                                                                             %
-%                ImageMagick Methods to Create Image Thumbnails               %
+%                MagickCore Methods to Create Image Thumbnails                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -60,6 +60,7 @@
 #include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/montage.h"
 #include "magick/option.h"
 #include "magick/quantize.h"
@@ -92,9 +93,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o montage_info: The montage info.
+%    o montage_info: the montage info.
 %
 */
 MagickExport MontageInfo *CloneMontageInfo(const ImageInfo *image_info,
@@ -258,7 +259,7 @@ MagickExport void GetMontageInfo(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o montage_info: Specifies a pointer to a MontageInfo structure.
 %
@@ -373,6 +374,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
 
   MagickBooleanType
     concatenate,
+    proceed,
     status;
 
   MagickOffsetType
@@ -446,14 +448,9 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       break;
     image_list[i]=thumbnail;
     (void) SetImageProgressMonitor(image,progress_monitor,image->client_data);
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(i,number_images) != MagickFalse))
-      {
-        status=image->progress_monitor(TileImageTag,i,number_images,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,TileImageTag,i,number_images);
+    if (proceed == MagickFalse)
+      break;
     image=DestroyImage(image);
   }
   if (i < (long) number_images)
@@ -580,8 +577,8 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   /*
     Allocate next structure.
   */
-  tile_image=AllocateImage(NULL);
-  montage=AllocateImage(clone_info);
+  tile_image=AcquireImage(NULL);
+  montage=AcquireImage(clone_info);
   montage->scene=0;
   images_per_page=(number_images-1)/(tiles_per_row*tiles_per_column)+1;
   tiles=0;
@@ -677,8 +674,8 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       (void) ConcatenateMagickString(montage->directory,"\n",extent);
       tile++;
     }
-    progress_monitor=SetImageProgressMonitor(montage,
-      (MagickProgressMonitor) NULL,montage->client_data);
+    progress_monitor=SetImageProgressMonitor(montage,(MagickProgressMonitor)
+      NULL,montage->client_data);
     if (texture != (Image *) NULL)
       (void) TextureImage(montage,texture);
     if (montage_info->title != (char *) NULL)
@@ -808,7 +805,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
               /*
                 Shadow image.
               */
-              (void) QueryColorDatabase("black",&image->background_color,
+              (void) QueryColorDatabase("#000000",&image->background_color,
                 exception);
               shadow_image=ShadowImage(image,80.0,2.0,5,5,exception);
               if (shadow_image != (Image *) NULL)
@@ -869,7 +866,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(clone_info,montage);
+        AcquireNextImage(clone_info,montage);
         if (GetNextImageInList(montage) == (Image *) NULL)
           {
             montage=DestroyImageList(montage);
