@@ -11,7 +11,7 @@
 %                                 N   N    T                                  %
 %                                                                             %
 %                                                                             %
-%                  Windows NT Feature Methods for ImageMagick                 %
+%                   Windows NT Feature Methods for MagickCore                 %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
@@ -53,6 +53,7 @@
 #include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantum.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -83,7 +84,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o geometry: Define the region of the image to crop with members
 %      x, y, width, and height.
@@ -100,7 +101,7 @@ MagickExport void *CropImageToHBITMAP(Image *image,
     y;
 
   MagickBooleanType
-    status;
+    proceed;
 
   RectangleInfo
     page;
@@ -167,8 +168,8 @@ MagickExport void *CropImageToHBITMAP(Image *image,
   bitmap.bmBitsPixel    = 32;
   bitmap.bmBits         = NULL;
 
-  bitmap_bitsH = (HANDLE) GlobalAlloc (GMEM_MOVEABLE | GMEM_DDESHARE,
-               page.width*page.height*bitmap.bmBitsPixel);
+  bitmap_bitsH=(HANDLE) GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE,page.width*
+    page.height*bitmap.bmBitsPixel);
   if (bitmap_bitsH == NULL)
     return(NULL);
   bitmap_bits=(RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
@@ -208,14 +209,9 @@ MagickExport void *CropImageToHBITMAP(Image *image,
           }
       }
 #endif
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,page.height) != MagickFalse))
-      {
-        status=image->progress_monitor(CropImageTag,y,page.height,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,CropImageTag,y,page.height);
+    if (proceed == MagickFalse)
+      break;
   }
   if (y < (long) page.height)
     {
@@ -223,9 +219,10 @@ MagickExport void *CropImageToHBITMAP(Image *image,
       GlobalFree((HGLOBAL) bitmap_bitsH);
       return((void *) NULL);
     }
-  bitmap.bmBits = bitmap_bits;
-  bitmapH = CreateBitmapIndirect( &bitmap );
+  bitmap.bmBits=bitmap_bits;
+  bitmapH=CreateBitmapIndirect(&bitmap);
   GlobalUnlock((HGLOBAL) bitmap_bitsH);
+  GlobalFree((HGLOBAL) bitmap_bitsH);
   return((void *) bitmapH);
 }
 
@@ -339,7 +336,7 @@ MagickExport MagickBooleanType NTLoadTypeLists(SplayTreeInfo *type_list,
   *font_root='\0';
   (void) CopyMagickString(buffer,system_root,MaxTextExtent);
   (void) ConcatenateMagickString(buffer,"\\fonts\\arial.ttf",MaxTextExtent);
-  if (IsAccessible(buffer) != MagickFalse)
+  if (IsPathAccessible(buffer) != MagickFalse)
     {
       (void) CopyMagickString(font_root,system_root,MaxTextExtent);
       (void) ConcatenateMagickString(font_root,"\\fonts\\",MaxTextExtent);
