@@ -48,23 +48,29 @@
 #include "magick/draw-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/gem.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
 #include "magick/string_.h"
 
+static inline double MagickMax(const double x,const double y)
+{
+    return( x > y ? x : y);
+}
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   P a i n t F l o o d f i l l I m a g e                                     %
+%   F l o o d f i l l P a i n t I m a g e                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  PaintFloodfill() changes the color value of any pixel that matches
+%  FloodfillPaintImage() changes the color value of any pixel that matches
 %  target and is an immediate neighbor.  If the method FillToBorderMethod is
 %  specified, the color value is changed for any neighbor pixel that does not
 %  match the bordercolor member of image.
@@ -76,32 +82,32 @@
 %  color red at intensities of 100 and 102 respectively are now
 %  interpreted as the same color for the purposes of the floodfill.
 %
-%  The format of the PaintFloodfillImage method is:
+%  The format of the FloodfillPaintImage method is:
 %
-%      MagickBooleanType PaintFloodfillImage(Image *image,
-%        const ChannelType channel,const MagickPixelPacket target,
-%        const long x_offset,const long y_offset,const DrawInfo *draw_info,
-%        const PaintMethod method)
+%      MagickBooleanType FloodfillPaintImage(Image *image,
+%        const ChannelType channel,const DrawInfo *draw_info,
+%        const MagickPixelPacket target,const long x_offset,const long y_offset,
+%        const MagickBooleanType invert)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o channel: The channel(s).
+%    o channel: the channel(s).
 %
-%    o target: The RGB value of the target color.
+%    o draw_info: the draw info.
 %
-%    o x_offset,y_offset: The starting location of the operation.
+%    o target: the RGB value of the target color.
 %
-%    o draw_info: The draw info.
+%    o x_offset,y_offset: the starting location of the operation.
 %
-%    o method: Choose either FloodfillMethod or FillToBorderMethod.
+%    o invert: paint any pixel that does not match the target color.
 %
 */
-MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
-  const ChannelType channel,const MagickPixelPacket *target,
-  const long x_offset,const long y_offset,const DrawInfo *draw_info,
-  const PaintMethod method)
+MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
+  const ChannelType channel,const DrawInfo *draw_info,
+  const MagickPixelPacket *target,const long x_offset,const long y_offset,
+  const MagickBooleanType invert)
 {
 #define MaxStacksize  (1UL << 15)
 #define PushSegmentStack(up,left,right,delta) \
@@ -175,7 +181,7 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
   if (image->matte == MagickFalse)
-    (void) SetImageOpacity(image,OpaqueOpacity);
+    (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
   /*
     Set floodfill state.
   */
@@ -183,7 +189,7 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
     &image->exception);
   if (floodplane_image == (Image *) NULL)
     return(MagickFalse);
-  (void) SetImageOpacity(floodplane_image,OpaqueOpacity);
+  (void) SetImageAlphaChannel(floodplane_image,OpaqueAlphaChannel);
   segment_stack=(SegmentInfo *) AcquireQuantumMemory(MaxStacksize,
     sizeof(*segment_stack));
   if (segment_stack == (SegmentInfo *) NULL)
@@ -228,14 +234,8 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
       if (q->opacity == (Quantum) TransparentOpacity)
         break;
       SetMagickPixelPacket(image,p,indexes+x,&pixel);
-      if (method == FloodfillMethod)
-        {
-          if (IsMagickColorSimilar(&pixel,target) == MagickFalse)
-            break;
-        }
-      else
-        if (IsMagickColorSimilar(&pixel,target) != MagickFalse)
-          break;
+      if (IsMagickColorSimilar(&pixel,target) == invert)
+        break;
       q->opacity=(Quantum) TransparentOpacity;
       p--;
       q--;
@@ -268,14 +268,8 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
                 if (q->opacity == (Quantum) TransparentOpacity)
                   break;
                 SetMagickPixelPacket(image,p,indexes+x,&pixel);
-                if (method == FloodfillMethod)
-                  {
-                    if (IsMagickColorSimilar(&pixel,target) == MagickFalse)
-                      break;
-                  }
-                else
-                  if (IsMagickColorSimilar(&pixel,target) != MagickFalse)
-                    break;
+                if (IsMagickColorSimilar(&pixel,target) == invert)
+                  break;
                 q->opacity=(Quantum) TransparentOpacity;
                 p++;
                 q++;
@@ -302,14 +296,8 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
             if (q->opacity == (Quantum) TransparentOpacity)
               break;
             SetMagickPixelPacket(image,p,indexes+x,&pixel);
-            if (method == FloodfillMethod)
-              {
-                if (IsMagickColorSimilar(&pixel,target) != MagickFalse)
-                  break;
-              }
-            else
-              if (IsMagickColorSimilar(&pixel,target) == MagickFalse)
-                break;
+            if (IsMagickColorSimilar(&pixel,target) != invert)
+              break;
             p++;
             q++;
           }
@@ -332,7 +320,7 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
     {
       if (p->opacity != OpaqueOpacity)
         {
-          fill_color=GetFillColor(draw_info,x,y);
+          (void) GetFillColor(draw_info,x,y,&fill_color);
           SetMagickPixelPacket(image,&fill_color,(IndexPacket *) NULL,&fill);
           if (image->colorspace == CMYKColorspace)
             ConvertRGBToCMYK(&fill);
@@ -363,12 +351,346 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
-%     P a i n t O p a q u e I m a g e                                         %
+%                                                                             %
++     G r a d i e n t I m a g e                                               %
+%                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  PaintOpaqueImage() changes any pixel that matches color with the color
+%  GradientImage() applies a continuously smooth color transitions along a
+%  vector from one color to another.
+%
+%  Note, the interface of this method will change in the future to support
+%  more than one transistion.
+%
+%  The format of the GradientImage method is:
+%
+%      MagickBooleanType GradientImage(Image *image,const GradientType type,
+%        const SpreadMethod method,const PixelPacket *start_color,
+%        const PixelPacket *stop_color)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o type: the gradient type: linear or radial.
+%
+%    o spread: the gradient spread meathod: pad, reflect, or repeat.
+%
+%    o start_color: the start color.
+%
+%    o stop_color: the stop color.
+%
+% This provides a good example of making use of the DrawGradientImage
+% function and the gradient structure in draw_info.
+*/
+MagickExport MagickBooleanType GradientImage(Image *image,
+  const GradientType type,const SpreadMethod method,
+  const PixelPacket *start_color,const PixelPacket *stop_color)
+{
+  DrawInfo
+    *draw_info;
+
+  GradientInfo
+    *gradient;
+
+  MagickBooleanType
+    status;
+
+  register long
+    i;
+
+  /*
+    Set gradient start-stop end points.
+  */
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(start_color != (const PixelPacket *) NULL);
+  assert(stop_color != (const PixelPacket *) NULL);
+  draw_info=AcquireDrawInfo();
+  gradient=(&draw_info->gradient);
+  gradient->type=type;
+  gradient->bounding_box.width=image->columns;
+  gradient->bounding_box.height=image->rows;
+  gradient->gradient_vector.x2=(double) image->columns-1.0;
+  gradient->gradient_vector.y2=(double) image->rows-1.0;
+  if ((type == LinearGradient) && (gradient->gradient_vector.y2 != 0.0))
+    gradient->gradient_vector.x2=0.0;
+  gradient->center.x=(double) gradient->gradient_vector.x2/2.0;
+  gradient->center.y=(double) gradient->gradient_vector.y2/2.0;
+  gradient->radius=MagickMax(gradient->center.x,gradient->center.y);
+  gradient->spread=method;
+  /*
+    Define the gradient to fill between the stops.
+  */
+  gradient->number_stops=2;
+  gradient->stops=(StopInfo *) AcquireQuantumMemory(gradient->number_stops,
+    sizeof(*gradient->stops));
+  if (gradient->stops == (StopInfo *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  (void) ResetMagickMemory(gradient->stops,0,gradient->number_stops*
+    sizeof(*gradient->stops));
+  for (i=0; i < (long) gradient->number_stops; i++)
+    GetMagickPixelPacket(image,&gradient->stops[i].color);
+  SetMagickPixelPacket(image,start_color,(IndexPacket *) NULL,
+    &gradient->stops[0].color);
+  gradient->stops[0].offset=0.0;
+  SetMagickPixelPacket(image,stop_color,(IndexPacket *) NULL,
+    &gradient->stops[1].color);
+  gradient->stops[1].offset=1.0;
+  /*
+    Draw a gradient on the image.
+  */
+  status=DrawGradientImage(image,draw_info);
+  draw_info=DestroyDrawInfo(draw_info);
+  if ((start_color->opacity == OpaqueOpacity) &&
+      (stop_color->opacity == OpaqueOpacity))
+    image->matte=MagickFalse;
+  if ((IsGrayPixel(start_color) != MagickFalse) &&
+      (IsGrayPixel(stop_color) != MagickFalse))
+    image->type=GrayscaleType;
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%     O i l P a i n t I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  OilPaintImage() applies a special effect filter that simulates an oil
+%  painting.  Each pixel is replaced by the most frequent color occurring
+%  in a circular region defined by radius.
+%
+%  The format of the OilPaintImage method is:
+%
+%      Image *OilPaintImage(const Image *image,const double radius,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o radius: the radius of the circular neighborhood.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+*/
+
+static unsigned long **DestroyHistogramThreadSet(unsigned long **histogram)
+{
+  register long
+    i;
+
+  assert(histogram != (unsigned long **) NULL);
+  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+    if (histogram[i] != (unsigned long *) NULL)
+      histogram[i]=(unsigned long *) RelinquishMagickMemory(histogram[i]);
+  return((unsigned long **) RelinquishMagickMemory(histogram));
+}
+
+static unsigned long **AcquireHistogramThreadSet(const size_t count)
+{
+  register long
+    i;
+
+  unsigned long
+    **histogram;
+
+  histogram=(unsigned long **) AcquireQuantumMemory(
+    GetCacheViewMaximumThreads(),sizeof(*histogram));
+  if (histogram == (unsigned long **) NULL)
+    return((unsigned long **) NULL);
+  (void) ResetMagickMemory(histogram,0,GetCacheViewMaximumThreads()*
+    sizeof(*histogram));
+  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  {
+    histogram[i]=(unsigned long *) AcquireQuantumMemory(count,
+      sizeof(**histogram));
+    if (histogram[i] == (unsigned long *) NULL)
+      return(DestroyHistogramThreadSet(histogram));
+  }
+  return(histogram);
+}
+
+MagickExport Image *OilPaintImage(const Image *image,const double radius,
+  ExceptionInfo *exception)
+{
+#define NumberPaintBins  256
+#define OilPaintImageTag  "OilPaint/Image"
+
+  Image
+    *paint_image;
+
+  long
+    progress,
+    y;
+
+  MagickBooleanType
+    status;
+
+  unsigned long
+    **histogram,
+    width;
+
+  ViewInfo
+    **image_view,
+    **paint_view;
+
+  /*
+    Initialize painted image attributes.
+  */
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  width=GetOptimalKernelWidth2D(radius,0.5);
+  if ((image->columns < width) || (image->rows < width))
+    ThrowImageException(OptionError,"ImageSmallerThanRadius");
+  paint_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
+  if (paint_image == (Image *) NULL)
+    return((Image *) NULL);
+  if (SetImageStorageClass(paint_image,DirectClass) == MagickFalse)
+    {
+      InheritException(exception,&paint_image->exception);
+      paint_image=DestroyImage(paint_image);
+      return((Image *) NULL);
+    }
+  histogram=AcquireHistogramThreadSet(NumberPaintBins);
+  if (histogram == (unsigned long **) NULL)
+    {
+      paint_image=DestroyImage(paint_image);
+      ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
+    }
+  /*
+    Oil paint image.
+  */
+  status=MagickTrue;
+  progress=0;
+  image_view=AcquireCacheViewThreadSet(image);
+  paint_view=AcquireCacheViewThreadSet(paint_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
+  for (y=0; y < (long) image->rows; y++)
+  {
+    const IndexPacket
+      *indexes;
+
+    IndexPacket
+      *paint_indexes;
+
+    register const PixelPacket
+      *p;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    p=AcquireCacheViewPixels(image_view[id],-((long) width/2L),y-(long)
+      (width/2L),image->columns+width,width,exception);
+    q=SetCacheViewPixels(paint_view[id],0,y,paint_image->columns,1);
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=AcquireCacheViewIndexes(image_view[id]);
+    paint_indexes=GetCacheViewIndexes(paint_view[id]);
+    for (x=0; x < (long) image->columns; x++)
+    {
+      long
+        j,
+        k,
+        v;
+
+      register long
+        i,
+        u;
+
+      unsigned long
+        count;
+
+      /*
+        Assign most frequent color.
+      */
+      i=0;
+      j=0;
+      count=0;
+      (void) ResetMagickMemory(histogram[id],0,NumberPaintBins*
+        sizeof(**histogram));
+      for (v=0; v < (long) width; v++)
+      {
+        for (u=0; u < (long) width; u++)
+        {
+          k=(long) ScaleQuantumToChar(PixelIntensityToQuantum(p+u+i));
+          histogram[id][k]++;
+          if (histogram[id][k] > count)
+            {
+              j=i+u;
+              count=histogram[id][k];
+            }
+        }
+        i+=image->columns+width;
+      }
+      *q=(*(p+j));
+      if (image->colorspace == CMYKColorspace)
+        paint_indexes[x]=indexes[x+j];
+      p++;
+      q++;
+    }
+    if (SyncCacheViewPixels(paint_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,OilPaintImageTag,progress++,image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
+  }
+  paint_view=DestroyCacheViewThreadSet(paint_view);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  histogram=DestroyHistogramThreadSet(histogram);
+  if (status == MagickFalse)
+    paint_image=DestroyImage(paint_image);
+  return(paint_image);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%     O p a q u e P a i n t I m a g e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  OpaquePaintImage() changes any pixel that matches color with the color
 %  defined by fill.
 %
 %  By default color must match a particular pixel color exactly.  However,
@@ -377,59 +699,55 @@ MagickExport MagickBooleanType PaintFloodfillImage(Image *image,
 %  For example, set fuzz to 10 and the color red at intensities of 100 and
 %  102 respectively are now interpreted as the same color.
 %
-%  The format of the PaintOpaqueImage method is:
+%  The format of the OpaquePaintImage method is:
 %
-%      MagickBooleanType PaintOpaqueImage(Image *image,
-%        const PixelPacket *target,const PixelPacket *fill)
-%      MagickBooleanType PaintOpaqueImageChannel(Image *image,
+%      MagickBooleanType OpaquePaintImage(Image *image,
+%        const PixelPacket *target,const PixelPacket *fill,
+%        const MagickBooleanType invert)
+%      MagickBooleanType OpaquePaintImageChannel(Image *image,
 %        const ChannelType channel,const PixelPacket *target,
-%        const PixelPacket *fill)
+%        const PixelPacket *fill,const MagickBooleanType invert)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o channel: The channel(s).
+%    o channel: the channel(s).
 %
-%    o target: The RGB value of the target color.
+%    o target: the RGB value of the target color.
 %
-%    o fill: The replacement color.
+%    o fill: the replacement color.
+%
+%    o invert: paint any pixel that does not match the target color.
 %
 */
 
-MagickExport MagickBooleanType PaintOpaqueImage(Image *image,
-  const MagickPixelPacket *target,const MagickPixelPacket *fill)
+MagickExport MagickBooleanType OpaquePaintImage(Image *image,
+  const MagickPixelPacket *target,const MagickPixelPacket *fill,
+  const MagickBooleanType invert)
 {
-  return(PaintOpaqueImageChannel(image,DefaultChannels,target,fill));
+  return(OpaquePaintImageChannel(image,AllChannels,target,fill,invert));
 }
 
-MagickExport MagickBooleanType PaintOpaqueImageChannel(Image *image,
+MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
   const ChannelType channel,const MagickPixelPacket *target,
-  const MagickPixelPacket *fill)
+  const MagickPixelPacket *fill,const MagickBooleanType invert)
 {
-#define PaintOpaqueImageTag  "Opaque/Image"
+#define OpaquePaintImageTag  "Opaque/Image"
 
   long
+    progress,
     y;
 
   MagickBooleanType
     status;
 
   MagickPixelPacket
-    pixel;
+    zero;
 
-  register IndexPacket
-    *indexes;
+  ViewInfo
+    **image_view;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  /*
-    Make image color opaque.
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   assert(target != (MagickPixelPacket *) NULL);
@@ -438,17 +756,46 @@ MagickExport MagickBooleanType PaintOpaqueImageChannel(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  GetMagickPixelPacket(image,&pixel);
+  /*
+    Make image color opaque.
+  */
+  status=MagickTrue;
+  progress=0;
+  GetMagickPixelPacket(image,&zero);
+  image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    MagickPixelPacket
+      pixel;
+
+    register IndexPacket
+      *indexes;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewIndexes(image_view[id]);
+    pixel=zero;
     for (x=0; x < (long) image->columns; x++)
     {
       SetMagickPixelPacket(image,q,indexes+x,&pixel);
-      if (IsMagickColorSimilar(&pixel,target) != MagickFalse)
+      if (IsMagickColorSimilar(&pixel,target) != invert)
         {
           if ((channel & RedChannel) != 0)
             q->red=RoundToQuantum(fill->red);
@@ -464,18 +811,24 @@ MagickExport MagickBooleanType PaintOpaqueImageChannel(Image *image,
         }
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(PaintOpaqueImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,OpaquePaintImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  return(MagickTrue);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  return(status);
 }
 
 /*
@@ -483,13 +836,13 @@ MagickExport MagickBooleanType PaintOpaqueImageChannel(Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%     P a i n t T r a n s p a r e n t I m a g e                               %
+%     T r a n s p a r e n t P a i n t I m a g e                               %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  PaintTransparentImage() changes the opacity value associated with any pixel
+%  TransparentPaintImage() changes the opacity value associated with any pixel
 %  that matches color to the value defined by opacity.
 %
 %  By default color must match a particular pixel color exactly.  However,
@@ -498,46 +851,42 @@ MagickExport MagickBooleanType PaintOpaqueImageChannel(Image *image,
 %  For example, set fuzz to 10 and the color red at intensities of 100 and
 %  102 respectively are now interpreted as the same color.
 %
-%  The format of the PaintTransparentImage method is:
+%  The format of the TransparentPaintImage method is:
 %
-%      MagickBooleanType PaintTransparentImage(Image *image,
-%        const MagickPixelPacket *target,const Quantum opacity)
+%      MagickBooleanType TransparentPaintImage(Image *image,
+%        const MagickPixelPacket *target,const Quantum opacity,
+%        const MagickBooleanType invert)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o target: The RGB value of the target color.
+%    o target: the target color.
 %
-%    o opacity: The replacement opacity value.
+%    o opacity: the replacement opacity value.
+%
+%    o invert: paint any pixel that does not match the target color.
 %
 */
-MagickExport MagickBooleanType PaintTransparentImage(Image *image,
-  const MagickPixelPacket *target,const Quantum opacity)
+MagickExport MagickBooleanType TransparentPaintImage(Image *image,
+  const MagickPixelPacket *target,const Quantum opacity,
+  const MagickBooleanType invert)
 {
-#define PaintTransparentImageTag  "Transparent/Image"
+#define TransparentPaintImageTag  "Transparent/Image"
 
   long
+    progress,
     y;
 
   MagickBooleanType
     status;
 
   MagickPixelPacket
-    pixel;
+    zero;
 
-  register IndexPacket
-    *indexes;
+  ViewInfo
+    **image_view;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  /*
-    Make image color transparent.
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   assert(target != (MagickPixelPacket *) NULL);
@@ -546,31 +895,66 @@ MagickExport MagickBooleanType PaintTransparentImage(Image *image,
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
   if (image->matte == MagickFalse)
-    (void) SetImageOpacity(image,OpaqueOpacity);
-  GetMagickPixelPacket(image,&pixel);
+    (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
+  /*
+    Make image color transparent.
+  */
+  status=MagickTrue;
+  progress=0;
+  GetMagickPixelPacket(image,&zero);
+  image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    MagickPixelPacket
+      pixel;
+
+    register IndexPacket
+      *indexes;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewIndexes(image_view[id]);
+    pixel=zero;
     for (x=0; x < (long) image->columns; x++)
     {
       SetMagickPixelPacket(image,q,indexes+x,&pixel);
-      if (IsMagickColorSimilar(&pixel,target) != MagickFalse)
+      if (IsMagickColorSimilar(&pixel,target) != invert)
         q->opacity=opacity;
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(PaintTransparentImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  return(MagickTrue);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  return(status);
 }
