@@ -25,13 +25,20 @@ extern "C" {
 #include "magick/color.h"
 #include "magick/draw.h"
 
+extern MagickExport Image
+  *OilPaintImage(const Image *,const double,ExceptionInfo *);
+
 extern MagickExport MagickBooleanType
-  PaintFloodfillImage(Image *,const ChannelType,const MagickPixelPacket *,
-    const long,const long,const DrawInfo *,const PaintMethod),
-  PaintOpaqueImage(Image *,const MagickPixelPacket *,const MagickPixelPacket *),
-  PaintOpaqueImageChannel(Image *,const ChannelType,const MagickPixelPacket *,
-    const MagickPixelPacket *),
-  PaintTransparentImage(Image *,const MagickPixelPacket *,const Quantum);
+  FloodfillPaintImage(Image *,const ChannelType,const DrawInfo *,
+    const MagickPixelPacket *,const long,const long,const MagickBooleanType),
+  GradientImage(Image *,const GradientType,const SpreadMethod,
+    const PixelPacket *,const PixelPacket *),
+  OpaquePaintImage(Image *,const MagickPixelPacket *,const MagickPixelPacket *,
+    const MagickBooleanType),
+  OpaquePaintImageChannel(Image *,const ChannelType,const MagickPixelPacket *,
+    const MagickPixelPacket *,const MagickBooleanType),
+  TransparentPaintImage(Image *,const MagickPixelPacket *,
+    const Quantum,const MagickBooleanType);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
