@@ -10,7 +10,7 @@
 %               P      R  R    OOO   F      IIIII  LLLLL  EEEEE               %
 %                                                                             %
 %                                                                             %
-%                      ImageMagick Image Profile Methods                      %
+%                       MagickCore Image Profile Methods                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -84,9 +84,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o clone_image: The clone image.
+%    o clone_image: the clone image.
 %
 */
 MagickExport MagickBooleanType CloneImageProfiles(Image *image,
@@ -127,9 +127,9 @@ MagickExport MagickBooleanType CloneImageProfiles(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o name: The profile name.
+%    o name: the profile name.
 %
 */
 MagickExport MagickBooleanType DeleteImageProfile(Image *image,
@@ -179,7 +179,7 @@ MagickExport MagickBooleanType DeleteImageProfile(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport void DestroyImageProfiles(Image *image)
@@ -207,9 +207,9 @@ MagickExport void DestroyImageProfiles(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o name: The profile name.
+%    o name: the profile name.
 %
 */
 MagickExport const StringInfo *GetImageProfile(const Image *image,
@@ -252,7 +252,7 @@ MagickExport const StringInfo *GetImageProfile(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o hash_info: The hash info.
+%    o hash_info: the hash info.
 %
 */
 MagickExport char *GetNextImageProfile(const Image *image)
@@ -278,12 +278,12 @@ MagickExport char *GetNextImageProfile(const Image *image)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  ProfileImage() associates, applies, or removes an ICM, IPTC, or generic
-%  profile with/to/from an image.  If the profile is NULL, it is removed from
-%  the image otherwise added or applied.  Use a name of '*' and a profile of
-%  NULL to remove all profiles from the image.
+%  profile with / to / from an image.  If the profile is NULL, it is removed
+%  from the image otherwise added or applied.  Use a name of '*' and a profile
+%  of NULL to remove all profiles from the image.
 %
-%  ICC and ICM profiles are are handled as follows: If the image does not
-%  have an associated color profile, the one you provide is associated with the
+%  ICC and ICM profiles are handled as follows: If the image does not have
+%  an associated color profile, the one you provide is associated with the
 %  image and the image pixels are not transformed.  Otherwise, the colorspace
 %  transform defined by the existing and new profile are applied to the image
 %  pixels and the new profile is associated with the image.
@@ -295,13 +295,13 @@ MagickExport char *GetNextImageProfile(const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o name: Name of profile to add or remove: ICC, IPTC, or generic profile.
 %
-%    o datum: The profile data.
+%    o datum: the profile data.
 %
-%    o length: The length of the profile.
+%    o length: the length of the profile.
 %
 %    o clone: should be MagickFalse.
 %
@@ -769,7 +769,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
         }
 #if !defined(MAGICKCORE_LCMS_DELEGATE)
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateWarning,"LCMSLibraryIsNotAvailable","`%s'",
+        MissingDelegateWarning,"DelegateLibrarySupportNotBuiltIn","`%s' (LCMS)",
         image->filename);
 #else
       if (icc_profile != (StringInfo *) NULL)
@@ -967,11 +967,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           (void) cmsCloseProfile(source_profile);
           (void) cmsCloseProfile(target_profile);
           if (transform == (cmsHTRANSFORM) NULL)
-            {
-              cmsDeleteTransform(transform);
-              ThrowBinaryException(ImageError,"UnableToCreateColorTransform",
-                name);
-            }
+            ThrowBinaryException(ImageError,"UnableToCreateColorTransform",
+              name);
           /*
             Transform image as dictated by the source and target image profiles.
           */
@@ -1038,6 +1035,29 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               break;
           }
           image->colorspace=target_colorspace;
+          switch (cmsGetColorSpace(target_profile))
+          {
+            case icSigRgbData:
+            {
+              image->type=image->matte == MagickFalse ? TrueColorType :
+                TrueColorMatteType;
+              break;
+            }
+            case icSigCmykData:
+            {
+              image->type=image->matte == MagickFalse ? ColorSeparationType :
+                ColorSeparationMatteType;
+              break;
+            }
+            case icSigGrayData:
+            {
+              image->type=image->matte == MagickFalse ? GrayscaleType :
+                GrayscaleMatteType;
+              break;
+            }
+            default:
+              break;
+          }
           target_pixels=(unsigned short *) RelinquishMagickMemory(
             target_pixels);
           source_pixels=(unsigned short *) RelinquishMagickMemory(
@@ -1071,9 +1091,9 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o name: The profile name.
+%    o name: the profile name.
 %
 */
 MagickExport StringInfo *RemoveImageProfile(Image *image,const char *name)
@@ -1129,7 +1149,7 @@ MagickExport StringInfo *RemoveImageProfile(Image *image,const char *name)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport void ResetImageProfileIterator(const Image *image)
@@ -1165,9 +1185,10 @@ MagickExport void ResetImageProfileIterator(const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o name: The profile name.
+%    o name: the profile name, for example icc, exif, and 8bim (8bim is the
+%      Photoshop wrapper for iptc profiles).
 %
 %    o profile: A StringInfo structure that contains the named profile.
 %
@@ -1178,6 +1199,152 @@ static void *DestroyProfile(void *profile)
   return((void *) DestroyStringInfo((StringInfo *) profile));
 }
 
+static inline const unsigned char *ReadResourceByte(const unsigned char *p,
+  unsigned char *quantum)
+{
+  *quantum=(*p++);
+  return(p);
+}
+
+static inline const unsigned char *ReadResourceBytes(const unsigned char *p,
+  const ssize_t count,unsigned char *quantum)
+{
+  register ssize_t
+    i;
+
+  for (i=0; i < count; i++)
+    *quantum++=(*p++);
+  return(p);
+}
+
+static inline const unsigned char *ReadResourceLong(const unsigned char *p,
+  unsigned long *quantum)
+{
+  *quantum=(unsigned long) (*p++ << 24);
+  *quantum|=(unsigned long) (*p++ << 16);
+  *quantum|=(unsigned long) (*p++ << 8);
+  *quantum|=(unsigned long) (*p++ << 0);
+  return(p);
+}
+
+static inline const unsigned char *ReadResourceShort(const unsigned char *p,
+  unsigned short *quantum)
+{
+  *quantum=(unsigned short) (*p++ << 8);
+  *quantum|=(unsigned short) (*p++ << 0);
+  return(p);
+}
+
+static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
+  const StringInfo *resource_block)
+{
+  const unsigned char
+    *datum;
+
+  register const unsigned char
+    *p;
+
+  size_t
+    length;
+
+  StringInfo
+    *profile;
+
+  unsigned char
+    length_byte;
+
+  unsigned long
+    count;
+
+  unsigned short
+    id;
+
+  datum=GetStringInfoDatum(resource_block);
+  length=GetStringInfoLength(resource_block);
+  for (p=datum; p < (datum+length-7); )
+  {
+    if (LocaleNCompare((char *) p,"8BIM",4) != 0)
+      break;
+    p+=4;
+    p=ReadResourceShort(p,&id);
+    p=ReadResourceByte(p,&length_byte);
+    p+=length_byte;
+    if (((length_byte+1) & 0x01) != 0)
+      p++;
+    if (p > (datum+length-4))
+      break;
+    p=ReadResourceLong(p,&count);
+    if (p > (datum+length-count))
+      break;
+    switch (id)
+    {
+      case 0x03ed:
+      {
+        unsigned short
+          resolution;
+
+        /*
+          Resolution.
+        */
+        p=ReadResourceShort(p,&resolution)+6;
+        image->x_resolution=(double) resolution;
+        p=ReadResourceShort(p,&resolution)+6;
+        image->y_resolution=(double) resolution;
+        break;
+      }
+      case 0x040c:
+      {
+        /*
+          Thumbnail.
+        */
+        p+=count;
+        break;
+      }
+      case 0x040f:
+      {
+        /*
+          ICC Profile.
+        */
+        profile=AcquireStringInfo(count);
+        SetStringInfoDatum(profile,p);
+        (void) SetImageProfile(image,"icc",profile);
+        p+=count;
+        break;
+      }
+      case 0x0422:
+      {
+        /*
+          EXIF Profile.
+        */
+        profile=AcquireStringInfo(count);
+        SetStringInfoDatum(profile,p);
+        (void) SetImageProfile(image,"exif",profile);
+        p+=count;
+        break;
+      }
+      case 0x0424:
+      {
+        /*
+          XMP Profile.
+        */
+        profile=AcquireStringInfo(count);
+        SetStringInfoDatum(profile,p);
+        (void) SetImageProfile(image,"xmp",profile);
+        p+=count;
+        break;
+      }
+      default:
+      {
+        p+=count;
+        break;
+      }
+    }
+    if ((count & 0x01) != 0)
+      p++;
+  }
+  return(MagickTrue);
+}
+
 MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
   const StringInfo *profile)
 {
@@ -1229,6 +1396,7 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
           image->iptc_profile.length=GetStringInfoLength(iptc_profile);
           image->iptc_profile.info=GetStringInfoDatum(iptc_profile);
         }
+      (void) GetProfilesFromResourceBlock(image,profile);
     }
   /*
     Inject profile into image properties.
@@ -1258,7 +1426,7 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 
@@ -1432,7 +1600,7 @@ MagickExport MagickBooleanType SyncImageProfiles(Image *image)
   if (ReadProfileShort(endian,exif+2) != 0x002a)
     return(MagickFalse);
   /*
-    This is the offset to the first IFD.
+    This the offset to the first IFD.
   */
   offset=(ssize_t) ReadProfileLong(endian,exif+4);
   if ((size_t) offset >= length)
