@@ -10,7 +10,7 @@
 %            P       R  R    OOO   P      EEEEE  R  R     T      Y            %
 %                                                                             %
 %                                                                             %
-%                        ImageMagick Property Methods                         %
+%                         MagickCore Property Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -65,7 +65,7 @@
 #include "magick/quantum.h"
 #include "magick/resource_.h"
 #include "magick/splay-tree.h"
-#include "magick/signature.h"
+#include "magick/signature-private.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -92,9 +92,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o clone_image: The clone image.
+%    o clone_image: the clone image.
 %
 */
 MagickExport MagickBooleanType CloneImageProperties(Image *image,
@@ -156,8 +156,13 @@ MagickExport MagickBooleanType CloneImageProperties(Image *image,
   image->error=clone_image->error;
   image->signature=clone_image->signature;
   if (clone_image->properties != (void *) NULL)
-    image->properties=CloneSplayTree((SplayTreeInfo *) clone_image->properties,
-      (void *(*)(void *)) ConstantString,(void *(*)(void *)) ConstantString);
+    {
+      if (image->properties != (void *) NULL)
+        DestroyImageProperties(image);
+      image->properties=CloneSplayTree((SplayTreeInfo *)
+        clone_image->properties,(void *(*)(void *)) ConstantString,
+        (void *(*)(void *)) ConstantString);
+    }
   return(MagickTrue);
 }
 
@@ -181,9 +186,9 @@ MagickExport MagickBooleanType CloneImageProperties(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o property: The image property.
+%    o property: the image property.
 %
 */
 MagickExport MagickBooleanType DefineImageProperty(Image *image,
@@ -228,9 +233,9 @@ MagickExport MagickBooleanType DefineImageProperty(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o property: The image property.
+%    o property: the image property.
 %
 */
 MagickExport MagickBooleanType DeleteImageProperty(Image *image,
@@ -266,7 +271,7 @@ MagickExport MagickBooleanType DeleteImageProperty(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport void DestroyImageProperties(Image *image)
@@ -364,9 +369,9 @@ MagickExport MagickBooleanType FormatImageProperty(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o key: The key.
+%    o key: the key.
 %
 */
 
@@ -891,10 +896,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
       {  0x80e5, "exif:ImageDepth" },
       {  0x80e6, "exif:TileDepth" },
       {  0x828d, "exif:CFARepeatPatternDim" },
-      {  0x828e, "exif:CFAPattern" },
-      {  0x828f, "exif:BatteryLevel" },
-      {  0x828d, "exif:CFARepeatPatternDim" },
-      {  0x828e, "exif:CFAPattern" },
+      {  0x828e, "exif:CFAPattern2" },
       {  0x828f, "exif:BatteryLevel" },
       {  0x8298, "exif:Copyright" },
       {  0x829a, "exif:ExposureTime" },
@@ -936,7 +938,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
       {  0x9213, "exif:ImageHistory" },
       {  0x9214, "exif:SubjectArea" },
       {  0x9215, "exif:ExposureIndex" },
-      {  0x9216, "exif:TIFF/EPStandardID" },
+      {  0x9216, "exif:TIFF-EPStandardID" },
       {  0x927c, "exif:MakerNote" },
       {  0x9C9b, "exif:WinXP-Title" },
       {  0x9C9c, "exif:WinXP-Comments" },
@@ -1093,7 +1095,6 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
       /*
         Try to match the text with a tag name instead.
       */
-      property+=5;
       for (i=0; ; i++)
       {
         if (Tags[i].tag == 0)
@@ -1141,7 +1142,7 @@ static MagickBooleanType GetEXIFProperty(const Image *image,
   if (ReadPropertyShort(endian,exif+2) != 0x002a)
     return(MagickFalse);
   /*
-    This is the offset to the first IFD.
+    This the offset to the first IFD.
   */
   offset=(ssize_t) ReadPropertyLong(endian,exif+4);
   if ((size_t) offset >= length)
@@ -1394,14 +1395,17 @@ static MagickBooleanType GetXMPProperty(const Image *image,
   MagickBooleanType
     status;
 
+  register const char
+    *p;
+
   register long
     i;
 
-  register unsigned char
-    *p;
-
   XMLTreeInfo
     *child,
+    *description,
+    *node,
+    *rdf,
     *xmp;
 
   profile=GetImageProfile(image,"xmp");
@@ -1409,29 +1413,48 @@ static MagickBooleanType GetXMPProperty(const Image *image,
     return(MagickFalse);
   if ((property == (const char *) NULL) || (*property == '\0'))
     return(MagickFalse);
-  p=GetStringInfoDatum(profile);
-  for (i=0; (i < (long) GetStringInfoLength(profile)) && (*p != '\0'); i++)
-    p++;
-  if (i == (long) GetStringInfoLength(profile))
-    return(MagickFalse);
   exception=AcquireExceptionInfo();
-  xmp=NewXMLTree((char *) ++p,exception);
+  p=(const char *) GetStringInfoDatum(profile);
+  for (i=0; i < (long) (GetStringInfoLength(profile)-1); i++)
+  {
+    if ((*p == '<') && (*(p+1) == 'x'))
+      break;
+    p++;
+  }
+  xmp=NewXMLTree((char *) p,exception);
   exception=DestroyExceptionInfo(exception);
   if (xmp == (XMLTreeInfo *) NULL)
     return(MagickFalse);
   status=MagickFalse;
-  child=GetXMLTreeChild(xmp,"rdf:RDF");
-  if (child != (XMLTreeInfo *) NULL)
+  rdf=GetXMLTreeChild(xmp,"rdf:RDF");
+  if (rdf != (XMLTreeInfo *) NULL)
     {
-      child=GetXMLTreeChild(child,"rdf:Description");
-      if (child != (XMLTreeInfo *) NULL)
+      if (image->properties == (void *) NULL)
+        ((Image *) image)->properties=NewSplayTree(CompareSplayTreeString,
+          RelinquishMagickMemory,RelinquishMagickMemory);
+      description=GetXMLTreeChild(rdf,"rdf:Description");
+      while (description != (XMLTreeInfo *) NULL)
+      {
+        node=GetXMLTreeChild(description,(const char *) NULL);
+        while (node != (XMLTreeInfo *) NULL)
         {
-          if (image->properties == (void *) NULL)
-            ((Image *) image)->properties=NewSplayTree(CompareSplayTreeString,
-              RelinquishMagickMemory,RelinquishMagickMemory);
-          status=GetXMLTreeAttributes(child,(SplayTreeInfo *)
-            image->properties);
+          child=GetXMLTreeChild(node,(const char *) NULL);
+          if (child == (XMLTreeInfo *) NULL)
+            (void) AddValueToSplayTree((SplayTreeInfo *) image->properties,
+              ConstantString(GetXMLTreeTag(node)),
+              ConstantString(GetXMLTreeContent(node)));
+          while (child != (XMLTreeInfo *) NULL)
+          {
+            if (LocaleCompare(GetXMLTreeTag(child),"rdf:Seq") != 0)
+              (void) AddValueToSplayTree((SplayTreeInfo *) image->properties,
+                ConstantString(GetXMLTreeTag(child)),
+                ConstantString(GetXMLTreeContent(child)));
+            child=GetXMLTreeSibling(child);
+          }
+          node=GetXMLTreeSibling(node);
         }
+        description=GetNextXMLTreeTag(description);
+      }
     }
   xmp=DestroyXMLTree(xmp);
   return(status);
@@ -1825,7 +1848,8 @@ MagickExport const char *GetImageProperty(const Image *image,
         image->properties);
       return(p);
     }
-  if (image->properties != (void *) NULL)
+  if ((image->properties != (void *) NULL) &&
+      (LocaleNCompare("fx:",property,3) != 0))
     {
       p=(const char *) GetValueFromSplayTree((SplayTreeInfo *)
         image->properties,property);
@@ -1880,11 +1904,10 @@ MagickExport const char *GetImageProperty(const Image *image,
               (void) FormatMagickString(value,MaxTextExtent,"%g",(double)
                 alpha);
               (void) SetImageProperty((Image *) image,property,value);
-              return(GetImageProperty(image,property));
-              p=(const char *) GetValueFromSplayTree((SplayTreeInfo *)
-                image->properties,property);
-              return(p);
             }
+          p=(const char *) GetValueFromSplayTree((SplayTreeInfo *)
+            image->properties,property);
+          return(p);
         }
       break;
     }
@@ -1936,8 +1959,8 @@ MagickExport const char *GetImageProperty(const Image *image,
               char
                 name[MaxTextExtent];
 
-              (void) QueryMagickColorname(image,&pixel,SVGCompliance,
-                MagickFalse,name,exception);
+              (void) QueryMagickColorname(image,&pixel,SVGCompliance,name,
+                exception);
               (void) SetImageProperty((Image *) image,property,name);
               return(GetImageProperty(image,property));
             }
@@ -1981,7 +2004,7 @@ MagickExport const char *GetImageProperty(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport char *GetNextImageProperty(const Image *image)
@@ -2017,11 +2040,11 @@ MagickExport char *GetNextImageProperty(const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o embed_text: The address of a character string containing the embedded
+%    o embed_text: the address of a character string containing the embedded
 %      formatting characters.
 %
 */
@@ -2046,6 +2069,38 @@ static const char *GetMagickProperty(const ImageInfo *image_info,Image *image,
         }
       break;
     }
+    case 'c':
+    {
+      if (LocaleNCompare("channels",property,8) == 0)
+        {
+          /*
+            Image channels.
+          */
+          (void) FormatMagickString(value,MaxTextExtent,"%s",
+            MagickOptionToMnemonic(MagickColorspaceOptions,(long)
+            image->colorspace));
+          LocaleLower(value);
+          if (image->matte != MagickFalse)
+            (void) ConcatenateMagickString(value,"a",MaxTextExtent);
+          break;
+        }
+      if (LocaleNCompare("colorspace",property,10) == 0)
+        {
+          ColorspaceType
+            colorspace;
+
+          /*
+            Image storage class and colorspace.
+          */
+          colorspace=image->colorspace;
+          if (IsGrayImage(image,&image->exception) != MagickFalse)
+            colorspace=GRAYColorspace;
+          (void) FormatMagickString(value,MaxTextExtent,"%s",
+            MagickOptionToMnemonic(MagickColorspaceOptions,(long) colorspace));
+          break;
+        }
+      break;
+    }
     case 'd':
     {
       if (LocaleNCompare("depth",property,5) == 0)
@@ -2107,6 +2162,39 @@ static const char *GetMagickProperty(const ImageInfo *image_info,Image *image,
           (void) CopyMagickString(value,image->magick,MaxTextExtent);
           break;
         }
+      if (LocaleNCompare("max",property,3) == 0)
+        {
+          double
+            maximum,
+            minimum;
+
+          (void) GetImageChannelRange(image,image_info->channel,&minimum,
+            &maximum,&image->exception);
+          (void) FormatMagickString(value,MaxTextExtent,"%g",maximum);
+          break;
+        }
+      if (LocaleNCompare("mean",property,4) == 0)
+        {
+          double
+            mean,
+            standard_deviation;
+
+          (void) GetImageChannelMean(image,image_info->channel,&mean,
+            &standard_deviation,&image->exception);
+          (void) FormatMagickString(value,MaxTextExtent,"%g",mean);
+          break;
+        }
+      if (LocaleNCompare("min",property,3) == 0)
+        {
+          double
+            maximum,
+            minimum;
+
+          (void) GetImageChannelRange(image,image_info->channel,&minimum,
+            &maximum,&image->exception);
+          (void) FormatMagickString(value,MaxTextExtent,"%g",minimum);
+          break;
+        }
       break;
     }
     case 'n':
@@ -2118,33 +2206,6 @@ static const char *GetMagickProperty(const ImageInfo *image_info,Image *image,
         }
      break;
     }
-    case 's':
-    {
-      if (LocaleNCompare("size",property,4) == 0)
-        {
-          char
-            format[MaxTextExtent];
-
-          (void) FormatMagickSize(GetBlobSize(image),format);
-          (void) FormatMagickString(value,MaxTextExtent,"%s",format);
-          break;
-        }
-      if (LocaleNCompare("scene",property,5) == 0)
-        {
-          (void) FormatMagickString(value,MaxTextExtent,"%lu",image->scene);
-          if (image_info->number_scenes != 0)
-            (void) FormatMagickString(value,MaxTextExtent,"%lu",
-              image_info->scene);
-          break;
-        }
-      if (LocaleNCompare("scenes",property,6) == 0)
-        {
-          (void) FormatMagickString(value,MaxTextExtent,"%lu",
-            (unsigned long) GetImageListLength(image));
-          break;
-        }
-       break;
-    }
     case 'o':
     {
       if (LocaleNCompare("output",property,6) == 0)
@@ -2172,6 +2233,46 @@ static const char *GetMagickProperty(const ImageInfo *image_info,Image *image,
         }
       break;
     }
+    case 's':
+    {
+      if (LocaleNCompare("size",property,4) == 0)
+        {
+          char
+            format[MaxTextExtent];
+
+          (void) FormatMagickSize(GetBlobSize(image),format);
+          (void) FormatMagickString(value,MaxTextExtent,"%s",format);
+          break;
+        }
+      if (LocaleNCompare("scene",property,5) == 0)
+        {
+          (void) FormatMagickString(value,MaxTextExtent,"%lu",image->scene);
+          if (image_info->number_scenes != 0)
+            (void) FormatMagickString(value,MaxTextExtent,"%lu",
+              image_info->scene);
+          break;
+        }
+      if (LocaleNCompare("scenes",property,6) == 0)
+        {
+          (void) FormatMagickString(value,MaxTextExtent,"%lu",
+            (unsigned long) GetImageListLength(image));
+          break;
+        }
+      if ((LocaleNCompare("standard-deviation",property,18) == 0) ||
+          (LocaleNCompare("standard_deviation",property,18) == 0))
+        {
+          double
+            mean,
+            standard_deviation;
+
+          (void) GetImageChannelMean(image,image_info->channel,&mean,
+            &standard_deviation,&image->exception);
+          (void) FormatMagickString(value,MaxTextExtent,"%g",
+            standard_deviation);
+          break;
+        }
+       break;
+    }
     case 'u':
     {
       if (LocaleNCompare("unique",property,6) == 0)
@@ -2260,7 +2361,7 @@ MagickExport char *InterpretImageProperties(const ImageInfo *image_info,
     return((char *) NULL);
   text=(char *) embed_text;
   if ((*text == '@') && ((*(text+1) == '-') ||
-      (IsAccessible(text+1) != MagickFalse)))
+      (IsPathAccessible(text+1) != MagickFalse)))
     return(FileToString(embed_text+1,~0,&image->exception));
   /*
     Translate any embedded format characters.
@@ -2583,10 +2684,18 @@ MagickExport char *InterpretImageProperties(const ImageInfo *image_info,
       case 'z':
       {
         /*
-          Unique filename.
+          Image depth.
         */
-        q+=FormatMagickString(q,extent,"%lu",GetImageDepth(image,
-          &image->exception));
+        q+=FormatMagickString(q,extent,"%lu",image->depth);
+        break;
+      }
+      case 'A':
+      {
+        /*
+          Image alpha channel.
+        */
+        q+=FormatMagickString(q,extent,"%s",MagickOptionToMnemonic(
+          MagickBooleanOptions,(long) image->matte));
         break;
       }
       case 'C':
@@ -2607,6 +2716,11 @@ MagickExport char *InterpretImageProperties(const ImageInfo *image_info,
           MagickDisposeOptions,(long) image->dispose));
         break;
       }
+      case 'H':
+      {
+        q+=FormatMagickString(q,extent,"%ld",image->page.height);
+        break;
+      }
       case 'O':
       {
         q+=FormatMagickString(q,extent,"%+ld%+ld",image->page.x,image->page.y);
@@ -2628,6 +2742,30 @@ MagickExport char *InterpretImageProperties(const ImageInfo *image_info,
         q+=FormatMagickString(q,extent,"%lu",image->delay);
         break;
       }
+      case 'W':
+      {
+        q+=FormatMagickString(q,extent,"%ld",image->page.width);
+        break;
+      }
+      case 'X':
+      {
+        q+=FormatMagickString(q,extent,"%+ld",image->page.x);
+        break;
+      }
+      case 'Y':
+      {
+        q+=FormatMagickString(q,extent,"%+ld",image->page.y);
+        break;
+      }
+      case 'Z':
+      {
+        /*
+          Unique filename.
+        */
+        (void) CopyMagickString(filename,text_info->zero,extent);
+        q+=CopyMagickString(q,filename,extent);
+        break;
+      }
       case '[':
       {
         char
@@ -2810,9 +2948,9 @@ MagickExport char *InterpretImageProperties(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o property: The image property.
+%    o property: the image property.
 %
 */
 MagickExport char *RemoveImageProperty(Image *image,
@@ -2854,7 +2992,7 @@ MagickExport char *RemoveImageProperty(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport void ResetImagePropertyIterator(const Image *image)
@@ -2889,11 +3027,11 @@ MagickExport void ResetImagePropertyIterator(const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o property: The image property.
+%    o property: the image property.
 %
-%    o values: The image property values.
+%    o values: the image property values.
 %
 */
 MagickExport MagickBooleanType SetImageProperty(Image *image,
@@ -2919,6 +3057,33 @@ MagickExport MagickBooleanType SetImageProperty(Image *image,
     ConstantString(property),ConstantString(value));
   switch (*property)
   {
+    case 'B':
+    case 'b':
+    {
+      if (LocaleCompare(property,"bias") == 0)
+        {
+          image->bias=StringToDouble(value,QuantumRange);
+          break;
+        }
+      break;
+    }
+    case 'C':
+    case 'c':
+    {
+      if (LocaleCompare(property,"colorspace") == 0)
+        {
+          image->colorspace=(ColorspaceType) ParseMagickOption(
+            MagickColorspaceOptions,MagickFalse,value);
+          break;
+        }
+      if (LocaleCompare(property,"compose") == 0)
+        {
+          image->compose=(CompositeOperator) ParseMagickOption(
+            MagickComposeOptions,MagickFalse,value);
+          break;
+        }
+      break;
+    }
     case 'D':
     case 'd':
     {
@@ -2986,6 +3151,16 @@ MagickExport MagickBooleanType SetImageProperty(Image *image,
         }
       break;
     }
+    case 'L':
+    case 'l':
+    {
+      if (LocaleCompare(property,"loop") == 0)
+        {
+          image->iterations=atol(value);
+          break;
+        }
+      break;
+    }
     case 'P':
     case 'p':
     {
