@@ -10,7 +10,7 @@
 %           R  R    EEEEE   SSSSS   OOO    UUU   R  R    CCCC  EEEEE          %
 %                                                                             %
 %                                                                             %
-%                        Get/Set ImageMagick Resources.                       %
+%                        Get/Set MagickCore Resources                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/configure.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/hashmap.h"
@@ -51,7 +52,7 @@
 #include "magick/registry.h"
 #include "magick/resource_.h"
 #include "magick/semaphore.h"
-#include "magick/signature.h"
+#include "magick/signature-private.h"
 #include "magick/string_.h"
 #include "magick/splay-tree.h"
 #include "magick/token.h"
@@ -80,6 +81,9 @@ typedef struct _ResourceInfo
 /*
   Global declarations.
 */
+static RandomInfo
+  *random_info = (RandomInfo *) NULL;
+
 static ResourceInfo
   resource_info =
   {
@@ -123,9 +127,9 @@ static SplayTreeInfo
 %
 %  A description of each parameter follows:
 %
-%    o type: The type of resource.
+%    o type: the type of resource.
 %
-%    o size: The number of bytes needed from for this resource.
+%    o size: the number of bytes needed from for this resource.
 %
 */
 MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
@@ -294,29 +298,29 @@ static MagickBooleanType GetPathTemplate(char *path)
   char
     *directory;
 
-  const char
-    *temporary_path;
-
-  ExceptionInfo
-    *exception;
-
-  int
+  MagickBooleanType
     status;
 
   register char
     *p;
 
   struct stat
-    file_info;
+    attributes;
 
   (void) CopyMagickString(path,"magick-XXXXXXXX",MaxTextExtent);
-  directory=(char *) NULL;
-  exception=AcquireExceptionInfo();
-  temporary_path=(const char *) GetImageRegistry(StringRegistryType,
-    "temporary-path",exception);
-  exception=DestroyExceptionInfo(exception);
-  if (temporary_path != (char *) NULL)
-    directory=ConstantString(temporary_path);
+  directory=GetEnvironmentValue("MAGICK_TEMPORARY_PATH");
+  if (directory == (char *) NULL)
+    directory=GetConfigureOption("MAGICK_TEMPORARY_PATH");
+  if (directory == (char *) NULL)
+    {
+      ExceptionInfo
+        *exception;
+
+      exception=AcquireExceptionInfo();
+      directory=(char *) GetImageRegistry(StringRegistryType,"temporary-path",
+        exception);
+      exception=DestroyExceptionInfo(exception);
+    }
   if (directory == (char *) NULL)
     directory=GetEnvironmentValue("MAGICK_TMPDIR");
   if (directory == (char *) NULL)
@@ -342,8 +346,8 @@ static MagickBooleanType GetPathTemplate(char *path)
       directory=DestroyString(directory);
       return(MagickTrue);
     }
-  status=stat(directory,&file_info);
-  if ((status != 0) || !S_ISDIR(file_info.st_mode))
+  status=GetPathAttributes(directory,&attributes);
+  if ((status == MagickFalse) || !S_ISDIR(attributes.st_mode))
     {
       directory=DestroyString(directory);
       return(MagickTrue);
@@ -385,13 +389,18 @@ MagickExport int AcquireUniqueFileResource(char *path)
 
   static const char
     portable_filename[65] =
-      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789._";
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_-";
+
+  StringInfo
+    *key;
 
   unsigned char
-    key[8];
+    *datum;
 
   assert(path != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s",path);
+  if (random_info == (RandomInfo *) NULL)
+    random_info=AcquireRandomInfo();
   file=(-1);
   for (i=0; i < TMP_MAX; i++)
   {
@@ -404,13 +413,15 @@ MagickExport int AcquireUniqueFileResource(char *path)
     if (file != -1)
       break;
 #endif
-    GetRandomKey(key,8);
+    key=GetRandomKey(random_info,8);
     p=path+strlen(path)-8;
+    datum=GetStringInfoDatum(key);
     for (i=0; i < 8; i++)
     {
-      c=(int) (key[i] & 0x3f);
+      c=(int) (datum[i] & 0x3f);
       *p++=portable_filename[c];
     }
+    key=DestroyStringInfo(key);
     file=open(path,O_RDWR | O_CREAT | O_EXCL | O_BINARY | O_NOFOLLOW,S_MODE);
     if ((file > 0) || (errno != EEXIST))
       break;
@@ -451,8 +462,10 @@ MagickExport void DestroyMagickResources(void)
   AcquireSemaphoreInfo(&resource_semaphore);
   if (temporary_resources != (SplayTreeInfo *) NULL)
     temporary_resources=DestroySplayTree(temporary_resources);
+  if (random_info != (RandomInfo *) NULL)
+    random_info=DestroyRandomInfo(random_info);
   RelinquishSemaphoreInfo(resource_semaphore);
-  resource_semaphore=DestroySemaphoreInfo(resource_semaphore);
+  DestroySemaphoreInfo(&resource_semaphore);
 }
 
 /*
@@ -466,7 +479,7 @@ MagickExport void DestroyMagickResources(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetMagickResource() returns the the specified resource.
+%  GetMagickResource() returns the specified resource.
 %
 %  The format of the GetMagickResource() method is:
 %
@@ -474,7 +487,7 @@ MagickExport void DestroyMagickResources(void)
 %
 %  A description of each parameter follows:
 %
-%    o type: The type of resource.
+%    o type: the type of resource.
 %
 */
 MagickExport MagickSizeType GetMagickResource(const ResourceType type)
@@ -529,7 +542,7 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetMagickResource() returns the the specified resource limit.
+%  GetMagickResource() returns the specified resource limit.
 %
 %  The format of the GetMagickResourceLimit() method is:
 %
@@ -537,7 +550,7 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
 %
 %  A description of each parameter follows:
 %
-%    o type: The type of resource.
+%    o type: the type of resource.
 %
 */
 MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
@@ -644,6 +657,8 @@ MagickExport void InitializeMagickResources(void)
   (void) SetMagickResourceLimit(MemoryResource,3UL*memory/2UL);
   (void) SetMagickResourceLimit(MapResource,4UL*memory);
   limit=GetEnvironmentValue("MAGICK_AREA_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetConfigureOption("MAGICK_AREA_LIMIT");
   if (limit != (char *) NULL)
     {
       (void) SetMagickResourceLimit(AreaResource,(MagickSizeType)
@@ -651,6 +666,8 @@ MagickExport void InitializeMagickResources(void)
       limit=DestroyString(limit);
     }
   limit=GetEnvironmentValue("MAGICK_MEMORY_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetConfigureOption("MAGICK_MEMORY_LIMIT");
   if (limit != (char *) NULL)
     {
       (void) SetMagickResourceLimit(MemoryResource,(MagickSizeType)
@@ -658,6 +675,8 @@ MagickExport void InitializeMagickResources(void)
       limit=DestroyString(limit);
     }
   limit=GetEnvironmentValue("MAGICK_MAP_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetConfigureOption("MAGICK_MAP_LIMIT");
   if (limit != (char *) NULL)
     {
       (void) SetMagickResourceLimit(MapResource,(MagickSizeType)
@@ -665,6 +684,8 @@ MagickExport void InitializeMagickResources(void)
       limit=DestroyString(limit);
     }
   limit=GetEnvironmentValue("MAGICK_DISK_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetConfigureOption("MAGICK_DISK_LIMIT");
   if (limit != (char *) NULL)
     {
       (void) SetMagickResourceLimit(DiskResource,(MagickSizeType)
@@ -677,14 +698,25 @@ MagickExport void InitializeMagickResources(void)
 #elif defined(MAGICKCORE_HAVE_GETDTABLESIZE) && defined(MAGICKCORE_POSIX_SUPPORT)
   files=getdtablesize();
 #endif
-  (void) SetMagickResourceLimit(FileResource,MagickMax(3UL*files/4UL,64UL));
+  (void) SetMagickResourceLimit(FileResource,MagickMax((unsigned long)
+    (3*files/4),64));
   limit=GetEnvironmentValue("MAGICK_FILE_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetConfigureOption("MAGICK_FILE_LIMIT");
   if (limit != (char *) NULL)
     {
       (void) SetMagickResourceLimit(FileResource,(MagickSizeType)
         StringToDouble(limit,100.0));
       limit=DestroyString(limit);
     }
+  limit=GetEnvironmentValue("MAGICK_THREAD_LIMIT");
+  if (limit == (char *) NULL)
+    limit=GetConfigureOption("MAGICK_THREAD_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      SetCacheViewMaximumThreads((unsigned long) atol(limit));
+      limit=DestroyString(limit);
+    }
 }
 
 /*
@@ -757,9 +789,9 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
 %
 %  A description of each parameter follows:
 %
-%    o type: The type of resource.
+%    o type: the type of resource.
 %
-%    o size: The size of the resource.
+%    o size: the size of the resource.
 %
 */
 MagickExport void RelinquishMagickResource(const ResourceType type,
@@ -895,9 +927,9 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
 %
 %  A description of each parameter follows:
 %
-%    o type: The type of resource.
+%    o type: the type of resource.
 %
-%    o limit: The maximum limit for the resource.
+%    o limit: the maximum limit for the resource.
 %
 */
 MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
