@@ -10,7 +10,7 @@
 %               SSSSS  EEEEE   GGGG  M   M  EEEEE  N   N    T                 %
 %                                                                             %
 %                                                                             %
-%        Methods to Segment an Image with Thresholding Fuzzy c-Means          %
+%    MagickCore Methods to Segment an Image with Thresholding Fuzzy c-Means   %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -91,6 +91,7 @@
 #include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantize.h"
 #include "magick/quantum.h"
 #include "magick/quantum-private.h"
@@ -217,7 +218,7 @@ static void
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o extrema:  Specifies a pointer to an array of integers.  They
 %      represent the peaks and valleys of the histogram for each color
@@ -260,7 +261,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     y;
 
   MagickBooleanType
-    status;
+    proceed;
 
   MagickRealType
     distance_squared,
@@ -390,14 +391,9 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           }
       p++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   /*
     Remove clusters that do not meet minimum cluster threshold.
@@ -491,7 +487,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Allocate image colormap.
   */
-  if (AllocateImageColormap(image,number_clusters) == MagickFalse)
+  if (AcquireImageColormap(image,number_clusters) == MagickFalse)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   cluster=head;
@@ -586,17 +582,12 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     }
     if (SyncImagePixels(image) == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster)
   {
@@ -867,13 +858,13 @@ static void DerivativeHistogram(const MagickRealType *histogram,
 %
 %  The format of the GetImageDynamicThreshold method is:
 %
-%      MagickPixelPacket GetImageDynamicThreshold(const Image *image,
+%      MagickBooleanType GetImageDynamicThreshold(const Image *image,
 %        const double cluster_threshold,const double smooth_threshold,
-%        ExceptionInfo *)
+%        MagickPixelPacket *pixel,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o cluster_threshold:  This MagickRealType represents the minimum number of
 %      pixels contained in a hexahedra before it can be considered valid
@@ -883,12 +874,14 @@ static void DerivativeHistogram(const MagickRealType *histogram,
 %      derivative of the histogram.  As the value is increased, you can expect a
 %      smoother second derivative.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o pixel: return the dynamic threshold here.
+%
+%    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
+MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
   const double cluster_threshold,const double smooth_threshold,
-  ExceptionInfo *exception)
+  MagickPixelPacket *pixel,ExceptionInfo *exception)
 {
   Cluster
     *background,
@@ -909,10 +902,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
     y;
 
   MagickBooleanType
-    status;
-
-  MagickPixelPacket
-    pixel;
+    proceed;
 
   MagickRealType
     threshold;
@@ -934,7 +924,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
+  GetMagickPixelPacket(image,pixel);
   for (i=0; i < MaxDimension; i++)
   {
     histogram[i]=(long *) AcquireQuantumMemory(256UL,sizeof(**histogram));
@@ -948,7 +938,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         }
         (void) ThrowMagickException(exception,GetMagickModule(),
           ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-        return(pixel);
+        return(MagickFalse);
       }
   }
   /*
@@ -996,7 +986,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
             (void) ThrowMagickException(exception,GetMagickModule(),
               ResourceLimitError,"MemoryAllocationFailed","`%s'",
               image->filename);
-            return(pixel);
+            return(MagickFalse);
           }
         /*
           Initialize a new class.
@@ -1019,7 +1009,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-          return(pixel);
+          return(MagickFalse);
         }
       /*
         Initialize a new class.
@@ -1071,14 +1061,9 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
           }
       p++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   /*
     Remove clusters that do not meet minimum cluster threshold.
@@ -1132,16 +1117,16 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
       }
     }
   threshold=(background->red.center+object->red.center)/2.0;
-  pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+  pixel->red=(MagickRealType) ScaleCharToQuantum((unsigned char)
     (threshold+0.5));
   threshold=(background->green.center+object->green.center)/2.0;
-  pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+  pixel->green=(MagickRealType) ScaleCharToQuantum((unsigned char)
     (threshold+0.5));
   threshold=(background->blue.center+object->blue.center)/2.0;
-  pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+  pixel->blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
     (threshold+0.5));
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster)
   {
@@ -1153,7 +1138,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
     extrema[i]=(short *) RelinquishMagickMemory(extrema[i]);
     histogram[i]=(long *) RelinquishMagickMemory(histogram[i]);
   }
-  return(pixel);
+  return(MagickTrue);
 }
 
 /*
@@ -1626,7 +1611,7 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
     average_tau+=list[i]->tau;
   average_tau/=(MagickRealType) number_nodes;
   /*
-    Free resources.
+    Relinquish resources.
   */
   FreeNodes(root);
   zero_crossing=(ZeroCrossing *) RelinquishMagickMemory(zero_crossing);
@@ -1719,7 +1704,7 @@ static void ScaleSpace(const long *histogram,const MagickRealType tau,
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o colorspace: Indicate the colorspace.
 %
@@ -1792,7 +1777,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
   if (colorspace != RGBColorspace)
     (void) SetImageColorspace(image,colorspace);
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (i=0; i < MaxDimension; i++)
   {
