@@ -10,7 +10,7 @@
 %        SSSSS  EEEEE  M   M  A   A  P      H   H   OOO   R  R   EEEEE        %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick Semaphore Methods                         %
+%                        MagickCore Semaphore Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                             William Radcliffe                               %
@@ -60,19 +60,22 @@
 struct SemaphoreInfo
 {
 #if defined(MAGICKCORE_HAVE_PTHREAD)
-  pthread_mutex_t
-    mutex;
-
   pthread_t
-    id;
-#endif
-#if defined(__WINDOWS__)
-  CRITICAL_SECTION
-    mutex;
-
+#elif defined(__WINDOWS__)
   DWORD
-    id;
+#else
+  unsigned long
 #endif
+    id;
+
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  pthread_mutex_t
+#elif defined(__WINDOWS__)
+  CRITICAL_SECTION
+#else
+  unsigned long
+#endif
+    mutex;
 
   long
     reference_count;
@@ -87,11 +90,12 @@ struct SemaphoreInfo
 #if defined(MAGICKCORE_HAVE_PTHREAD)
 static pthread_mutex_t
   semaphore_mutex = PTHREAD_MUTEX_INITIALIZER;
-#endif
-
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
 static LONG
   semaphore_mutex = 0;
+#else
+static long
+  semaphore_mutex = 0;
 #endif
 
 /*
@@ -125,8 +129,7 @@ MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
       (void) fprintf(stderr,"pthread_mutex_lock failed %s\n",strerror(errno));
       _exit(1);
     }
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   while (InterlockedCompareExchange(&semaphore_mutex,1L,0L) != 0)
     Sleep(10);
 #endif
@@ -138,8 +141,7 @@ MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
       (void) fprintf(stderr,"pthread_mutex_unlock failed %s\n",strerror(errno));
       _exit(1);
     }
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   InterlockedExchange(&semaphore_mutex,0L);
 #endif
   (void) LockSemaphoreInfo(*semaphore_info);
@@ -198,16 +200,6 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
         ThrowFatalException(ResourceLimitFatalError,
           "UnableToInitializeSemaphore");
       }
-#if defined(PTHREAD_MUTEX_RECURSIVE)
-    status=pthread_mutexattr_settype(&mutex_info,PTHREAD_MUTEX_RECURSIVE);
-    if (status != 0)
-      {
-        (void) pthread_mutexattr_destroy(&mutex_info);
-        free(semaphore_info);
-        ThrowFatalException(ResourceLimitFatalError,
-          "UnableToInitializeSemaphore");
-      }
-#endif
     status=pthread_mutex_init(&semaphore_info->mutex,&mutex_info);
     (void) pthread_mutexattr_destroy(&mutex_info);
     if (status != 0)
@@ -216,12 +208,11 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
         ThrowFatalException(ResourceLimitFatalError,
           "UnableToInitializeSemaphore");
       }
-    semaphore_info->id=0;
+    semaphore_info->id=pthread_self();
   }
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   InitializeCriticalSection(&semaphore_info->mutex);
-  semaphore_info->id=0;
+  semaphore_info->id=GetCurrentThreadId();
 #endif
   semaphore_info->reference_count=0;
   semaphore_info->signature=MagickSignature;
@@ -273,40 +264,37 @@ MagickExport void DestroySemaphore(void)
 %
 %  The format of the DestroySemaphoreInfo method is:
 %
-%      SemaphoreInfo *DestroySemaphoreInfo(SemaphoreInfo *semaphore_info)
+%      void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 %
 %  A description of each parameter follows:
 %
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
 */
-MagickExport SemaphoreInfo *DestroySemaphoreInfo(SemaphoreInfo *semaphore_info)
+MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
-  assert(semaphore_info != (SemaphoreInfo *) NULL);
-  assert(semaphore_info->signature == MagickSignature);
+  assert(semaphore_info != (SemaphoreInfo **) NULL);
+  assert((*semaphore_info) != (SemaphoreInfo *) NULL);
+  assert((*semaphore_info)->signature == MagickSignature);
 #if defined(MAGICKCORE_HAVE_PTHREAD)
   (void) pthread_mutex_lock(&semaphore_mutex);
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   while (InterlockedCompareExchange(&semaphore_mutex,1L,0L) != 0)
     Sleep(10);
 #endif
 #if defined(MAGICKCORE_HAVE_PTHREAD)
-  (void) pthread_mutex_destroy(&semaphore_info->mutex);
+  (void) pthread_mutex_destroy(&(*semaphore_info)->mutex);
+#elif defined(__WINDOWS__)
+  DeleteCriticalSection(&(*semaphore_info)->mutex);
 #endif
-#if defined(__WINDOWS__)
-  DeleteCriticalSection(&semaphore_info->mutex);
-#endif
-  semaphore_info->signature=(~MagickSignature);
-  free(semaphore_info);
-  semaphore_info=(SemaphoreInfo *) NULL;
+  (*semaphore_info)->signature=(~MagickSignature);
+  free(*semaphore_info);
+  *semaphore_info=(SemaphoreInfo *) NULL;
 #if defined(MAGICKCORE_HAVE_PTHREAD)
   (void) pthread_mutex_unlock(&semaphore_mutex);
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   InterlockedExchange(&semaphore_mutex,0L);
 #endif
-  return(semaphore_info);
 }
 
 /*
@@ -374,15 +362,15 @@ MagickExport MagickBooleanType LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
     if (status != 0)
       return(MagickFalse);
     id=pthread_self();
-    if ((semaphore_info->reference_count > 0) && (semaphore_info->id == id))
+    if ((semaphore_info->reference_count > 0) &&
+        (pthread_equal(semaphore_info->id,id) != 0))
       {
         (void) fprintf(stderr,"Warning: recursive semaphore lock detected!\n");
         (void) fflush(stderr);
       }
     semaphore_info->id=id;
   }
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   {
     DWORD
       id;
@@ -453,7 +441,11 @@ MagickExport MagickBooleanType UnlockSemaphoreInfo(
   assert(semaphore_info != (SemaphoreInfo *) NULL);
   assert(semaphore_info->signature == MagickSignature);
   if (semaphore_info->reference_count == 0)
-    return(MagickFalse);
+    {
+      (void) fprintf(stderr,"Warning: semaphore lock already unlocked!\n");
+      (void) fflush(stderr);
+      return(MagickFalse);
+    }
   semaphore_info->reference_count--;
 #if defined(MAGICKCORE_HAVE_PTHREAD)
   {
@@ -468,8 +460,7 @@ MagickExport MagickBooleanType UnlockSemaphoreInfo(
         return(MagickFalse);
       }
   }
-#endif
-#if defined(__WINDOWS__)
+#elif defined(__WINDOWS__)
   assert(semaphore_info->id == GetCurrentThreadId());
   LeaveCriticalSection(&semaphore_info->mutex);
 #endif
