@@ -10,7 +10,7 @@
 %                  SSSSS    T    R  R   EEEEE  A   A  M   M                   %
 %                                                                             %
 %                                                                             %
-%                      ImageMagick Pixel Stream Methods                       %
+%                       MagickCore Pixel Stream Methods                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -144,8 +144,8 @@ static void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AcquireIndexesFromStream() returns the indexes associated with the last call to
-%  SetPixelStream() or AcquirePixelStream().
+%  AcquireIndexesFromStream() returns the indexes associated with the last
+%  call to SetPixelStream() or AcquirePixelStream().
 %
 %  The format of the AcquireIndexesFromStream() method is:
 %
@@ -156,7 +156,7 @@ static void
 %    o indexes: return the indexes associated with the last call to
 %      SetPixelStream() or AcquirePixelStream().
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 static const IndexPacket *AcquireIndexesFromStream(const Image *image)
@@ -198,9 +198,9 @@ static const IndexPacket *AcquireIndexesFromStream(const Image *image)
 %    o pixels: AcquireOnePixelFromStream() returns a pixel at the specified
 %      (x,y) location.
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o virtual_pixel_method: The virtual pixel method.
+%    o virtual_pixel_method: the virtual pixel method.
 %
 %    o x,y:  These values define the location of the pixel to return.
 %
@@ -247,9 +247,9 @@ static PixelPacket AcquireOnePixelFromStream(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o virtual_pixel_method: The virtual pixel method.
+%    o virtual_pixel_method: the virtual pixel method.
 %
 %    o x,y,columns,rows:  These values define the perimeter of a region of
 %      pixels.
@@ -262,19 +262,15 @@ static inline void AcquireStreamPixels(CacheInfo *cache_info)
 {
   assert(cache_info != (CacheInfo *) NULL);
   assert(cache_info->length == (MagickSizeType) ((size_t) cache_info->length));
-  cache_info->pixels=(PixelPacket *) MapBlob(-1,IOMode,0,(size_t)
-    cache_info->length);
-  if (cache_info->pixels != (PixelPacket *) NULL)
-    {
-      cache_info->mapped=MagickTrue;
-      return;
-    }
+  cache_info->mapped=MagickFalse;
   cache_info->pixels=(PixelPacket *) AcquireMagickMemory((size_t)
     cache_info->length);
-  if (cache_info->pixels != (PixelPacket *) NULL)
-    (void) ResetMagickMemory(cache_info->pixels,0,(size_t)
-      cache_info->length);
-  cache_info->mapped=MagickFalse;
+  if (cache_info->pixels == (PixelPacket *) NULL)
+    {
+      cache_info->mapped=MagickTrue;
+      cache_info->pixels=(PixelPacket *) MapBlob(-1,IOMode,0,(size_t)
+        cache_info->length);
+    }
 }
 
 static const PixelPacket *AcquirePixelStream(const Image *image,
@@ -352,7 +348,7 @@ static const PixelPacket *AcquirePixelStream(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 */
 MagickExport StreamInfo *AcquireStreamInfo(const ImageInfo *image_info)
@@ -370,7 +366,7 @@ MagickExport StreamInfo *AcquireStreamInfo(const ImageInfo *image_info)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   stream_info->map=ConstantString("RGB");
   stream_info->storage_type=CharPixel;
-  stream_info->stream=AllocateImage(image_info);
+  stream_info->stream=AcquireImage(image_info);
   stream_info->signature=MagickSignature;
   return(stream_info);
 }
@@ -394,7 +390,7 @@ MagickExport StreamInfo *AcquireStreamInfo(const ImageInfo *image_info)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 
@@ -424,11 +420,11 @@ static void DestroyPixelStream(Image *image)
   cache_info=(CacheInfo *) image->cache;
   assert(cache_info->signature == MagickSignature);
   destroy=MagickFalse;
-  AcquireSemaphoreInfo(&cache_info->semaphore);
+  (void) LockSemaphoreInfo(cache_info->semaphore);
   cache_info->reference_count--;
   if (cache_info->reference_count == 0)
     destroy=MagickTrue;
-  RelinquishSemaphoreInfo(cache_info->semaphore);
+  (void) UnlockSemaphoreInfo(cache_info->semaphore);
   if (destroy == MagickFalse)
     return;
   RelinquishStreamPixels(cache_info);
@@ -442,8 +438,12 @@ static void DestroyPixelStream(Image *image)
       cache_info->nexus_info=(NexusInfo *) RelinquishMagickMemory(
         cache_info->nexus_info);
     }
+  if (cache_info->disk_semaphore != (SemaphoreInfo *) NULL)
+    DestroySemaphoreInfo(&cache_info->disk_semaphore);
+  if (cache_info->nexus_semaphore != (SemaphoreInfo *) NULL)
+    DestroySemaphoreInfo(&cache_info->nexus_semaphore);
   if (cache_info->semaphore != (SemaphoreInfo *) NULL)
-    cache_info->semaphore=DestroySemaphoreInfo(cache_info->semaphore);
+    DestroySemaphoreInfo(&cache_info->semaphore);
   cache_info=(CacheInfo *) RelinquishMagickMemory(cache_info);
 }
 
@@ -467,7 +467,7 @@ static void DestroyPixelStream(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
 */
 MagickExport StreamInfo *DestroyStreamInfo(StreamInfo *stream_info)
@@ -516,7 +516,7 @@ MagickExport StreamInfo *DestroyStreamInfo(StreamInfo *stream_info)
 %    o indexes: Method GetIndexesFromStream() returns the indexes associated
 %      with the last call to SetPixelStream() or GetPixelStream().
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 static IndexPacket *GetIndexesFromStream(const Image *image)
@@ -557,7 +557,7 @@ static IndexPacket *GetIndexesFromStream(const Image *image)
 %    o pixels: Method GetOnePixelFromStream returns a pixel at the specified
 %      (x,y) location.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o x,y:  These values define the location of the pixel to return.
 %
@@ -598,7 +598,7 @@ static PixelPacket GetOnePixelFromStream(Image *image,const long x,const long y)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o x,y,columns,rows:  These values define the perimeter of a region of
 %      pixels.
@@ -641,7 +641,7 @@ static PixelPacket *GetPixelStream(Image *image,const long x,const long y,
 %    o pixels: Method GetPixelsFromStream returns the pixels associated with
 %      the last call to SetPixelStream() or GetPixelStream().
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 static PixelPacket *GetPixelsFromStream(const Image *image)
@@ -677,7 +677,7 @@ static PixelPacket *GetPixelsFromStream(const Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
 */
 MagickExport const void *GetStreamInfoClientData(StreamInfo *stream_info)
@@ -707,11 +707,11 @@ MagickExport const void *GetStreamInfoClientData(StreamInfo *stream_info)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
-%    o filename: The stream filename.
+%    o filename: the stream filename.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -749,7 +749,7 @@ MagickExport MagickBooleanType OpenStream(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o stream: a callback method.
 %
@@ -825,7 +825,7 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
 %    o pixels: Method SetPixelStream returns a pointer to the pixels is
 %      returned if the pixels are transferred, otherwise a NULL is returned.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o x,y,columns,rows:  These values define the perimeter of a region of
 %      pixels.
@@ -903,7 +903,7 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
       }
   if (cache_info->pixels == (void *) NULL)
     ThrowFatalException(ResourceLimitFatalError,
-      "UnableToAllocateImagePixels");
+      "UnableToAcquireImagePixels");
   cache_info->indexes=(IndexPacket *) NULL;
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
@@ -931,9 +931,9 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
 %
 %  A description of each parameter follows:
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
-%    o client_data: The client data.
+%    o client_data: the client data.
 %
 */
 MagickExport void SetStreamInfoClientData(StreamInfo *stream_info,
@@ -963,9 +963,9 @@ MagickExport void SetStreamInfoClientData(StreamInfo *stream_info,
 %
 %  A description of each parameter follows:
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
-%    o map: The map.
+%    o map: the map.
 %
 */
 MagickExport void SetStreamInfoMap(StreamInfo *stream_info,const char *map)
@@ -995,9 +995,9 @@ MagickExport void SetStreamInfoMap(StreamInfo *stream_info,const char *map)
 %
 %  A description of each parameter follows:
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
-%    o storage_type: The storage type.
+%    o storage_type: the storage type.
 %
 */
 MagickExport void SetStreamInfoStorageType(StreamInfo *stream_info,
@@ -1029,13 +1029,14 @@ MagickExport void SetStreamInfoStorageType(StreamInfo *stream_info,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
 */
+
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
@@ -1143,10 +1144,10 @@ MagickExport Image *StreamImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   read_info=CloneImageInfo(image_info);
   stream_info->image_info=image_info;
-  stream_info->quantum_info=AcquireQuantumInfo(image_info);
   stream_info->exception=exception;
   read_info->client_data=(void *) stream_info;
   image=ReadStream(read_info,&WriteStreamImage,exception);
+  stream_info->quantum_info=AcquireQuantumInfo(image_info,image);
   read_info=DestroyImageInfo(read_info);
   return(image);
 }
@@ -1173,9 +1174,9 @@ MagickExport Image *StreamImage(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o stream_info: The stream info.
+%    o stream_info: the stream info.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -1329,7 +1330,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToChar(p->blue);
             *q++=ScaleQuantumToChar(p->green);
@@ -1343,7 +1344,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToChar(p->blue);
             *q++=ScaleQuantumToChar(p->green);
@@ -1358,7 +1359,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
               break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToChar(p->blue);
             *q++=ScaleQuantumToChar(p->green);
@@ -1373,7 +1374,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToChar(PixelIntensityToQuantum(p));
             p++;
@@ -1385,7 +1386,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToChar(p->red);
             *q++=ScaleQuantumToChar(p->green);
@@ -1397,9 +1398,9 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (LocaleCompare(stream_info->map,"RGBA") == 0)
         {
           p=GetPixels(image);
-            if (p == (const PixelPacket *) NULL)
+          if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToChar(p->red);
             *q++=ScaleQuantumToChar(p->green);
@@ -1414,7 +1415,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToChar(p->red);
             *q++=ScaleQuantumToChar(p->green);
@@ -1428,7 +1429,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (p == (const PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (long) GetPixelCacheArea(image); x++)
+      for (x=0; x < (long) GetPixelCacheExtent(image); x++)
       {
         for (i=0; i < (long) length; i++)
         {
@@ -1494,7 +1495,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(double) ((QuantumScale*p->blue)*quantum_info->scale+
               quantum_info->minimum);
@@ -1511,7 +1512,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(double) ((QuantumScale*p->blue)*quantum_info->scale+
               quantum_info->minimum);
@@ -1530,7 +1531,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(double) ((QuantumScale*p->blue)*quantum_info->scale+
               quantum_info->minimum);
@@ -1548,7 +1549,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(double) ((QuantumScale*PixelIntensityToQuantum(p))*
               quantum_info->scale+quantum_info->minimum);
@@ -1561,7 +1562,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(double) ((QuantumScale*p->red)*quantum_info->scale+
               quantum_info->minimum);
@@ -1578,7 +1579,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(double) ((QuantumScale*p->red)*quantum_info->scale+
               quantum_info->minimum);
@@ -1597,7 +1598,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(double) ((QuantumScale*p->red)*quantum_info->scale+
               quantum_info->minimum);
@@ -1614,7 +1615,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (p == (const PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (long) GetPixelCacheArea(image); x++)
+      for (x=0; x < (long) GetPixelCacheExtent(image); x++)
       {
         for (i=0; i < (long) length; i++)
         {
@@ -1687,7 +1688,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(float) ((QuantumScale*p->blue)*quantum_info->scale+
               quantum_info->minimum);
@@ -1704,7 +1705,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(float) ((QuantumScale*p->blue)*quantum_info->scale+
               quantum_info->minimum);
@@ -1723,7 +1724,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(float) ((QuantumScale*p->blue)*quantum_info->scale+
               quantum_info->minimum);
@@ -1741,7 +1742,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(float) ((QuantumScale*PixelIntensityToQuantum(p))*
               quantum_info->scale+quantum_info->minimum);
@@ -1754,7 +1755,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(float) ((QuantumScale*p->red)*quantum_info->scale+
               quantum_info->minimum);
@@ -1771,7 +1772,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(float) ((QuantumScale*p->red)*quantum_info->scale+
               quantum_info->minimum);
@@ -1790,7 +1791,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(float) ((QuantumScale*p->red)*quantum_info->scale+
               quantum_info->minimum);
@@ -1807,7 +1808,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (p == (const PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (long) GetPixelCacheArea(image); x++)
+      for (x=0; x < (long) GetPixelCacheExtent(image); x++)
       {
         for (i=0; i < (long) length; i++)
         {
@@ -1880,7 +1881,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(unsigned int) ScaleQuantumToLong(p->blue);
             *q++=(unsigned int) ScaleQuantumToLong(p->green);
@@ -1894,7 +1895,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(unsigned int) ScaleQuantumToLong(p->blue);
             *q++=(unsigned int) ScaleQuantumToLong(p->green);
@@ -1910,7 +1911,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(unsigned int) ScaleQuantumToLong(p->blue);
             *q++=(unsigned int) ScaleQuantumToLong(p->green);
@@ -1925,7 +1926,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(unsigned int) ScaleQuantumToLong(
               PixelIntensityToQuantum(p));
@@ -1938,7 +1939,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(unsigned int) ScaleQuantumToLong(p->red);
             *q++=(unsigned int) ScaleQuantumToLong(p->green);
@@ -1952,7 +1953,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(unsigned int) ScaleQuantumToLong(p->red);
             *q++=(unsigned int) ScaleQuantumToLong(p->green);
@@ -1968,7 +1969,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=(unsigned int) ScaleQuantumToLong(p->red);
             *q++=(unsigned int) ScaleQuantumToLong(p->green);
@@ -1982,7 +1983,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (p == (const PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (long) GetPixelCacheArea(image); x++)
+      for (x=0; x < (long) GetPixelCacheExtent(image); x++)
       {
         for (i=0; i < (long) length; i++)
         {
@@ -2050,7 +2051,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToLong(p->blue);
             *q++=ScaleQuantumToLong(p->green);
@@ -2064,7 +2065,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToLong(p->blue);
             *q++=ScaleQuantumToLong(p->green);
@@ -2079,7 +2080,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToLong(p->blue);
             *q++=ScaleQuantumToLong(p->green);
@@ -2094,7 +2095,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToLong(PixelIntensityToQuantum(p));
             p++;
@@ -2106,7 +2107,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToLong(p->red);
             *q++=ScaleQuantumToLong(p->green);
@@ -2120,7 +2121,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToLong(p->red);
             *q++=ScaleQuantumToLong(p->green);
@@ -2135,7 +2136,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToLong(p->red);
             *q++=ScaleQuantumToLong(p->green);
@@ -2149,7 +2150,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (p == (const PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (long) GetPixelCacheArea(image); x++)
+      for (x=0; x < (long) GetPixelCacheExtent(image); x++)
       {
         for (i=0; i < (long) length; i++)
         {
@@ -2215,7 +2216,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=p->blue;
             *q++=p->green;
@@ -2229,7 +2230,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=p->blue;
             *q++=p->green;
@@ -2244,7 +2245,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=p->blue;
             *q++=p->green;
@@ -2259,7 +2260,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=PixelIntensityToQuantum(p);
             p++;
@@ -2271,7 +2272,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=p->red;
             *q++=p->green;
@@ -2285,7 +2286,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=p->red;
             *q++=p->green;
@@ -2300,7 +2301,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=p->red;
             *q++=p->green;
@@ -2314,7 +2315,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (p == (const PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (long) GetPixelCacheArea(image); x++)
+      for (x=0; x < (long) GetPixelCacheExtent(image); x++)
       {
         for (i=0; i < (long) length; i++)
         {
@@ -2380,7 +2381,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToShort(p->blue);
             *q++=ScaleQuantumToShort(p->green);
@@ -2394,7 +2395,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToShort(p->blue);
             *q++=ScaleQuantumToShort(p->green);
@@ -2409,7 +2410,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
             if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToShort(p->blue);
             *q++=ScaleQuantumToShort(p->green);
@@ -2424,7 +2425,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToShort(PixelIntensityToQuantum(p));
             p++;
@@ -2436,7 +2437,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToShort(p->red);
             *q++=ScaleQuantumToShort(p->green);
@@ -2450,7 +2451,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToShort(p->red);
             *q++=ScaleQuantumToShort(p->green);
@@ -2465,7 +2466,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
           p=GetPixels(image);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) GetPixelCacheArea(image); x++)
+          for (x=0; x < (long) GetPixelCacheExtent(image); x++)
           {
             *q++=ScaleQuantumToShort(p->red);
             *q++=ScaleQuantumToShort(p->green);
@@ -2479,7 +2480,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
       if (p == (const PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (long) GetPixelCacheArea(image); x++)
+      for (x=0; x < (long) GetPixelCacheExtent(image); x++)
       {
         for (i=0; i < (long) length; i++)
         {
@@ -2566,7 +2567,7 @@ static MagickBooleanType StreamImagePixels(const StreamInfo *stream_info,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 static MagickBooleanType SyncPixelStream(Image *image)
@@ -2619,7 +2620,7 @@ static MagickBooleanType SyncPixelStream(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o stream: A callback method.
 %
