@@ -10,7 +10,7 @@
 %         T    H   H  R  R   EEEEE  SSSSS  H   H   OOO   LLLLL  DDDD          %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Threshold Methods                     %
+%                      MagickCore Image Threshold Methods                     %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
@@ -63,6 +63,7 @@
 #include "magick/log.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/montage.h"
 #include "magick/pixel-private.h"
 #include "magick/quantize.h"
@@ -72,7 +73,7 @@
 #include "magick/resource_.h"
 #include "magick/segment.h"
 #include "magick/shear.h"
-#include "magick/signature.h"
+#include "magick/signature-private.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
 #include "magick/threshold.h"
@@ -80,7 +81,12 @@
 #include "magick/xml-tree.h"
 
 /*
-  Structure declatations.
+  Define declarations.
+*/
+#define ThresholdsFilename  "thresholds.xml"
+
+/*
+  Typedef declarations.
 */
 struct _ThresholdMap
 {
@@ -88,17 +94,15 @@ struct _ThresholdMap
     *map_id,
     *description;
 
-  int
+  unsigned long
     width,
-    height,
+    height;
+
+  long
     divisor,
     *levels;
 };
 
-/*
-  Define declarations.
-*/
-#define ThresholdsFilename  "thresholds.xml"
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -123,13 +127,13 @@ struct _ThresholdMap
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o width: The width of the local neighborhood.
+%    o width: the width of the local neighborhood.
 %
-%    o height: The height of the local neighborhood.
+%    o height: the height of the local neighborhood.
 %
-%    o offset: The mean offset.
+%    o offset: the mean offset.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -143,38 +147,23 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
   Image
     *threshold_image;
 
-  IndexPacket
-    *indexes,
-    *threshold_indexes;
-
   long
-    u,
-    v,
+    progress,
     y;
 
   MagickBooleanType
     status;
 
   MagickPixelPacket
-    pixel,
-    mean;
+    zero;
 
   MagickRealType
     number_pixels;
 
-  register const PixelPacket
-    *p,
-    *r;
+  ViewInfo
+    **image_view,
+    **threshold_view;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  /*
-    Initialize thresholded image attributes.
-  */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -193,22 +182,65 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       return((Image *) NULL);
     }
   /*
-    Threshold each row of the image.
+    Local adaptive threshold.
   */
-  GetMagickPixelPacket(image,&mean);
-  number_pixels=(MagickRealType) (width*height);
+  status=MagickTrue;
+  progress=0;
+  GetMagickPixelPacket(image,&zero);
+  number_pixels=(MagickRealType) width*height;
+  image_view=AcquireCacheViewThreadSet(image);
+  threshold_view=AcquireCacheViewThreadSet(threshold_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,-((long) width/2L),y-height/2L,image->columns+
-      width,height,exception);
-    q=GetImagePixels(threshold_image,0,y,threshold_image->columns,1);
+    register const IndexPacket
+      *indexes;
+
+    register const PixelPacket
+      *p;
+
+    register IndexPacket
+      *threshold_indexes;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    p=AcquireCacheViewPixels(image_view[id],-((long) width/2L),y-height/2L,
+      image->columns+width,height,exception);
+    q=GetCacheViewPixels(threshold_view[id],0,y,threshold_image->columns,1);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
-      break;
-    indexes=GetIndexes(image);
-    threshold_indexes=GetIndexes(threshold_image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=AcquireCacheViewIndexes(image_view[id]);
+    threshold_indexes=GetCacheViewIndexes(threshold_view[id]);
     for (x=0; x < (long) image->columns; x++)
     {
-      GetMagickPixelPacket(image,&pixel);
+      long
+        v;
+
+      MagickPixelPacket
+        mean,
+        pixel;
+
+      register const PixelPacket
+        *r;
+
+      register long
+        u;
+
+      pixel=zero;
+      mean=zero;
       r=p;
       for (v=0; v < (long) height; v++)
       {
@@ -229,31 +261,40 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       mean.opacity=(MagickRealType) (pixel.opacity/number_pixels+offset);
       if (image->colorspace == CMYKColorspace)
         mean.index=(MagickRealType) (pixel.index/number_pixels+offset);
-      q->red=(Quantum) (((MagickRealType)
-        q->red <= mean.red) ? 0 : QuantumRange);
-      q->green=(Quantum) (((MagickRealType)
-        q->green <= mean.green) ? 0 : QuantumRange);
-      q->blue=(Quantum) (((MagickRealType)
-        q->blue <= mean.blue) ? 0 : QuantumRange);
-      q->opacity=(Quantum) (((MagickRealType)
-        q->opacity <= mean.opacity) ? 0 : QuantumRange);
+      q->red=(Quantum) (((MagickRealType) q->red <= mean.red) ?
+        0 : QuantumRange);
+      q->green=(Quantum) (((MagickRealType) q->green <= mean.green) ?
+        0 : QuantumRange);
+      q->blue=(Quantum) (((MagickRealType) q->blue <= mean.blue) ?
+        0 : QuantumRange);
+      q->opacity=(Quantum) (((MagickRealType) q->opacity <= mean.opacity) ?
+        0 : QuantumRange);
       if (image->colorspace == CMYKColorspace)
         threshold_indexes[x]=(IndexPacket) (((MagickRealType)
           threshold_indexes[x] <= mean.index) ? 0 : QuantumRange);
       p++;
       q++;
     }
-    if (SyncImagePixels(threshold_image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(threshold_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(ThresholdImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
+  threshold_view=DestroyCacheViewThreadSet(threshold_view);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  if (status == MagickFalse)
+    threshold_image=DestroyImage(threshold_image);
   return(threshold_image);
 }
 
@@ -271,6 +312,16 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
 %  BilevelImage() changes the value of individual pixels based on the
 %  intensity of each pixel channel.  The result is a high-contrast image.
 %
+%  More precisely each channel value of the image is 'thresholded' so that if
+%  it is equal to or less than the given value it is set to zero, while any
+%  value greater than that give is set to it maximum or QuantumRange.
+%
+%  This function is what is used to implement the "-threshold" operator for
+%  the command line API.
+%
+%  If the default channel setting is given the image is thresholded using just
+%  the gray 'intensity' of the image, rather than the individual channels.
+%
 %  The format of the BilevelImageChannel method is:
 %
 %      MagickBooleanType BilevelImage(Image *image,const double threshold)
@@ -279,12 +330,15 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o channel: The channel type.
+%    o channel: the channel type.
 %
 %    o threshold: define the threshold values.
 %
+%  Aside: You can get the same results as operator using LevelImageChannels()
+%  with the 'threshold' value for both the black_point and the white_point.
+%
 */
 
 MagickExport MagickBooleanType BilevelImage(Image *image,const double threshold)
@@ -302,41 +356,58 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 #define ThresholdImageTag  "Threshold/Image"
 
   long
+    progress,
     y;
 
   MagickBooleanType
     status;
 
-  register IndexPacket
-    *indexes;
+  ViewInfo
+    **image_view;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  /*
-    Threshold image.
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
+  /*
+    Bilevel threshold image.
+  */
+  status=MagickTrue;
+  progress=0;
+  image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    register IndexPacket
+      *indexes;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewIndexes(image_view[id]);
     if (channel == DefaultChannels)
       {
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=(Quantum) ((MagickRealType)
-            PixelIntensityToQuantum(q) <= threshold ? 0 : QuantumRange);
+          q->red=(Quantum) ((MagickRealType) PixelIntensityToQuantum(q) <=
+            threshold ? 0 : QuantumRange);
           q->green=q->red;
           q->blue=q->red;
           q++;
@@ -346,41 +417,47 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
       for (x=0; x < (long) image->columns; x++)
       {
         if ((channel & RedChannel) != 0)
-          q->red=(Quantum) ((MagickRealType)
-            q->red <= threshold ? 0 : QuantumRange);
+          q->red=(Quantum) ((MagickRealType) q->red <= threshold ? 0 :
+            QuantumRange);
         if ((channel & GreenChannel) != 0)
-          q->green=(Quantum) ((MagickRealType)
-            q->green <= threshold ? 0 : QuantumRange);
+          q->green=(Quantum) ((MagickRealType) q->green <= threshold ? 0 :
+            QuantumRange);
         if ((channel & BlueChannel) != 0)
-          q->blue=(Quantum) ((MagickRealType)
-            q->blue <= threshold ? 0 : QuantumRange);
+          q->blue=(Quantum) ((MagickRealType) q->blue <= threshold ? 0 :
+            QuantumRange);
         if ((channel & OpacityChannel) != 0)
           {
             if (image->matte == MagickFalse)
-              q->opacity=(Quantum) ((MagickRealType)
-                q->opacity <= threshold ? 0 : QuantumRange);
+              q->opacity=(Quantum) ((MagickRealType) q->opacity <= threshold ?
+                0 : QuantumRange);
             else
-              q->opacity=(Quantum) ((MagickRealType)
-                q->opacity <= threshold ?  OpaqueOpacity : TransparentOpacity);
+              q->opacity=(Quantum) ((MagickRealType) q->opacity <= threshold ?
+                OpaqueOpacity : TransparentOpacity);
           }
         if (((channel & IndexChannel) != 0) &&
             (image->colorspace == CMYKColorspace))
-          indexes[x]=(IndexPacket) ((MagickRealType)
-            indexes[x] <= threshold ? 0 : QuantumRange);
+          indexes[x]=(IndexPacket) ((MagickRealType) indexes[x] <= threshold ?
+            0 : QuantumRange);
         q++;
       }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(ThresholdImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  return(MagickTrue);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  return(status);
 }
 
 /*
@@ -404,7 +481,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o threshold: Define the threshold value.
 %
@@ -418,6 +495,7 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
     geometry_info;
 
   long
+    progress,
     y;
 
   MagickBooleanType
@@ -429,15 +507,9 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
   MagickStatusType
     flags;
 
-  register long
-    x;
+  ViewInfo
+    **image_view;
 
-  register PixelPacket
-    *q;
-
-  /*
-    Threshold image.
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -460,16 +532,38 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
     pixel.opacity=(MagickRealType) OpaqueOpacity;
   if ((flags & PercentValue) != 0)
     {
-      pixel.red*=QuantumRange/100.0f;
-      pixel.green*=QuantumRange/100.0f;
-      pixel.blue*=QuantumRange/100.0f;
-      pixel.opacity*=QuantumRange/100.0f;
+      pixel.red*=(MagickRealType) (QuantumRange/100.0);
+      pixel.green*=(MagickRealType) (QuantumRange/100.0);
+      pixel.blue*=(MagickRealType) (QuantumRange/100.0);
+      pixel.opacity*=(MagickRealType) (QuantumRange/100.0);
     }
+  /*
+    Black threshold image.
+  */
+  status=MagickTrue;
+  progress=0;
+  image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFalse;
+        continue;
+      }
     if (IsMagickGray(&pixel) != MagickFalse)
       for (x=0; x < (long) image->columns; x++)
       {
@@ -494,18 +588,24 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
           q->opacity=(Quantum) 0;
         q++;
       }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(ThresholdImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  return(MagickTrue);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  return(status);
 }
 
 /*
@@ -532,14 +632,14 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
 */
 MagickExport ThresholdMap *DestroyThresholdMap(ThresholdMap *map)
 {
-  if ( map && map->description )
-    map->map_id = DestroyString(map->map_id);
-  if ( map && map->description )
-    map->description = DestroyString(map->description);
-  if ( map && map->levels )
-    map->levels = (int *) RelinquishMagickMemory(map->levels);
-  if ( map )
-    map = (ThresholdMap *) RelinquishMagickMemory(map);
+  assert(map != (ThresholdMap *) NULL);
+  if (map->map_id != (char *) NULL)
+    map->map_id=DestroyString(map->map_id);
+  if (map->description != (char *) NULL)
+    map->description=DestroyString(map->description);
+  if (map->levels != (long *) NULL)
+    map->levels=(long *) RelinquishMagickMemory(map->levels);
+  map=(ThresholdMap *) RelinquishMagickMemory(map);
   return(map);
 }
 
@@ -621,7 +721,7 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireThresholdMap");
   map->map_id = (char *)NULL;
   map->description = (char *)NULL;
-  map->levels = (int *)NULL;
+  map->levels = (long *) NULL;
 
   /* Assign Basic Attributes */
   attr = GetXMLTreeAttribute(threshold, "map");
@@ -640,7 +740,7 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
     map = DestroyThresholdMap(map);
     return(map);
   }
-  map->width = atoi(attr);
+  map->width = (unsigned long) atoi(attr);
   if ( map->width == 0 ) {
     (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
      "XmlInvalidAttribute", "<levels width>, map \"%s\"", map_id);
@@ -657,7 +757,7 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
     map = DestroyThresholdMap(map);
     return(map);
   }
-  map->height = atoi(attr);
+  map->height = (unsigned long) atoi(attr);
   if ( map->height == 0 ) {
     (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
       "XmlInvalidAttribute", "<levels height>, map \"%s\"", map_id);
@@ -683,7 +783,7 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
     return(map);
   }
 
-  /* Allocate the theshold levels array */
+  /* Allocate theshold levels array */
   content = GetXMLTreeContent(levels);
   if ( content == (char *)NULL ) {
     (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
@@ -692,14 +792,14 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
     map = DestroyThresholdMap(map);
     return(map);
   }
-  map->levels=(int *) AcquireQuantumMemory((size_t) map->width,map->height*
+  map->levels=(long *) AcquireQuantumMemory((size_t) map->width,map->height*
     sizeof(*map->levels));
-  if ( map->levels == (int *)NULL )
+  if ( map->levels == (long *)NULL )
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireThresholdMap");
   { /* parse levels into integer array */
     int i;
     char *p;
-    for( i=0; i<map->width*map->height; i++) {
+    for( i=0; i< (long) (map->width*map->height); i++) {
       map->levels[i] = (int)strtol(content, &p, 10);
       if ( p == content ) {
         (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
@@ -710,7 +810,7 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
       }
       if ( map->levels[i] < 0 || map->levels[i] > map->divisor ) {
         (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
-          "XmlInvalidContent", "<level> %d out of range, map \"%s\"",
+          "XmlInvalidContent", "<level> %ld out of range, map \"%s\"",
           map->levels[i], map_id);
         thresholds = DestroyXMLTree(thresholds);
         map = DestroyThresholdMap(map);
@@ -942,9 +1042,9 @@ MagickExport MagickBooleanType ListThresholdMaps(FILE *file,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o channel: The channel or channels to be thresholded.
+%    o channel: the channel or channels to be thresholded.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -998,9 +1098,9 @@ MagickExport MagickBooleanType OrderedDitherImageChannel(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o channel: The channel or channels to be thresholded.
+%    o channel: the channel or channels to be thresholded.
 %
 %    o threshold_map: A string containing the name of the threshold dither
 %      map to use, followed by zero or more numbers representing the number
@@ -1039,20 +1139,22 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
   const ChannelType channel,const char *threshold_map,ExceptionInfo *exception)
 {
 #define DitherImageTag  "Dither/Image"
-  ThresholdMap
-    *map;
+
+  long
+    progress,
+    y;
 
   LongPixelPacket
     levels;
 
-  long
-    y;
+  MagickBooleanType
+    status;
 
-  register long
-    x;
+  ThresholdMap
+    *map;
 
-  register IndexPacket
-    *indexes;
+  ViewInfo
+    **image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1060,10 +1162,8 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-
   if (threshold_map == (const char *) NULL)
     return(MagickTrue);
-
   {
     char token[MaxTextExtent], *p;
 
@@ -1088,7 +1188,7 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
     }
   }
   /* Set channel levels from extra comma seperated arguments
-     Default to 2, or single value given, or individual channel values
+     Default to 2, the single value given, or individual channel values
   */
 #if 1
   { /* parse directly as a comma seperated list of integers */
@@ -1096,7 +1196,7 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
 
     p = strchr(threshold_map,',');
     if ( p != (char *)NULL && isdigit((int) ((unsigned char) *(++p))) )
-      levels.index = 1UL*strtol(p, &p, 10);
+      levels.index = (unsigned long) strtol(p, &p, 10);
     else
       levels.index = 2;
 
@@ -1117,15 +1217,15 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
         levels.green = (unsigned long) strtol(p, &p, 10), (void)(*p == ',' && p++);
       if ((channel & BlueChannel) != 0)
         levels.blue = (unsigned long) strtol(p, &p, 10),  (void)(*p == ',' && p++);
-      if ((channel & OpacityChannel) != 0)
-        levels.opacity = (unsigned long) strtol(p, &p, 10), (void)(*p == ',' && p++);
       if ((channel & IndexChannel) != 0 && image->colorspace == CMYKColorspace)
         levels.index=(unsigned long) strtol(p, &p, 10), (void)(*p == ',' && p++);
+      if ((channel & OpacityChannel) != 0)
+        levels.opacity = (unsigned long) strtol(p, &p, 10), (void)(*p == ',' && p++);
     }
   }
 #else
   /* Parse level values as a geometry */
-  /* This is difficult!
+  /* This difficult!
    * How to map   GeometryInfo structure elements into
    * LongPixelPacket structure elements, but according to channel?
    * Note the channels list may skip elements!!!!
@@ -1160,19 +1260,41 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
         InheritException(exception,&image->exception);
         return(MagickFalse);
       }
+    status=MagickTrue;
+    progress=0;
+    image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+    #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
     for (y=0; y < (long) image->rows; y++)
     {
+      register long
+        id,
+        x;
+
+      register IndexPacket
+        *indexes;
+
       register PixelPacket
         *q;
 
-      q=GetImagePixels(image,0,y,image->columns,1);
+      if (status == MagickFalse)
+        continue;
+      id=GetCacheViewThreadId();
+      q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
       if (q == (PixelPacket *) NULL)
-        break;
-      indexes=GetIndexes(image);
+        {
+          status=MagickFalse;
+          continue;
+        }
+      indexes=GetCacheViewIndexes(image_view[id]);
       for (x=0; x < (long) image->columns; x++)
       {
         register int
-          threshold, t, l;
+          threshold,
+          t,
+          l;
+
         /*
           Figure out the dither threshold for this pixel
           This must be a integer from 1 to map->divisor-1
@@ -1219,18 +1341,22 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
         }
         q++;
       }
-      if (SyncImagePixels(image) == MagickFalse)
-        break;
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
+      if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+        status=MagickFalse;
+      if (image->progress_monitor != (MagickProgressMonitor) NULL)
         {
           MagickBooleanType
-            status=image->progress_monitor(DitherImageTag,y,image->rows,
-               image->client_data);
-          if (status == MagickFalse)
-            break;
+            proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+          #pragma omp critical
+#endif
+          proceed=SetImageProgress(image,DitherImageTag,progress++,image->rows);
+          if (proceed == MagickFalse)
+            status=MagickFalse;
         }
     }
+    image_view=DestroyCacheViewThreadSet(image_view);
   }
   map=DestroyThresholdMap(map);
   return(MagickTrue);
@@ -1261,9 +1387,9 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
-%    o channel: The channel or channels to be thresholded.
+%    o channel: the channel or channels to be thresholded.
 %
 %    o thresholds: a geometry string containing low,high thresholds.  If the
 %      string contains 2x2, 3x3, or 4x4, an ordered dither of order 2, 3, or 4
@@ -1296,6 +1422,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     flags;
 
   long
+    progress,
     y;
 
   MagickBooleanType
@@ -1308,14 +1435,8 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     min_threshold,
     max_threshold;
 
-  register long
-    x;
-
-  register IndexPacket
-    *indexes;
-
-  register PixelPacket
-    *q;
+  ViewInfo
+    **image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1335,12 +1456,12 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     max_threshold=min_threshold;
   if (strchr(thresholds,'%') != (char *) NULL)
     {
-      max_threshold*=(0.01f*QuantumRange);
-      min_threshold*=(0.01f*QuantumRange);
+      max_threshold*=(MagickRealType) (0.01*QuantumRange);
+      min_threshold*=(MagickRealType) (0.01*QuantumRange);
     }
   else
-    if ((min_threshold <= 8) && ((max_threshold == min_threshold) ||
-        (max_threshold == 1)))
+    if (((max_threshold == min_threshold) || (max_threshold == 1)) &&
+        (min_threshold <= 8))
       {
         /*
           Backward Compatibility -- ordered-dither -- IM v 6.2.9-6.
@@ -1348,60 +1469,112 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
         status=OrderedPosterizeImageChannel(image,channel,thresholds,exception);
         return(status);
       }
+  /*
+    Random threshold image.
+  */
+  status=MagickTrue;
+  progress=0;
   if (channel == AllChannels)
     {
-      IndexPacket
-        index;
-
-      MagickRealType
-        intensity;
-
-      if (AllocateImageColormap(image,2) == MagickFalse)
+      if (AcquireImageColormap(image,2) == MagickFalse)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
+      image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+      #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
       for (y=0; y < (long) image->rows; y++)
       {
-        q=GetImagePixels(image,0,y,image->columns,1);
+        register IndexPacket
+          *indexes;
+
+        register long
+          id,
+          x;
+
+        register PixelPacket
+          *q;
+
+        if (status == MagickFalse)
+          continue;
+        id=GetCacheViewThreadId();
+        q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
-          break;
-        indexes=GetIndexes(image);
+          {
+            status=MagickFalse;
+            continue;
+          }
+        indexes=GetCacheViewIndexes(image_view[id]);
         for (x=0; x < (long) image->columns; x++)
         {
+          IndexPacket
+            index;
+
+          MagickRealType
+            intensity;
+
           intensity=(MagickRealType) PixelIntensityToQuantum(q);
           if (intensity < min_threshold)
             threshold.index=min_threshold;
           else if (intensity > max_threshold)
             threshold.index=max_threshold;
           else
-            threshold.index=(MagickRealType)(QuantumRange*GetRandomValue());
+            threshold.index=(MagickRealType)(QuantumRange*
+              GetPseudoRandomValue());
           index=(IndexPacket) (intensity <= threshold.index ? 0 : 1);
           indexes[x]=index;
           *q++=image->colormap[(long) index];
         }
-        if (SyncImagePixels(image) == MagickFalse)
-          break;
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
+        if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+          status=MagickFalse;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
-            status=image->progress_monitor(ThresholdImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
+            MagickBooleanType
+              proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+            #pragma omp critical
+#endif
+            proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+              image->rows);
+            if (proceed == MagickFalse)
+              status=MagickFalse;
           }
       }
-      return(MagickTrue);
+      image_view=DestroyCacheViewThreadSet(image_view);
+      return(status);
     }
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     {
       InheritException(exception,&image->exception);
       return(MagickFalse);
     }
+  image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    register IndexPacket
+      *indexes;
+
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewIndexes(image_view[id]);
     for (x=0; x < (long) image->columns; x++)
     {
       if ((channel & RedChannel) != 0)
@@ -1413,7 +1586,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.red=max_threshold;
             else
               threshold.red=(MagickRealType) (QuantumRange*
-                GetRandomValue());
+                GetPseudoRandomValue());
         }
       if ((channel & GreenChannel) != 0)
         {
@@ -1424,7 +1597,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.green=max_threshold;
             else
               threshold.green=(MagickRealType) (QuantumRange*
-                GetRandomValue());
+                GetPseudoRandomValue());
         }
       if ((channel & BlueChannel) != 0)
         {
@@ -1435,7 +1608,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.blue=max_threshold;
             else
               threshold.blue=(MagickRealType) (QuantumRange*
-                GetRandomValue());
+                GetPseudoRandomValue());
         }
       if ((channel & OpacityChannel) != 0)
         {
@@ -1446,7 +1619,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.opacity=max_threshold;
             else
               threshold.opacity=(MagickRealType) (QuantumRange*
-                GetRandomValue());
+                GetPseudoRandomValue());
         }
       if (((channel & IndexChannel) != 0) &&
           (image->colorspace == CMYKColorspace))
@@ -1458,7 +1631,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.index=max_threshold;
             else
               threshold.index=(MagickRealType) (QuantumRange*
-                GetRandomValue());
+                GetPseudoRandomValue());
         }
       if ((channel & RedChannel) != 0)
         q->red=(Quantum) ((MagickRealType) q->red <= threshold.red ? 0 :
@@ -1478,18 +1651,24 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           threshold.index ? 0 : QuantumRange);
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(ThresholdImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  return(MagickTrue);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  return(status);
 }
 
 /*
@@ -1513,7 +1692,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o threshold: Define the threshold value
 %
@@ -1527,6 +1706,7 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
     geometry_info;
 
   long
+    progress,
     y;
 
   MagickBooleanType
@@ -1538,15 +1718,9 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
   MagickStatusType
     flags;
 
-  register long
-    x;
+  ViewInfo
+    **image_view;
 
-  register PixelPacket
-    *q;
-
-  /*
-    Threshold image.
-  */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -1569,16 +1743,38 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
     pixel.opacity=(MagickRealType) OpaqueOpacity;
   if ((flags & PercentValue) != 0)
     {
-      pixel.red*=QuantumRange/100.0f;
-      pixel.green*=QuantumRange/100.0f;
-      pixel.blue*=QuantumRange/100.0f;
-      pixel.opacity*=QuantumRange/100.0f;
+      pixel.red*=(MagickRealType) (QuantumRange/100.0);
+      pixel.green*=(MagickRealType) (QuantumRange/100.0);
+      pixel.blue*=(MagickRealType) (QuantumRange/100.0);
+      pixel.opacity*=(MagickRealType) (QuantumRange/100.0);
     }
+  /*
+    White threshold image.
+  */
+  status=MagickTrue;
+  progress=0;
+  image_view=AcquireCacheViewThreadSet(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(static,64) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    register long
+      id,
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    id=GetCacheViewThreadId();
+    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
-      break;
+      {
+        status=MagickFalse;
+        continue;
+      }
     if (IsMagickGray(&pixel) != MagickFalse)
       for (x=0; x < (long) image->columns; x++)
       {
@@ -1603,16 +1799,22 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
           q->opacity=(Quantum) QuantumRange;
         q++;
       }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
+    if (SyncCacheViewPixels(image_view[id]) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(ThresholdImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
-  return(MagickTrue);
+  image_view=DestroyCacheViewThreadSet(image_view);
+  return(status);
 }
