@@ -10,7 +10,7 @@
 %                      T    IIIII  M   M  EEEEE  R  R                         %
 %                                                                             %
 %                                                                             %
-%                        ImageMagick Timing Methods                           %
+%                         MagickCore Timing Methods                           %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
@@ -41,6 +41,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
 #include "magick/timer.h"
@@ -69,6 +71,39 @@ static void
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A c q u i r e T i m e r I n f o                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireTimerInfo() initializes the TimerInfo structure.  It effectively
+%  creates a stopwatch and starts it.
+%
+%  The format of the AcquireTimerInfo method is:
+%
+%      TimerInfo *AcquireTimerInfo(void)
+%
+*/
+MagickExport TimerInfo *AcquireTimerInfo(void)
+{
+  TimerInfo
+    *timer_info;
+
+  timer_info=(TimerInfo *) AcquireMagickMemory(sizeof(*timer_info));
+  if (timer_info == (TimerInfo *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
+  (void) ResetMagickMemory(timer_info,0,sizeof(*timer_info));
+  timer_info->signature=MagickSignature;
+  GetTimerInfo(timer_info);
+  return(timer_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C o n t i n u e T i m e r                                                 %
 %                                                                             %
 %                                                                             %
@@ -108,6 +143,37 @@ MagickExport MagickBooleanType ContinueTimer(TimerInfo *time_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   D e s t r o y T i m e r I n f o                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  DestroyTimerInfo() zeros memory associated with the TimerInfo structure.
+%
+%  The format of the DestroyTimerInfo method is:
+%
+%      TimerInfo *DestroyTimerInfo(TimerInfo *timer_info)
+%
+%  A description of each parameter follows:
+%
+%    o timer_info: The cipher context.
+%
+*/
+MagickExport TimerInfo *DestroyTimerInfo(TimerInfo *timer_info)
+{
+  assert(timer_info != (TimerInfo *) NULL);
+  assert(timer_info->signature == MagickSignature);
+  timer_info->signature=(~MagickSignature);
+  timer_info=(TimerInfo *) RelinquishMagickMemory(timer_info);
+  return(timer_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   E l a p s e d T i m e                                                     %
 %                                                                             %
 %                                                                             %
@@ -178,7 +244,7 @@ MagickExport double GetElapsedTime(TimerInfo *time_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e t T i m e r I n f o                                                   %
++   G e t T i m e r I n f o                                                   %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -194,7 +260,6 @@ MagickExport double GetElapsedTime(TimerInfo *time_info)
 %
 %    o  time_info: Timer statistics structure.
 %
-%
 */
 MagickExport void GetTimerInfo(TimerInfo *time_info)
 {
