@@ -10,14 +10,14 @@
 %                          T      Y    P      EEEEE                           %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Type Methods                          %
+%                       MagickCore Image Type Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,7 +49,6 @@
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt-feature.h"
 #include "magick/option.h"
 #include "magick/semaphore.h"
 #include "magick/splay-tree.h"
@@ -57,6 +56,42 @@
 #include "magick/type.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
+#if defined(MAGICKCORE_FONTCONFIG_DELEGATE)
+# include "fontconfig/fontconfig.h"
+#if (FC_VERSION < 20209)
+#undef FC_WEIGHT_LIGHT
+#define FC_WIDTH                  "width"    /* Int */
+#define FC_WIDTH_ULTRACONDENSED    50
+#define FC_WIDTH_EXTRACONDENSED    63
+#define FC_WIDTH_CONDENSED         75
+#define FC_WIDTH_SEMICONDENSED     87
+#define FC_WIDTH_NORMAL            100
+#define FC_WIDTH_SEMIEXPANDED      113
+#define FC_WIDTH_EXPANDED          125
+#define FC_WIDTH_EXTRAEXPANDED     150
+#define FC_WIDTH_ULTRAEXPANDED     200
+
+#define FC_WEIGHT_THIN             0
+#define FC_WEIGHT_EXTRALIGHT       40
+#define FC_WEIGHT_ULTRALIGHT       FC_WEIGHT_EXTRALIGHT
+#define FC_WEIGHT_LIGHT            50
+#define FC_WEIGHT_BOOK             75
+#define FC_WEIGHT_REGULAR          80
+#define FC_WEIGHT_NORMAL           FC_WEIGHT_REGULAR
+#define FC_WEIGHT_MEDIUM           100
+#define FC_WEIGHT_DEMIBOLD         180
+#define FC_WEIGHT_SEMIBOLD         FC_WEIGHT_DEMIBOLD
+#define FC_WEIGHT_BOLD             200
+#define FC_WEIGHT_EXTRABOLD        205
+#define FC_WEIGHT_ULTRABOLD        FC_WEIGHT_EXTRABOLD
+#define FC_WEIGHT_BLACK            210
+#define FC_WEIGHT_HEAVY            FC_WEIGHT_BLACK
+#endif
+#endif
+#if defined(__WINDOWS__)
+# include "magick/nt-feature.h"
+#endif
 
 /*
   Define declarations.
@@ -67,10 +102,11 @@
   Declare type map.
 */
 static const char
-  *TypeMap = (char *)
+  *TypeMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<typemap>"
-    "  <type stealth=\"True\" />"
+    "  <type stealth=\"True\" name=\"fixed\" family=\"helvetica\"/>"
+    "  <type stealth=\"True\" name=\"helvetica\" family=\"helvetica\"/>"
     "</typemap>";
 
 /*
@@ -109,7 +145,6 @@ static MagickBooleanType
 %
 %      DestroyTypeList(void)
 %
-%
 */
 MagickExport void DestroyTypeList(void)
 {
@@ -118,7 +153,7 @@ MagickExport void DestroyTypeList(void)
     type_list=DestroySplayTree(type_list);
   instantiate_type=MagickFalse;
   RelinquishSemaphoreInfo(type_semaphore);
-  type_semaphore=DestroySemaphoreInfo(type_semaphore);
+  DestroySemaphoreInfo(&type_semaphore);
 }
 
 /*
@@ -141,18 +176,14 @@ MagickExport void DestroyTypeList(void)
 %
 %  A description of each parameter follows:
 %
-%    o name: The type name.
+%    o name: the type name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport const TypeInfo *GetTypeInfo(const char *name,
   ExceptionInfo *exception)
 {
-  register const TypeInfo
-    *p;
-
   assert(exception != (ExceptionInfo *) NULL);
   if ((type_list == (SplayTreeInfo *) NULL) ||
       (instantiate_type == MagickFalse))
@@ -166,20 +197,7 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
       ResetSplayTreeIterator(type_list);
       return((const TypeInfo *) GetNextValueInSplayTree(type_list));
     }
-  /*
-    Search for requested type.
-  */
-  AcquireSemaphoreInfo(&type_semaphore);
-  ResetSplayTreeIterator(type_list);
-  p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
-  while (p != (const TypeInfo *) NULL)
-  {
-    if (LocaleCompare(name,p->name) == 0)
-      break;
-    p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
-  }
-  RelinquishSemaphoreInfo(type_semaphore);
-  return(p);
+  return((const TypeInfo *) GetValueFromSplayTree(type_list,name));
 }
 
 /*
@@ -206,25 +224,41 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
 %
 %  A description of each parameter follows:
 %
-%    o family: The type family.
+%    o family: the type family.
 %
-%    o style: The type style.
+%    o style: the type style.
 %
-%    o stretch: The type stretch.
+%    o stretch: the type stretch.
 %
-%    o weight: The type weight.
+%    o weight: the type weight.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
+
+static inline unsigned long MagickMax(const unsigned long x,
+  const unsigned long y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline unsigned long MagickMin(const unsigned long x,
+  const unsigned long y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   const StyleType style,const StretchType stretch,const unsigned long weight,
   ExceptionInfo *exception)
 {
   typedef struct _Fontmap
   {
-    char
+    const char
       *name,
       *substitute;
   } Fontmap;
@@ -244,14 +278,14 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   static Fontmap
     fontmap[] =
     {
-      { (char *) "fixed", (char *) "courier" },
-      { (char *) "modern",(char *) "courier" },
-      { (char *) "monotype corsiva", (char *) "courier" },
-      { (char *) "news gothic", (char *) "helvetica" },
-      { (char *) "system", (char *) "courier" },
-      { (char *) "terminal", (char *) "courier" },
-      { (char *) "wingdings", (char *) "symbol" },
-      { (char *) NULL, (char *) NULL }
+      { "fixed", "courier" },
+      { "modern","courier" },
+      { "monotype corsiva", "courier" },
+      { "news gothic", "helvetica" },
+      { "system", "courier" },
+      { "terminal", "courier" },
+      { "wingdings", "symbol" },
+      { NULL, NULL }
     };
 
   unsigned long
@@ -351,15 +385,15 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
     if (weight == 0)
       score+=16;
     else
-      score+=(16*(800-((long) Max(Min(weight,900),p->weight)-
-        (long) Min(Min(weight,900),p->weight))))/800;
+      score+=(16*(800-((long) MagickMax(MagickMin(weight,900),p->weight)-
+        (long) MagickMin(MagickMin(weight,900),p->weight))))/800;
     if ((stretch == UndefinedStretch) || (stretch == AnyStretch))
       score+=8;
     else
       {
         range=(long) UltraExpandedStretch-(long) NormalStretch;
-        score+=(8*(range-((long) Max(stretch,p->stretch)-
-          (long) Min(stretch,p->stretch))))/range;
+        score+=(8*(range-((long) MagickMax(stretch,p->stretch)-
+          (long) MagickMin(stretch,p->stretch))))/range;
       }
     if (score > max_score)
       {
@@ -472,8 +506,8 @@ MagickExport const TypeInfo **GetTypeInfoList(const char *pattern,
   p=GetTypeInfo("*",exception);
   if (p == (const TypeInfo *) NULL)
     return((const TypeInfo **) NULL);
-  fonts=(const TypeInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfNodesInSplayTree(type_list)+1)*sizeof(*fonts));
+  fonts=(const TypeInfo **) AcquireQuantumMemory((size_t)
+    GetNumberOfNodesInSplayTree(type_list)+1UL,sizeof(*fonts));
   if (fonts == (const TypeInfo **) NULL)
     return((const TypeInfo **) NULL);
   /*
@@ -485,7 +519,7 @@ MagickExport const TypeInfo **GetTypeInfoList(const char *pattern,
   for (i=0; p != (const TypeInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
       fonts[i++]=p;
     p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
   }
@@ -530,12 +564,12 @@ extern "C" {
 
 static int TypeCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -565,8 +599,8 @@ MagickExport char **GetTypeList(const char *pattern,unsigned long *number_fonts,
   p=GetTypeInfo("*",exception);
   if (p == (const TypeInfo *) NULL)
     return((char **) NULL);
-  fonts=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfNodesInSplayTree(type_list)+1)*sizeof(*fonts));
+  fonts=(char **) AcquireQuantumMemory((size_t)
+    GetNumberOfNodesInSplayTree(type_list)+1UL,sizeof(*fonts));
   if (fonts == (char **) NULL)
     return((char **) NULL);
   /*
@@ -578,8 +612,8 @@ MagickExport char **GetTypeList(const char *pattern,unsigned long *number_fonts,
   for (i=0; p != (const TypeInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
-      fonts[i++]=ConstantString(AcquireString(p->name));
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
+      fonts[i++]=ConstantString(p->name);
     p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
   }
   RelinquishSemaphoreInfo(type_semaphore);
@@ -611,6 +645,155 @@ MagickExport char **GetTypeList(const char *pattern,unsigned long *number_fonts,
 %    o exception: Return any errors or warnings in this structure.
 %
 */
+
+#if defined(MAGICKCORE_FONTCONFIG_DELEGATE)
+MagickExport MagickBooleanType LoadFontConfigFonts(SplayTreeInfo *type_list,
+  ExceptionInfo *exception)
+{
+  char
+    extension[MaxTextExtent],
+    name[MaxTextExtent];
+
+  FcChar8
+    *family,
+    *file,
+    *style;
+
+  FcConfig
+    *font_config;
+
+  FcFontSet
+    *font_set;
+
+  FcObjectSet
+    *object_set;
+
+  FcPattern
+    *pattern;
+
+  FcResult
+    status;
+
+  int
+    slant,
+    width,
+    weight;
+
+  register long
+    i;
+
+  TypeInfo
+    *type_info;
+
+  /*
+    Load system fonts.
+  */
+  (void) exception;
+  font_config=FcInitLoadConfigAndFonts();
+  if (font_config == (FcConfig *) NULL)
+    return(MagickFalse);
+  font_set=(FcFontSet *) NULL;
+  object_set=FcObjectSetBuild(FC_FAMILY,FC_STYLE,FC_SLANT,FC_WIDTH,FC_WEIGHT,
+    FC_FILE,(char *) NULL);
+  if (object_set != (FcObjectSet *) NULL)
+    {
+      pattern=FcPatternCreate();
+      if (pattern != (FcPattern *) NULL)
+        {
+          font_set=FcFontList(0,pattern,object_set);
+          FcPatternDestroy(pattern);
+        }
+      FcObjectSetDestroy(object_set);
+    }
+  if (font_set == (FcFontSet *) NULL)
+    {
+      FcConfigDestroy(font_config);
+      return(MagickFalse);
+    }
+  for (i=0; i < (long) font_set->nfont; i++)
+  {
+    status=FcPatternGetString(font_set->fonts[i],FC_FAMILY,0,&family);
+    if (status != FcResultMatch)
+      continue;
+    status=FcPatternGetString(font_set->fonts[i],FC_FILE,0,&file);
+    if (status != FcResultMatch)
+      continue;
+    *extension='\0';
+    GetPathComponent((const char *) file,ExtensionPath,extension);
+    if ((*extension != '\0') && (LocaleCompare(extension,"gz") == 0))
+      continue;
+    type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
+    if (type_info == (TypeInfo *) NULL)
+      continue;
+    (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
+    type_info->path=ConstantString("System Fonts");
+    type_info->signature=MagickSignature;
+    (void) CopyMagickString(name,(const char *) family,MaxTextExtent);
+    (void) ConcatenateMagickString(name," ",MaxTextExtent);
+    status=FcPatternGetString(font_set->fonts[i],FC_STYLE,0,&style);
+    if (status == FcResultMatch)
+      (void) ConcatenateMagickString(name,(const char *) style,MaxTextExtent);
+    type_info->name=ConstantString(name);
+    (void) SubstituteString(&type_info->name," ","-");
+    (void) SubstituteString(&type_info->name,"-L-","-");
+    (void) SubstituteString(&type_info->name,"semicondensed","SemiCondensed");
+    type_info->family=ConstantString((const char *) family);
+    (void) SubstituteString(&type_info->family," L","");
+    status=FcPatternGetInteger(font_set->fonts[i],FC_SLANT,0,&slant);
+    type_info->style=NormalStyle;
+    if (slant == FC_SLANT_ITALIC)
+      type_info->style=ItalicStyle;
+    if (slant == FC_SLANT_OBLIQUE)
+      type_info->style=ObliqueStyle;
+    status=FcPatternGetInteger(font_set->fonts[i],FC_WIDTH,0,&width);
+    type_info->stretch=NormalStretch;
+    if (width >= FC_WIDTH_ULTRACONDENSED)
+      type_info->stretch=UltraCondensedStretch;
+    if (width >= FC_WIDTH_EXTRACONDENSED)
+      type_info->stretch=ExtraCondensedStretch;
+    if (width >= FC_WIDTH_CONDENSED)
+      type_info->stretch=CondensedStretch;
+    if (width >= FC_WIDTH_SEMICONDENSED)
+      type_info->stretch=SemiCondensedStretch;
+    if (width >= FC_WIDTH_NORMAL)
+      type_info->stretch=NormalStretch;
+    if (width >= FC_WIDTH_SEMIEXPANDED)
+      type_info->stretch=SemiExpandedStretch;
+    if (width >= FC_WIDTH_EXPANDED)
+      type_info->stretch=ExpandedStretch;
+    if (width >= FC_WIDTH_EXTRAEXPANDED)
+      type_info->stretch=ExtraExpandedStretch;
+    if (width >= FC_WIDTH_ULTRAEXPANDED)
+      type_info->stretch=UltraExpandedStretch;
+    type_info->weight=400;
+    status=FcPatternGetInteger(font_set->fonts[i],FC_WEIGHT,0,&weight);
+    if (weight >= FC_WEIGHT_THIN)
+      type_info->weight=100;
+    if (weight >= FC_WEIGHT_EXTRALIGHT)
+      type_info->weight=200;
+    if (weight >= FC_WEIGHT_LIGHT)
+      type_info->weight=300;
+    if (weight >= FC_WEIGHT_NORMAL)
+      type_info->weight=400;
+    if (weight >= FC_WEIGHT_MEDIUM)
+      type_info->weight=500;
+    if (weight >= FC_WEIGHT_DEMIBOLD)
+      type_info->weight=600;
+    if (weight >= FC_WEIGHT_BOLD)
+      type_info->weight=700;
+    if (weight >= FC_WEIGHT_EXTRABOLD)
+      type_info->weight=800;
+    if (weight >= FC_WEIGHT_BLACK)
+      type_info->weight=900;
+    type_info->glyphs=ConstantString((const char *) file);
+    (void) AddValueToSplayTree(type_list,type_info->name,type_info);
+  }
+  FcFontSetDestroy(font_set);
+  FcConfigDestroy(font_config);
+  return(MagickTrue);
+}
+#endif
+
 static MagickBooleanType InitializeTypeList(ExceptionInfo *exception)
 {
   if ((type_list == (SplayTreeInfo *) NULL) &&
@@ -623,12 +806,15 @@ static MagickBooleanType InitializeTypeList(ExceptionInfo *exception)
           (void) LoadTypeLists(MagickTypeFilename,exception);
 #if defined(__WINDOWS__)
           (void) NTLoadTypeLists(type_list,exception);
+#endif
+#if defined(MAGICKCORE_FONTCONFIG_DELEGATE)
+          (void) LoadFontConfigFonts(type_list,exception);
 #endif
           instantiate_type=MagickTrue;
         }
       RelinquishSemaphoreInfo(type_semaphore);
     }
-  return((MagickBooleanType) (type_list != (SplayTreeInfo *) NULL));
+  return(type_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -654,7 +840,6 @@ static MagickBooleanType InitializeTypeList(ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
 {
@@ -663,6 +848,7 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
 
   const char
     *family,
+    *glyphs,
     *name,
     *path,
     *stretch,
@@ -689,16 +875,10 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
   {
     if (type_info[i]->stealth != MagickFalse)
       continue;
-    if ((path == (const char *) NULL) ||
-        (LocaleCompare(path,type_info[i]->path) != 0))
-      {
-        if (type_info[i]->path != (char *) NULL)
-          (void) fprintf(file,"\nPath: %s\n\n",type_info[i]->path);
-        (void) fprintf(file,"%-32.32s %-23.23s %-7.7s %-8s %-3s\n",
-          "Name","Family","Style","Stretch","Weight");
-        (void) fprintf(file,"--------------------------------------------------"
-          "------------------------------\n");
-      }
+    if (((path == (const char *) NULL) ||
+         (LocaleCompare(path,type_info[i]->path) != 0)) &&
+         (type_info[i]->path != (char *) NULL))
+      (void) fprintf(file,"\nPath: %s\n",type_info[i]->path);
     path=type_info[i]->path;
     name="unknown";
     if (type_info[i]->name != (char *) NULL)
@@ -708,9 +888,16 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
       family=type_info[i]->family;
     style=MagickOptionToMnemonic(MagickStyleOptions,type_info[i]->style);
     stretch=MagickOptionToMnemonic(MagickStretchOptions,type_info[i]->stretch);
+    glyphs="unknown";
+    if (type_info[i]->glyphs != (char *) NULL)
+      glyphs=type_info[i]->glyphs;
     (void) FormatMagickString(weight,MaxTextExtent,"%lu",type_info[i]->weight);
-    (void) fprintf(file,"%-32.32s %-23.23s %-7.7s %-9s %-3s\n",name,family,
-      style,stretch,weight);
+    (void) fprintf(file,"  Font: %s\n",name);
+    (void) fprintf(file,"    family: %s\n",family);
+    (void) fprintf(file,"    style: %s\n",style);
+    (void) fprintf(file,"    stretch: %s\n",stretch);
+    (void) fprintf(file,"    weight: %s\n",weight);
+    (void) fprintf(file,"    glyphs: %s\n",glyphs);
   }
   (void) fflush(file);
   type_info=(const TypeInfo **) RelinquishMagickMemory((void *) type_info);
@@ -746,7 +933,6 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 static void *DestroyTypeNode(void *type_info)
@@ -756,23 +942,23 @@ static void *DestroyTypeNode(void *type_info)
 
   p=(TypeInfo *) type_info;
   if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
+    p->path=DestroyString(p->path);
   if (p->name != (char *) NULL)
-    p->name=(char *) RelinquishMagickMemory(p->name);
+    p->name=DestroyString(p->name);
   if (p->description != (char *) NULL)
-    p->description=(char *) RelinquishMagickMemory(p->description);
+    p->description=DestroyString(p->description);
   if (p->family != (char *) NULL)
-    p->family=(char *) RelinquishMagickMemory(p->family);
+    p->family=DestroyString(p->family);
   if (p->encoding != (char *) NULL)
-    p->encoding=(char *) RelinquishMagickMemory(p->encoding);
+    p->encoding=DestroyString(p->encoding);
   if (p->foundry != (char *) NULL)
-    p->foundry=(char *) RelinquishMagickMemory(p->foundry);
+    p->foundry=DestroyString(p->foundry);
   if (p->format != (char *) NULL)
-    p->format=(char *) RelinquishMagickMemory(p->format);
+    p->format=DestroyString(p->format);
   if (p->metrics != (char *) NULL)
-    p->metrics=(char *) RelinquishMagickMemory(p->metrics);
+    p->metrics=DestroyString(p->metrics);
   if (p->glyphs != (char *) NULL)
-    p->glyphs=(char *) RelinquishMagickMemory(p->glyphs);
+    p->glyphs=DestroyString(p->glyphs);
   return(RelinquishMagickMemory(p));
 }
 
@@ -780,29 +966,29 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
 {
   char
-#if defined(__WINDOWS__)
     font_path[MaxTextExtent],
-#endif
     keyword[MaxTextExtent],
-    *q,
     *token;
 
-  MagickStatusType
+  const char
+    *q;
+
+  MagickBooleanType
     status;
 
   TypeInfo
-    *type_info = (TypeInfo *) NULL;
+    *type_info;
 
   /*
     Load the type map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading type file \"%s\" ...",filename);
+    "Loading type configure file \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (type_list == (SplayTreeInfo *) NULL)
     {
-      type_list=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
+      type_list=NewSplayTree(CompareSplayTreeString,(void *(*)(void *)) NULL,
         DestroyTypeNode);
       if (type_list == (SplayTreeInfo *) NULL)
         {
@@ -812,7 +998,8 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
-  token=AcquireString((char *) xml);
+  type_info=(TypeInfo *) NULL;
+  token=AcquireString(xml);
 #if defined(__WINDOWS__)
   /*
     Determine the Ghostscript font path.
@@ -830,6 +1017,15 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
@@ -862,15 +1058,23 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
                     path[MaxTextExtent],
                     *xml;
 
+                  ExceptionInfo
+                    *sans_exception;
+
                   *path='\0';
                   GetPathComponent(filename,HeadPath,path);
                   if (*path != '\0')
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
-                  status|=LoadTypeList(xml,path,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
+                  sans_exception=AcquireExceptionInfo();
+                  xml=FileToString(path,~0,sans_exception);
+                  sans_exception=DestroyExceptionInfo(sans_exception);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadTypeList(xml,path,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
                 }
             }
         }
@@ -879,14 +1083,13 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
     if (LocaleCompare(keyword,"<type") == 0)
       {
         /*
-          Allocate memory for the type list.
+          Type element.
         */
         type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
         if (type_info == (TypeInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
-        type_info->path=ConstantString(AcquireString(filename));
+        type_info->path=ConstantString(filename);
         type_info->signature=MagickSignature;
         continue;
       }
@@ -894,14 +1097,13 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       continue;
     if (LocaleCompare(keyword,"/>") == 0)
       {
-        status=AddValueToSplayTree(type_list,
-          ConstantString(AcquireString(type_info->name)),type_info);
+        status=AddValueToSplayTree(type_list,type_info->name,type_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",type_info->name);
         type_info=(TypeInfo *) NULL;
       }
-    GetMagickToken(q,(char **) NULL,token);
+    GetMagickToken(q,(const char **) NULL,token);
     if (*token != '=')
       continue;
     GetMagickToken(q,&q,token);
@@ -913,7 +1115,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"encoding") == 0)
           {
-            type_info->encoding=ConstantString(AcquireString(token));
+            type_info->encoding=ConstantString(token);
             break;
           }
         break;
@@ -928,22 +1130,22 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
           }
         if (LocaleCompare((char *) keyword,"family") == 0)
           {
-            type_info->family=ConstantString(AcquireString(token));
+            type_info->family=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
           {
-            type_info->format=ConstantString(AcquireString(token));
+            type_info->format=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"foundry") == 0)
           {
-            type_info->foundry=ConstantString(AcquireString(token));
+            type_info->foundry=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"fullname") == 0)
           {
-            type_info->description=ConstantString(AcquireString(token));
+            type_info->description=ConstantString(token);
             break;
           }
         break;
@@ -954,14 +1156,26 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
         if (LocaleCompare((char *) keyword,"glyphs") == 0)
           {
             char
-              *glyphs;
+              *path;
 
-            glyphs=AcquireString(token);
+            path=ConstantString(token);
 #if defined(__WINDOWS__)
-            if (strchr(glyphs,'@') != (char *) NULL)
-              SubstituteString(&glyphs,"@ghostscript_font_dir@",font_path);
+            if (strchr(path,'@') != (char *) NULL)
+              SubstituteString(&path,"@ghostscript_font_path@",font_path);
 #endif
-            type_info->glyphs=ConstantString(glyphs);
+            if (IsPathAccessible(path) == MagickFalse)
+              {
+                /*
+                  Relative path.
+                */
+                path=DestroyString(path);
+                GetPathComponent(filename,HeadPath,font_path);
+                (void) ConcatenateMagickString(font_path,DirectorySeparator,
+                  MaxTextExtent);
+                (void) ConcatenateMagickString(font_path,token,MaxTextExtent);
+                path=ConstantString(font_path);
+              }
+            type_info->glyphs=path;
             break;
           }
         break;
@@ -972,14 +1186,26 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
         if (LocaleCompare((char *) keyword,"metrics") == 0)
           {
             char
-              *metrics;
+              *path;
 
-            metrics=AcquireString(token);
+            path=ConstantString(token);
 #if defined(__WINDOWS__)
-            if (strchr(metrics,'@') != (char *) NULL)
-              SubstituteString(&metrics,"@ghostscript_font_dir@",font_path);
+            if (strchr(path,'@') != (char *) NULL)
+              SubstituteString(&path,"@ghostscript_font_path@",font_path);
 #endif
-            type_info->metrics=ConstantString(metrics);
+            if (IsPathAccessible(path) == MagickFalse)
+              {
+                /*
+                  Relative path.
+                */
+                path=DestroyString(path);
+                GetPathComponent(filename,HeadPath,font_path);
+                (void) ConcatenateMagickString(font_path,DirectorySeparator,
+                  MaxTextExtent);
+                (void) ConcatenateMagickString(font_path,token,MaxTextExtent);
+                path=ConstantString(font_path);
+              }
+            type_info->metrics=path;
             break;
           }
         break;
@@ -989,7 +1215,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            type_info->name=ConstantString(AcquireString(token));
+            type_info->name=ConstantString(token);
             break;
           }
         break;
@@ -999,20 +1225,19 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"stealth") == 0)
           {
-            type_info->stealth=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
+            type_info->stealth=IsMagickTrue(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"stretch") == 0)
           {
-            type_info->stretch=(StretchType)
-              ParseMagickOption(MagickStretchOptions,MagickFalse,token);
+            type_info->stretch=(StretchType) ParseMagickOption(
+              MagickStretchOptions,MagickFalse,token);
             break;
           }
         if (LocaleCompare((char *) keyword,"style") == 0)
           {
-            type_info->style=(StyleType)
-              ParseMagickOption(MagickStyleOptions,MagickFalse,token);
+            type_info->style=(StyleType) ParseMagickOption(MagickStyleOptions,
+              MagickFalse,token);
             break;
           }
         break;
@@ -1036,9 +1261,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
     }
   }
   token=(char *) RelinquishMagickMemory(token);
-  if (type_list == (SplayTreeInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  return(status);
 }
 
 /*
@@ -1053,7 +1276,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  LoadTypeList() loads one or more type configuration files which provides a
-%  mapping between type attributes a type name.
+%  mapping between type attributes and a type name.
 %
 %  The format of the LoadTypeLists method is:
 %
@@ -1062,24 +1285,21 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: the font file name.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadTypeLists(const char *filename,
   ExceptionInfo *exception)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   return(LoadTypeList(TypeMap,"built-in",0,exception));
 #else
   char
+    *font_path,
     path[MaxTextExtent];
 
-  const char
-    *p;
-
   const StringInfo
     *option;
 
@@ -1095,42 +1315,34 @@ static MagickBooleanType LoadTypeLists(const char *filename,
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    (void) CopyMagickString(path,option->path,MaxTextExtent);
-    status|=LoadTypeList((const char *) option->datum,option->path,0,exception);
+    (void) CopyMagickString(path,GetStringInfoPath(option),MaxTextExtent);
+    status|=LoadTypeList((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
-  p=getenv("MAGICK_FONT_PATH");
-  if (p != (const char *) NULL)
+  font_path=GetEnvironmentValue("MAGICK_FONT_PATH");
+  if (font_path != (char *) NULL)
     {
+      char
+        *option;
+
       /*
         Search MAGICK_FONT_PATH.
       */
-      (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",p,
+      (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",font_path,
         DirectorySeparator,filename);
-      if (IsAccessible(path) != MagickFalse)
+      option=FileToString(path,~0,exception);
+      if (option != (void *) NULL)
         {
-          char
-            *option;
-
-          size_t
-            length;
-
-          option=(char *) FileToBlob(path,&length,exception);
-          if (option != (void *) NULL)
-            {
-              status|=LoadTypeList(option,path,0,exception);
-              option=(char *) RelinquishMagickMemory(option);
-            }
+          status|=LoadTypeList(option,path,0,exception);
+          option=DestroyString(option);
         }
+      font_path=DestroyString(font_path);
     }
-  if ((type_list == (SplayTreeInfo *) NULL) || 
+  if ((type_list == (SplayTreeInfo *) NULL) ||
       (GetNumberOfNodesInSplayTree(type_list) == 0))
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-        "UnableToOpenConfigureFile","`%s'",path);
-      status|=LoadTypeList(TypeMap,"built-in",0,exception);
-    }
+    status|=LoadTypeList(TypeMap,"built-in",0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
 #endif
 }
