@@ -150,11 +150,11 @@ static char
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
-%    o tag: The tag.
+%    o tag: the tag.
 %
-%    o offset: The tag offset.
+%    o offset: the tag offset.
 %
 */
 MagickExport XMLTreeInfo *AddChildToXMLTree(XMLTreeInfo *xml_info,
@@ -198,11 +198,11 @@ MagickExport XMLTreeInfo *AddChildToXMLTree(XMLTreeInfo *xml_info,
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
-%    o path: The path.
+%    o path: the path.
 %
-%    o offset: The tag offset.
+%    o offset: the tag offset.
 %
 */
 MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
@@ -277,11 +277,16 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
 %
 %  The format of the CanonicalXMLContent method is:
 %
-%      char *CanonicalXMLContent(const char *content)
+%
+%      char *CanonicalXMLContent(const char *content,
+%        const MagickBooleanType pedantic)
 %
 %  A description of each parameter follows:
 %
-%    o content: The content.
+%    o content: the content.
+%
+%    o pedantic: if true, replace newlines and tabs with their respective
+%      entities.
 %
 */
 
@@ -340,14 +345,12 @@ MagickExport char *CanonicalXMLContent(const char *content,
     i;
 
   size_t
+    extent,
     length;
 
   unsigned char
     *utf8;
 
-  unsigned long
-    extent;
-
   utf8=ConvertLatin1ToUTF8((const unsigned char *) content);
   if (utf8 == (unsigned char *) NULL)
     return((char *) NULL);
@@ -380,8 +383,8 @@ MagickExport char *CanonicalXMLContent(const char *content,
     if ((i+MaxTextExtent) > (long) extent)
       {
         extent+=MaxTextExtent;
-        canonical_content=(char *) ResizeQuantumMemory(canonical_content,
-          extent,sizeof(*canonical_content));
+        canonical_content=(char *) ResizeQuantumMemory(canonical_content,extent,
+          sizeof(*canonical_content));
         if (canonical_content == (char *) NULL)
           return(canonical_content);
       }
@@ -463,7 +466,7 @@ MagickExport char *CanonicalXMLContent(const char *content,
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 
@@ -540,17 +543,21 @@ MagickExport XMLTreeInfo *DestroyXMLTree(XMLTreeInfo *xml_info)
       }
       if (root->attributes[0] != (char **) NULL)
         root->attributes=(char ***) RelinquishMagickMemory(root->attributes);
-      for (i=0; root->processing_instructions[i] != (char **) NULL; i++)
-      {
-        for (j=1; root->processing_instructions[i][j]; j++);
-        root->processing_instructions[i][j+1]=(char *)
-          RelinquishMagickMemory(root->processing_instructions[i][j+1]);
-        root->processing_instructions[i]=(char **)
-          RelinquishMagickMemory(root->processing_instructions[i]);
-      }
       if (root->processing_instructions[0] != (char **) NULL)
-        root->processing_instructions=(char ***)
-          RelinquishMagickMemory(root->processing_instructions);
+        {
+          for (i=0; root->processing_instructions[i] != (char **) NULL; i++)
+          {
+            for (j=0; root->processing_instructions[i][j] != (char *) NULL; j++)
+              root->processing_instructions[i][j]=DestroyString(
+                root->processing_instructions[i][j]);
+            root->processing_instructions[i][j+1]=DestroyString(
+              root->processing_instructions[i][j+1]);
+            root->processing_instructions[i]=(char **) RelinquishMagickMemory(
+              root->processing_instructions[i]);
+          }
+          root->processing_instructions=(char ***) RelinquishMagickMemory(
+            root->processing_instructions);
+        }
     }
   xml_info->attributes=DestroyXMLTreeAttributes(xml_info->attributes);
   xml_info->content=DestroyString(xml_info->content);
@@ -578,7 +585,7 @@ MagickExport XMLTreeInfo *DestroyXMLTree(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport XMLTreeInfo *GetNextXMLTreeTag(XMLTreeInfo *xml_info)
@@ -610,9 +617,9 @@ MagickExport XMLTreeInfo *GetNextXMLTreeTag(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
-%    o tag: The attribute tag.
+%    o tag: the attribute tag.
 %
 */
 MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
@@ -678,9 +685,9 @@ MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
-%    o attributes: The attribute splay-tree.
+%    o attributes: the attribute splay-tree.
 %
 */
 MagickExport MagickBooleanType GetXMLTreeAttributes(const XMLTreeInfo *xml_info,
@@ -727,7 +734,7 @@ MagickExport MagickBooleanType GetXMLTreeAttributes(const XMLTreeInfo *xml_info,
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport XMLTreeInfo *GetXMLTreeChild(XMLTreeInfo *xml_info,const char *tag)
@@ -762,11 +769,11 @@ MagickExport XMLTreeInfo *GetXMLTreeChild(XMLTreeInfo *xml_info,const char *tag)
 %
 %  The format of the GetXMLTreeContent method is:
 %
-%      XMLTreeInfo *GetXMLTreeContent(XMLTreeInfo *xml_info)
+%      const char *GetXMLTreeContent(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport const char *GetXMLTreeContent(XMLTreeInfo *xml_info)
@@ -797,7 +804,7 @@ MagickExport const char *GetXMLTreeContent(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport XMLTreeInfo *GetXMLTreeOrdered(XMLTreeInfo *xml_info)
@@ -829,9 +836,9 @@ MagickExport XMLTreeInfo *GetXMLTreeOrdered(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
-%    o path: The path (e.g. property/elapsed-time).
+%    o path: the path (e.g. property/elapsed-time).
 %
 */
 MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
@@ -900,12 +907,12 @@ MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
 %
 %  The format of the GetXMLTreeProcessingInstructions method is:
 %
-%      XMLTreeInfo *GetXMLTreeProcessingInstructions(XMLTreeInfo *xml_info,
+%      const char **GetXMLTreeProcessingInstructions(XMLTreeInfo *xml_info,
 %        const char *target)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport const char **GetXMLTreeProcessingInstructions(
@@ -952,7 +959,7 @@ MagickExport const char **GetXMLTreeProcessingInstructions(
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport XMLTreeInfo *GetXMLTreeSibling(XMLTreeInfo *xml_info)
@@ -979,11 +986,11 @@ MagickExport XMLTreeInfo *GetXMLTreeSibling(XMLTreeInfo *xml_info)
 %
 %  The format of the GetXMLTreeTag method is:
 %
-%      XMLTreeInfo *GetXMLTreeTag(XMLTreeInfo *xml_info)
+%      const char *GetXMLTreeTag(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport const char *GetXMLTreeTag(XMLTreeInfo *xml_info)
@@ -1016,11 +1023,11 @@ MagickExport const char *GetXMLTreeTag(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
-%    o child: The child tag.
+%    o child: the child tag.
 %
-%    o offset: The tag offset.
+%    o offset: the tag offset.
 %
 */
 MagickExport XMLTreeInfo *InsertTagIntoXMLTree(XMLTreeInfo *xml_info,
@@ -1134,7 +1141,7 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
     i;
 
   size_t
-    max_length;
+    extent;
 
   utf8=(char *) AcquireQuantumMemory(*length,sizeof(*utf8));
   if (utf8 == (char *) NULL)
@@ -1145,11 +1152,11 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
       /*
         Already UTF-8.
       */
-      (void) CopyMagickMemory(utf8,content,*length);
+      (void) CopyMagickMemory(utf8,content,*length*sizeof(*utf8));
       return(utf8);
     }
   j=0;
-  max_length=(*length);
+  extent=(*length);
   for (i=2; i < (long) (*length-1); i+=2)
   {
     c=(encoding != 0) ? ((content[i] & 0xff) << 8) | (content[i+1] & 0xff) :
@@ -1161,10 +1168,10 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
           (content[i] & 0xff);
         c=(((c & 0x3ff) << 10) | (byte & 0x3ff))+0x10000;
       }
-    if ((size_t) (j+MaxTextExtent) > max_length)
+    if ((size_t) (j+MaxTextExtent) > extent)
       {
-        max_length=(size_t) j+MaxTextExtent;
-        utf8=(char *) ResizeQuantumMemory(utf8,max_length,sizeof(*utf8));
+        extent=(size_t) j+MaxTextExtent;
+        utf8=(char *) ResizeQuantumMemory(utf8,extent,sizeof(*utf8));
         if (utf8 == (char *) NULL)
           return(utf8);
       }
@@ -1193,7 +1200,7 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
   return((char *) ResizeQuantumMemory(utf8,*length,sizeof(*utf8)));
 }
 
-static char *ParseEntities(char *xml,char **entities,char state)
+static char *ParseEntities(char *xml,char **entities,int state)
 {
   char
     *entity;
@@ -1302,7 +1309,7 @@ static char *ParseEntities(char *xml,char **entities,char state)
               */
               length=strlen(entities[i]);
               entity=strchr(xml,';');
-              if (length > (size_t) (entity-xml))
+              if ((length-1L) >= (size_t) (entity-xml))
                 {
                   offset=(ssize_t) (xml-p);
                   extent=(size_t) (offset+length+strlen(entity));
@@ -1496,8 +1503,8 @@ static void ParseProcessingInstructions(XMLTreeRoot *root,char *xml,
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) CopyMagickString(root->processing_instructions[i][j+2]+j-1,
     root->root.tag != (char *) NULL ? ">" : "<",2);
-  root->processing_instructions[i][j+1]=(char *) NULL;
   root->processing_instructions[i][j]=ConstantString(xml);
+  root->processing_instructions[i][j+1]=(char *) NULL;
 }
 
 static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
@@ -1551,7 +1558,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
             continue;
           }
         entities=(*c == '%') ? predefined_entitites : root->entities;
-        for (i=0; entities[i] != (char *) NULL; i++);
+        for (i=0; entities[i] != (char *) NULL; i++) ;
         entities=(char **) ResizeQuantumMemory(entities,(size_t) (i+3),
           sizeof(*entities));
         if (entities == (char **) NULL)
@@ -1578,8 +1585,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
               entities[i+1]=DestroyString(entities[i+1]);
             (void) ThrowMagickException(exception,GetMagickModule(),
               OptionWarning,"ParseError","circular entity declaration &%s",n);
-            predefined_entitites=(char **)
-              RelinquishMagickMemory(predefined_entitites);
+            predefined_entitites=(char **) RelinquishMagickMemory(
+              predefined_entitites);
             return(MagickFalse);
           }
         }
@@ -1594,8 +1601,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
               {
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   OptionWarning,"ParseError","unclosed <!ATTLIST");
-                predefined_entitites=(char **)
-                  RelinquishMagickMemory(predefined_entitites);
+                predefined_entitites=(char **) RelinquishMagickMemory(
+                  predefined_entitites);
                 return(MagickFalse);
               }
             xml=t+strcspn(t,XMLWhitespace ">");
@@ -1606,7 +1613,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
             while ((root->attributes[i] != (char **) NULL) &&
                     (strcmp(n,root->attributes[i][0]) != 0))
               i++;
-            while (*(n=++xml+strspn(xml,XMLWhitespace)) && (*n != '>'))
+            while (*(n=(++xml)+strspn(xml,XMLWhitespace)) && (*n != '>'))
             {
               xml=n+strcspn(n,XMLWhitespace);
               if (*xml != '\0')
@@ -1615,8 +1622,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                 {
                   (void) ThrowMagickException(exception,GetMagickModule(),
                     OptionWarning,"ParseError","malformed <!ATTLIST");
-                  predefined_entitites=(char **)
-                    RelinquishMagickMemory(predefined_entitites);
+                  predefined_entitites=(char **) RelinquishMagickMemory(
+                    predefined_entitites);
                   return(MagickFalse);
                 }
               xml+=strspn(xml+1,XMLWhitespace)+1;
@@ -1629,8 +1636,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                 {
                   (void) ThrowMagickException(exception,GetMagickModule(),
                     OptionWarning,"ParseError","malformed <!ATTLIST");
-                  predefined_entitites=(char **)
-                    RelinquishMagickMemory(predefined_entitites);
+                  predefined_entitites=(char **) RelinquishMagickMemory(
+                    predefined_entitites);
                   return(MagickFalse);
                 }
               xml+=strspn(xml,XMLWhitespace ")");
@@ -1651,8 +1658,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                   {
                     (void) ThrowMagickException(exception,GetMagickModule(),
                       OptionWarning,"ParseError","malformed <!ATTLIST");
-                    predefined_entitites=(char **)
-                      RelinquishMagickMemory(predefined_entitites);
+                    predefined_entitites=(char **) RelinquishMagickMemory(
+                      predefined_entitites);
                     return(MagickFalse);
                   }
               if (root->attributes[i] == (char **) NULL)
@@ -1679,7 +1686,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                   root->attributes[i][1]=(char *) NULL;
                   root->attributes[i+1]=(char **) NULL;
                 }
-              for (j=1; root->attributes[i][j] != (char *) NULL; j+=3);
+              for (j=1; root->attributes[i][j] != (char *) NULL; j+=3) ;
               root->attributes[i]=(char **) ResizeQuantumMemory(
                 root->attributes[i],(size_t) (j+4),sizeof(*root->attributes));
               if (root->attributes[i] == (char **) NULL)
@@ -1790,6 +1797,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
         "ParseError","root tag missing");
+      utf8=DestroyString(utf8);
       return((XMLTreeInfo *) NULL);
     }
   attribute=(char **) NULL;
@@ -1807,6 +1815,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
           {
             (void) ThrowMagickException(exception,GetMagickModule(),
               OptionWarning,"ParseError","root tag missing");
+            utf8=DestroyString(utf8);
             return(&root->root);
           }
         p+=strcspn(p,XMLWhitespace "/>");
@@ -1837,6 +1846,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
             {
               (void) ThrowMagickException(exception,GetMagickModule(),
                 ResourceLimitError,"MemoryAllocationFailed","`%s'","");
+              utf8=DestroyString(utf8);
               return(&root->root);
             }
           attributes[l+2]=(char *) NULL;
@@ -1863,9 +1873,12 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                     *p++='\0';
                   else
                     {
+                      attributes[l]=ConstantString("");
+                      attributes[l+1]=ConstantString("");
                       (void) DestroyXMLTreeAttributes(attributes);
                       (void) ThrowMagickException(exception,GetMagickModule(),
                         OptionWarning,"ParseError","missing %c",c);
+                      utf8=DestroyString(utf8);
                       return(&root->root);
                     }
                   j=1;
@@ -1895,6 +1908,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                   (void) DestroyXMLTreeAttributes(attributes);
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   OptionWarning,"ParseError","missing >");
+                utf8=DestroyString(utf8);
                 return(&root->root);
               }
             ParseOpenTag(root,tag,attributes);
@@ -1915,6 +1929,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                   (void) DestroyXMLTreeAttributes(attributes);
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   OptionWarning,"ParseError","missing >");
+                utf8=DestroyString(utf8);
                 return(&root->root);
               }
           }
@@ -1932,11 +1947,15 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
             {
               (void) ThrowMagickException(exception,GetMagickModule(),
                 OptionWarning,"ParseError","missing >");
+              utf8=DestroyString(utf8);
               return(&root->root);
             }
           *p='\0';
           if (ParseCloseTag(root,tag,p,exception) != (XMLTreeInfo *) NULL)
-            return(&root->root);
+            {
+              utf8=DestroyString(utf8);
+              return(&root->root);
+            }
           *p=c;
           if (isspace((int) ((unsigned char) *p)) != 0)
             p+=strspn(p,XMLWhitespace);
@@ -1953,6 +1972,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
               {
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   OptionWarning,"ParseError","unclosed <!--");
+                utf8=DestroyString(utf8);
                 return(&root->root);
               }
           }
@@ -1972,6 +1992,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                 {
                   (void) ThrowMagickException(exception,GetMagickModule(),
                     OptionWarning,"ParseError","unclosed <![CDATA[");
+                  utf8=DestroyString(utf8);
                   return(&root->root);
                 }
             }
@@ -1990,6 +2011,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                   {
                     (void) ThrowMagickException(exception,GetMagickModule(),
                       OptionWarning,"ParseError","unclosed <!DOCTYPE");
+                    utf8=DestroyString(utf8);
                     return(&root->root);
                   }
                 if (l != 0)
@@ -1999,7 +2021,10 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                     status=ParseInternalDoctype(root,tag,(size_t) (p-tag),
                       exception);
                     if (status == MagickFalse)
-                      return(&root->root);
+                      {
+                        utf8=DestroyString(utf8);
+                        return(&root->root);
+                      }
                     p++;
                   }
               }
@@ -2021,6 +2046,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                     {
                       (void) ThrowMagickException(exception,GetMagickModule(),
                         OptionWarning,"ParseError","unclosed <?");
+                      utf8=DestroyString(utf8);
                       return(&root->root);
                     }
                   ParseProcessingInstructions(root,tag+1,(size_t) (p-tag-2));
@@ -2029,6 +2055,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                 {
                   (void) ThrowMagickException(exception,GetMagickModule(),
                     OptionWarning,"ParseError","unexpected <");
+                  utf8=DestroyString(utf8);
                   return(&root->root);
                 }
      if ((p == (char *) NULL) || (*p == '\0'))
@@ -2083,7 +2110,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o tag: The tag.
+%    o tag: the tag.
 %
 */
 MagickExport XMLTreeInfo *NewXMLTreeTag(const char *tag)
@@ -2140,7 +2167,7 @@ MagickExport XMLTreeInfo *NewXMLTreeTag(const char *tag)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 MagickExport XMLTreeInfo *PruneTagFromXMLTree(XMLTreeInfo *xml_info)
@@ -2209,7 +2236,7 @@ MagickExport XMLTreeInfo *PruneTagFromXMLTree(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 %    o tag:  The attribute tag.
 %
@@ -2263,7 +2290,7 @@ MagickExport XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
   /*
     Add new value to an existing attribute.
   */
-  for (j=i; xml_info->attributes[j] != (char *) NULL; j+=2);
+  for (j=i; xml_info->attributes[j] != (char *) NULL; j+=2) ;
   if (xml_info->attributes[i+1] != (char *) NULL)
     xml_info->attributes[i+1]=DestroyString(xml_info->attributes[i+1]);
   if (value != (const char *) NULL)
@@ -2275,13 +2302,13 @@ MagickExport XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
     xml_info->attributes[i]=DestroyString(xml_info->attributes[i]);
   (void) CopyMagickMemory(xml_info->attributes+i,xml_info->attributes+i+2,
     (size_t) (j-i)*sizeof(*xml_info->attributes));
+  j-=2;
   xml_info->attributes=(char **) ResizeQuantumMemory(xml_info->attributes,
     (size_t) (j+2),sizeof(*xml_info->attributes));
   if (xml_info->attributes == (char **) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  j-=2;
   (void) CopyMagickMemory(xml_info->attributes[j+1]+(i/2),
-    xml_info->attributes[j+1]+(i/2)+1,(size_t) (j/2)-(i/2)*
+    xml_info->attributes[j+1]+(i/2)+1,(size_t) ((j/2)-(i/2))*
     sizeof(*xml_info->attributes));
   return(xml_info);
 }
@@ -2307,7 +2334,7 @@ MagickExport XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 %    o content:  The content.
 %
@@ -2340,17 +2367,16 @@ MagickExport XMLTreeInfo *SetXMLTreeContent(XMLTreeInfo *xml_info,
 %
 %  The format of the XMLTreeInfoToXML method is:
 %
-%      XMLTreeInfo *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
+%      char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
-%    o xml_info: The xml info.
+%    o xml_info: the xml info.
 %
 */
 
 static char *EncodePredefinedEntities(const char *source,ssize_t offset,
-  char **destination,size_t *length,size_t *max_length,
-  MagickBooleanType pedantic)
+  char **destination,size_t *length,size_t *extent,MagickBooleanType pedantic)
 {
   char
     *canonical_content;
@@ -2369,22 +2395,22 @@ static char *EncodePredefinedEntities(const char *source,ssize_t offset,
     }
   if (canonical_content == (char *) NULL)
     return(*destination);
-  if ((*length+strlen(canonical_content)+MaxTextExtent) > *max_length)
+  if ((*length+strlen(canonical_content)+MaxTextExtent) > *extent)
     {
-      *max_length=(*length)+strlen(canonical_content)+MaxTextExtent;
-      *destination=(char *) ResizeQuantumMemory(*destination,*max_length,
+      *extent=(*length)+strlen(canonical_content)+MaxTextExtent;
+      *destination=(char *) ResizeQuantumMemory(*destination,*extent,
         sizeof(**destination));
       if (*destination == (char *) NULL)
         return(*destination);
     }
-  *length+=FormatMagickString(*destination+(*length),*max_length,"%s",
+  *length+=FormatMagickString(*destination+(*length),*extent,"%s",
     canonical_content);
   canonical_content=DestroyString(canonical_content);
   return(*destination);
 }
 
 static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
-  size_t *max_length,size_t start,char ***attributes)
+  size_t *extent,size_t start,char ***attributes)
 {
   char
     *content;
@@ -2406,34 +2432,32 @@ static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
     content=xml_info->parent->content;
   offset=0;
   *source=EncodePredefinedEntities(content+start,(ssize_t) (xml_info->offset-
-    start),source,length,max_length,MagickFalse);
-  if ((*length+strlen(xml_info->tag)+MaxTextExtent) > *max_length)
+    start),source,length,extent,MagickFalse);
+  if ((*length+strlen(xml_info->tag)+MaxTextExtent) > *extent)
     {
-      *max_length=(*length)+strlen(xml_info->tag)+MaxTextExtent;
-      *source=(char *) ResizeQuantumMemory(*source,*max_length,sizeof(*source));
+      *extent=(*length)+strlen(xml_info->tag)+MaxTextExtent;
+      *source=(char *) ResizeQuantumMemory(*source,*extent,sizeof(*source));
       if (*source == (char *) NULL)
         return(*source);
     }
-  *length+=FormatMagickString(*source+(*length),*max_length,"<%s",
-    xml_info->tag);
+  *length+=FormatMagickString(*source+(*length),*extent,"<%s",xml_info->tag);
   for (i=0; xml_info->attributes[i]; i+=2)
   {
     attribute=GetXMLTreeAttribute(xml_info,xml_info->attributes[i]);
     if (attribute != xml_info->attributes[i+1])
       continue;
-    if ((*length+strlen(xml_info->attributes[i])+MaxTextExtent) > *max_length)
+    if ((*length+strlen(xml_info->attributes[i])+MaxTextExtent) > *extent)
       {
-        *max_length=(*length)+strlen(xml_info->attributes[i])+MaxTextExtent;
-        *source=(char *) ResizeQuantumMemory(*source,*max_length,
-          sizeof(**source));
+        *extent=(*length)+strlen(xml_info->attributes[i])+MaxTextExtent;
+        *source=(char *) ResizeQuantumMemory(*source,*extent,sizeof(**source));
         if (*source == (char *) NULL)
           return((char *) NULL);
       }
-    *length+=FormatMagickString(*source+(*length),*max_length," %s=\"",
+    *length+=FormatMagickString(*source+(*length),*extent," %s=\"",
       xml_info->attributes[i]);
     (void) EncodePredefinedEntities(xml_info->attributes[i+1],-1,source,length,
-      max_length,MagickTrue);
-    *length+=FormatMagickString(*source+(*length),*max_length,"\"");
+      extent,MagickTrue);
+    *length+=FormatMagickString(*source+(*length),*extent,"\"");
   }
   i=0;
   while ((attributes[i] != (char **) NULL) &&
@@ -2449,47 +2473,44 @@ static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
         j+=3;
         continue;
       }
-    if ((*length+strlen(attributes[i][j])+MaxTextExtent) > *max_length)
+    if ((*length+strlen(attributes[i][j])+MaxTextExtent) > *extent)
       {
-        *max_length=(*length)+strlen(attributes[i][j])+MaxTextExtent;
-        *source=(char *) ResizeQuantumMemory(*source,*max_length,
-          sizeof(**source));
+        *extent=(*length)+strlen(attributes[i][j])+MaxTextExtent;
+        *source=(char *) ResizeQuantumMemory(*source,*extent,sizeof(**source));
         if (*source == (char *) NULL)
           return((char *) NULL);
       }
-    *length+=FormatMagickString(*source+(*length),*max_length," %s=\"",
+    *length+=FormatMagickString(*source+(*length),*extent," %s=\"",
       attributes[i][j]);
-    (void) EncodePredefinedEntities(attributes[i][j+1],-1,source,length,
-      max_length,MagickTrue);
-    *length+=FormatMagickString(*source+(*length),*max_length,"\"");
+    (void) EncodePredefinedEntities(attributes[i][j+1],-1,source,length,extent,
+      MagickTrue);
+    *length+=FormatMagickString(*source+(*length),*extent,"\"");
     j+=3;
   }
-  *length+=FormatMagickString(*source+(*length),*max_length,*xml_info->content ?
+  *length+=FormatMagickString(*source+(*length),*extent,*xml_info->content ?
     ">" : "/>");
   if (xml_info->child != (XMLTreeInfo *) NULL)
-    *source=XMLTreeTagToXML(xml_info->child,source,length,max_length,0,
-      attributes);
+    *source=XMLTreeTagToXML(xml_info->child,source,length,extent,0,attributes);
   else
-    *source=EncodePredefinedEntities(xml_info->content,-1,source,length,
-      max_length,MagickFalse);
-  if ((*length+strlen(xml_info->tag)+MaxTextExtent) > *max_length)
+    *source=EncodePredefinedEntities(xml_info->content,-1,source,length,extent,
+      MagickFalse);
+  if ((*length+strlen(xml_info->tag)+MaxTextExtent) > *extent)
     {
-      *max_length=(*length)+strlen(xml_info->tag)+MaxTextExtent;
-      *source=(char *) ResizeQuantumMemory(*source,*max_length,
-        sizeof(**source));
+      *extent=(*length)+strlen(xml_info->tag)+MaxTextExtent;
+      *source=(char *) ResizeQuantumMemory(*source,*extent,sizeof(**source));
       if (*source == (char *) NULL)
         return((char *) NULL);
     }
   if (*xml_info->content != '\0')
-    *length+=FormatMagickString(*source+(*length),*max_length,"</%s>",
-     xml_info->tag);
+    *length+=FormatMagickString(*source+(*length),*extent,"</%s>",
+      xml_info->tag);
   while ((content[offset] != '\0') && (offset < xml_info->offset))
     offset++;
   if (xml_info->ordered != (XMLTreeInfo *) NULL)
-    content=XMLTreeTagToXML(xml_info->ordered,source,length,max_length,offset,
+    content=XMLTreeTagToXML(xml_info->ordered,source,length,extent,offset,
       attributes);
   else
-    content=EncodePredefinedEntities(content+offset,-1,source,length,max_length,
+    content=EncodePredefinedEntities(content+offset,-1,source,length,extent,
       MagickFalse);
   return(content);
 }
@@ -2511,8 +2532,8 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
     i;
 
   size_t
-    length,
-    max_length;
+    extent,
+    length;
 
   XMLTreeInfo
     *ordered,
@@ -2529,7 +2550,7 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
     return((char *) NULL);
   xml=AcquireString((char *) NULL);
   length=0;
-  max_length=MaxTextExtent;
+  extent=MaxTextExtent;
   root=(XMLTreeRoot *) xml_info;
   while (root->root.parent != (XMLTreeInfo *) NULL)
     root=(XMLTreeRoot *) root->root.parent;
@@ -2542,7 +2563,7 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
       /*
         Pre-root processing instructions.
       */
-      for (k=2; root->processing_instructions[i][k-1]; k++);
+      for (k=2; root->processing_instructions[i][k-1]; k++) ;
       p=root->processing_instructions[i][1];
       for (j=1; p != (char *) NULL; j++)
       {
@@ -2552,14 +2573,14 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
             continue;
           }
         q=root->processing_instructions[i][0];
-        if ((length+strlen(p)+strlen(q)+MaxTextExtent) > max_length)
+        if ((length+strlen(p)+strlen(q)+MaxTextExtent) > extent)
           {
-            max_length=length+strlen(p)+strlen(q)+MaxTextExtent;
-            xml=(char *) ResizeQuantumMemory(xml,max_length,sizeof(*xml));
+            extent=length+strlen(p)+strlen(q)+MaxTextExtent;
+            xml=(char *) ResizeQuantumMemory(xml,extent,sizeof(*xml));
             if (xml == (char *) NULL)
               return(xml);
           }
-        length+=FormatMagickString(xml+length,max_length,"<?%s%s%s?>\n",q,
+        length+=FormatMagickString(xml+length,extent,"<?%s%s%s?>\n",q,
           *p != '\0' ? " " : "",p);
         p=root->processing_instructions[i][j];
       }
@@ -2569,7 +2590,7 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
     ordered=xml_info->ordered;
   xml_info->parent=(XMLTreeInfo *) NULL;
   xml_info->ordered=(XMLTreeInfo *) NULL;
-  xml=XMLTreeTagToXML(xml_info,&xml,&length,&max_length,0,root->attributes);
+  xml=XMLTreeTagToXML(xml_info,&xml,&length,&extent,0,root->attributes);
   xml_info->parent=parent;
   xml_info->ordered=ordered;
   if (parent == (XMLTreeInfo *) NULL)
@@ -2578,7 +2599,7 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
       /*
         Post-root processing instructions.
       */
-      for (k=2; root->processing_instructions[i][k-1]; k++);
+      for (k=2; root->processing_instructions[i][k-1]; k++) ;
       p=root->processing_instructions[i][1];
       for (j=1; p != (char *) NULL; j++)
       {
@@ -2588,14 +2609,14 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
             continue;
           }
         q=root->processing_instructions[i][0];
-        if ((length+strlen(p)+strlen(q)+MaxTextExtent) > max_length)
+        if ((length+strlen(p)+strlen(q)+MaxTextExtent) > extent)
           {
-            max_length=length+strlen(p)+strlen(q)+MaxTextExtent;
-            xml=(char *) ResizeQuantumMemory(xml,max_length,sizeof(*xml));
+            extent=length+strlen(p)+strlen(q)+MaxTextExtent;
+            xml=(char *) ResizeQuantumMemory(xml,extent,sizeof(*xml));
             if (xml == (char *) NULL)
               return(xml);
           }
-        length+=FormatMagickString(xml+length,max_length,"\n<?%s%s%s?>",q,
+        length+=FormatMagickString(xml+length,extent,"\n<?%s%s%s?>",q,
           *p != '\0' ? " " : "",p);
         p=root->processing_instructions[i][j];
       }
