@@ -54,7 +54,10 @@ int main ( int argc, char **argv )
     *imageInfo;
 
   ExceptionInfo
-    exception;
+    *exception;
+
+  MagickBooleanType
+    status;    
 
   if (LocaleNCompare("rwfile",argv[0],7) == 0)
     MagickCoreGenesis((char *) NULL,MagickTrue);
@@ -62,7 +65,7 @@ int main ( int argc, char **argv )
     MagickCoreGenesis(*argv,MagickTrue);
 
   imageInfo=CloneImageInfo(0);
-  GetExceptionInfo( &exception );
+  exception=AcquireExceptionInfo();
 
   for (arg=1; arg < argc; arg++)
     {
@@ -135,7 +138,7 @@ if (0)
             }
           else if (LocaleCompare("verbose",option+1) == 0)
             {
-              imageInfo->verbose+=1;
+              imageInfo->verbose=MagickTrue;
             }
         }
       else
@@ -162,16 +165,16 @@ if (0)
   /*
    * Read original image
    */
-
-  imageInfo->dither = 0;
+  imageInfo=DestroyImageInfo(imageInfo);
+  imageInfo=CloneImageInfo(0);
+  ClearMagickException(exception);
   (void) strncpy( imageInfo->filename, infile, MaxTextExtent-1 );
-  (void) fflush(stdout);
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
     "Reading image %s", imageInfo->filename);
-  original = ReadImage ( imageInfo, &exception );
-  if (exception.severity >= ErrorException)
+  original = ReadImage ( imageInfo, exception );
+  if (exception->severity >= ErrorException)
     {
-      CatchException(&exception);
+      CatchException(exception);
       exit_status = 1;
       goto program_exit;
     }
@@ -189,7 +192,7 @@ if (0)
   rows    = original->rows;
   columns = original->columns;
   size[0] = '\0';
-  magick_info=GetMagickInfo(format,&exception);
+  magick_info=GetMagickInfo(format,exception);
   if (magick_info && magick_info->raw)
     FormatMagickString( size, MaxTextExtent, "%dx%d", columns, rows );
 
@@ -203,10 +206,17 @@ if (0)
   (void) fflush(stdout);
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
     "Writing image %s", original->filename);
-  WriteImage ( imageInfo, original );
+  status=WriteImage ( imageInfo, original );
+  if ( status == MagickFalse )
+    {
+      (void) printf ( "Failed to write file in format %s\n", imageInfo->magick )
+;
+      (void) fflush(stdout);
+      exit_status = 1;
+      goto program_exit;
+    }
   imageInfo->depth=original->depth;
-   original =DestroyImage( original );
-  original = (Image*)NULL;
+  original=DestroyImageList(original);
 
   /*
    * Read image back from file
@@ -216,10 +226,10 @@ if (0)
   if ( size[0] != '\0' )
     CloneString( &imageInfo->size, size );
   (void) fflush(stdout);
-  original = ReadImage ( imageInfo, &exception );
-  if (exception.severity >= ErrorException)
+  original = ReadImage ( imageInfo, exception );
+  if (exception->severity >= ErrorException)
     {
-      CatchException(&exception);
+      CatchException(exception);
       exit_status = 1;
       goto program_exit;
     }
@@ -240,7 +250,15 @@ if (0)
   (void) strncpy( original->filename, filename, MaxTextExtent-1 );
   original->delay = 10;
   (void) fflush(stdout);
-  WriteImage ( imageInfo, original );
+  status=WriteImage (imageInfo,original);
+  if ( status == MagickFalse )
+    {
+      (void) printf ( "Failed to write file in format %s\n", imageInfo->magick )
+;
+      (void) fflush(stdout);
+      exit_status = 1;
+      goto program_exit;
+    }
 
   /*
    * Read image back from file
@@ -252,10 +270,10 @@ if (0)
   (void) fflush(stdout);
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
     "Reading image %s", imageInfo->filename);
-  final = ReadImage ( imageInfo, &exception );
-  if (exception.severity >= ErrorException)
+  final = ReadImage ( imageInfo, exception );
+  if (exception->severity >= ErrorException)
     {
-      CatchException(&exception);
+      CatchException(exception);
       exit_status = 1;
       goto program_exit;
     }
@@ -310,14 +328,11 @@ if (0)
 
  program_exit:
   if (original)
-     original =DestroyImage( original );
-  original = (Image*)NULL;
+    original=DestroyImageList(original);
   if (final)
-     final =DestroyImage( final );
-  final = (Image*)NULL;
-
-  imageInfo =DestroyImageInfo( imageInfo );
-  DestroyExceptionInfo( &exception );
+    final =DestroyImageList(final);
+  imageInfo=DestroyImageInfo(imageInfo);
+  exception=DestroyExceptionInfo(exception);
   MagickCoreTerminus();
 
   if (pause)
