@@ -10,17 +10,22 @@ PACKAGE_NAME='ImageMagick'
 #
 # Package version.  This is is the numeric version suffix applied to
 # PACKAGE_NAME (e.g. "1.0.0").
-PACKAGE_VERSION='6.3.7'
-PACKAGE_LIB_VERSION="0x637"
-PACKAGE_LIB_VERSION_NUMBER="6,3,7,9"
-PACKAGE_RELEASE_DATE=`date +%x`
+PACKAGE_VERSION='6.4.5'
+PACKAGE_LIB_VERSION="0x645"
+PACKAGE_LIB_VERSION_NUMBER="6,4,5,4"
+PACKAGE_RELEASE="4"
+PACKAGE_RELEASE_DATE=`date +%F`
+
+#
+# Date of last ChangeLog update
+#
+PACKAGE_CHANGE_DATE=`awk '/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]/ { print substr($1,1,4) substr($1,6,2) substr($1,9,2); exit; }' ${srcdir}/ChangeLog`
 
 #
 # Package version addendum.  This is an arbitrary suffix (if any) appended
 # to the package version. (e.g. "beta1") `echo -snapshot-``date '+%g%m%d'`
 CHANGE_DATE=`find ${srcdir}/ChangeLog -printf '%Ty%Tm%Td%\n' 2> /dev/null`
-if test -n "$CHANGE_DATE"
-then
+if test -n "$CHANGE_DATE"; then
   PACKAGE_VERSION_ADDENDUM=""
 else
   PACKAGE_VERSION_ADDENDUM=""
@@ -46,6 +51,6 @@ fi
 #     increment age.
 #  6. If any interfaces have been removed since the last public release,
 #     then set age to 0.
-LIBRARY_CURRENT=10
-LIBRARY_REVISION=9
-LIBRARY_AGE=0
+MAGICK_LIBRARY_CURRENT=1
+MAGICK_LIBRARY_REVISION=0
+MAGICK_LIBRARY_AGE=0
