@@ -66,9 +66,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o argc: The number of elements in the argument vector.
+%    o argc: the number of elements in the argument vector.
 %
 %    o argv: A text array containing the command line arguments.
 %
@@ -95,27 +95,37 @@ static void CompareUsage(void)
     *settings[]=
     {
       "-alpha option        activate, deactivate, reset, or set the alpha channel",
-      "-authenticate value  decrypt image with this password",
+      "-authenticate password",
+      "                     decipher image with this password",
       "-channel type        apply option to select image channels",
       "-colorspace type     alternate image colorspace",
+      "-compose operator    set image composite operator",
       "-compress type       type of pixel compression when writing the image",
+      "-decipher filename   convert cipher pixels to plain pixels",
       "-define format:option",
       "                     define one or more image format options",
       "-density geometry    horizontal and vertical density of the image",
       "-depth value         image depth",
+      "-encipher filename   convert plain pixels to cipher pixels",
       "-extract geometry    extract area from image",
       "-format \"string\"     output formatted image characteristics",
       "-fuzz distance       colors within this distance are considered equal",
+      "-highlight-color color",
+      "                     empasize pixel differences with this color",
       "-identify            identify the format and characteristics of the image",
       "-interlace type      type of image interlacing scheme",
       "-limit type value    pixel cache resource limit",
+      "-lowlight-color color",
+      "                     de-emphasize pixel differences with this color",
       "-metric type         measure differences between images with this metric",
       "-monitor             monitor progress",
+      "-passphrase filename get the passphrase from this file",
       "-profile filename    add, delete, or apply an image profile",
       "-quality value       JPEG/MIFF/PNG compression level",
       "-quiet               suppress all warning messages",
       "-quantize colorspace reduce colors in this colorspace",
       "-regard-warnings     pay attention to warning messages",
+      "-respect-parenthesis settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-seed value          seed a new sequence of pseudo-random numbers",
@@ -158,8 +168,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
 {
 #define DestroyCompare() \
 { \
-  for ( ; k >= 0; k--) \
-    image_stack[k]=DestroyImageList(image_stack[k]); \
+  DestroyImageStack(); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
@@ -188,7 +197,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     *format;
 
   ChannelType
-    channel;
+    channels;
 
   double
     distortion;
@@ -196,9 +205,11 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   Image
     *difference_image,
     *image,
-    *image_stack[MaxImageStackDepth+1],
     *reconstruct_image;
 
+  ImageStack
+    image_stack[MaxImageStackDepth+1];
+
   long
     j,
     k;
@@ -233,19 +244,19 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
           (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-          return(MagickTrue);
+          return(MagickFalse);
         }
     }
   if (argc < 3)
     CompareUsage();
-  channel=AllChannels;
+  channels=AllChannels;
   difference_image=NewImageList();
   distortion=0.0;
   format=(char *) NULL;
   j=1;
   k=0;
-  image_stack[k]=NewImageList();
   metric=UndefinedMetric;
+  NewImageStack();
   option=(char *) NULL;
   pend=MagickFalse;
   reconstruct_image=NewImageList();
@@ -256,59 +267,50 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
+      strerror(errno));
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
     if (LocaleCompare(option,"(") == 0)
       {
+        FireImageStack(MagickTrue,MagickTrue,pend);
         if (k == MaxImageStackDepth)
           ThrowCompareException(OptionError,"ParenthesisNestedTooDeeply",
             option);
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
-        k++;
-        image_stack[k]=NewImageList();
+        PushImageStack();
         continue;
       }
     if (LocaleCompare(option,")") == 0)
       {
+        FireImageStack(MagickTrue,MagickTrue,MagickTrue);
         if (k == 0)
           ThrowCompareException(OptionError,"UnableToParseExpression",option);
-        if (image_stack[k] != (Image *) NULL)
-          {
-            MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-            AppendImageToList(&image_stack[k-1],image_stack[k]);
-          }
-        k--;
+        PopImageStack();
         continue;
       }
     if (IsMagickOption(option) == MagickFalse)
       {
+        Image
+          *images;
+
         /*
           Read input image.
         */
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
+        FireImageStack(MagickTrue,MagickTrue,pend);
         filename=argv[i];
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
-        image=ReadImage(image_info,exception);
-        status&=(image != (Image *) NULL) &&
+        images=ReadImages(image_info,exception);
+        status&=(images != (Image *) NULL) &&
           (exception->severity < ErrorException);
-        if (image == (Image *) NULL)
+        if (images == (Image *) NULL)
           continue;
-        AppendImageToList(&image_stack[k],image);
+        AppendImageStack(images);
         continue;
       }
-    pend=image_stack[k] != (Image *) NULL ? MagickTrue : MagickFalse;
+    pend=image != (Image *) NULL ? MagickTrue : MagickFalse;
     switch (*(option+1))
     {
       case 'a':
@@ -367,6 +369,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             if (channel < 0)
               ThrowCompareException(OptionError,"UnrecognizedChannelType",
                 argv[i]);
+            channels=(ChannelType) channel;
             break;
           }
         if (LocaleCompare("colorspace",option+1) == 0)
@@ -386,6 +389,23 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("compose",option+1) == 0)
+          {
+            long
+              compose;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            compose=ParseMagickOption(MagickComposeOptions,MagickFalse,
+              argv[i]);
+            if (compose < 0)
+              ThrowCompareException(OptionError,"UnrecognizedComposeOperator",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("compress",option+1) == 0)
           {
             long
@@ -423,6 +443,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("decipher",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("define",option+1) == 0)
           {
             i++;
@@ -466,6 +495,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       }
       case 'e':
       {
+        if (LocaleCompare("encipher",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("extract",option+1) == 0)
           {
             if (*option == '+')
@@ -509,6 +547,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
           CompareUsage();
+        if (LocaleCompare("highlight-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'i':
@@ -571,6 +618,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("lowlight-color",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'm':
@@ -600,6 +656,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       }
       case 'p':
       {
+        if (LocaleCompare("passphrase",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("profile",option+1) == 0)
           {
             i++;
@@ -647,6 +712,11 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("regard-warnings",option+1) == 0)
           break;
+        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+          {
+            respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 's':
@@ -664,22 +734,13 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("seed",option+1) == 0)
           {
-            unsigned long
-              seed;
-
             if (*option == '+')
-              {
-                seed=(unsigned long) time((time_t *) NULL);
-                SeedRandomReservoir(seed);
-                break;
-              }
+              break;
             i++;
             if (i == (long) (argc-1))
               ThrowCompareException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompareInvalidArgumentException(option,argv[i]);
-            seed=(unsigned long) atol(argv[i]);
-            SeedRandomReservoir(seed);
             break;
           }
         if (LocaleCompare("set",option+1) == 0)
@@ -772,22 +833,22 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       default:
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
     }
-    fire=(MagickBooleanType) ParseMagickOption(MagickMogrifyOptions,
-      MagickFalse,option+1);
-    if (fire == MagickTrue)
-      MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ? 
+      MagickFalse : MagickTrue;
+    if (fire != MagickFalse)
+      FireImageStack(MagickTrue,MagickTrue,MagickTrue);
   }
   if (k != 0)
     ThrowCompareException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i-- != (long) (argc-1))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
-  if ((image_stack[k] == (Image *) NULL) ||
-      (GetImageListLength(image_stack[k]) < 2))
+  if ((image == (Image *) NULL) ||
+      (GetImageListLength(image) < 2))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
-  MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-  image=GetImageFromList(image_stack[k],0);
-  reconstruct_image=GetImageFromList(image_stack[k],1);
-  difference_image=CompareImageChannels(image,reconstruct_image,channel,
+  FinalizeImageSettings(image_info,image);
+  image=GetImageFromList(image,0);
+  reconstruct_image=GetImageFromList(image,1);
+  difference_image=CompareImageChannels(image,reconstruct_image,channels,
     metric,&distortion,exception);
   if (difference_image == (Image *) NULL)
     status=0;
@@ -803,47 +864,178 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
 
           text=InterpretImageProperties(image_info,difference_image,format);
           if (text == (char *) NULL)
-            {
-              char
-                *message;
-
-              message=GetExceptionMessage(errno);
-              ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-                message);
-              message=DestroyString(message);
-            }
+            ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
+              strerror(errno));
           (void) ConcatenateString(&(*metadata),text);
           (void) ConcatenateString(&(*metadata),"\n");
           text=DestroyString(text);
         }
       difference_image=DestroyImageList(difference_image);
-      switch (metric)
-      {
-        case MeanAbsoluteErrorMetric:
-        case RootMeanSquaredErrorMetric:
-        case PeakAbsoluteErrorMetric:
+      if (image_info->verbose == MagickFalse)
+        switch (metric)
         {
-          (void) fprintf(stderr,"%g (%g)\n",distortion,(double) (QuantumScale*
-            distortion));
-          break;
+          case MeanAbsoluteErrorMetric:
+          case MeanSquaredErrorMetric:
+          case RootMeanSquaredErrorMetric:
+          case PeakAbsoluteErrorMetric:
+          {
+            (void) fprintf(stderr,"%g (%g)\n",distortion,(double) (QuantumScale*
+              distortion));
+            break;
+          }
+          case AbsoluteErrorMetric:
+          case PeakSignalToNoiseRatioMetric:
+          {
+            (void) fprintf(stderr,"%g\n",distortion);
+            break;
+          }
+          case MeanErrorPerPixelMetric:
+          {
+            (void) fprintf(stderr,"%g (%g, %g)\n",distortion,
+              image->error.normalized_mean_error,
+              image->error.normalized_maximum_error);
+            break;
+          }
+          case UndefinedMetric:
+            break;
         }
-        case AbsoluteErrorMetric:
-        case MeanSquaredErrorMetric:
-        case PeakSignalToNoiseRatioMetric:
+      else
         {
-          (void) fprintf(stderr,"%g\n",distortion);
-          break;
+          double
+            *channel_distortion;
+
+ 
+          channel_distortion=GetImageChannelDistortions(image,reconstruct_image,
+            metric,&image->exception);
+          (void) fprintf(stderr,"Image: %s\n",image->filename);
+          (void) fprintf(stderr,"  Channel distortion: %s\n",
+            MagickOptionToMnemonic(MagickMetricOptions,(long) metric));
+          switch (metric)
+          {
+            case MeanAbsoluteErrorMetric:
+            case MeanSquaredErrorMetric:
+            case RootMeanSquaredErrorMetric:
+            case PeakAbsoluteErrorMetric:
+            {
+              switch (image->colorspace)
+              {
+                case RGBColorspace:
+                default:
+                {
+                  (void) fprintf(stderr,"    red: %g (%g)\n",
+                    channel_distortion[RedChannel],(double) QuantumScale*
+                    channel_distortion[RedChannel]);
+                  (void) fprintf(stderr,"    green: %g (%g)\n",
+                    channel_distortion[GreenChannel],(double) QuantumScale*
+                    channel_distortion[GreenChannel]);
+                  (void) fprintf(stderr,"    blue: %g (%g)\n",
+                    channel_distortion[BlueChannel],(double) QuantumScale*
+                    channel_distortion[BlueChannel]);
+                  if (image->matte != MagickFalse)
+                    (void) fprintf(stderr,"    alpha: %g (%g)\n",
+                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      channel_distortion[OpacityChannel]);
+                  break;
+                }
+                case CMYKColorspace:
+                {
+                  (void) fprintf(stderr,"    cyan: %g (%g)\n",
+                    channel_distortion[CyanChannel],(double) QuantumScale*
+                    channel_distortion[CyanChannel]);
+                  (void) fprintf(stderr,"    magenta: %g (%g)\n",
+                    channel_distortion[MagentaChannel],(double) QuantumScale*
+                    channel_distortion[MagentaChannel]);
+                  (void) fprintf(stderr,"    yellow: %g (%g)\n",
+                    channel_distortion[YellowChannel],(double) QuantumScale*
+                    channel_distortion[YellowChannel]);
+                  (void) fprintf(stderr,"    black: %g (%g)\n",
+                    channel_distortion[BlackChannel],(double) QuantumScale*
+                    channel_distortion[BlackChannel]);
+                  if (image->matte != MagickFalse)
+                    (void) fprintf(stderr,"    alpha: %g (%g)\n",
+                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      channel_distortion[OpacityChannel]);
+                  break;
+                }
+                case GRAYColorspace:
+                {
+                  (void) fprintf(stderr,"    gray: %g (%g)\n",
+                    channel_distortion[GrayChannel],(double) QuantumScale*
+                    channel_distortion[GrayChannel]);
+                  if (image->matte != MagickFalse)
+                    (void) fprintf(stderr,"    alpha: %g (%g)\n",
+                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      channel_distortion[OpacityChannel]);
+                  break;
+                }
+              }
+              (void) fprintf(stderr,"    all: %g (%g)\n",
+                channel_distortion[AllChannels],(double) QuantumScale*
+                channel_distortion[AllChannels]);
+              break;
+            }
+            case AbsoluteErrorMetric:
+            case PeakSignalToNoiseRatioMetric:
+            {
+              switch (image->colorspace)
+              {
+                case RGBColorspace:
+                default:
+                {
+                  (void) fprintf(stderr,"    red: %g\n",
+                    channel_distortion[RedChannel]);
+                  (void) fprintf(stderr,"    green: %g\n",
+                    channel_distortion[GreenChannel]);
+                  (void) fprintf(stderr,"    blue: %g\n",
+                    channel_distortion[BlueChannel]);
+                  if (image->matte != MagickFalse)
+                    (void) fprintf(stderr,"    alpha: %g\n",
+                      channel_distortion[OpacityChannel]);
+                  break;
+                }
+                case CMYKColorspace:
+                {
+                  (void) fprintf(stderr,"    cyan: %g\n",
+                    channel_distortion[CyanChannel]);
+                  (void) fprintf(stderr,"    magenta: %g\n",
+                    channel_distortion[MagentaChannel]);
+                  (void) fprintf(stderr,"    yellow: %g\n",
+                    channel_distortion[YellowChannel]);
+                  (void) fprintf(stderr,"    black: %g\n",
+                    channel_distortion[BlackChannel]);
+                  if (image->matte != MagickFalse)
+                    (void) fprintf(stderr,"    alpha: %g\n",
+                      channel_distortion[OpacityChannel]);
+                  break;
+                }
+                case GRAYColorspace:
+                {
+                  (void) fprintf(stderr,"    gray: %g\n",
+                    channel_distortion[GrayChannel]);
+                  if (image->matte != MagickFalse)
+                    (void) fprintf(stderr,"    alpha: %g\n",
+                      channel_distortion[OpacityChannel]);
+                  break;
+                }
+              }
+              (void) fprintf(stderr,"    all: %g\n",
+                channel_distortion[AllChannels]);
+              break;
+            }
+            case MeanErrorPerPixelMetric:
+            {
+              (void) fprintf(stderr,"    %g (%g, %g)\n",
+                channel_distortion[AllChannels],
+                image->error.normalized_mean_error,
+                image->error.normalized_maximum_error);
+              break;
+            }
+            case UndefinedMetric:
+              break;
+          }
+          channel_distortion=(double *) RelinquishMagickMemory(
+            channel_distortion);
         }
-        case MeanErrorPerPixelMetric:
-        {
-          (void) fprintf(stderr,"%g (%g, %g)\n",distortion,
-            image->error.normalized_mean_error,
-            image->error.normalized_maximum_error);
-          break;
-        }
-        case UndefinedMetric:
-          break;
-      }
     }
   DestroyCompare();
   return(status != 0 ? MagickTrue : MagickFalse);
