@@ -3,14 +3,14 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%        CCCC   OOO  M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE          %
-%       C      O   O MM MM  P   P  O   O  SS       I      T    E              %
-%       C      O   O M M M  PPPP   O   O   SSS     I      T    EEE            %
-%       C      O   O M   M  P      O   O     SS    I      T    E              %
-%        CCCC   OOO  M   M  P       OOO   SSSSS  IIIII    T    EEEEE          %
+%        CCCC    OOO   M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE        %
+%       C       O   O  MM MM  P   P  O   O  SS       I      T    E            %
+%       C       O   O  M M M  PPPP   O   O   SSS     I      T    EEE          %
+%       C       O   O  M   M  P      O   O     SS    I      T    E            %
+%        CCCC    OOO   M   M  P       OOO   SSSSS  IIIII    T    EEEEE        %
 %                                                                             %
 %                                                                             %
-%                    ImageMagick Image Composite Methods                      %
+%                      MagickWand Image Composite Methods                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -95,9 +95,9 @@ typedef struct _CompositeOptions
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o argc: The number of elements in the argument vector.
+%    o argc: the number of elements in the argument vector.
 %
 %    o argv: A text array containing the command line arguments.
 %
@@ -238,9 +238,13 @@ static void CompositeUsage(void)
     *operators[]=
     {
       "-blend geometry      blend images",
+      "-border geometry     surround image with a border of color",
+      "-bordercolor color   border color",
       "-colors value        preferred number of colors in the image",
+      "-decipher filename    convert cipher pixels to plain pixels",
       "-displace geometry   shift image pixels defined by a displacement map",
       "-dissolve value      dissolve the two images a given percent",
+      "-encipher filename   convert plain pixels to cipher pixels",
       "-extract geometry    extract area from image",
       "-geometry geometry   location of the composite image",
       "-identify            identify the format and characteristics of the image",
@@ -252,6 +256,7 @@ static void CompositeUsage(void)
       "-rotate degrees      apply Paeth rotation to the image",
       "-resize geometry     resize the image",
       "-sharpen geometry    sharpen the image",
+      "-shave geometry      shave pixels from the image edges",
       "-stegano offset      hide watermark within an image",
       "-stereo              combine two image to create a stereo anaglyph",
       "-strip               strip image of all profiles and comments",
@@ -267,7 +272,8 @@ static void CompositeUsage(void)
     {
       "-affine matrix       affine transform matrix",
       "-alpha option        activate, deactivate, reset, or set the alpha channel",
-      "-authenticate value  decrypt image with this password",
+      "-authenticate password",
+      "                     decipher image with this password",
       "-blue-primary point  chromaticity blue primary point",
       "-channel type        apply option to select image channels",
       "-colorspace type     alternate image colorspace",
@@ -280,7 +286,7 @@ static void CompositeUsage(void)
       "-density geometry    horizontal and vertical density of the image",
       "-display server      get image or font from this X server",
       "-dispose method      layer disposal method",
-      "-dither              apply Floyd/Steinberg error diffusion to image",
+      "-dither method       apply error diffusion to image",
       "-encoding type       text encoding type",
       "-endian type         endianness (MSB or LSB) of the image",
       "-filter type         use this filter when resizing an image",
@@ -294,10 +300,12 @@ static void CompositeUsage(void)
       "-limit type value    pixel cache resource limit",
       "-monitor             monitor progress",
       "-page geometry       size and location of an image canvas (setting)",
+      "-pointsize value     font point size",
       "-quality value       JPEG/MIFF/PNG compression level",
       "-quiet               suppress all warning messages",
       "-red-primary point   chromaticity red primary point",
       "-regard-warnings     pay attention to warning messages",
+      "-respect-parenthesis settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-scene value         image scene number",
@@ -313,8 +321,14 @@ static void CompositeUsage(void)
       "                     virtual pixel access method",
       "-white-point point   chromaticity white point",
       (char *) NULL
+    },
+    *stack_operators[]=
+    {
+      "-swap indexes        swap two images in the image sequence",
+      (char *) NULL
     };
 
+
   (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
   (void) printf("Copyright: %s\n\n",GetMagickCopyright());
   (void) printf("Usage: %s [options ...] image [options ...] composite\n"
@@ -326,6 +340,9 @@ static void CompositeUsage(void)
   (void) printf("\nImage Operators:\n");
   for (p=operators; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
+  (void) printf("\nImage Stack Operators:\n");
+  for (p=stack_operators; *p != (char *) NULL; p++)
+    (void) printf("  %s\n",*p);
   (void) printf("\nMiscellaneous Options:\n");
   for (p=miscellaneous; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
@@ -377,8 +394,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
 #define DestroyComposite() \
 { \
   RelinquishCompositeOptions(&composite_options); \
-  for ( ; k >= 0; k--) \
-    image_stack[k]=DestroyImageList(image_stack[k]); \
+  DestroyImageStack(); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
@@ -386,7 +402,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
 #define ThrowCompositeException(asperity,tag,option) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),asperity,tag,"`%s'", \
-    option); \
+     option == (char *) NULL ? strerror(errno) : option); \
   DestroyComposite(); \
   return(MagickFalse); \
 }
@@ -411,9 +427,12 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
   Image
     *composite_image,
     *image,
-    *image_stack[MaxImageStackDepth+1],
+    *images,
     *mask_image;
 
+  ImageStack
+    image_stack[MaxImageStackDepth+1];
+
   MagickBooleanType
     fire,
     pend;
@@ -445,7 +464,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
           (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-          return(MagickTrue);
+          return(MagickFalse);
         }
     }
   if (argc < 4)
@@ -455,7 +474,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
   format="%w,%h,%m";
   j=1;
   k=0;
-  image_stack[k]=NewImageList();
+  NewImageStack();
   option=(char *) NULL;
   pend=MagickFalse;
   status=MagickTrue;
@@ -468,62 +487,50 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
+      strerror(errno));
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
     if (LocaleCompare(option,"(") == 0)
       {
+        FireImageStack(MagickFalse,MagickTrue,pend);
         if (k == MaxImageStackDepth)
           ThrowCompositeException(OptionError,"ParenthesisNestedTooDeeply",
             option);
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
-        k++;
-        image_stack[k]=NewImageList();
+        PushImageStack();
         continue;
       }
     if (LocaleCompare(option,")") == 0)
       {
+        FireImageStack(MagickFalse,MagickTrue,MagickTrue);
         if (k == 0)
           ThrowCompositeException(OptionError,"UnableToParseExpression",option);
-        if (image_stack[k] != (Image *) NULL)
-          {
-            MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-            AppendImageToList(&image_stack[k-1],image_stack[k]);
-          }
-        k--;
+        PopImageStack();
         continue;
       }
     if (IsMagickOption(option) == MagickFalse)
       {
         Image
-          *image;
+          *images;
 
         /*
           Read input image.
         */
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
+        FireImageStack(MagickFalse,MagickTrue,pend);
         filename=argv[i];
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
-        image=ReadImage(image_info,exception);
-        status&=(image != (Image *) NULL) &&
+        images=ReadImages(image_info,exception);
+        status&=(images != (Image *) NULL) &&
           (exception->severity < ErrorException);
-        if (image == (Image *) NULL)
+        if (images == (Image *) NULL)
           continue;
-        AppendImageToList(&image_stack[k],image);
+        AppendImageStack(images);
         continue;
       }
-    pend=image_stack[k] != (Image *) NULL ? MagickTrue : MagickFalse;
+    pend=image != (Image *) NULL ? MagickTrue : MagickFalse;
     switch (*(option+1))
     {
       case 'a':
@@ -602,6 +609,26 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("border",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            break;
+          }
+        if (LocaleCompare("bordercolor",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            break;
+          }
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 'c':
@@ -731,6 +758,15 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             (void) SetLogEventMask(argv[i]);
             break;
           }
+        if (LocaleCompare("decipher",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("define",option+1) == 0)
           {
             i++;
@@ -825,12 +861,35 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             composite_options.compose=DissolveCompositeOp;
             break;
           }
-        if (LocaleCompare("dither",option+1) == 0)
-          break;
+        if (LocaleCompare("method",option+1) == 0)
+          {
+            long
+              method;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            method=ParseMagickOption(MagickDitherOptions,MagickFalse,argv[i]);
+            if (method < 0)
+              ThrowCompositeException(OptionError,"UnrecognizedDitherMethod",
+                argv[i]);
+            break;
+          }
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 'e':
       {
+        if (LocaleCompare("encipher",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("encoding",option+1) == 0)
           {
             if (*option == '+')
@@ -1080,6 +1139,17 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("pointsize",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("process",option+1) == 0)
           {
             if (*option == '+')
@@ -1160,6 +1230,11 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+          {
+            respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
+            break;
+          }
         if (LocaleCompare("resize",option+1) == 0)
           {
             if (*option == '+')
@@ -1208,22 +1283,13 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("seed",option+1) == 0)
           {
-            unsigned long
-              seed;
-
             if (*option == '+')
-              {
-                seed=(unsigned long) time((time_t *) NULL);
-                SeedRandomReservoir(seed);
-                break;
-              }
+              break;
             i++;
             if (i == (long) (argc-1))
               ThrowCompositeException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowCompositeInvalidArgumentException(option,argv[i]);
-            seed=(unsigned long) atol(argv[i]);
-            SeedRandomReservoir(seed);
             break;
           }
         if (LocaleCompare("sharpen",option+1) == 0)
@@ -1235,6 +1301,17 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("shave",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("size",option+1) == 0)
           {
             if (*option == '+')
@@ -1272,6 +1349,17 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             i++;  /* deprecated */
             break;
           }
+        if (LocaleCompare("swap",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 't':
@@ -1442,60 +1530,68 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
       default:
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
     }
-    fire=(MagickBooleanType) ParseMagickOption(MagickMogrifyOptions,
-      MagickFalse,option+1);
-    if (fire == MagickTrue)
-      MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ?
+      MagickFalse : MagickTrue;
+    if (fire != MagickFalse)
+      FireImageStack(MagickFalse,MagickTrue,MagickTrue);
   }
   if (k != 0)
     ThrowCompositeException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i-- != (long) (argc-1))
     ThrowCompositeException(OptionError,"MissingAnImageFilename",argv[i]);
-  if ((image_stack[k] == (Image *) NULL) ||
-      (GetImageListLength(image_stack[k]) < 2))
+  if ((image == (Image *) NULL) ||
+      (GetImageListLength(image) < 2))
     ThrowCompositeException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
+  FinalizeImageSettings(image_info,image);
   /*
     Composite images.
   */
-  composite_image=RemoveFirstImageFromList(image_stack+k);
-  image=RemoveFirstImageFromList(image_stack+k);
+  RemoveImageStack(composite_image);
+  RemoveImageStack(images);
   (void) TransformImage(&composite_image,(char *) NULL,
     composite_image->geometry);
-  mask_image=RemoveFirstImageFromList(image_stack+k);
+  RemoveImageStack(mask_image);
   if (mask_image != (Image *) NULL)
     {
-      image->mask=mask_image;
-      (void) NegateImage(image->mask,MagickFalse);
+      if (composite_options.compose == DisplaceCompositeOp)
+        {
+          /*
+            Merge Y displacement into X displacement image.
+          */
+          (void) CompositeImage(composite_image,CopyGreenCompositeOp,
+            mask_image,0,0);
+          mask_image=DestroyImage(mask_image);
+        }
+      else
+        {
+          /*
+            Set a blending mask for the composition.
+          */
+          images->mask=mask_image;
+          (void) NegateImage(images->mask,MagickFalse);
+        }
     }
-  status&=CompositeImageList(image_info,&image,composite_image,
+  status&=CompositeImageList(image_info,&images,composite_image,
     &composite_options,exception);
   composite_image=DestroyImage(composite_image);
   /*
     Write composite images.
   */
-  status&=WriteImages(image_info,image,argv[argc-1],exception);
+  status&=WriteImages(image_info,images,argv[argc-1],exception);
   if (metadata != (char **) NULL)
     {
       char
         *text;
 
-      text=InterpretImageProperties(image_info,image,format);
+      text=InterpretImageProperties(image_info,images,format);
       if (text == (char *) NULL)
-        {
-          char
-            *message;
-
-          message=GetExceptionMessage(errno);
-          ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
-            message);
-          message=DestroyString(message);
-        }
+        ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
+          strerror(errno));
       (void) ConcatenateString(&(*metadata),text);
       (void) ConcatenateString(&(*metadata),"\n");
       text=DestroyString(text);
     }
-  image=DestroyImage(image);
+  images=DestroyImage(images);
   RelinquishCompositeOptions(&composite_options);
   DestroyComposite();
   return(status != 0 ? MagickTrue : MagickFalse);
