@@ -83,9 +83,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o argc: The number of elements in the argument vector.
+%    o argc: the number of elements in the argument vector.
 %
 %    o argv: A text array containing the command line arguments.
 %
@@ -111,7 +111,7 @@ static MagickBooleanType ConcatenateImages(int argc,char **argv,
   /*
     Open output file.
   */
-  output=MagickOpenStream(argv[argc-1],"wb");
+  output=OpenMagickStream(argv[argc-1],"wb");
   if (output == (FILE *) NULL)
     {
       ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
@@ -120,7 +120,7 @@ static MagickBooleanType ConcatenateImages(int argc,char **argv,
     }
   for (i=2; i < (long) (argc-1); i++)
   {
-    input=MagickOpenStream(argv[i],"rb");
+    input=OpenMagickStream(argv[i],"rb");
     if (input == (FILE *) NULL)
       ThrowFileException(exception,FileOpenError,"UnableToOpenFile",argv[i]);
     for (c=fgetc(input); c != EOF; c=fgetc(input))
@@ -149,8 +149,7 @@ static void ConvertUsage(void)
       "-adaptive-blur geometry",
       "                     adaptively blur pixels; decrease effect near edges",
       "-adaptive-resize geometry",
-      "                     adaptively resize image with data dependent",
-      "                     triangulation",
+      "                     adaptively resize image using 'mesh' interpolation",
       "-adaptive-sharpen geometry",
       "                     adaptively sharpen pixels; increase effect near edges",
       "-alpha option        activate, deactivate, reset, or set the alpha channel",
@@ -162,6 +161,7 @@ static void ConvertUsage(void)
       "                     force all pixels below the threshold into black",
       "-blur geometry       reduce image noise and reduce detail levels",
       "-border geometry     surround image with a border of color",
+      "-bordercolor color   border color",
       "-charcoal radius     simulate a charcoal drawing",
       "-chop geometry       remove pixels from the image interior",
       "-clip                clip along the first path from the 8BIM profile",
@@ -174,11 +174,14 @@ static void ConvertUsage(void)
       "-convolve coefficients",
       "                     apply a convolution kernel to the image",
       "-cycle amount        cycle the image colormap",
+      "-decipher filename   convert cipher pixels to plain pixels",
+      "-deskew threshold    straighten an image",
       "-despeckle           reduce the speckles within an image",
-      "-distort type coefficients",
-      "                     distort the image",
+      "-distort method args",
+      "                     distort images according to given method ad args",
       "-draw string         annotate the image with a graphic primitive",
       "-edge radius         apply a filter to detect edges in the image",
+      "-encipher filename   convert plain pixels to cipher pixels",
       "-emboss radius       emboss an image",
       "-enhance             apply a digital filter to enhance a noisy image",
       "-equalize            perform histogram equalization to an image",
@@ -201,8 +204,12 @@ static void ConvertUsage(void)
       "-lat geometry        local adaptive thresholding",
       "-layers method       optimize, merge,  or compare image layers",
       "-level value         adjust the level of image contrast",
+      "-level-colors color,color",
+      "                     level image using given colors",
       "-linear-stretch geometry",
       "                     improve contrast by `stretching with saturation'",
+      "-liquid-rescale geometry",
+      "                     rescale image with seam-carving",
       "-median radius       apply a median filter to the image",
       "-modulate value      vary the brightness, saturation, and hue",
       "-monochrome          transform image to black and white",
@@ -245,10 +252,11 @@ static void ConvertUsage(void)
       "-shave geometry      shave pixels from the image edges",
       "-shear geometry      slide one edge of the image along the X or Y axis",
       "-sigmoidal-contrast geometry",
-      "                     lightness rescaling using sigmoidal contrast",
-      "                     enhancement",
+      "                     increase the contrast without saturating highlights or shadows",
       "-sketch geometry     simulate a pencil sketch",
       "-solarize threshold  negate all pixels above the threshold level",
+      "-sparse-color method args",
+      "                     fill in a image based on a few color points",
       "-splice geometry     splice the background color into the image",
       "-spread radius       displace image pixels by a random amount",
       "-strip               strip image of all profiles and comments",
@@ -283,7 +291,6 @@ static void ConvertUsage(void)
       "-deconstruct         break down an image sequence into constituent parts",
       "-flatten             flatten a sequence of images",
       "-fx expression       apply mathematical expression to an image channel(s)",
-      "-map filename        transform image colors to match this set of colors",
       "-morph value         morph an image sequence",
       "-mosaic              create a mosaic from an image sequence",
       "-process arguments   process the image with a custom image filter",
@@ -298,7 +305,8 @@ static void ConvertUsage(void)
       "-affine matrix       affine transform matrix",
       "-alpha option        activate, deactivate, reset, or set the alpha channel",
       "-antialias           remove pixel-aliasing",
-      "-authenticate value  decrypt image with this password",
+      "-authenticate password",
+      "                     decipher image with this password",
       "-attenuate value     lessen (or intensify) when adding noise to an image",
       "-background color    background color",
       "-bias value          add bias when convolving an image",
@@ -320,7 +328,7 @@ static void ConvertUsage(void)
       "-depth value         image depth",
       "-display server      get image or font from this X server",
       "-dispose method      layer disposal method",
-      "-dither              apply Floyd/Steinberg error diffusion to image",
+      "-dither method       apply error diffusion to image",
       "-encoding type       text encoding type",
       "-endian type         endianness (MSB or LSB) of the image",
       "-family name         render text with this font family",
@@ -349,6 +357,8 @@ static void ConvertUsage(void)
       "-quiet               suppress all warning messages",
       "-red-primary point   chromaticity red primary point",
       "-regard-warnings     pay attention to warning messages",
+      "-remap filename      transform image colors to match this set of colors",
+      "-respect-parenthesis settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-scene value         image scene number",
@@ -424,8 +434,7 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
 #define NotInitialized  (unsigned int) (~0)
 #define DestroyConvert() \
 { \
-  for ( ; k >= 0; k--) \
-    image_stack[k]=DestroyImageList(image_stack[k]); \
+  DestroyImageStack(); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
@@ -453,7 +462,10 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
     *format;
 
   Image
-    *image_stack[MaxImageStackDepth+1];
+    *image;
+
+  ImageStack
+    image_stack[MaxImageStackDepth+1];
 
   long
     j,
@@ -495,7 +507,7 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
   format="%w,%h,%m";
   j=1;
   k=0;
-  image_stack[k]=NewImageList();
+  NewImageStack();
   option=(char *) NULL;
   pend=MagickFalse;
   status=MagickTrue;
@@ -505,15 +517,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowConvertException(ResourceLimitError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowConvertException(ResourceLimitError,"MemoryAllocationFailed",
+      strerror(errno));
   if ((argc > 2) && (LocaleCompare("-concatenate",argv[1]) == 0))
     return(ConcatenateImages(argc,argv,exception));
   for (i=1; i < (long) (argc-1); i++)
@@ -521,51 +526,46 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
     option=argv[i];
     if (LocaleCompare(option,"(") == 0)
       {
+        FireImageStack(MagickTrue,MagickTrue,pend);
         if (k == MaxImageStackDepth)
           ThrowConvertException(OptionError,"ParenthesisNestedTooDeeply",
             option);
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
-        k++;
-        image_stack[k]=NewImageList();
+        PushImageStack();
         continue;
       }
     if (LocaleCompare(option,")") == 0)
       {
+        FireImageStack(MagickTrue,MagickTrue,MagickTrue);
         if (k == 0)
           ThrowConvertException(OptionError,"UnableToParseExpression",option);
-        if (image_stack[k] != (Image *) NULL)
-          {
-            MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-            AppendImageToList(&image_stack[k-1],image_stack[k]);
-          }
-        k--;
+        PopImageStack();
         continue;
       }
     if (IsMagickOption(option) == MagickFalse)
       {
         Image
-          *image;
+          *images;
 
         /*
           Read input image.
         */
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
+        FireImageStack(MagickTrue,MagickTrue,pend);
         filename=argv[i];
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
         if (image_info->ping != MagickFalse)
-          image=PingImage(image_info,exception);
+          images=PingImages(image_info,exception);
         else
-          image=ReadImage(image_info,exception);
-        status&=(image != (Image *) NULL) &&
+          images=ReadImages(image_info,exception);
+        status&=(images != (Image *) NULL) &&
           (exception->severity < ErrorException);
-        if (image == (Image *) NULL)
+        if (images == (Image *) NULL)
           continue;
-        AppendImageToList(&image_stack[k],image);
+        AppendImageStack(images);
         continue;
       }
-    pend=image_stack[k] != (Image *) NULL ? MagickTrue : MagickFalse;
+    pend=image != (Image *) NULL ? MagickTrue : MagickFalse;
     switch (*(option+1))
     {
       case 'a':
@@ -841,9 +841,9 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
             Image
               *clone_images;
 
-            clone_images=image_stack[k];
+            clone_images=image;
             if (k != 0)
-              clone_images=image_stack[k-1];
+              clone_images=image_stack[k-1].image;
             if (clone_images == (Image *) NULL)
               ThrowConvertException(ImageError,"ImageSequenceRequired",option);
             if (*option == '+')
@@ -859,8 +859,8 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               }
             if (clone_images == (Image *) NULL)
               ThrowConvertException(OptionError,"NoSuchImage",option);
-            MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-            AppendImageToList(&image_stack[k],clone_images);
+            FireImageStack(MagickTrue,MagickTrue,MagickTrue);
+            AppendImageStack(clone_images);
             break;
           }
         if (LocaleCompare("clut",option+1) == 0)
@@ -1000,6 +1000,15 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
       }
       case 'd':
       {
+        if (LocaleCompare("decipher",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("deconstruct",option+1) == 0)
           break;
         if (LocaleCompare("debug",option+1) == 0)
@@ -1080,6 +1089,17 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("deskew",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConvertInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("despeckle",option+1) == 0)
           break;
         if (LocaleCompare("display",option+1) == 0)
@@ -1120,7 +1140,7 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertException(OptionError,"MissingArgument",option);
             op=ParseMagickOption(MagickDistortOptions,MagickFalse,argv[i]);
             if (op < 0)
-              ThrowConvertException(OptionError,"UnrecognizedDistortType",
+              ThrowConvertException(OptionError,"UnrecognizedDistortMethod",
                 argv[i]);
             i++;
             if (i == (long) (argc-1))
@@ -1131,7 +1151,21 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
             break;
           }
         if (LocaleCompare("dither",option+1) == 0)
-          break;
+          {
+            long
+              method;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            method=ParseMagickOption(MagickDitherOptions,MagickFalse,argv[i]);
+            if (method < 0)
+              ThrowConvertException(OptionError,"UnrecognizedDitherMethod",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("draw",option+1) == 0)
           {
             if (*option == '+')
@@ -1167,6 +1201,15 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("encipher",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("encoding",option+1) == 0)
           {
             if (*option == '+')
@@ -1293,9 +1336,9 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowConvertInvalidArgumentException(option,argv[i]);
+            i++;
             if (i == (long) (argc-1))
               ThrowConvertException(OptionError,"MissingArgument",option);
-            i++;
             break;
           }
         if (LocaleCompare("font",option+1) == 0)
@@ -1545,6 +1588,13 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("level-colors",option+1) == 0)
+          {
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("linewidth",option+1) == 0)
           {
             if (*option == '+')
@@ -1556,6 +1606,15 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("liquid-rescale",option+1) == 0)
+          {
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowConvertInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("limit",option+1) == 0)
           {
             char
@@ -1749,8 +1808,6 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("opaque",option+1) == 0)
           {
-            if (*option == '+')
-              break;
             i++;
             if (i == (long) (argc-1))
               ThrowConvertException(OptionError,"MissingArgument",option);
@@ -1985,6 +2042,15 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("remap",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("render",option+1) == 0)
           break;
         if (LocaleCompare("repage",option+1) == 0)
@@ -2020,6 +2086,11 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+          {
+            respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
+            break;
+          }
         if (LocaleCompare("reverse",option+1) == 0)
           break;
         if (LocaleCompare("roll",option+1) == 0)
@@ -2092,22 +2163,13 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("seed",option+1) == 0)
           {
-            unsigned long
-              seed;
-
             if (*option == '+')
-              {
-                seed=(unsigned long) time((time_t *) NULL);
-                SeedRandomReservoir(seed);
-                break;
-              }
+              break;
             i++;
             if (i == (long) (argc-1))
               ThrowConvertException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowConvertInvalidArgumentException(option,argv[i]);
-            seed=(unsigned long) atol(argv[i]);
-            SeedRandomReservoir(seed);
             break;
           }
         if (LocaleCompare("segment",option+1) == 0)
@@ -2237,6 +2299,28 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
               ThrowConvertInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("sparse-color",option+1) == 0)
+          {
+            long
+              op;
+
+            i++;
+            if (i == (long) argc)
+              ThrowConvertException(OptionError,"MissingArgument",option);
+            op=ParseMagickOption(MagickSparseColorOptions,MagickFalse,argv[i]);
+            if (op < 0)
+              ThrowConvertException(OptionError,"UnrecognizedSparseColorMethod",
+                argv[i]);
+            i++;
+            if (i == (long) (argc-1))
+              ThrowConvertException(OptionError,"MissingArgument",option);
+/*  This does not work if first number is a '%' escape!
+            (void) strtod(argv[i],&p);
+            if (p == argv[i])
+              ThrowConvertInvalidArgumentException(option,argv[i]);
+*/
+            break;
+          }
         if (LocaleCompare("splice",option+1) == 0)
           {
             if (*option == '+')
@@ -2613,37 +2697,28 @@ WandExport MagickBooleanType ConvertImageCommand(ImageInfo *image_info,
       default:
         ThrowConvertException(OptionError,"UnrecognizedOption",option)
     }
-    fire=(MagickBooleanType) ParseMagickOption(MagickMogrifyOptions,
-      MagickFalse,option+1);
-    if (fire == MagickTrue)
-      MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ?
+      MagickFalse : MagickTrue;
+    if (fire != MagickFalse)
+      FireImageStack(MagickTrue,MagickTrue,MagickTrue);
   }
   if (k != 0)
     ThrowConvertException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i-- != (long) (argc-1))
     ThrowConvertException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  if (image_stack[k] == (Image *) NULL)
+  if (image == (Image *) NULL)
     ThrowConvertException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-  if (image_stack[k] == (Image *) NULL)
-    ThrowConvertException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  status&=WriteImages(image_info,image_stack[k],argv[argc-1],exception);
+  FinalizeImageSettings(image_info,image);
+  status&=WriteImages(image_info,image,argv[argc-1],exception);
   if (metadata != (char **) NULL)
     {
       char
         *text;
 
-      text=InterpretImageProperties(image_info,image_stack[k],format);
+      text=InterpretImageProperties(image_info,image,format);
       if (text == (char *) NULL)
-        {
-          char
-            *message;
-
-          message=GetExceptionMessage(errno);
-          ThrowConvertException(ResourceLimitError,"MemoryAllocationFailed",
-            message);
-          message=DestroyString(message);
-        }
+        ThrowConvertException(ResourceLimitError,"MemoryAllocationFailed",
+          strerror(errno));
       (void) ConcatenateString(&(*metadata),text);
       text=DestroyString(text);
     }
