@@ -80,7 +80,7 @@
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 %    o pathname: name of clipping path resource. If name is preceded by #, use
 %      clipping path numbered by name.
@@ -114,7 +114,7 @@ WandExport MagickBooleanType MagickClipPathImage(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The drawing wand.
+%    o wand: the drawing wand.
 %
 */
 WandExport double DrawGetFillAlpha(const DrawingWand *wand)
@@ -141,7 +141,7 @@ WandExport double DrawGetFillAlpha(const DrawingWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The drawing wand.
+%    o wand: the drawing wand.
 */
 WandExport double DrawGetStrokeAlpha(const DrawingWand *wand)
 {
@@ -167,7 +167,7 @@ WandExport double DrawGetStrokeAlpha(const DrawingWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The drawing wand.
+%    o wand: the drawing wand.
 %
 */
 WandExport DrawInfo *DrawPeekGraphicWand(const DrawingWand *wand)
@@ -197,7 +197,7 @@ WandExport DrawInfo *DrawPeekGraphicWand(const DrawingWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The drawing wand.
+%    o wand: the drawing wand.
 %
 */
 WandExport void DrawPopGraphicContext(DrawingWand *wand)
@@ -217,7 +217,7 @@ WandExport void DrawPopGraphicContext(DrawingWand *wand)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  DrawPushGraphicContext() clones the current drawing wand to create a new
-%  drawing wand.  The original drawing drawing wand(s) may be returned to by
+%  drawing wand.  The original drawing wand(s) may be returned to by
 %  invoking PopDrawingWand().  The drawing wands are stored on a drawing wand
 %  stack.  For every Pop there must have already been an equivalent Push.
 %
@@ -227,7 +227,7 @@ WandExport void DrawPopGraphicContext(DrawingWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The drawing wand.
+%    o wand: the drawing wand.
 %
 */
 WandExport void DrawPushGraphicContext(DrawingWand *wand)
@@ -255,7 +255,7 @@ WandExport void DrawPushGraphicContext(DrawingWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The drawing wand.
+%    o wand: the drawing wand.
 %
 %    o fill_alpha: fill alpha
 %
@@ -284,7 +284,7 @@ WandExport void DrawSetFillAlpha(DrawingWand *wand,const double fill_alpha)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The drawing wand.
+%    o wand: the drawing wand.
 %
 %    o stroke_alpha: stroke alpha.  The value 1.0 is opaque.
 %
@@ -318,9 +318,9 @@ WandExport void DrawSetStrokeAlpha(DrawingWand *wand,const double stroke_alpha)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o fill: The floodfill color pixel wand.
+%    o fill: the floodfill color pixel wand.
 %
 %    o fuzz: By default target must match a particular pixel color
 %      exactly.  However, in many cases two colors may differ by a small amount.
@@ -329,9 +329,9 @@ WandExport void DrawSetStrokeAlpha(DrawingWand *wand,const double stroke_alpha)
 %      color red at intensities of 100 and 102 respectively are now interpreted
 %      as the same color for the purposes of the floodfill.
 %
-%    o bordercolor: The border color pixel wand.
+%    o bordercolor: the border color pixel wand.
 %
-%    o x,y: The starting location of the operation.
+%    o x,y: the starting location of the operation.
 %
 */
 WandExport MagickBooleanType MagickColorFloodfillImage(MagickWand *wand,
@@ -355,8 +355,8 @@ WandExport MagickBooleanType MagickColorFloodfillImage(MagickWand *wand,
     ThrowWandException(WandError,"ContainsNoImages",wand->name);
   draw_info=CloneDrawInfo(wand->image_info,(DrawInfo *) NULL);
   PixelGetQuantumColor(fill,&draw_info->fill);
-  target=AcquireOnePixel(wand->images,x % wand->images->columns,
-    y % wand->images->rows,wand->exception);
+  (void) AcquireOnePixel(wand->images,x % wand->images->columns,
+    y % wand->images->rows,&target,wand->exception);
   if (bordercolor != (PixelWand *) NULL)
     PixelGetQuantumColor(bordercolor,&target);
   wand->images->fuzz=fuzz;
@@ -388,7 +388,7 @@ WandExport MagickBooleanType MagickColorFloodfillImage(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport char *MagickDescribeImage(MagickWand *wand)
@@ -407,7 +407,7 @@ WandExport char *MagickDescribeImage(MagickWand *wand)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickFlattenImages() merges a sequence of images.  This is useful for
+%  MagickFlattenImages() merges a sequence of images.  This useful for
 %  combining Photoshop layers into a single image.
 %
 %  The format of the MagickFlattenImages method is:
@@ -416,7 +416,7 @@ WandExport char *MagickDescribeImage(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 
@@ -488,9 +488,9 @@ WandExport MagickWand *MagickFlattenImages(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o property: The property.
+%    o property: the property.
 %
 */
 WandExport char *MagickGetImageAttribute(MagickWand *wand,const char *property)
@@ -517,7 +517,7 @@ WandExport char *MagickGetImageAttribute(MagickWand *wand,const char *property)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport long MagickGetImageIndex(MagickWand *wand)
@@ -546,9 +546,9 @@ WandExport long MagickGetImageIndex(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o channel: The image channel(s).
+%    o channel: the image channel(s).
 %
 %    o minima:  The minimum pixel value for the specified channel(s).
 %
@@ -592,7 +592,7 @@ WandExport MagickBooleanType MagickGetImageChannelExtrema(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 %    o minima:  The minimum pixel value for the specified channel(s).
 %
@@ -620,6 +620,87 @@ WandExport MagickBooleanType MagickGetImageExtrema(MagickWand *wand,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k G e t I m a g e M a t t e                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetImageMatte() returns MagickTrue if the image has a matte channel
+%  otherwise MagickFalse.
+%
+%  The format of the MagickGetImageMatte method is:
+%
+%      unsigned long MagickGetImageMatte(MagickWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: the magick wand.
+%
+*/
+WandExport MagickBooleanType MagickGetImageMatte(MagickWand *wand)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  if (wand->images == (Image *) NULL)
+    ThrowWandException(WandError,"ContainsNoImages",wand->name);
+  return(wand->images->matte);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k M a p I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickMapImage() replaces the colors of an image with the closest color
+%  from a reference image.
+%
+%  The format of the MagickMapImage method is:
+%
+%      MagickBooleanType MagickMapImage(MagickWand *wand,
+%        const MagickWand *map_wand,const MagickBooleanType dither)
+%
+%  A description of each parameter follows:
+%
+%    o wand: the magick wand.
+%
+%    o map: the map wand.
+%
+%    o dither: Set this integer value to something other than zero to dither
+%      the mapped image.
+%
+*/
+WandExport MagickBooleanType MagickMapImage(MagickWand *wand,
+  const MagickWand *map_wand,const MagickBooleanType dither)
+{
+  MagickBooleanType
+    status;
+
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  if ((wand->images == (Image *) NULL) || (map_wand->images == (Image *) NULL))
+    ThrowWandException(WandError,"ContainsNoImages",wand->name);
+  status=MapImage(wand->images,map_wand->images,dither);
+  if (status == MagickFalse)
+    InheritException(wand->exception,&wand->images->exception);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k M a t t e F l o o d f i l l I m a g e                         %
 %                                                                             %
 %                                                                             %
@@ -639,9 +720,9 @@ WandExport MagickBooleanType MagickGetImageExtrema(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o alpha: The level of transparency: 1.0 is fully opaque and 0.0 is fully
+%    o alpha: the level of transparency: 1.0 is fully opaque and 0.0 is fully
 %      transparent.
 %
 %    o fuzz: By default target must match a particular pixel color
@@ -651,9 +732,9 @@ WandExport MagickBooleanType MagickGetImageExtrema(MagickWand *wand,
 %      color red at intensities of 100 and 102 respectively are now interpreted
 %      as the same color for the purposes of the floodfill.
 %
-%    o bordercolor: The border color pixel wand.
+%    o bordercolor: the border color pixel wand.
 %
-%    o x,y: The starting location of the operation.
+%    o x,y: the starting location of the operation.
 %
 */
 WandExport MagickBooleanType MagickMatteFloodfillImage(MagickWand *wand,
@@ -676,8 +757,8 @@ WandExport MagickBooleanType MagickMatteFloodfillImage(MagickWand *wand,
   if (wand->images == (Image *) NULL)
     ThrowWandException(WandError,"ContainsNoImages",wand->name);
   draw_info=CloneDrawInfo(wand->image_info,(DrawInfo *) NULL);
-  target=AcquireOnePixel(wand->images,x % wand->images->columns,
-    y % wand->images->rows,wand->exception);
+  (void) AcquireOnePixel(wand->images,x % wand->images->columns,
+    y % wand->images->rows,&target,wand->exception);
   if (bordercolor != (PixelWand *) NULL)
     PixelGetQuantumColor(bordercolor,&target);
   wand->images->fuzz=fuzz;
@@ -711,7 +792,7 @@ WandExport MagickBooleanType MagickMatteFloodfillImage(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport MagickWand *MagickMosaicImages(MagickWand *wand)
@@ -752,13 +833,13 @@ WandExport MagickWand *MagickMosaicImages(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o channel: The channel(s).
+%    o channel: the channel(s).
 %
 %    o target: Change this target color to the fill color within the image.
 %
-%    o fill: The fill pixel wand.
+%    o fill: the fill pixel wand.
 %
 %    o fuzz: By default target must match a particular pixel color
 %      exactly.  However, in many cases two colors may differ by a small amount.
@@ -779,6 +860,162 @@ WandExport MagickBooleanType MagickOpaqueImage(MagickWand *wand,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k P a i n t F l o o d f i l l I m a g e                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickPaintFloodfillImage() changes the color value of any pixel that matches
+%  target and is an immediate neighbor.  If the method FillToBorderMethod is
+%  specified, the color value is changed for any neighbor pixel that does not
+%  match the bordercolor member of image.
+%
+%  The format of the MagickPaintFloodfillImage method is:
+%
+%      MagickBooleanType MagickPaintFloodfillImage(MagickWand *wand,
+%        const ChannelType channel,const PixelWand *fill,const double fuzz,
+%        const PixelWand *bordercolor,const long x,const long y)
+%
+%  A description of each parameter follows:
+%
+%    o wand: the magick wand.
+% 
+%    o channel: the channel(s).
+%
+%    o fill: the floodfill color pixel wand.
+%
+%    o fuzz: By default target must match a particular pixel color
+%      exactly.  However, in many cases two colors may differ by a small amount.
+%      The fuzz member of image defines how much tolerance is acceptable to
+%      consider two colors as the same.  For example, set fuzz to 10 and the
+%      color red at intensities of 100 and 102 respectively are now interpreted
+%      as the same color for the purposes of the floodfill.
+%
+%    o bordercolor: the border color pixel wand.
+%
+%    o x,y: the starting location of the operation.
+%
+*/
+WandExport MagickBooleanType MagickPaintFloodfillImage(MagickWand *wand,
+  const ChannelType channel,const PixelWand *fill,const double fuzz,
+  const PixelWand *bordercolor,const long x,const long y)
+{
+  MagickBooleanType
+    status;
+
+  status=MagickFloodfillPaintImage(wand,channel,fill,fuzz,bordercolor,x,y,
+    MagickFalse);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k P a i n t O p a q u e I m a g e                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickPaintOpaqueImage() changes any pixel that matches color with the color
+%  defined by fill.
+%
+%  The format of the MagickPaintOpaqueImage method is:
+%
+%      MagickBooleanType MagickPaintOpaqueImage(MagickWand *wand,
+%        const PixelWand *target,const PixelWand *fill,const double fuzz)
+%      MagickBooleanType MagickPaintOpaqueImageChannel(MagickWand *wand,
+%        const ChannelType channel,const PixelWand *target,
+%        const PixelWand *fill,const double fuzz)
+%
+%  A description of each parameter follows:
+%
+%    o wand: the magick wand.
+%
+%    o channel: the channel(s).
+%
+%    o target: Change this target color to the fill color within the image.
+%
+%    o fill: the fill pixel wand.
+%
+%    o fuzz: By default target must match a particular pixel color
+%      exactly.  However, in many cases two colors may differ by a small amount.
+%      The fuzz member of image defines how much tolerance is acceptable to
+%      consider two colors as the same.  For example, set fuzz to 10 and the
+%      color red at intensities of 100 and 102 respectively are now interpreted
+%      as the same color for the purposes of the floodfill.
+%
+*/
+
+WandExport MagickBooleanType MagickPaintOpaqueImage(MagickWand *wand,
+  const PixelWand *target,const PixelWand *fill,const double fuzz)
+{
+  return(MagickPaintOpaqueImageChannel(wand,DefaultChannels,target,fill,fuzz));
+}
+
+WandExport MagickBooleanType MagickPaintOpaqueImageChannel(MagickWand *wand,
+  const ChannelType channel,const PixelWand *target,const PixelWand *fill,
+  const double fuzz)
+{
+  MagickBooleanType
+    status;
+
+  status=MagickOpaquePaintImageChannel(wand,channel,target,fill,fuzz,
+    MagickFalse);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k P a i n t T r a n s p a r e n t I m a g e                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickPaintTransparentImage() changes any pixel that matches color with the
+%  color defined by fill.
+%
+%  The format of the MagickPaintTransparentImage method is:
+%
+%      MagickBooleanType MagickPaintTransparentImage(MagickWand *wand,
+%        const PixelWand *target,const double alpha,const double fuzz)
+%
+%  A description of each parameter follows:
+%
+%    o wand: the magick wand.
+%
+%    o target: Change this target color to specified opacity value within
+%      the image.
+%
+%    o alpha: the level of transparency: 1.0 is fully opaque and 0.0 is fully
+%      transparent.
+%
+%    o fuzz: By default target must match a particular pixel color
+%      exactly.  However, in many cases two colors may differ by a small amount.
+%      The fuzz member of image defines how much tolerance is acceptable to
+%      consider two colors as the same.  For example, set fuzz to 10 and the
+%      color red at intensities of 100 and 102 respectively are now interpreted
+%      as the same color for the purposes of the floodfill.
+%
+*/
+WandExport MagickBooleanType MagickPaintTransparentImage(MagickWand *wand,
+  const PixelWand *target,const double alpha,const double fuzz)
+{
+  return(MagickTransparentPaintImage(wand,target,alpha,fuzz,MagickFalse));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k S e t I m a g e A t t r i b u t e                             %
 %                                                                             %
 %                                                                             %
@@ -794,11 +1031,11 @@ WandExport MagickBooleanType MagickOpaqueImage(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o property: The property.
+%    o property: the property.
 %
-%    o value: The value.
+%    o value: the value.
 %
 */
 WandExport MagickBooleanType MagickSetImageAttribute(MagickWand *wand,
@@ -827,9 +1064,9 @@ WandExport MagickBooleanType MagickSetImageAttribute(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o index: The scene number.
+%    o index: the scene number.
 %
 */
 WandExport MagickBooleanType MagickSetImageIndex(MagickWand *wand,
@@ -859,9 +1096,9 @@ WandExport MagickBooleanType MagickSetImageIndex(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o format: The image format.
+%    o format: the image format.
 %
 %    o key:  The key.
 %
@@ -903,12 +1140,12 @@ WandExport MagickBooleanType MagickSetImageOption(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 %    o target: Change this target color to specified opacity value within
 %      the image.
 %
-%    o alpha: The level of transparency: 1.0 is fully opaque and 0.0 is fully
+%    o alpha: the level of transparency: 1.0 is fully opaque and 0.0 is fully
 %      transparent.
 %
 %    o fuzz: By default target must match a particular pixel color
@@ -937,7 +1174,7 @@ WandExport MagickBooleanType MagickTransparentImage(MagickWand *wand,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  MagickRegionOfInterestImage() extracts a region of the image and returns it
-%  as a a new wand.
+%  as a new wand.
 %
 %  The format of the MagickRegionOfInterestImage method is:
 %
@@ -947,15 +1184,15 @@ WandExport MagickBooleanType MagickTransparentImage(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o width: The region width.
+%    o width: the region width.
 %
-%    o height: The region height.
+%    o height: the region height.
 %
-%    o x: The region x offset.
+%    o x: the region x offset.
 %
-%    o y: The region y offset.
+%    o y: the region y offset.
 %
 */
 WandExport MagickWand *MagickRegionOfInterestImage(MagickWand *wand,
@@ -985,9 +1222,9 @@ WandExport MagickWand *MagickRegionOfInterestImage(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o length: The image length in bytes.
+%    o length: the image length in bytes.
 %
 */
 WandExport MagickSizeType MagickGetImageSize(MagickWand *wand)
@@ -1024,9 +1261,9 @@ WandExport MagickSizeType MagickGetImageSize(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o length: The length of the blob.
+%    o length: the length of the blob.
 %
 */
 WandExport unsigned char *MagickWriteImageBlob(MagickWand *wand,size_t *length)
@@ -1054,9 +1291,9 @@ WandExport unsigned char *MagickWriteImageBlob(MagickWand *wand,size_t *length)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o method: The image virtual pixel method : UndefinedVirtualPixelMethod,
+%    o method: the image virtual pixel method : UndefinedVirtualPixelMethod,
 %      ConstantVirtualPixelMethod,  EdgeVirtualPixelMethod,
 %      MirrorVirtualPixelMethod, or TileVirtualPixelMethod.
 %
@@ -1094,9 +1331,9 @@ WandExport VirtualPixelMethod MagickSetImageVirtualPixelMethod(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o number_wands: The number of pixel wands.
+%    o number_wands: the number of pixel wands.
 %
 */
 WandExport PixelWand **PixelGetNextRow(PixelIterator *iterator)
@@ -1128,9 +1365,9 @@ WandExport PixelWand **PixelGetNextRow(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o severity: The severity of the error is returned here.
+%    o severity: the severity of the error is returned here.
 %
 */
 WandExport char *PixelIteratorGetException(const PixelIterator *iterator,
