@@ -45,12 +45,17 @@ extern WandExport DisposeType
   MagickGetImageDispose(MagickWand *);
 
 extern WandExport double
+  *MagickGetImageChannelDistortions(MagickWand *,const MagickWand *,
+    const MetricType),
   MagickGetImageGamma(MagickWand *),
   MagickGetImageTotalInkDensity(MagickWand *);
 
+extern WandExport GravityType
+  MagickGetImageGravity(MagickWand *);
+
 extern WandExport Image
   *MagickDestroyImage(Image *),
-  *GetImageFromMagickWand(MagickWand *);
+  *GetImageFromMagickWand(const MagickWand *);
 
 extern WandExport ImageType
   MagickGetImageType(MagickWand *);
@@ -109,6 +114,8 @@ extern WandExport MagickBooleanType
   MagickCropImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
   MagickCycleColormapImage(MagickWand *,const long),
+  MagickDecipherImage(MagickWand *,const char *),
+  MagickDeskewImage(MagickWand *,const double),
   MagickDespeckleImage(MagickWand *),
   MagickDisplayImage(MagickWand *,const char *),
   MagickDisplayImages(MagickWand *,const char *),
@@ -117,6 +124,7 @@ extern WandExport MagickBooleanType
   MagickDrawImage(MagickWand *,const DrawingWand *),
   MagickEdgeImage(MagickWand *,const double),
   MagickEmbossImage(MagickWand *,const double,const double),
+  MagickEncipherImage(MagickWand *,const char *),
   MagickEnhanceImage(MagickWand *),
   MagickEqualizeImage(MagickWand *),
   MagickEqualizeImageChannel(MagickWand *,const ChannelType),
@@ -126,6 +134,9 @@ extern WandExport MagickBooleanType
   MagickExtentImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
   MagickFlipImage(MagickWand *),
+  MagickFloodfillPaintImage(MagickWand *,const ChannelType,const PixelWand *,
+    const double,const PixelWand *,const long,const long,
+    const MagickBooleanType),
   MagickFlopImage(MagickWand *),
   MagickFrameImage(MagickWand *,const PixelWand *,const unsigned long,
     const unsigned long,const long,const long),
@@ -134,6 +145,7 @@ extern WandExport MagickBooleanType
   MagickGaussianBlurImage(MagickWand *,const double,const double),
   MagickGaussianBlurImageChannel(MagickWand *,const ChannelType,const double,
     const double),
+  MagickGetImageAlphaChannel(MagickWand *),
   MagickGetImageBackgroundColor(MagickWand *,PixelWand *),
   MagickGetImageBluePrimary(MagickWand *,double *,double *),
   MagickGetImageBorderColor(MagickWand *,PixelWand *),
@@ -145,7 +157,6 @@ extern WandExport MagickBooleanType
   MagickGetImageChannelRange(MagickWand *,const ChannelType,double *,double *),
   MagickGetImageColormapColor(MagickWand *,const unsigned long,PixelWand *),
   MagickGetImageGreenPrimary(MagickWand *,double *,double *),
-  MagickGetImageMatte(MagickWand *),
   MagickGetImageMatteColor(MagickWand *,PixelWand *),
   MagickGetImageLength(MagickWand *,MagickSizeType *),
   MagickGetImagePage(MagickWand *,unsigned long *,unsigned long *,long *,
@@ -165,12 +176,15 @@ extern WandExport MagickBooleanType
   MagickLevelImageChannel(MagickWand *,const ChannelType,const double,
     const double,const double),
   MagickLinearStretchImage(MagickWand *,const double,const double),
+  MagickLiquidRescaleImage(MagickWand *,const unsigned long,const unsigned long,
+    const double,const double),
   MagickMagnifyImage(MagickWand *),
-  MagickMapImage(MagickWand *,const MagickWand *,const MagickBooleanType),
   MagickMedianFilterImage(MagickWand *,const double),
   MagickMinifyImage(MagickWand *),
   MagickModulateImage(MagickWand *,const double,const double,const double),
   MagickMotionBlurImage(MagickWand *,const double,const double,const double),
+  MagickMotionBlurImageChannel(MagickWand *,const ChannelType,const double,
+    const double,const double),
   MagickNegateImage(MagickWand *,const MagickBooleanType),
   MagickNegateImageChannel(MagickWand *,const ChannelType,
     const MagickBooleanType),
@@ -180,17 +194,15 @@ extern WandExport MagickBooleanType
   MagickNormalizeImage(MagickWand *),
   MagickNormalizeImageChannel(MagickWand *,const ChannelType),
   MagickOilPaintImage(MagickWand *,const double),
+  MagickOpaquePaintImage(MagickWand *,const PixelWand *,const PixelWand *,
+    const double,const MagickBooleanType),
+  MagickOpaquePaintImageChannel(MagickWand *,const ChannelType,
+    const PixelWand *,const PixelWand *,const double,const MagickBooleanType),
   MagickOrderedPosterizeImage(MagickWand *,const char *),
   MagickOrderedPosterizeImageChannel(MagickWand *,const ChannelType,
     const char *),
-  MagickPaintFloodfillImage(MagickWand *,const ChannelType,const PixelWand *,
-    const double,const PixelWand *,const long,const long),
-  MagickPaintOpaqueImage(MagickWand *,const PixelWand *,const PixelWand *,
-    const double),
-  MagickPaintOpaqueImageChannel(MagickWand *,const ChannelType,
-    const PixelWand *,const PixelWand *,const double),
-  MagickPaintTransparentImage(MagickWand *,const PixelWand *,const double,
-    const double),
+  MagickTransparentPaintImage(MagickWand *,const PixelWand *,
+    const double,const double,const MagickBooleanType invert),
   MagickPingImage(MagickWand *,const char *),
   MagickPingImageBlob(MagickWand *,const void *,const size_t),
   MagickPingImageFile(MagickWand *,FILE *),
@@ -214,6 +226,7 @@ extern WandExport MagickBooleanType
   MagickReadImageFile(MagickWand *,FILE *),
   MagickRecolorImage(MagickWand *,const unsigned long,const double *),
   MagickReduceNoiseImage(MagickWand *,const double),
+  MagickRemapImage(MagickWand *,const MagickWand *,const DitherMethod),
   MagickRemoveImage(MagickWand *),
   MagickResampleImage(MagickWand *,const double,const double,const FilterTypes,
     const double),
@@ -249,6 +262,7 @@ extern WandExport MagickBooleanType
   MagickSetImageGamma(MagickWand *,const double),
   MagickSetImageExtent(MagickWand *,const unsigned long,const unsigned long),
   MagickSetImageFilename(MagickWand *,const char *),
+  MagickSetImageGravity(MagickWand *,const GravityType),
   MagickSetImageFormat(MagickWand *,const char *),
   MagickSetImageInterlaceScheme(MagickWand *,const InterlaceType),
   MagickSetImageInterpolateMethod(MagickWand *,
@@ -286,6 +300,8 @@ extern WandExport MagickBooleanType
     const MagickBooleanType,const double,const double),
   MagickSketchImage(MagickWand *,const double,const double,const double),
   MagickSolarizeImage(MagickWand *,const double),
+  MagickSparseColorImage(MagickWand *,const ChannelType,const SparseColorMethod,
+    const unsigned long,const double *),
   MagickSpliceImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
   MagickSpreadImage(MagickWand *,const double),
