@@ -16,7 +16,7 @@
 %                        W   W  A   A  N   N  DDDD                            %
 %                                                                             %
 %                                                                             %
-%                 ImageMagick MagickWand Programming Interface                %
+%                           MagickWand Wand Methods                           %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
@@ -86,8 +86,7 @@ static unsigned long
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
-%
+%    o wand: the magick wand.
 %
 */
 WandExport void ClearMagickWand(MagickWand *wand)
@@ -124,7 +123,7 @@ WandExport void ClearMagickWand(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
@@ -138,15 +137,8 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   clone_wand=(MagickWand *) AcquireMagickMemory(sizeof(*clone_wand));
   if (clone_wand == (MagickWand *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      wand->name);
   (void) ResetMagickMemory(clone_wand,0,sizeof(*clone_wand));
   clone_wand->id=AcquireWandId();
   (void) FormatMagickString(clone_wand->name,MaxTextExtent,"%s-%lu",
@@ -182,8 +174,7 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
-%
+%    o wand: the magick wand.
 %
 */
 WandExport MagickWand *DestroyMagickWand(MagickWand *wand)
@@ -221,7 +212,7 @@ WandExport MagickWand *DestroyMagickWand(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport MagickBooleanType IsMagickWand(const MagickWand *wand)
@@ -254,7 +245,7 @@ WandExport MagickBooleanType IsMagickWand(const MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport MagickBooleanType MagickClearException(MagickWand *wand)
@@ -287,9 +278,9 @@ WandExport MagickBooleanType MagickClearException(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o severity: The severity of the error is returned here.
+%    o severity: the severity of the error is returned here.
 %
 */
 WandExport char *MagickGetException(const MagickWand *wand,
@@ -346,7 +337,7 @@ WandExport char *MagickGetException(const MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport ExceptionType MagickGetExceptionType(const MagickWand *wand)
@@ -378,7 +369,7 @@ WandExport ExceptionType MagickGetExceptionType(const MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport long MagickGetIteratorIndex(MagickWand *wand)
@@ -416,7 +407,7 @@ WandExport long MagickGetIteratorIndex(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o option: The option name.
+%    o option: the option name.
 %
 */
 WandExport char *MagickQueryConfigureOption(const char *option)
@@ -524,11 +515,11 @@ WandExport char **MagickQueryConfigureOptions(const char *pattern,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The Magick wand.
+%    o wand: the Magick wand.
 %
-%    o drawing_wand: The drawing wand.
+%    o drawing_wand: the drawing wand.
 %
-%    o text: The text.
+%    o text: the text.
 %
 */
 WandExport double *MagickQueryFontMetrics(MagickWand *wand,
@@ -632,11 +623,11 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The Magick wand.
+%    o wand: the Magick wand.
 %
-%    o drawing_wand: The drawing wand.
+%    o drawing_wand: the drawing wand.
 %
-%    o text: The text.
+%    o text: the text.
 %
 */
 WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
@@ -834,7 +825,7 @@ WandExport void *MagickRelinquishMemory(void *memory)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport void MagickResetIterator(MagickWand *wand)
@@ -867,7 +858,7 @@ WandExport void MagickResetIterator(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport void MagickSetFirstIterator(MagickWand *wand)
@@ -902,9 +893,9 @@ WandExport void MagickSetFirstIterator(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
-%    o index: The scene number.
+%    o index: the scene number.
 %
 */
 WandExport MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
@@ -949,7 +940,7 @@ WandExport MagickBooleanType MagickSetIteratorIndex(MagickWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport void MagickSetLastIterator(MagickWand *wand)
@@ -976,9 +967,9 @@ WandExport void MagickSetLastIterator(MagickWand *wand)
 %
 %  MagickWandGenesis() initializes the MagickWand environment.
 %
-%  The format of the MaickWandGenesis method is:
+%  The format of the MagickWandGenesis method is:
 %
-%      void MaickWandGenesis(void)
+%      void MagickWandGenesis(void)
 %
 */
 WandExport void MagickWandGenesis(void)
@@ -1047,15 +1038,8 @@ WandExport MagickWand *NewMagickWand(void)
     ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
   wand=(MagickWand *) AcquireMagickMemory(sizeof(*wand));
   if (wand == (MagickWand *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      strerror(errno));
   (void) ResetMagickMemory(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
   (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%lu",MagickWandId,
@@ -1090,7 +1074,7 @@ WandExport MagickWand *NewMagickWand(void)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 WandExport MagickWand *NewMagickWandFromImage(const Image *image)
