@@ -10,14 +10,14 @@
 %               M   M   OOO   N   N    T    A   A   GGG   EEEEE               %
 %                                                                             %
 %                                                                             %
-%                ImageMagick Methods to Create Image Thumbnails               %
+%                MagickWand Methods to Create Image Thumbnails                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -66,9 +66,9 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
-%    o argc: The number of elements in the argument vector.
+%    o argc: the number of elements in the argument vector.
 %
 %    o argv: A text array containing the command line arguments.
 %
@@ -90,7 +90,7 @@ static void MontageUsage(void)
       "-help                print program options",
       "-list type           print a list of supported option arguments",
       "-log format          format of debugging information",
-			"-version             print version information",
+      "-version             print version information",
       (char *) NULL
     },
     *operators[]=
@@ -120,7 +120,8 @@ static void MontageUsage(void)
       "-adjoin              join images into a single multi-image file",
       "-affine matrix       affine transform matrix",
       "-alpha option        activate, deactivate, reset, or set the alpha channel",
-      "-authenticate value  decrypt image with this password",
+      "-authenticate password",
+      "                     decipher image with this password",
       "-blue-primary point  chromaticity blue primary point",
       "-bordercolor color   border color",
       "-caption string      assign a caption to an image",
@@ -136,7 +137,7 @@ static void MontageUsage(void)
       "-depth value         image depth",
       "-display server      query font from this X server",
       "-dispose method      layer disposal method",
-      "-dither              apply Floyd/Steinberg error diffusion to image",
+      "-dither method       apply error diffusion to image",
       "-draw string         annotate the image with a graphic primitive",
       "-encoding type       text encoding type",
       "-endian type         endianness (MSB or LSB) of the image",
@@ -166,6 +167,7 @@ static void MontageUsage(void)
       "-quiet               suppress all warning messages",
       "-red-primary point   chromaticity red primary point",
       "-regard-warnings     pay attention to warning messages",
+      "-respect-parenthesis settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-scenes range        image scene range",
@@ -191,6 +193,7 @@ static void MontageUsage(void)
     *sequence_operators[]=
     {
       "-coalesce            merge a sequence of images",
+      (char *) NULL
     },
     *stack_operators[]=
     {
@@ -242,8 +245,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
 { \
   if (montage_image != (Image *) NULL) \
     montage_image=DestroyImageList(montage_image); \
-  for ( ; k >= 0; k--) \
-    image_stack[k]=DestroyImageList(image_stack[k]); \
+  DestroyImageStack(); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
   argv=(char **) RelinquishMagickMemory(argv); \
@@ -271,9 +273,12 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
     *format;
 
   Image
-    *image_stack[MaxImageStackDepth+1],
+    *image,
     *montage_image;
 
+  ImageStack
+    image_stack[MaxImageStackDepth+1];
+
   long
     first_scene,
     j,
@@ -291,9 +296,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
   MontageInfo
     *montage_info;
 
-  QuantizeInfo
-    quantize_info;
-
   register long
     i;
 
@@ -314,7 +316,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
           (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-          return(MagickTrue);
+          return(MagickFalse);
         }
     }
   if (argc < 3)
@@ -323,14 +325,12 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
   first_scene=0;
   j=1;
   k=0;
-  image_stack[k]=NewImageList();
   last_scene=0;
   montage_image=NewImageList();
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
+  NewImageStack();
   option=(char *) NULL;
   pend=MagickFalse;
-  GetQuantizeInfo(&quantize_info);
-  quantize_info.number_colors=0;
   scene=0;
   status=MagickFalse;
   transparent_color=(char *) NULL;
@@ -340,46 +340,34 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowMontageException(ResourceLimitError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowMontageException(ResourceLimitError,"MemoryAllocationFailed",
+      strerror(errno));
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
     if (LocaleCompare(option,"(") == 0)
       {
+        FireImageStack(MagickTrue,MagickTrue,pend);
         if (k == MaxImageStackDepth)
           ThrowMontageException(OptionError,"ParenthesisNestedTooDeeply",
             option);
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
-        k++;
-        image_stack[k]=NewImageList();
+        PushImageStack();
         continue;
       }
     if (LocaleCompare(option,")") == 0)
       {
+        FireImageStack(MagickTrue,MagickTrue,MagickTrue);
         if (k == 0)
           ThrowMontageException(OptionError,"UnableToParseExpression",option);
-        if (image_stack[k] != (Image *) NULL)
-          {
-            MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-            AppendImageToList(&image_stack[k-1],image_stack[k]);
-          }
-        k--;
+        PopImageStack();
         continue;
       }
     if (IsMagickOption(option) == MagickFalse)
       {
         Image
-         *image;
+          *images;
 
-        MogrifyImageStack(image_stack[k],MagickTrue,pend);
+        FireImageStack(MagickTrue,MagickTrue,pend);
         for (scene=first_scene; scene <= last_scene ; scene++)
         {
           char
@@ -409,16 +397,16 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
                 MaxTextExtent);
             }
           (void) CloneString(&image_info->font,montage_info->font);
-          image=ReadImage(image_info,exception);
-          status&=(image != (Image *) NULL) &&
+          images=ReadImages(image_info,exception);
+          status&=(images != (Image *) NULL) &&
             (exception->severity < ErrorException);
-          if (image == (Image *) NULL)
+          if (images == (Image *) NULL)
             continue;
-          AppendImageToList(&image_stack[k],image);
+          AppendImageStack(images);
         }
         continue;
       }
-    pend=image_stack[k] != (Image *) NULL ? MagickTrue : MagickFalse;
+    pend=image != (Image *) NULL ? MagickTrue : MagickFalse;
     switch (*(option+1))
     {
       case 'a':
@@ -594,9 +582,9 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             Image
               *clone_images;
 
-            clone_images=image_stack[k];
+            clone_images=image;
             if (k != 0)
-              clone_images=image_stack[k-1];
+              clone_images=image_stack[k-1].image;
             if (clone_images == (Image *) NULL)
               ThrowMontageException(ImageError,"ImageSequenceRequired",option);
             if (*option == '+')
@@ -612,15 +600,14 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               }
             if (clone_images == (Image *) NULL)
               ThrowMontageException(OptionError,"NoSuchImage",option);
-            MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
-            AppendImageToList(&image_stack[k],clone_images);
+            FireImageStack(MagickTrue,MagickTrue,MagickTrue);
+            AppendImageStack(clone_images);
             break;
           }
         if (LocaleCompare("coalesce",option+1) == 0)
           break;
         if (LocaleCompare("colors",option+1) == 0)
           {
-            quantize_info.number_colors=0;
             if (*option == '+')
               break;
             i++;
@@ -628,7 +615,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowMontageInvalidArgumentException(option,argv[i]);
-            quantize_info.number_colors=(unsigned long) atol(argv[i]);
             break;
           }
         if (LocaleCompare("colorspace",option+1) == 0)
@@ -788,7 +774,18 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("dither",option+1) == 0)
           {
-            quantize_info.dither=(*option == '-') ? MagickTrue : MagickFalse;
+            long
+              method;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            method=ParseMagickOption(MagickDitherOptions,MagickFalse,argv[i]);
+            if (method < 0)
+              ThrowMontageException(OptionError,"UnrecognizedDitherMethod",
+                argv[i]);
             break;
           }
         if (LocaleCompare("draw",option+1) == 0)
@@ -1020,6 +1017,9 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("limit",option+1) == 0)
           {
+            char
+              *p;
+
             long
               resource;
 
@@ -1036,8 +1036,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowMontageException(OptionError,"MissingArgument",option);
-            if ((LocaleCompare("unlimited",argv[i]) != 0) &&
-                (IsGeometry(argv[i]) == MagickFalse))
+            (void) strtod(argv[i],&p);
+            if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowMontageInvalidArgumentException(option,argv[i]);
             break;
           }
@@ -1115,8 +1115,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           {
             if (*option == '+')
               break;
-            quantize_info.number_colors=2;
-            quantize_info.colorspace=GRAYColorspace;
             break;
           }
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
@@ -1259,6 +1257,11 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+          {
+            respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
+            break;
+          }
         if (LocaleCompare("rotate",option+1) == 0)
           {
             i++;
@@ -1285,22 +1288,13 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("seed",option+1) == 0)
           {
-            unsigned long
-              seed;
-
             if (*option == '+')
-              {
-                seed=(unsigned long) time((time_t *) NULL);
-                SeedRandomReservoir(seed);
-                break;
-              }
+              break;
             i++;
             if (i == (long) (argc-1))
               ThrowMontageException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowMontageInvalidArgumentException(option,argv[i]);
-            seed=(unsigned long) atol(argv[i]);
-            SeedRandomReservoir(seed);
             break;
           }
         if (LocaleCompare("scenes",option+1) == 0)
@@ -1312,7 +1306,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowMontageException(OptionError,"MissingArgument",option);
-            if (IsGeometry(argv[i]) == MagickFalse)
+            if (IsSceneGeometry(argv[i],MagickFalse) == MagickFalse)
               ThrowMontageInvalidArgumentException(option,argv[i]);
             first_scene=atol(argv[i]);
             last_scene=first_scene;
@@ -1486,7 +1480,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("treedepth",option+1) == 0)
           {
-            quantize_info.tree_depth=0;
             if (*option == '+')
               break;
             i++;
@@ -1494,7 +1487,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageException(OptionError,"MissingArgument",option);
             if (IsGeometry(argv[i]) == MagickFalse)
               ThrowMontageInvalidArgumentException(option,argv[i]);
-            quantize_info.tree_depth=(unsigned long) atol(argv[i]);
             break;
           }
         if (LocaleCompare("trim",option+1) == 0)
@@ -1591,21 +1583,20 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
       default:
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
     }
-    fire=(MagickBooleanType) ParseMagickOption(MagickMogrifyOptions,
-      MagickFalse,option+1);
-    if (fire == MagickTrue)
-      MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ? 
+      MagickFalse : MagickTrue;
+    if (fire != MagickFalse)
+      FireImageStack(MagickTrue,MagickTrue,MagickTrue);
   }
   if (k != 0)
     ThrowMontageException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i-- != (long) (argc-1))
     ThrowMontageException(OptionError,"MissingAnImageFilename",argv[i]);
-  if (image_stack[k] == (Image *) NULL)
+  if (image == (Image *) NULL)
     ThrowMontageException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  MogrifyImageStack(image_stack[k],MagickTrue,MagickTrue);
+  FinalizeImageSettings(image_info,image);
   (void) CopyMagickString(montage_info->filename,argv[argc-1],MaxTextExtent);
-  montage_image=MontageImageList(image_info,montage_info,image_stack[k],
-    exception);
+  montage_image=MontageImageList(image_info,montage_info,image,exception);
   if (montage_image == (Image *) NULL)
     status=MagickFalse;
   else
@@ -1625,15 +1616,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
 
           text=InterpretImageProperties(image_info,montage_image,format);
           if (text == (char *) NULL)
-            {
-              char
-                *message;
-
-              message=GetExceptionMessage(errno);
-              ThrowMontageException(ResourceLimitError,"MemoryAllocationFailed",
-                message);
-              message=DestroyString(message);
-            }
+            ThrowMontageException(ResourceLimitError,"MemoryAllocationFailed",
+              strerror(errno));
           (void) ConcatenateString(&(*metadata),text);
           (void) ConcatenateString(&(*metadata),"\n");
           text=DestroyString(text);
