@@ -23,7 +23,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -113,7 +113,7 @@ struct _PixelIterator
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
 */
 WandExport void ClearPixelIterator(PixelIterator *iterator)
@@ -150,7 +150,7 @@ WandExport void ClearPixelIterator(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The magick iterator.
+%    o iterator: the magick iterator.
 %
 */
 WandExport PixelIterator *ClonePixelIterator(const PixelIterator *iterator)
@@ -164,15 +164,8 @@ WandExport PixelIterator *ClonePixelIterator(const PixelIterator *iterator)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   clone_iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*clone_iterator));
   if (clone_iterator == (PixelIterator *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      iterator->name);
   (void) ResetMagickMemory(clone_iterator,0,sizeof(*clone_iterator));
   clone_iterator->id=AcquireWandId();
   (void) FormatMagickString(clone_iterator->name,MaxTextExtent,"%s-%lu",
@@ -212,7 +205,7 @@ WandExport PixelIterator *ClonePixelIterator(const PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
 */
 WandExport PixelIterator *DestroyPixelIterator(PixelIterator *iterator)
@@ -221,7 +214,7 @@ WandExport PixelIterator *DestroyPixelIterator(PixelIterator *iterator)
   assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->view=CloseCacheView(iterator->view);
+  iterator->view=DestroyCacheView(iterator->view);
   iterator->pixel_wands=DestroyPixelWands(iterator->pixel_wands,
     iterator->region.width);
   iterator->exception=DestroyExceptionInfo(iterator->exception);
@@ -251,7 +244,7 @@ WandExport PixelIterator *DestroyPixelIterator(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The magick iterator.
+%    o iterator: the magick iterator.
 %
 */
 WandExport MagickBooleanType IsPixelIterator(const PixelIterator *iterator)
@@ -288,7 +281,7 @@ WandExport MagickBooleanType IsPixelIterator(const PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
@@ -316,20 +309,13 @@ WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
   image=GetImageFromMagickWand(wand);
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
-  view=OpenCacheView(image);
+  view=AcquireCacheView(image);
   if (view == (ViewInfo *) NULL)
     return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      strerror(errno));
   (void) ResetMagickMemory(iterator,0,sizeof(*iterator));
   iterator->id=AcquireWandId();
   (void) FormatMagickString(iterator->name,MaxTextExtent,"%s-%lu",
@@ -370,7 +356,7 @@ WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport MagickBooleanType PixelClearIteratorException(
@@ -400,19 +386,18 @@ WandExport MagickBooleanType PixelClearIteratorException(
 %  The format of the NewPixelRegionIterator method is:
 %
 %      PixelIterator NewPixelRegionIterator(MagickWand *wand,const long x,
-%        const long y,const unsigned long columns,const unsigned long rows,
-%        const MagickBooleanType modify)
+%        const long y,const unsigned long width,const unsigned long height)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 %    o x,y,columns,rows:  These values define the perimeter of a region of
 %      pixels.
 %
 */
 WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
-  const long y,const unsigned long columns,const unsigned long rows)
+  const long y,const unsigned long width,const unsigned long height)
 {
   const char
     *quantum;
@@ -434,25 +419,18 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
   quantum=GetMagickQuantumDepth(&depth);
   if (depth != MAGICKCORE_QUANTUM_DEPTH)
     ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
-  if ((columns == 0) || (rows == 0))
+  if ((width == 0) || (width == 0))
     ThrowWandFatalException(WandError,"ZeroRegionSize",quantum);
   image=GetImageFromMagickWand(wand);
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
-  view=OpenCacheView(image);
+  view=AcquireCacheView(image);
   if (view == (ViewInfo *) NULL)
     return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      wand->name);
   (void) ResetMagickMemory(iterator,0,sizeof(*iterator));
   iterator->id=AcquireWandId();
   (void) FormatMagickString(iterator->name,MaxTextExtent,"%s-%lu",
@@ -460,8 +438,8 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
   iterator->exception=AcquireExceptionInfo();
   iterator->view=view;
   SetGeometry(image,&iterator->region);
-  iterator->region.width=columns;
-  iterator->region.height=rows;
+  iterator->region.width=width;
+  iterator->region.height=height;
   iterator->region.x=x;
   iterator->region.y=y;
   iterator->pixel_wands=NewPixelWands(iterator->region.width);
@@ -494,9 +472,9 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o number_wands: The number of pixel wands.
+%    o number_wands: the number of pixel wands.
 %
 */
 WandExport PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
@@ -506,7 +484,7 @@ WandExport PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
     *indexes;
 
   register const PixelPacket
-    *p;
+    *pixels;
 
   register long
     x;
@@ -517,21 +495,24 @@ WandExport PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   *number_wands=0;
   iterator->active=MagickTrue;
-  p=AcquireCacheViewPixels(iterator->view,iterator->region.x,iterator->region.y+
-    iterator->y,iterator->region.width,1,iterator->exception);
-  if (p == (const PixelPacket *) NULL)
-    return((PixelWand **) NULL);
+  pixels=AcquireCacheViewPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1,
+    iterator->exception);
+  if (pixels == (const PixelPacket *) NULL)
+    {
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
+      return((PixelWand **) NULL);
+    }
   indexes=AcquireCacheViewIndexes(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
-  {
-    PixelSetQuantumColor(iterator->pixel_wands[x],p);
-    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
       PixelSetBlackQuantum(iterator->pixel_wands[x],indexes[x]);
-    else
-      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
-        PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
-    p++;
-  }
+  if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+    for (x=0; x < (long) iterator->region.width; x++)
+      PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
   *number_wands=iterator->region.width;
   return(iterator->pixel_wands);
 }
@@ -557,9 +538,9 @@ WandExport PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o severity: The severity of the error is returned here.
+%    o severity: the severity of the error is returned here.
 %
 */
 WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
@@ -577,15 +558,8 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
   description=(char *) AcquireQuantumMemory(2UL*MaxTextExtent,
     sizeof(*description));
   if (description == (char *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      iterator->name);
   *description='\0';
   if (iterator->exception->reason != (char *) NULL)
     (void) CopyMagickString(description,GetLocaleExceptionMessage(
@@ -621,7 +595,7 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport ExceptionType PixelGetIteratorExceptionType(
@@ -653,7 +627,7 @@ WandExport ExceptionType PixelGetIteratorExceptionType(
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
 */
 WandExport long PixelGetIteratorRow(PixelIterator *iterator)
@@ -686,9 +660,9 @@ WandExport long PixelGetIteratorRow(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o number_wands: The number of pixel wands.
+%    o number_wands: the number of pixel wands.
 %
 */
 WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
@@ -698,7 +672,7 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
     *indexes;
 
   register const PixelPacket
-    *p;
+    *pixels;
 
   register long
     x;
@@ -712,21 +686,24 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
     iterator->y++;
   if (PixelSetIteratorRow(iterator,iterator->y) == MagickFalse)
     return((PixelWand **) NULL);
-  p=AcquireCacheViewPixels(iterator->view,iterator->region.x,iterator->region.y+
-    iterator->y,iterator->region.width,1,iterator->exception);
-  if (p == (const PixelPacket *) NULL)
-    return((PixelWand **) NULL);
+  pixels=AcquireCacheViewPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1,
+    iterator->exception);
+  if (pixels == (const PixelPacket *) NULL)
+    {
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
+      return((PixelWand **) NULL);
+    }
   indexes=AcquireCacheViewIndexes(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
-  {
-    PixelSetQuantumColor(iterator->pixel_wands[x],p);
-    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
       PixelSetBlackQuantum(iterator->pixel_wands[x],indexes[x]);
-    else
-      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+  if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+    for (x=0; x < (long) iterator->region.width; x++)
         PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
-    p++;
-  }
   *number_wands=iterator->region.width;
   return(iterator->pixel_wands);
 }
@@ -752,9 +729,9 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o number_wands: The number of pixel wands.
+%    o number_wands: the number of pixel wands.
 %
 */
 
@@ -773,7 +750,7 @@ WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
     *indexes;
 
   register const PixelPacket
-    *p;
+    *pixels;
 
   register long
     x;
@@ -787,21 +764,24 @@ WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
     iterator->y--;
   if (PixelSetIteratorRow(iterator,iterator->y) == MagickFalse)
     return((PixelWand **) NULL);
-  p=AcquireCacheViewPixels(iterator->view,iterator->region.x,iterator->region.y+
-    iterator->y,iterator->region.width,1,iterator->exception);
-  if (p == (const PixelPacket *) NULL)
-    return((PixelWand **) NULL);
+  pixels=AcquireCacheViewPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1,
+    iterator->exception);
+  if (pixels == (const PixelPacket *) NULL)
+    {
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
+      return((PixelWand **) NULL);
+    }
   indexes=AcquireCacheViewIndexes(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
-  {
-    PixelSetQuantumColor(iterator->pixel_wands[x],p);
-    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
       PixelSetBlackQuantum(iterator->pixel_wands[x],indexes[x]);
-    else
-      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
-        PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
-    p++;
-  }
+  if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+    for (x=0; x < (long) iterator->region.width; x++)
+      PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
   *number_wands=iterator->region.width;
   return(iterator->pixel_wands);
 }
@@ -827,7 +807,7 @@ WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
 */
 WandExport void PixelResetIterator(PixelIterator *iterator)
@@ -859,7 +839,7 @@ WandExport void PixelResetIterator(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The magick iterator.
+%    o iterator: the magick iterator.
 %
 */
 WandExport void PixelSetFirstIteratorRow(PixelIterator *iterator)
@@ -892,7 +872,7 @@ WandExport void PixelSetFirstIteratorRow(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
 */
 WandExport MagickBooleanType PixelSetIteratorRow(PixelIterator *iterator,
@@ -928,7 +908,7 @@ WandExport MagickBooleanType PixelSetIteratorRow(PixelIterator *iterator,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The magick iterator.
+%    o iterator: the magick iterator.
 %
 */
 WandExport void PixelSetLastIteratorRow(PixelIterator *iterator)
@@ -960,7 +940,7 @@ WandExport void PixelSetLastIteratorRow(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
 */
 WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
@@ -972,7 +952,7 @@ WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
     x;
 
   register PixelPacket
-    *p;
+    *pixels;
 
   assert(iterator != (const PixelIterator *) NULL);
   assert(iterator->signature == WandSignature);
@@ -980,9 +960,9 @@ WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   if (SetCacheViewStorageClass(iterator->view,DirectClass) == MagickFalse)
     return(MagickFalse);
-  p=GetCacheViewPixels(iterator->view,iterator->region.x,iterator->region.y+
-    iterator->y,iterator->region.width,1);
-  if (p == (PixelPacket *) NULL)
+  pixels=GetCacheViewPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1);
+  if (pixels == (PixelPacket *) NULL)
     {
       InheritException(iterator->exception,GetCacheViewException(
         iterator->view));
@@ -990,16 +970,11 @@ WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
     }
   indexes=GetCacheViewIndexes(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
-  {
-    PixelGetQuantumColor(iterator->pixel_wands[x],p);
-    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    PixelGetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
       indexes[x]=PixelGetBlackQuantum(iterator->pixel_wands[x]);
-    else
-      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
-        indexes[x]=PixelGetIndex(iterator->pixel_wands[x]);
-    p++;
-  }
-  if (SyncCacheView(iterator->view) == MagickFalse)
+  if (SyncCacheViewPixels(iterator->view) == MagickFalse)
     {
       InheritException(iterator->exception,GetCacheViewException(
         iterator->view));
