@@ -16,7 +16,7 @@
 %                         W   W  A   A  N   N  DDDD                           %
 %                                                                             %
 %                                                                             %
-%                   ImageMagick Image Pixel Wand Methods                      %
+%                    MagickWand Image Pixel Wand Methods                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -103,7 +103,7 @@ struct _PixelWand
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport void ClearPixelWand(PixelWand *wand)
@@ -136,7 +136,7 @@ WandExport void ClearPixelWand(PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport PixelWand *ClonePixelWand(const PixelWand *wand)
@@ -150,15 +150,8 @@ WandExport PixelWand *ClonePixelWand(const PixelWand *wand)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   clone_wand=(PixelWand *) AcquireMagickMemory(sizeof(*clone_wand));
   if (clone_wand == (PixelWand *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      wand->name);
   (void) ResetMagickMemory(clone_wand,0,sizeof(*clone_wand));
   clone_wand->id=AcquireWandId();
   (void) FormatMagickString(clone_wand->name,MaxTextExtent,"%s-%lu",PixelWandId,
@@ -194,9 +187,9 @@ WandExport PixelWand *ClonePixelWand(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wands: The magick wands.
+%    o wands: the magick wands.
 %
-%    o number_wands: The number of wands.
+%    o number_wands: the number of wands.
 %
 */
 WandExport PixelWand **ClonePixelWands(const PixelWand **wands,
@@ -211,15 +204,8 @@ WandExport PixelWand **ClonePixelWands(const PixelWand **wands,
   clone_wands=(PixelWand **) AcquireQuantumMemory((size_t) number_wands,
     sizeof(*clone_wands));
   if (clone_wands == (PixelWand **) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      strerror(errno));
   for (i=0; i < (long) number_wands; i++)
     clone_wands[i]=ClonePixelWand(wands[i]);
   return(clone_wands);
@@ -244,7 +230,7 @@ WandExport PixelWand **ClonePixelWands(const PixelWand **wands,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport PixelWand *DestroyPixelWand(PixelWand *wand)
@@ -281,9 +267,9 @@ WandExport PixelWand *DestroyPixelWand(PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o number_wands: The number of wands.
+%    o number_wands: the number of wands.
 %
 */
 WandExport PixelWand **DestroyPixelWands(PixelWand **wand,
@@ -324,9 +310,9 @@ WandExport PixelWand **DestroyPixelWands(PixelWand **wand,
 %
 %  A description of each parameter follows:
 %
-%    o p: The pixel wand.
+%    o p: the pixel wand.
 %
-%    o q: The pixel wand.
+%    o q: the pixel wand.
 %
 %    o fuzz: any two colors that are less than or equal to this distance
 %      squared are consider similar.
@@ -367,7 +353,7 @@ WandExport MagickBooleanType IsPixelWandSimilar(PixelWand *p,PixelWand *q,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport MagickBooleanType IsPixelWand(const PixelWand *wand)
@@ -416,15 +402,8 @@ WandExport PixelWand *NewPixelWand(void)
     ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
   wand=(PixelWand *) AcquireMagickMemory(sizeof(*wand));
   if (wand == (PixelWand *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      strerror(errno));
   (void) ResetMagickMemory(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
   (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%lu",PixelWandId,
@@ -457,7 +436,7 @@ WandExport PixelWand *NewPixelWand(void)
 %
 %  A description of each parameter follows:
 %
-%    o number_wands: The number of wands.
+%    o number_wands: the number of wands.
 %
 */
 WandExport PixelWand **NewPixelWands(const unsigned long number_wands)
@@ -471,15 +450,8 @@ WandExport PixelWand **NewPixelWands(const unsigned long number_wands)
   wands=(PixelWand **) AcquireQuantumMemory((size_t) number_wands,
     sizeof(*wands));
   if (wands == (PixelWand **) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      strerror(errno));
   for (i=0; i < (long) number_wands; i++)
     wands[i]=NewPixelWand();
   return(wands);
@@ -504,7 +476,7 @@ WandExport PixelWand **NewPixelWands(const unsigned long number_wands)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport MagickBooleanType PixelClearException(PixelWand *wand)
@@ -536,7 +508,7 @@ WandExport MagickBooleanType PixelClearException(PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetAlpha(const PixelWand *wand)
@@ -567,7 +539,7 @@ WandExport double PixelGetAlpha(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetAlphaQuantum(const PixelWand *wand)
@@ -598,7 +570,7 @@ WandExport Quantum PixelGetAlphaQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetBlack(const PixelWand *wand)
@@ -629,7 +601,7 @@ WandExport double PixelGetBlack(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetBlackQuantum(const PixelWand *wand)
@@ -660,7 +632,7 @@ WandExport Quantum PixelGetBlackQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetBlue(const PixelWand *wand)
@@ -691,7 +663,7 @@ WandExport double PixelGetBlue(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetBlueQuantum(const PixelWand *wand)
@@ -722,7 +694,7 @@ WandExport Quantum PixelGetBlueQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport char *PixelGetColorAsString(const PixelWand *wand)
@@ -763,7 +735,7 @@ WandExport char *PixelGetColorAsString(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport char *PixelGetColorAsNormalizedString(const PixelWand *wand)
@@ -807,7 +779,7 @@ WandExport char *PixelGetColorAsNormalizedString(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport unsigned long PixelGetColorCount(const PixelWand *wand)
@@ -838,7 +810,7 @@ WandExport unsigned long PixelGetColorCount(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetCyan(const PixelWand *wand)
@@ -869,7 +841,7 @@ WandExport double PixelGetCyan(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetCyanQuantum(const PixelWand *wand)
@@ -901,9 +873,9 @@ WandExport Quantum PixelGetCyanQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o severity: The severity of the error is returned here.
+%    o severity: the severity of the error is returned here.
 %
 */
 WandExport char *PixelGetException(const PixelWand *wand,
@@ -921,15 +893,8 @@ WandExport char *PixelGetException(const PixelWand *wand,
   description=(char *) AcquireQuantumMemory(2UL*MaxTextExtent,
     sizeof(*description));
   if (description == (char *) NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-        message);
-      message=DestroyString(message);
-    }
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      wand->name);
   *description='\0';
   if (wand->exception->reason != (char *) NULL)
     (void) CopyMagickString(description,GetLocaleExceptionMessage(
@@ -964,7 +929,7 @@ WandExport char *PixelGetException(const PixelWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport ExceptionType PixelGetExceptionType(const PixelWand *wand)
@@ -995,7 +960,7 @@ WandExport ExceptionType PixelGetExceptionType(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetFuzz(const PixelWand *wand)
@@ -1026,7 +991,7 @@ WandExport double PixelGetFuzz(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetGreen(const PixelWand *wand)
@@ -1057,7 +1022,7 @@ WandExport double PixelGetGreen(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetGreenQuantum(const PixelWand *wand)
@@ -1085,18 +1050,18 @@ WandExport Quantum PixelGetGreenQuantum(const PixelWand *wand)
 %  The format of the PixelGetHSL method is:
 %
 %      void PixelGetHSL(const PixelWand *wand,double *hue,double *saturation,
-%        double *luminosity)
+%        double *lightness)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o hue,saturation,luminosity: Return the pixel hue, saturation, and
+%    o hue,saturation,lightness: Return the pixel hue, saturation, and
 %      brightness.
 %
 */
 WandExport void PixelGetHSL(const PixelWand *wand,double *hue,
-  double *saturation,double *luminosity)
+  double *saturation,double *lightness)
 {
   assert(wand != (const PixelWand *) NULL);
   assert(wand->signature == WandSignature);
@@ -1104,7 +1069,7 @@ WandExport void PixelGetHSL(const PixelWand *wand,double *hue,
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   ConvertRGBToHSL(RoundToQuantum(wand->pixel.red),RoundToQuantum(
     wand->pixel.green),RoundToQuantum(wand->pixel.blue),hue,saturation,
-    luminosity);
+    lightness);
 }
 
 /*
@@ -1126,7 +1091,7 @@ WandExport void PixelGetHSL(const PixelWand *wand,double *hue,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport IndexPacket PixelGetIndex(const PixelWand *wand)
@@ -1157,7 +1122,7 @@ WandExport IndexPacket PixelGetIndex(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetMagenta(const PixelWand *wand)
@@ -1188,7 +1153,7 @@ WandExport double PixelGetMagenta(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetMagentaQuantum(const PixelWand *wand)
@@ -1219,7 +1184,7 @@ WandExport Quantum PixelGetMagentaQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 %    o color:  The pixel wand color is returned here.
 %
@@ -1254,7 +1219,7 @@ WandExport void PixelGetMagickColor(const PixelWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetOpacity(const PixelWand *wand)
@@ -1285,7 +1250,7 @@ WandExport double PixelGetOpacity(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetOpacityQuantum(const PixelWand *wand)
@@ -1316,7 +1281,7 @@ WandExport Quantum PixelGetOpacityQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 %    o color:  The pixel wand color is returned here.
 %
@@ -1363,7 +1328,7 @@ WandExport void PixelGetQuantumColor(const PixelWand *wand,PixelPacket *color)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetRed(const PixelWand *wand)
@@ -1394,7 +1359,7 @@ WandExport double PixelGetRed(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetRedQuantum(const PixelWand *wand)
@@ -1425,7 +1390,7 @@ WandExport Quantum PixelGetRedQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport double PixelGetYellow(const PixelWand *wand)
@@ -1456,7 +1421,7 @@ WandExport double PixelGetYellow(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport Quantum PixelGetYellowQuantum(const PixelWand *wand)
@@ -1487,9 +1452,9 @@ WandExport Quantum PixelGetYellowQuantum(const PixelWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o alpha: The level of transparency: 1.0 is fully opaque and 0.0 is fully
+%    o alpha: the level of transparency: 1.0 is fully opaque and 0.0 is fully
 %      transparent.
 %
 */
@@ -1523,9 +1488,9 @@ WandExport void PixelSetAlpha(PixelWand *wand,const double alpha)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o opacity: The opacity color.
+%    o opacity: the opacity color.
 %
 */
 WandExport void PixelSetAlphaQuantum(PixelWand *wand,const Quantum opacity)
@@ -1556,9 +1521,9 @@ WandExport void PixelSetAlphaQuantum(PixelWand *wand,const Quantum opacity)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o black: The black color.
+%    o black: the black color.
 %
 */
 WandExport void PixelSetBlack(PixelWand *wand,const double black)
@@ -1590,9 +1555,9 @@ WandExport void PixelSetBlack(PixelWand *wand,const double black)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o black: The black color.
+%    o black: the black color.
 %
 */
 WandExport void PixelSetBlackQuantum(PixelWand *wand,const Quantum black)
@@ -1623,9 +1588,9 @@ WandExport void PixelSetBlackQuantum(PixelWand *wand,const Quantum black)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o blue: The blue color.
+%    o blue: the blue color.
 %
 */
 WandExport void PixelSetBlue(PixelWand *wand,const double blue)
@@ -1657,9 +1622,9 @@ WandExport void PixelSetBlue(PixelWand *wand,const double blue)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o blue: The blue color.
+%    o blue: the blue color.
 %
 */
 WandExport void PixelSetBlueQuantum(PixelWand *wand,const Quantum blue)
@@ -1691,9 +1656,9 @@ WandExport void PixelSetBlueQuantum(PixelWand *wand,const Quantum blue)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o color: The pixel wand color.
+%    o color: the pixel wand color.
 %
 */
 WandExport MagickBooleanType PixelSetColor(PixelWand *wand,const char *color)
@@ -1733,9 +1698,9 @@ WandExport MagickBooleanType PixelSetColor(PixelWand *wand,const char *color)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o count: The number of this particular color.
+%    o count: the number of this particular color.
 %
 */
 WandExport void PixelSetColorCount(PixelWand *wand,const unsigned long count)
@@ -1766,7 +1731,7 @@ WandExport void PixelSetColorCount(PixelWand *wand,const unsigned long count)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 %    o color: set the pixel wand color here.
 %
@@ -1800,9 +1765,9 @@ WandExport void PixelSetColorFromWand(PixelWand *wand,const PixelWand *color)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o cyan: The cyan color.
+%    o cyan: the cyan color.
 %
 */
 WandExport void PixelSetCyan(PixelWand *wand,const double cyan)
@@ -1834,9 +1799,9 @@ WandExport void PixelSetCyan(PixelWand *wand,const double cyan)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o cyan: The cyan color.
+%    o cyan: the cyan color.
 %
 */
 WandExport void PixelSetCyanQuantum(PixelWand *wand,const Quantum cyan)
@@ -1867,9 +1832,9 @@ WandExport void PixelSetCyanQuantum(PixelWand *wand,const Quantum cyan)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o fuzz: The fuzz value.
+%    o fuzz: the fuzz value.
 %
 */
 WandExport void PixelSetFuzz(PixelWand *wand,const double fuzz)
@@ -1900,9 +1865,9 @@ WandExport void PixelSetFuzz(PixelWand *wand,const double fuzz)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o green: The green color.
+%    o green: the green color.
 %
 */
 WandExport void PixelSetGreen(PixelWand *wand,const double green)
@@ -1934,9 +1899,9 @@ WandExport void PixelSetGreen(PixelWand *wand,const double green)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o green: The green color.
+%    o green: the green color.
 %
 */
 WandExport void PixelSetGreenQuantum(PixelWand *wand,const Quantum green)
@@ -1964,18 +1929,18 @@ WandExport void PixelSetGreenQuantum(PixelWand *wand,const Quantum green)
 %  The format of the PixelSetHSL method is:
 %
 %      void PixelSetHSL(PixelWand *wand,const double hue,
-%        const double saturation,const double luminosity)
+%        const double saturation,const double lightness)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o hue,saturation,luminosity: Return the pixel hue, saturation, and
+%    o hue,saturation,lightness: Return the pixel hue, saturation, and
 %      brightness.
 %
 */
 WandExport void PixelSetHSL(PixelWand *wand,const double hue,
-  const double saturation,const double luminosity)
+  const double saturation,const double lightness)
 {
   Quantum
     blue,
@@ -1986,7 +1951,7 @@ WandExport void PixelSetHSL(PixelWand *wand,const double hue,
   assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  ConvertHSLToRGB(hue,saturation,luminosity,&red,&green,&blue);
+  ConvertHSLToRGB(hue,saturation,lightness,&red,&green,&blue);
   wand->pixel.red=(MagickRealType) red;
   wand->pixel.green=(MagickRealType) green;
   wand->pixel.blue=(MagickRealType) blue;
@@ -2011,9 +1976,9 @@ WandExport void PixelSetHSL(PixelWand *wand,const double hue,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o index: The colormap index.
+%    o index: the colormap index.
 %
 */
 WandExport void PixelSetIndex(PixelWand *wand,const IndexPacket index)
@@ -2044,9 +2009,9 @@ WandExport void PixelSetIndex(PixelWand *wand,const IndexPacket index)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o magenta: The magenta color.
+%    o magenta: the magenta color.
 %
 */
 WandExport void PixelSetMagenta(PixelWand *wand,const double magenta)
@@ -2079,9 +2044,9 @@ WandExport void PixelSetMagenta(PixelWand *wand,const double magenta)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o magenta: The green magenta.
+%    o magenta: the green magenta.
 %
 */
 WandExport void PixelSetMagentaQuantum(PixelWand *wand,const Quantum magenta)
@@ -2112,9 +2077,9 @@ WandExport void PixelSetMagentaQuantum(PixelWand *wand,const Quantum magenta)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o color: The pixel wand color.
+%    o color: the pixel wand color.
 %
 */
 WandExport void PixelSetMagickColor(PixelWand *wand,
@@ -2147,9 +2112,9 @@ WandExport void PixelSetMagickColor(PixelWand *wand,
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o opacity: The opacity color.
+%    o opacity: the opacity color.
 %
 */
 WandExport void PixelSetOpacity(PixelWand *wand,const double opacity)
@@ -2183,9 +2148,9 @@ WandExport void PixelSetOpacity(PixelWand *wand,const double opacity)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o opacity: The opacity color.
+%    o opacity: the opacity color.
 %
 */
 WandExport void PixelSetOpacityQuantum(PixelWand *wand,const Quantum opacity)
@@ -2216,9 +2181,9 @@ WandExport void PixelSetOpacityQuantum(PixelWand *wand,const Quantum opacity)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o color: The pixel wand color.
+%    o color: the pixel wand color.
 %
 */
 WandExport void PixelSetQuantumColor(PixelWand *wand,const PixelPacket *color)
@@ -2254,9 +2219,9 @@ WandExport void PixelSetQuantumColor(PixelWand *wand,const PixelPacket *color)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o red: The red color.
+%    o red: the red color.
 %
 */
 WandExport void PixelSetRed(PixelWand *wand,const double red)
@@ -2288,9 +2253,9 @@ WandExport void PixelSetRed(PixelWand *wand,const double red)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o red: The red color.
+%    o red: the red color.
 %
 */
 WandExport void PixelSetRedQuantum(PixelWand *wand,const Quantum red)
@@ -2321,9 +2286,9 @@ WandExport void PixelSetRedQuantum(PixelWand *wand,const Quantum red)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o yellow: The yellow color.
+%    o yellow: the yellow color.
 %
 */
 WandExport void PixelSetYellow(PixelWand *wand,const double yellow)
@@ -2355,9 +2320,9 @@ WandExport void PixelSetYellow(PixelWand *wand,const double yellow)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
-%    o yellow: The yellow color.
+%    o yellow: the yellow color.
 %
 */
 WandExport void PixelSetYellowQuantum(PixelWand *wand,const Quantum yellow)
