@@ -15,7 +15,7 @@
 %                         WW WW  A   A  N  NN  D   D                          %
 %                         W   W  A   A  N   N  DDDD                           %
 %                                                                             %
-%                    ImageMagick MagickWand Validation Tests                  %
+%                         MagickWand Validation Tests                         %
 %                                                                             %
 %                                                                             %
 %                              Software Design                                %
@@ -427,7 +427,22 @@ int main(int argc,char **argv)
       exit(1);
     }
   (void) fprintf(stdout,"Reading images...\n");
-  status=MagickReadImage(magick_wand,"sequence.miff");
+  {
+    char
+      *p,
+      path[MaxTextExtent];
+
+    path[0]=0;
+    p=getenv("SRCDIR");
+    if (p != (char *) NULL)
+      {
+        (void) strcpy(path,p);
+        if (path[strlen(path)-1] != '/')
+          (void) strcat(path,"/");
+      }
+    (void) strcat(path,"sequence.miff");
+    status=MagickReadImage(magick_wand,path);
+  }
   if (status == MagickFalse)
     ThrowAPIException(magick_wand);
   if (MagickGetNumberImages(magick_wand) != 5)
@@ -486,8 +501,8 @@ int main(int argc,char **argv)
   border=NewPixelWand();
   (void) PixelSetColor(background,"green");
   (void) PixelSetColor(border,"black");
-  status=MagickPaintFloodfillImage(magick_wand,AllChannels,background,
-    0.01*QuantumRange,border,0,0);
+  status=MagickFloodfillPaintImage(magick_wand,AllChannels,background,
+    0.01*QuantumRange,border,0,0,MagickFalse);
   if (status == MagickFalse)
     ThrowAPIException(magick_wand);
   background=DestroyPixelWand(background);
@@ -559,8 +574,8 @@ int main(int argc,char **argv)
   }
   (void) PixelSyncIterator(iterator);
   iterator=DestroyPixelIterator(iterator);
-  (void) fprintf(stdout,"Write to image.miff...\n");
-  status=MagickWriteImages(magick_wand,"image.miff",MagickTrue);
+  (void) fprintf(stdout,"Write to wandtest_out.miff...\n");
+  status=MagickWriteImages(magick_wand,"wandtest_out.miff",MagickTrue);
   if (status == MagickFalse)
     ThrowAPIException(magick_wand);
   (void) fprintf(stdout,"Change image format from \"MIFF\" to \"GIF\"...\n");
@@ -578,8 +593,8 @@ int main(int argc,char **argv)
       (void) fprintf( stderr,"Get delay does not match set delay\n");
       exit(1);
     }
-  (void) fprintf(stdout,"Write to image_0.gif...\n");
-  status=MagickWriteImages(magick_wand,"image.gif",MagickTrue);
+  (void) fprintf(stdout,"Write to wandtest_out_0.gif...\n");
+  status=MagickWriteImages(magick_wand,"wandtest_out.gif",MagickTrue);
   if (status == MagickFalse)
     ThrowAPIException(magick_wand);
   (void) fprintf(stdout,"Set, list, get, and delete wand option...\n");
@@ -645,7 +660,7 @@ int main(int argc,char **argv)
       (void) fprintf( stderr,"Profile does not match\n");
       exit(1);
     }
-  profile=MagickRelinquishMemory(profile);
+  profile=(unsigned char *) MagickRelinquishMemory(profile);
   profiles=MagickGetImageProfiles(magick_wand,"*",&number_profiles);
   if (profiles != (char **) NULL)
     {
@@ -664,7 +679,7 @@ int main(int argc,char **argv)
       (void) fprintf( stderr,"Profile does not match\n");
       exit(1);
     }
-  profile=MagickRelinquishMemory(profile);
+  profile=(unsigned char *) MagickRelinquishMemory(profile);
   magick_wand=DestroyMagickWand(magick_wand);
   (void) fprintf(stdout,"Wand tests pass.\n");
   return(0); 
